@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -48,13 +48,13 @@
 #include <selection.h>
 #include <position.h>
 #include <ranges.h>
-#include <io-context.h>
 #include <command-context.h>
 #include <workbook-view.h>
 #include <workbook.h>
-#include <error-info.h>
 #include <parse-util.h>
 
+#include <goffice/app/io-context.h>
+#include <goffice/app/error-info.h>
 #include <gsf/gsf-input-textline.h>
 #include <string.h>
 #include <stdlib.h>
@@ -148,6 +148,68 @@ applix_parse_value (char *buf, char **follow)
 	return buf;
 }
 
+/* A..Z, AA..ZZ */
+#define APPLIX_SHEET_MAX_COLS 702
+
+static gboolean
+valid_col (int c)
+{
+	return c >= 0 && c < SHEET_MAX_COLS;
+}
+
+static gboolean
+valid_row (int r)
+{
+	return r >= 0 && r < SHEET_MAX_ROWS;
+}
+
+static gboolean
+valid_cellpos (const GnmCellPos *cpos)
+{
+	return valid_col (cpos->col) && valid_row (cpos->row);
+}
+
+static char const *
+applix_col_parse (char const *str, int *res, unsigned char *relative)
+{
+	char const *ptr, *start = str;
+	int col = -1;
+
+	if (!(*relative = (*start != '$')))
+		start++;
+
+	for (ptr = start; col < APPLIX_SHEET_MAX_COLS ; ptr++)
+		if (('a' <= *ptr && *ptr <= 'z'))
+			col = 26 * (col + 1) + (*ptr - 'a');
+		else if (('A' <= *ptr && *ptr <= 'Z'))
+			col = 26 * (col + 1) + (*ptr - 'A');
+		else if (ptr != start) {
+			*res = col;
+			return ptr;
+		} else
+			return NULL;
+	return NULL;
+}
+
+static char const *
+applix_cellpos_parse (char const *cell_str, GnmCellPos *res, gboolean strict)
+{
+	unsigned char dummy_relative;
+
+	cell_str = applix_col_parse (cell_str, &res->col, &dummy_relative);
+	if (!cell_str)
+		return NULL;
+
+	cell_str = row_parse (cell_str, &res->row, &dummy_relative);
+	if (!cell_str)
+		return NULL;
+
+	if (*cell_str != 0 && strict)
+		return NULL;
+
+	return cell_str;
+}
+
 static char const *
 applix_sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb)
 {
@@ -171,7 +233,8 @@ applix_sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb)
 }
 
 static char const *
-applix_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp)
+applix_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
+		       GnmExprConventions const *convention)
 {
 	char const *ptr = start, *tmp1, *tmp2;
 	Workbook *wb = pp->wb;
@@ -351,7 +414,7 @@ applix_get_color (ApplixReadState *state, char **buf)
 	int num = strtol (start, buf, 10);
 
 	if (start == *buf) {
-		(void) applix_parse_error (state, "Invalid color");
+		applix_parse_error (state, "Invalid color");
 		return NULL;
 	}
 
@@ -391,14 +454,14 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 	}
 	if ((is_protected || is_invisible)) {
 		if (*tmp != ' ') {
-			(void) applix_parse_error (state, "Invalid format, protection problem");
+			applix_parse_error (state, "Invalid format, protection problem");
 			return NULL;
 		}
 		tmp = ++start;
 	}
 
 	if (*tmp != '(') {
-		(void) applix_parse_error (state, "Invalid format, missing '('");
+		applix_parse_error (state, "Invalid format, missing '('");
 		return NULL;
 	}
 
@@ -406,7 +469,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 		;
 
 	if (tmp[0] != ')' || tmp[1] != ' ') {
-		(void) applix_parse_error (state, "Invalid format missing ')'");
+		applix_parse_error (state, "Invalid format missing ')'");
 		return NULL;
 	}
 
@@ -418,10 +481,10 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 		char *sep = start;
 
 		/* Allocate the new style */
-		style = mstyle_new_default ();
+		style = gnm_style_new_default ();
 
-		mstyle_set_content_locked (style, is_protected);
-		mstyle_set_content_hidden (style, is_invisible);
+		gnm_style_set_contents_locked (style, is_protected);
+		gnm_style_set_contents_hidden (style, is_invisible);
 
 		if (sep[1] == '\'')
 			sep += 2;
@@ -437,29 +500,29 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 			}
 
 			if (g_ascii_isdigit (*sep)) {
-				StyleHAlignFlags a;
+				GnmHAlign a;
 				switch (*sep) {
 				case '1' : a = HALIGN_LEFT; break;
 				case '2' : a = HALIGN_RIGHT; break;
 				case '3' : a = HALIGN_CENTER; break;
 				case '4' : a = HALIGN_FILL; break;
 				default :
-					(void) applix_parse_error (state, "Unknown horizontal alignment '%c'", *sep);
+					applix_parse_error (state, "Unknown horizontal alignment '%c'", *sep);
 					return NULL;
-				};
-				mstyle_set_align_h (style, a);
+				}
+				gnm_style_set_align_h (style, a);
 				++sep;
 			} else if (*sep == 'V') {
-				StyleVAlignFlags a;
+				GnmVAlign a;
 				switch (sep[1]) {
 				case 'T' : a = VALIGN_TOP; break;
 				case 'C' : a = VALIGN_CENTER; break;
 				case 'B' : a = VALIGN_BOTTOM; break;
 				default :
-					(void) applix_parse_error (state, "Unknown vertical alignment '%c'", *sep);
+					applix_parse_error (state, "Unknown vertical alignment '%c'", *sep);
 					return NULL;
-				};
-				mstyle_set_align_v (style, a);
+				}
+				gnm_style_set_align_v (style, a);
 				sep += 2;
 				break;
 			} else {
@@ -497,7 +560,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					    (0 == (id = strtol (sep+1, &end, 10))) ||
 					    sep+1 == end ||
 					    id < 1 || id > 16)
-						(void) applix_parse_error (state, "Unknown format %d", id);
+						applix_parse_error (state, "Unknown format %d", id);
 
 					format = date_formats[id - 1];
 					sep = end;
@@ -511,14 +574,14 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					case '2' : format = "hh:mm:ss";		break;
 					case '3' : format = "hh:mm";		break;
 					default :
-						(void) applix_parse_error (state, "Unknown time format '%c'", sep[1]);
+						applix_parse_error (state, "Unknown time format '%c'", sep[1]);
 						return NULL;
-					};
+					}
 					sep += 2;
 					break;
 				}
 				case 'G' : /* general */
-					mstyle_set_format_text (style, "General");
+					gnm_style_set_format_text (style, "General");
 
 					/* What is 'Gf' ? */
 					if (sep[1] == 'f')
@@ -562,7 +625,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					format = g_strconcat (format_prefix, decimal, prec,
 							      format_suffix, NULL);
 
-					mstyle_set_format_text (style, format);
+					gnm_style_set_format_text (style, format);
 					g_free (format);
 					break;
 				}
@@ -580,11 +643,11 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					   }
 					   /* Fall through */
 				default :
-					(void) applix_parse_error (state, "Unknown format '%c'", *sep);
+					applix_parse_error (state, "Unknown format '%c'", *sep);
 					return NULL;
-				};
+				}
 				if (format)
-					mstyle_set_format_text (style, format);
+					gnm_style_set_format_text (style, format);
 			}
 		}
 
@@ -594,19 +657,19 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 			/* check for the 1 character modifiers */
 			switch (*sep) {
 			case 'B' :
-				mstyle_set_font_bold (style, TRUE);
+				gnm_style_set_font_bold (style, TRUE);
 				++sep;
 				break;
 			case 'I' :
-				mstyle_set_font_italic (style, TRUE);
+				gnm_style_set_font_italic (style, TRUE);
 				++sep;
 				break;
 			case 'U' :
-				mstyle_set_font_uline (style, UNDERLINE_SINGLE);
+				gnm_style_set_font_uline (style, UNDERLINE_SINGLE);
 				++sep;
 				break;
 			case 'D' :
-				mstyle_set_font_uline (style, UNDERLINE_DOUBLE);
+				gnm_style_set_font_uline (style, UNDERLINE_DOUBLE);
 				++sep;
 				break;
 			case 'f' :
@@ -614,8 +677,8 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					/* TODO : what is this ?? */
 					sep += 2;
 					break;
-				};
-				(void) applix_parse_error (state, "Unknown font modifier 'f%c'", sep[1]);
+				}
+				applix_parse_error (state, "Unknown font modifier 'f%c'", sep[1]);
 				return NULL;
 
 			case 'F' :
@@ -623,10 +686,10 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					GnmColor *color = applix_get_color (state, &sep);
 					if (color == NULL)
 						return NULL;
-					mstyle_set_color (style, MSTYLE_COLOR_FORE, color);
+					gnm_style_set_font_color (style, color);
 					break;
 				}
-				(void) applix_parse_error (state, "Unknown font modifier F%c", sep[1]);
+				applix_parse_error (state, "Unknown font modifier F%c", sep[1]);
 				return NULL;
 
 			case 'P' : {
@@ -634,10 +697,10 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				double size = strtod (start, &sep);
 
 				if (start != sep && size > 0.) {
-					mstyle_set_font_size (style, size / gnm_app_dpi_to_pixels ());
+					gnm_style_set_font_size (style, size / gnm_app_dpi_to_pixels ());
 					break;
 				}
-				(void) applix_parse_error (state, "Invalid font size '%s", start);
+				applix_parse_error (state, "Invalid font size '%s", start);
 				return NULL;
 			}
 
@@ -648,11 +711,11 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 						sep +=3;
 						break;
 					}
-					mstyle_set_wrap_text (style, TRUE);
+					gnm_style_set_wrap_text (style, TRUE);
 					sep +=2;
 					break;
 				}
-				(void) applix_parse_error (state, "Unknown font modifier W%c", sep[1]);
+				applix_parse_error (state, "Unknown font modifier W%c", sep[1]);
 				return NULL;
 
 			case 'T' :
@@ -662,26 +725,26 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 
 					font_id = strtol (start, &sep, 10);
 					if (start == sep || font_id < 0 || font_id >= (int)state->font_names->len)
-						(void) applix_parse_error (state, "Unknown font index %s", start);
+						applix_parse_error (state, "Unknown font index %s", start);
 					break;
 				}
 
 
 			default :
-				(void) applix_parse_error (state, "Unknown font modifier");
+				applix_parse_error (state, "Unknown font modifier");
 				return NULL;
-			};
+			}
 
 			if (*sep == ',')
 				++sep;
 		}
 
 		if (*sep != '|' && *sep != ')') {
-			(void) applix_parse_error (state, "Invalid font specification");
+			applix_parse_error (state, "Invalid font specification");
 			return NULL;
 		}
 
-		mstyle_set_font_name (style, g_ptr_array_index (state->font_names, font_id));
+		gnm_style_set_font_name (style, g_ptr_array_index (state->font_names, font_id));
 
 		/* Background, pattern, and borders */
 		for (++sep ; *sep && *sep != ')' ; ) {
@@ -699,26 +762,26 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				int num = strtol (sep += 2, &end, 10);
 
 				if (sep == end || 0 >= num || num >= (int)G_N_ELEMENTS (map)) {
-					(void) applix_parse_error (state, "Unknown pattern %s", sep);
+					applix_parse_error (state, "Unknown pattern %s", sep);
 					return NULL;
 				}
 
 				num = map[num];
-				mstyle_set_pattern (style, num);
+				gnm_style_set_pattern (style, num);
 				sep = end;
 
 				if (sep[0] == 'F' && sep[1] == 'G' ) {
 					GnmColor *color = applix_get_color (state, &sep);
 					if (color == NULL)
 						return NULL;
-					mstyle_set_color (style, MSTYLE_COLOR_PATTERN, color);
+					gnm_style_set_pattern_color (style, color);
 				}
 
 				if (sep[0] == 'B' && sep[1] == 'G') {
 					GnmColor *color = applix_get_color (state, &sep);
 					if (color == NULL)
 						return NULL;
-					mstyle_set_color (style, MSTYLE_COLOR_BACK, color);
+					gnm_style_set_back_color (style, color);
 				}
 			} else if (sep[0] == 'T' || sep[0] == 'B' || sep[0] == 'L' || sep[0] == 'R') {
 				/* A map from applix border indicies to gnumeric. */
@@ -731,7 +794,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				};
 
 				GnmColor *color;
-				MStyleElementType const type =
+				GnmStyleElement const type =
 					(sep[0] == 'T') ? MSTYLE_BORDER_TOP :
 					(sep[0] == 'B') ? MSTYLE_BORDER_BOTTOM :
 					(sep[0] == 'L') ? MSTYLE_BORDER_LEFT : MSTYLE_BORDER_RIGHT;
@@ -741,7 +804,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				int num = strtol (++sep, &end, 10);
 
 				if (sep == end || 0 >= num || num >= (int)G_N_ELEMENTS (map)) {
-					(void) applix_parse_error (state, "Unknown border style %s", sep);
+					applix_parse_error (state, "Unknown border style %s", sep);
 					return NULL;
 				}
 				sep = end;
@@ -753,20 +816,20 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				} else
 					color = style_color_black ();
 
-				mstyle_set_border (style, type,
+				gnm_style_set_border (style, type,
 						   style_border_fetch (map[num], color, orient));
 			}
 
 			if (*sep == ',')
 				++sep;
 			else if (*sep != ')') {
-				(void) applix_parse_error (state, "Invalid pattern, background, or border");
+				applix_parse_error (state, "Invalid pattern, background, or border");
 				return NULL;
 			}
 		}
 
 		if (*sep != ')') {
-			(void) applix_parse_error (state, "Invalid pattern or background");
+			applix_parse_error (state, "Invalid pattern or background");
 			return NULL;
 		}
 
@@ -777,7 +840,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 	g_return_val_if_fail (style != NULL, NULL);
 
 	*buffer = tmp + 2;
-	mstyle_ref (style);
+	gnm_style_ref (style);
 	return style;
 }
 
@@ -816,8 +879,8 @@ applix_fetch_sheet (ApplixReadState *state, char const *name)
 
 	if (sheet == NULL) {
 		sheet = sheet_new (state->wb, name);
-		workbook_sheet_attach (state->wb, sheet, NULL);
-		sheet_set_zoom_factor (sheet, (double )(state->zoom) / 100., FALSE, FALSE);
+		workbook_sheet_attach (state->wb, sheet);
+		g_object_set (sheet, "zoom-factor", state->zoom / 100.0, NULL);
 		sheet_flag_recompute_spans (sheet);
 	}
 
@@ -834,7 +897,7 @@ applix_parse_sheet (ApplixReadState *state, unsigned char **buffer,
 	char *tmp = strchr (*buffer, separator);
 
 	if (tmp == NULL) {
-		(void) applix_parse_error (state, "Invalid sheet name.");
+		applix_parse_error (state, "Invalid sheet name.");
 		return NULL;
 	}
 
@@ -853,7 +916,7 @@ applix_parse_cellref (ApplixReadState *state, unsigned char *buffer,
 
 	/* Get cell addr */
 	if (*sheet) {
-		buffer = (unsigned char *)cellpos_parse (buffer, pos, FALSE);
+		buffer = (unsigned char *)applix_cellpos_parse (buffer, pos, FALSE);
 		if (buffer)
 			return buffer;
 	}
@@ -911,12 +974,14 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 
 		if (!a_strncmp (buffer, "View Top Left: ")) {
 			GnmCellPos pos;
-			if (applix_parse_cellref (state, buffer+15, &sheet, &pos, ':'))
+			if (applix_parse_cellref (state, buffer+15, &sheet, &pos, ':') &&
+			    valid_cellpos (&pos))
 				sv_set_initial_top_left (sheet_get_view (sheet, state->wb_view),
 							 pos.col, pos.row);
 		} else if (!a_strncmp (buffer, "View Open Cell: ")) {
 			GnmCellPos pos;
-			if (applix_parse_cellref (state, buffer+16, &sheet, &pos, ':'))
+			if (applix_parse_cellref (state, buffer+16, &sheet, &pos, ':') &&
+			    valid_cellpos (&pos))
 				sv_selection_set (sheet_get_view (sheet, state->wb_view),
 						  &pos, pos.col, pos.row, pos.col, pos.row);
 		} else if (!a_strncmp (buffer, "View Default Column Width ")) {
@@ -1009,16 +1074,23 @@ applix_read_cells (ApplixReadState *state)
 		if (style == NULL)
 			return -1;
 		if (ptr == NULL) {
-			mstyle_unref (style);
+			gnm_style_unref (style);
 			return -1;
 		}
 
 		/* Get cell */
 		ptr = applix_parse_cellref (state, ptr, &sheet, &pos, '!');
 		if (ptr == NULL) {
-			mstyle_unref (style);
+			gnm_style_unref (style);
 			return applix_parse_error (state, "Expression did not specify target cell");
 		}
+
+		if (!valid_cellpos (&pos)) {
+			gnm_style_unref (style);
+			g_warning ("Ignoring sheet contents beyond allowed range.");
+			continue;
+		}
+
 		cell = sheet_cell_fetch (sheet, pos.col, pos.row);
 
 		/* Apply the formating */
@@ -1028,7 +1100,6 @@ applix_read_cells (ApplixReadState *state)
 		case ';' : /* First of a shared formula */
 		case '.' : { /* instance of a shared formula */
 			GnmParsePos	 pos;
-			GnmExpr	const	*expr;
 			GnmValue		*val = NULL;
 			GnmRange		 r;
 			char *expr_string;
@@ -1052,6 +1123,7 @@ applix_read_cells (ApplixReadState *state)
 #endif
 
 			if (content_type == ';') {
+				GnmExprTop const *texpr;
 				gboolean	is_array = FALSE;
 
 				if (*expr_string == '~') {
@@ -1059,22 +1131,27 @@ applix_read_cells (ApplixReadState *state)
 					tmp = applix_parse_cellref (state, expr_string+1, &start_sheet,
 								    &r.start, ':');
 					if (start_sheet == NULL || tmp == NULL || tmp[0] != '.' || tmp[1] != '.') {
-						(void) applix_parse_error (state, "Invalid array expression");
+						applix_parse_error (state, "Invalid array expression");
 						continue;
 					}
 
 					tmp = applix_parse_cellref (state, tmp+2, &end_sheet,
 								    &r.end, ':');
 					if (end_sheet == NULL || tmp == NULL || tmp[0] != '~') {
-						(void) applix_parse_error (state, "Invalid array expression");
+						applix_parse_error (state, "Invalid array expression");
 						continue;
 					}
 
 					if (start_sheet != end_sheet) {
-						(void) applix_parse_error (state, "3D array functions are not supported.");
+						applix_parse_error (state, "3D array functions are not supported.");
 						continue;
 					}
 
+					if (!valid_cellpos (&r.start) || !valid_cellpos (&r.end)) {
+						g_warning ("Ignoring sheet contents beyond allowed range.");
+						continue;						
+					}
+
 					is_array = TRUE;
 					expr_string = tmp+3; /* ~addr~<space><space>=expr */
 				}
@@ -1084,52 +1161,57 @@ applix_read_cells (ApplixReadState *state)
 				 * 'Formula ' lines confuse the parser
 				 */
 				if (*expr_string != '=' && *expr_string != '+') {
-					(void) applix_parse_error (state, _("Expression did not start with '=' ? '%s'"),
+					applix_parse_error (state, _("Expression did not start with '=' ? '%s'"),
 								   expr_string);
-					expr = gnm_expr_new_constant (value_new_string (expr_string));
+					texpr = gnm_expr_top_new_constant (value_new_string (expr_string));
 				} else
-					expr = gnm_expr_parse_str (expr_string+1,
+					texpr = gnm_expr_parse_str (expr_string+1,
 						parse_pos_init_cell (&pos, cell),
 								   GNM_EXPR_PARSE_DEFAULT,
 								   state->exprconv,
 								   parse_error_init (&perr));
 
-				if (expr == NULL) {
-					(void) applix_parse_error (state, _("%s!%s : unable to parse '%s'\n     %s"),
+				if (texpr == NULL) {
+					applix_parse_error (state, _("%s!%s : unable to parse '%s'\n     %s"),
 								   cell->base.sheet->name_quoted, cell_name (cell),
 								   expr_string, perr.err->message);
 					parse_error_free (&perr);
-					expr = gnm_expr_new_constant (value_new_string (expr_string));
+					texpr = gnm_expr_top_new_constant (value_new_string (expr_string));
 				} else if (is_array) {
-					gnm_expr_ref (expr);
+					gnm_expr_top_ref (texpr);
 					cell_set_array_formula (sheet,
 								r.start.col, r.start.row,
 								r.end.col, r.end.row,
-								expr);
+								texpr);
 					cell_assign_value (cell, val);
-				} else
-					cell_set_expr_and_value (cell, expr, val, TRUE);
+					/* Leak? */
+				} else {
+					cell_set_expr_and_value (cell, texpr, val, TRUE);
+					/* Leak? */
+				}
 
 				if (!applix_get_line (state) ||
 				    a_strncmp (state->buffer, "Formula: ")) {
-					(void) applix_parse_error (state, "Missing formula ID");
+					applix_parse_error (state, "Missing formula ID");
 					continue;
 				}
 
 				ptr = state->buffer + 9;
 
 				/* Store the newly parsed expresion along with its descriptor */
-				g_hash_table_insert (state->exprs, g_strdup (ptr),
-						     (gpointer)expr);
+				g_hash_table_insert (state->exprs,
+						     g_strdup (ptr),
+						     (gpointer)texpr);
 			} else {
+				GnmExprTop const *texpr;
 				char const *key = expr_string + strlen (expr_string);
 				while (key > expr_string && !g_ascii_isspace (key[-1]))
 					key--;
 #if 0
 				printf ("Shared '%s'\n", expr_string);
 #endif
-				expr = g_hash_table_lookup (state->exprs, key);
-				cell_set_expr_and_value (cell, expr, val, TRUE);
+				texpr = g_hash_table_lookup (state->exprs, key);
+				cell_set_expr_and_value (cell, texpr, val, TRUE);
 			}
 			break;
 		}
@@ -1156,7 +1238,7 @@ applix_read_cells (ApplixReadState *state)
 
 		default :
 			g_warning ("Unknown cell type '%c'", content_type);
-		};
+		}
 	}
 
 	return 0;
@@ -1191,7 +1273,7 @@ applix_read_row_list (ApplixReadState *state, unsigned char *ptr)
 		attr_index = strtol (ptr = tmp+1, (char **)&tmp, 10);
 		if (tmp != ptr && attr_index >= 2 && attr_index < state->attrs->len+2) {
 			GnmStyle *style = g_ptr_array_index(state->attrs, attr_index-2);
-			mstyle_ref (style);
+			gnm_style_ref (style);
 			sheet_style_set_range (sheet, &r, style);
 		} else if (attr_index != 1) /* TODO : What the hell is attr 1 ?? */
 			return applix_parse_error (state, "Invalid row format attr index");
@@ -1248,7 +1330,7 @@ applix_read_absolute_name (ApplixReadState *state, char *buffer)
 	char *end;
 	GnmRangeRef ref;
 	GnmParsePos pp;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 
 	/* .ABCDe. Coordinate: A:B2..A:C4 */
 	/* Spec guarantees that there are no dots in the name */
@@ -1263,12 +1345,14 @@ applix_read_absolute_name (ApplixReadState *state, char *buffer)
 	if (end == NULL)
 		return TRUE;
 	applix_rangeref_parse (&ref, end+2,
-		parse_pos_init (&pp, state->wb, NULL, 0, 0));
+		parse_pos_init (&pp, state->wb, NULL, 0, 0),
+		state->exprconv);
 	ref.a.col_relative = ref.b.col_relative =
 		ref.a.row_relative = ref.b.row_relative = FALSE;
 
-	expr = gnm_expr_new_constant (value_new_cellrange_unsafe (&ref.a, &ref.b));
-	expr_name_add (&pp, buffer, expr, NULL, TRUE);
+	texpr = gnm_expr_top_new_constant
+		(value_new_cellrange_unsafe (&ref.a, &ref.b));
+	expr_name_add (&pp, buffer, texpr, NULL, TRUE, NULL);
 
 	return FALSE;
 }
@@ -1280,7 +1364,7 @@ applix_read_relative_name (ApplixReadState *state, char *buffer)
 	char *end;
 	GnmRangeRef ref, flag;
 	GnmParsePos pp;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 
 	/* .abcdE. tCol:0 tRow:0 tSheet:0 bCol:1 bRow:2 bSheet: 0 tColAbs:0 tRowAbs:0 tSheetAbs:1 bColAbs:0 bRowAbs:0 bSheetAbs:1 */
 	/* Spec guarantees that there are no dots in the name */
@@ -1303,10 +1387,11 @@ applix_read_relative_name (ApplixReadState *state, char *buffer)
 	ref.b.row_relative = (flag.b.row == 0);
 
 	ref.a.sheet = ref.b.sheet = NULL;
-	expr = gnm_expr_new_constant (value_new_cellrange_unsafe (&ref.a, &ref.b));
+	texpr = gnm_expr_top_new_constant
+		(value_new_cellrange_unsafe (&ref.a, &ref.b));
 	parse_pos_init (&pp, state->wb, NULL,
 		MAX (-ref.a.col, 0), MAX (-ref.a.row, 0));
-	expr_name_add (&pp, buffer, expr, NULL, TRUE);
+	expr_name_add (&pp, buffer, texpr, NULL, TRUE, NULL);
 
 	return FALSE;
 }
@@ -1437,24 +1522,25 @@ applix_read_impl (ApplixReadState *state)
 		return -1;
 
 	/* We only need the sheet, the visible cell, and edit pos are already set */
-	if (applix_parse_cellref (state, cur_cell_addr, &sheet, &pos, ':'))
+	if (applix_parse_cellref (state, cur_cell_addr, &sheet, &pos, ':') &&
+	    valid_cellpos (&pos))
 		wb_view_sheet_focus (state->wb_view, sheet);
 
 	return 0;
 }
 
 static gboolean
-cb_remove_expr (gpointer key, gpointer value, gpointer user_data)
+cb_remove_texpr (gpointer key, gpointer value, gpointer user_data)
 {
 	g_free (key);
-	gnm_expr_unref (value);
+	gnm_expr_top_unref (value);
 	return TRUE;
 }
 static gboolean
 cb_remove_style (gpointer key, gpointer value, gpointer user_data)
 {
 	g_free (key);
-	mstyle_unref (value);
+	gnm_style_unref (value);
 	return TRUE;
 }
 
@@ -1465,7 +1551,7 @@ static struct {
 	{ "IPAYMT", "IPMT" },
 	{ "PAYMT", "PMT" },
 	{ "PPAYMT", "PPMT" },
-	{ 0, 0 }
+	{ NULL, NULL }
 };
 
 static GnmExpr const *
@@ -1495,7 +1581,7 @@ applix_conventions (void)
 {
 	GnmExprConventions *res = gnm_expr_conventions_new ();
 
-	res->ignore_whitespace = TRUE;
+	res->intersection_char = 0;
 	res->accept_hash_logicals = TRUE;
 	res->allow_absolute_sheet_references = TRUE;
 	res->range_sep_dotdot = TRUE;
@@ -1523,7 +1609,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	state.input	  = (GsfInputTextline *)gsf_input_textline_new (src);
 	state.parse_error = NULL;
 	state.wb_view     = wb_view;
-	state.wb          = wb_view_workbook (wb_view);
+	state.wb          = wb_view_get_workbook (wb_view);
 	state.exprs       = g_hash_table_new (&g_str_hash, &g_str_equal);
 	state.styles      = g_hash_table_new (&g_str_hash, &g_str_equal);
 	state.colors      = g_ptr_array_new ();
@@ -1556,7 +1642,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	renamed_sheets = g_slist_reverse (renamed_sheets);
 	workbook_sheet_rename (state.wb, renamed_sheets,
 			       state.real_names, 
-			       GNM_CMD_CONTEXT (io_context));
+			       GO_CMD_CONTEXT (io_context));
 	g_slist_free (renamed_sheets);
 	g_slist_foreach (state.std_names, (GFunc)g_free, NULL);
 	g_slist_free (state.std_names);
@@ -1564,7 +1650,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	g_slist_free (state.real_names);
 
 	/* Release the shared expressions and styles */
-	g_hash_table_foreach_remove (state.exprs, &cb_remove_expr, NULL);
+	g_hash_table_foreach_remove (state.exprs, &cb_remove_texpr, NULL);
 	g_hash_table_destroy (state.exprs);
 	g_hash_table_foreach_remove (state.styles, &cb_remove_style, NULL);
 	g_hash_table_destroy (state.styles);
@@ -1574,7 +1660,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	g_ptr_array_free (state.colors, TRUE);
 
 	for (i = state.attrs->len; --i >= 0 ; )
-		mstyle_unref (g_ptr_array_index(state.attrs, i));
+		gnm_style_unref (g_ptr_array_index(state.attrs, i));
 	g_ptr_array_free (state.attrs, TRUE);
 
 	for (i = state.font_names->len; --i >= 0 ; )
