@@ -60,7 +60,7 @@ dif_input_context_new (IOContext *io_context, Workbook *wb, GsfInput *input)
 
 	ctxt->line_no        = 1;
 	ctxt->line           = NULL;
-	ctxt->sheet          = workbook_sheet_add (wb, -1, FALSE);
+	ctxt->sheet          = workbook_sheet_add (wb, -1);
 	ctxt->converter      = g_iconv_open ("UTF-8", "ISO-8859-1");
 
 	io_progress_message (io_context, _("Reading file..."));
@@ -254,7 +254,7 @@ void
 dif_file_open (GOFileOpener const *fo, IOContext *io_context,
                WorkbookView *wbv, GsfInput *input)
 {
-	Workbook *wb = wb_view_workbook (wbv);
+	Workbook *wb = wb_view_get_workbook (wbv);
 	DifInputContext *ctxt = dif_input_context_new (io_context, wb, input);
 
 	workbook_set_saveinfo (wb, FILE_FL_MANUAL_REMEMBER,
