@@ -26,6 +26,7 @@
 #include <parse-util.h>
 #include <sheet.h>
 #include <workbook.h>
+#include <goffice/app/go-doc.h>
 #include <gsf/gsf-utils.h>
 
 #undef G_LOG_DOMAIN
@@ -49,7 +50,7 @@ ExcelFuncDesc const excel_func_desc [] = {
 /* 8 */	  { "ROW",		 0,  1, XL_STD, 1, 'V', "R" },
 /* 9 */	  { "COLUMN",		 0,  1, XL_STD, 1, 'V', "R" },
 
-/* 10 */  { "NA",		 0,  0, XL_STD,  0, 'V', 0 },
+/* 10 */  { "NA",		 0,  0, XL_STD,  0, 'V', NULL },
 /* 11 */  { "NPV",		 2, 30, XL_STD, 2, 'V', "VR" },
 /* 12 */  { "STDEV",		 1, 30, XL_STD, 1, 'V', "R" },
 /* 13 */  { "DOLLAR",		 1,  2, XL_STD, 1, 'V', "V" },
@@ -58,7 +59,7 @@ ExcelFuncDesc const excel_func_desc [] = {
 /* 16 */  { "COS",		 1,  1, XL_STD,  1, 'V', "V" },
 /* 17 */  { "TAN",		 1,  1, XL_STD,  1, 'V', "V" },
 /* 18 */  { "ATAN",		 1,  1, XL_STD,  1, 'V', "V" },
-/* 19 */  { "PI",		 0,  0, XL_STD,  0, 'V', 0 },
+/* 19 */  { "PI",		 0,  0, XL_STD,  0, 'V', NULL },
 
 /* 20 */  { "SQRT",		 1,  1, XL_STD,  1, 'V', "V" },
 /* 21 */  { "EXP",		 1,  1, XL_STD,  1, 'V', "V" },
@@ -75,8 +76,8 @@ ExcelFuncDesc const excel_func_desc [] = {
 /* 31 */  { "MID",		 3,  3, XL_STD,  3, 'V', "VVV" },
 /* 32 */  { "LEN",		 1,  1, XL_STD,  1, 'V', "V" },
 /* 33 */  { "VALUE",		 1,  1, XL_STD,  1, 'V', "V" },
-/* 34 */  { "TRUE",		 0,  0, XL_STD,  0, 'V', 0 },
-/* 35 */  { "FALSE",		 0,  0, XL_STD,  0, 'V', 0 },
+/* 34 */  { "TRUE",		 0,  0, XL_STD,  0, 'V', NULL },
+/* 35 */  { "FALSE",		 0,  0, XL_STD,  0, 'V', NULL },
 /* 36 */  { "AND",		 1, 30, XL_STD, 1, 'V', "R" },
 /* 37 */  { "OR",		 1, 30, XL_STD, 1, 'V', "R" },
 /* 38 */  { "NOT",		 1,  1, XL_STD,  1, 'V', "V" },
@@ -107,7 +108,7 @@ ExcelFuncDesc const excel_func_desc [] = {
 /* 60 */  { "RATE",		 3,  6, XL_STD, 6, 'V', "VVVVVV" },	/* guess is optional */
 /* 61 */  { "MIRR",		 3,  3, XL_STD,  3, 'V', "RVV" },
 /* 62 */  { "IRR",		 1,  2, XL_STD, 2, 'V', "RV" },	/* guess is optional */
-/* 63 */  { "RAND", 		 0,  0, XL_VOLATILE,  0, 'V',  0 },
+/* 63 */  { "RAND", 		 0,  0, XL_VOLATILE,  0, 'V',  NULL },
 /* 64 */  { "MATCH",		 2,  3, XL_STD, 3, 'V', "VRR" },	/* match_type is optional */
 /* 65 */  { "DATE",		 3,  3, XL_STD,  3, 'V', "VVV" },
 /* 66 */  { "TIME",		 3,  3, XL_STD,  3, 'V', "VVV" },
@@ -119,7 +120,7 @@ ExcelFuncDesc const excel_func_desc [] = {
 /* 71 */  { "HOUR",		 1,  1, XL_STD,  1, 'V', "V" },
 /* 72 */  { "MINUTE",		 1,  1, XL_STD,  1, 'V', "V" },
 /* 73 */  { "SECOND",		 1,  1, XL_STD,  1, 'V', "V" },
-/* 74 */  { "NOW",  		 0,  0, XL_VOLATILE,  0, 'V',  0 },
+/* 74 */  { "NOW",  		 0,  0, XL_VOLATILE,  0, 'V',  NULL },
 /* 75 */  { "AREAS",		 1,  1, XL_STD,  1, 'V', "R" },
 /* 76 */  { "ROWS",		 1,  1, XL_STD,  1, 'V', "R" },
 /* 77 */  { "COLUMNS",		 1,  1, XL_STD,  1, 'V', "R" },
@@ -283,7 +284,7 @@ ExcelFuncDesc const excel_func_desc [] = {
 /* 219 */ { "ADDRESS",		 2,  5, XL_STD, 5, 'V', "VVVVV" },
 
 /* 220 */ { "DAYS360",		 2,  3, XL_STD, 3, 'V', "VVV" },	/* pre-biff5/7 VV */
-/* 221 */ { "TODAY", 		 0,  0, XL_VOLATILE,  0, 'V', 0 },
+/* 221 */ { "TODAY", 		 0,  0, XL_VOLATILE,  0, 'V', NULL },
 /* 222 */ { "VDB",		 5,  7, XL_STD, 7, 'V', "VVVVVVV" },
 /* 223 */ { "ELSE",		-1, -1, XL_XLM },
 /* 224 */ { "ELSE.IF",		-1, -1, XL_XLM },
@@ -620,7 +621,7 @@ static void
 parse_list_free (GnmExprList **list)
 {
 	while (*list)
-		gnm_expr_unref (parse_list_pop(list));
+		gnm_expr_free (parse_list_pop(list));
 }
 
 static gboolean
@@ -640,16 +641,16 @@ make_function (GnmExprList **stack, int fn_idx, int numargs, Workbook *wb)
 		char const *f_name = NULL;
 
 		if (tmp != NULL) {
-			if (tmp->any.oper == GNM_EXPR_OP_CONSTANT &&
-			    tmp->constant.value->type == VALUE_STRING)
+			if (GNM_EXPR_GET_OPER (tmp) == GNM_EXPR_OP_CONSTANT &&
+			    VALUE_IS_STRING (tmp->constant.value))
 				f_name = tmp->constant.value->v_str.val->str;
-			else if (tmp->any.oper == GNM_EXPR_OP_NAME)
+			else if (GNM_EXPR_GET_OPER (tmp) == GNM_EXPR_OP_NAME)
 				f_name = tmp->name.name->name->str;
 		}
 
 		if (f_name == NULL) {
 			if (tmp)
-				gnm_expr_unref (tmp);
+				gnm_expr_free (tmp);
 			parse_list_free (&args);
 			parse_list_push_raw (stack,
 				value_new_error (NULL, _("Broken function")));
@@ -661,7 +662,7 @@ make_function (GnmExprList **stack, int fn_idx, int numargs, Workbook *wb)
 		if (name == NULL)
 			name = gnm_func_add_placeholder (wb, f_name, "UNKNOWN", TRUE);
 
-		gnm_expr_unref (tmp);
+		gnm_expr_free (tmp);
 		parse_list_push (stack, gnm_expr_new_funcall (name, args));
 		return TRUE;
 	} else if (fn_idx >= 0 && fn_idx < excel_func_desc_size) {
@@ -692,7 +693,7 @@ make_function (GnmExprList **stack, int fn_idx, int numargs, Workbook *wb)
 		if (fd->name) {
 			name = gnm_func_lookup (fd->name, wb);
 			if (name == NULL)
-				name = gnm_func_add_placeholder (wb, fd->name, "0NKNOWN", TRUE);
+				name = gnm_func_add_placeholder (wb, fd->name, "UNKNOWN", TRUE);
 		}
 		/* This should not happen */
 		if (!name) {
@@ -723,8 +724,8 @@ ms_excel_dump_cellname (GnmXLImporter const *importer, ExcelReadSheet const *esh
 {
 	if (esheet && esheet->sheet && esheet->sheet->name_unquoted)
 		fprintf (stderr, "%s!", esheet->sheet->name_unquoted);
-	else if (importer && importer->wb && workbook_get_uri (importer->wb)) {
-		fprintf (stderr, "[%s]", workbook_get_uri (importer->wb));
+	else if (importer && importer->wb && go_doc_get_uri (GO_DOC (importer->wb))) {
+		fprintf (stderr, "[%s]", go_doc_get_uri (GO_DOC (importer->wb)));
 		return;
 	}
 	fprintf (stderr, "%s%d : ", col_name(fn_col), fn_row+1);
@@ -846,13 +847,13 @@ static char const *ptg_name[] = {
  * Parse that RP Excel formula, see S59E2B.HTM
  * Return a dynamicly allocated GnmExpr containing the formula, or NULL
  **/
-GnmExpr const *
-excel_parse_formula (MSContainer const *container,
-		     ExcelReadSheet const *esheet,
-		     int fn_col, int fn_row,
-		     guint8 const *mem, guint16 length,
-		     gboolean shared,
-		     gboolean *array_element)
+static GnmExpr const *
+excel_parse_formula1 (MSContainer const *container,
+		      ExcelReadSheet const *esheet,
+		      int fn_col, int fn_row,
+		      guint8 const *mem, guint16 length,
+		      gboolean shared,
+		      gboolean *array_element)
 {
 	MsBiffVersion const ver = container->importer->ver;
 
@@ -923,12 +924,8 @@ excel_parse_formula (MSContainer const *container,
 				return NULL;
 			}
 
-#ifndef NO_DEBUG_EXCEL
-			if (ms_excel_formula_debug > 0) {
-				fprintf (stderr, "Parse shared formula\n");
-			}
-#endif
-			expr = excel_parse_formula (container, esheet, fn_col, fn_row,
+			d (0, fprintf (stderr, "Parse shared formula\n"););
+			expr = excel_parse_formula1 (container, esheet, fn_col, fn_row,
 				sf->data, sf->data_len, TRUE, array_element);
 
 			parse_list_push (&stack, expr);
@@ -984,13 +981,19 @@ excel_parse_formula (MSContainer const *container,
 
 			/* not exactly legal, but should be reasonable
 			 * XL has union operator we have sets.
-			 */
-			if (l->any.oper != GNM_EXPR_OP_SET) {
+ 			 */
+			if (GNM_EXPR_GET_OPER (l) != GNM_EXPR_OP_SET) {
 				GnmExprList *args = gnm_expr_list_prepend (NULL, r);
 				args = gnm_expr_list_prepend (args, l);
 				parse_list_push (&stack, gnm_expr_new_set (args));
 			} else {
-				gnm_expr_list_append (l->set.set, r);
+				/* Barf!  -- MW.  */
+				GnmExpr *ll = (GnmExpr *)l;
+				ll->set.argc++;
+				ll->set.argv = g_renew (GnmExprConstPtr,
+							ll->set.argv,
+							ll->set.argc);
+				ll->set.argv[ll->set.argc - 1] = r;
 				parse_list_push (&stack, l);
 			}
 			break;
@@ -1031,57 +1034,39 @@ excel_parse_formula (MSContainer const *container,
 					"bracket a 1x1 array formula.  please send us this file.\n"
 					"Flags = 0x%X\n", w);
 			} else if (grbit & 0x01) {
-#ifndef NO_DEBUG_EXCEL
-				if (ms_excel_formula_debug > 0) {
-					fprintf (stderr, "A volatile function: so what\n");
-				}
-#endif
-			} else if (grbit & 0x02) { /* AttrIf: 'optimised' IF function */
-				/* Who cares if the TRUE expr has a goto at the end */
-				GnmExpr const *tr;
-#ifndef NO_DEBUG_EXCEL
-				if (ms_excel_formula_debug > 2) {
-					fprintf (stderr, "Optimised IF 0x%x 0x%x\n", grbit, w);
-					gsf_mem_dump (mem, length);
-				}
-#endif
-				tr = w ? excel_parse_formula (container, esheet, fn_col, fn_row,
-					   cur+ptg_length, w, shared, NULL)
-					: gnm_expr_new_constant (value_new_string (""));
-				parse_list_push (&stack, tr);
-				ptg_length += w;
-			} else if (grbit & 0x04) { /* Optimized CHOOSE function */
-				/* Ignore the optimzation to specificy which arg to use */
-				ptg_length = 2 + GSF_LE_GET_GUINT8 (cur+1) + 1;
-			} else if (grbit & 0x08) { /* AttrGoto */
-#ifndef NO_DEBUG_EXCEL
-				if (ms_excel_formula_debug > 2) {
-					fprintf (stderr, "Goto %d: cur = 0x%x\n", w,
-						(int)(cur-mem));
-					gsf_mem_dump (mem, length);
-				}
-#endif
-				/* Not right prior to Excel 4.0 ? */
-				if (ver <= MS_BIFF_V3) break;
-				ptg_length = w;
-			} else if (grbit & 0x10) { /* AttrSum: 'optimised' SUM function */
+				d (2, fprintf (stderr, "A volatile function\n"););
+
+			/* AttrIf: stores jump to FALSE condition */
+			} else if (grbit & 0x02) {
+				/* Ignore cached result */
+				d (2, fprintf (stderr, "ATTR IF\n"););
+
+			/* AttrChoose : stores table of inputs */
+			} else if (grbit & 0x04) {
+				/* Ignore the optimzation to specify which arg to use */
+				d (2, fprintf (stderr, "ATTR CHOOSE\n"););
+				ptg_length = 2 * ((w + 1) /* args */ + 1 /* count */) + 1;
+
+			/* AttrGoto : bytes/words to skip during _evaluation_.
+			 * We still need to parse them */
+			} else if (grbit & 0x08) {
+				d (2, fprintf (stderr, "ATTR GOTO\n"););
+
+			/* AttrSum: 'optimised' SUM function */
+			} else if (grbit & 0x10) {
 				if (!make_function (&stack, 0x04, 1, container->importer->wb)) {
 					error = TRUE;
 					fprintf (stderr, "Error in optimised SUM\n");
 				}
-			} else if (grbit & 0x40) { /* AttrSpace */
-				guint8 num_space = GSF_LE_GET_GUINT8(cur+2);
-				guint8 attrs     = GSF_LE_GET_GUINT8(cur+1);
-				if (attrs == 00) /* bitFSpace : ignore it */
-				/* Could perhaps pop top arg & append space ? */;
+
+			/* AttrSpace */
+			} else if (grbit & 0x40) {
+				guint8 num_space = GSF_LE_GET_GUINT8 (cur+2);
+				guint8 attrs     = GSF_LE_GET_GUINT8 (cur+1);
+				if (attrs == 0) /* bitFSpace : ignore it for now */
+					;
 				else
-#ifndef NO_DEBUG_EXCEL
-					if (ms_excel_formula_debug > 1) {
-						fprintf (stderr, "Redundant whitespace in formula 0x%x count %d\n", attrs, num_space);
-					}
-#else
-				;
-#endif
+					d (2, fprintf (stderr, "Redundant whitespace in formula 0x%x count %d\n", attrs, num_space););
 			} else {
 				ms_excel_dump_cellname (container->importer, esheet, fn_col, fn_row);
 				fprintf (stderr, "Unknown PTG Attr gr = 0x%x, w = 0x%x ptg = 0x%x\n", grbit, w, ptg);
@@ -1106,12 +1091,12 @@ excel_parse_formula (MSContainer const *container,
 			break;
 
 		case FORMULA_PTG_INT:
-			parse_list_push_raw (&stack, value_new_int (GSF_LE_GET_GUINT16(cur)));
+			parse_list_push_raw (&stack, value_new_int (GSF_LE_GET_GUINT16 (cur)));
 			ptg_length = 2;
 			break;
 
 		case FORMULA_PTG_BOOL:
-			parse_list_push_raw (&stack, value_new_bool (GSF_LE_GET_GUINT8(cur)));
+			parse_list_push_raw (&stack, value_new_bool (GSF_LE_GET_GUINT8 (cur)));
 			ptg_length = 1;
 			break;
 
@@ -1237,7 +1222,7 @@ excel_parse_formula (MSContainer const *container,
 				break;
 			case 0x10 : ptg_length += 4;	/* eptgElfRadicalLel, No, Err */
 				/* does not seem to put anything on the stack */
-				gnm_expr_unref (
+				gnm_expr_free (
 					xl_expr_err (esheet, fn_col, fn_row,
 						     "undocumented extended ptg 0x10", "#REF!"));
 				break;
@@ -1345,10 +1330,10 @@ excel_parse_formula (MSContainer const *container,
 
 			if (ver >= MS_BIFF_V4) {
 				ptg_length = 2;
-				iftab = GSF_LE_GET_GUINT16(cur);
+				iftab = GSF_LE_GET_GUINT16 (cur);
 			} else {
 				ptg_length = 1;
-				iftab = GSF_LE_GET_GUINT8(cur);
+				iftab = GSF_LE_GET_GUINT8 (cur);
 			}
 
 			if (!make_function (&stack, iftab, -1, container->importer->wb)) {
@@ -1359,21 +1344,21 @@ excel_parse_formula (MSContainer const *container,
 		}
 
 		case FORMULA_PTG_FUNC_VAR: {
-			int const numargs = (GSF_LE_GET_GUINT8( cur ) & 0x7f);
+			int const numargs = (GSF_LE_GET_GUINT8 ( cur ) & 0x7f);
 			/* index into fn table */
 			int iftab;
 #if 0
 			/* Prompts the user ?  */
-			int const prompt  = (GSF_LE_GET_GUINT8( cur ) & 0x80);
+			int const prompt  = (GSF_LE_GET_GUINT8 ( cur ) & 0x80);
 			/* is a command equiv.?*/
-			int const cmdquiv = (GSF_LE_GET_GUINT16(cur+1) & 0x8000);
+			int const cmdquiv = (GSF_LE_GET_GUINT16 (cur+1) & 0x8000);
 #endif
 			if (ver >= MS_BIFF_V4) {
 				ptg_length = 3;
-				iftab = (GSF_LE_GET_GUINT16(cur+1) & 0x7fff);
+				iftab = (GSF_LE_GET_GUINT16 (cur+1) & 0x7fff);
 			} else {
 				ptg_length = 2;
-				iftab = GSF_LE_GET_GUINT8(cur+1);
+				iftab = GSF_LE_GET_GUINT8 (cur+1);
 			}
 
 			if (!make_function (&stack, iftab, numargs, container->importer->wb)) {
@@ -1442,14 +1427,14 @@ excel_parse_formula (MSContainer const *container,
 			GnmCellRef ref;
 			if (ver >= MS_BIFF_V8) {
 				getRefV8 (&ref,
-					  GSF_LE_GET_GUINT16(cur),
-					  GSF_LE_GET_GUINT16(cur + 2),
+					  GSF_LE_GET_GUINT16 (cur),
+					  GSF_LE_GET_GUINT16 (cur + 2),
 					  fn_col, fn_row, ptgbase == FORMULA_PTG_REFN);
 				ptg_length = 4;
 			} else {
 				getRefV7 (&ref,
-					  GSF_LE_GET_GUINT8(cur+2),
-					  GSF_LE_GET_GUINT16(cur),
+					  GSF_LE_GET_GUINT8 (cur+2),
+					  GSF_LE_GET_GUINT16 (cur),
 					  fn_col, fn_row, ptgbase == FORMULA_PTG_REFN);
 				ptg_length = 3;
 			}
@@ -1461,22 +1446,22 @@ excel_parse_formula (MSContainer const *container,
 			GnmCellRef first, last;
 			if (ver >= MS_BIFF_V8) {
 				getRefV8 (&first,
-					  GSF_LE_GET_GUINT16(cur+0),
-					  GSF_LE_GET_GUINT16(cur+4),
+					  GSF_LE_GET_GUINT16 (cur+0),
+					  GSF_LE_GET_GUINT16 (cur+4),
 					  fn_col, fn_row, ptgbase == FORMULA_PTG_AREAN);
 				getRefV8 (&last,
-					  GSF_LE_GET_GUINT16(cur+2),
-					  GSF_LE_GET_GUINT16(cur+6),
+					  GSF_LE_GET_GUINT16 (cur+2),
+					  GSF_LE_GET_GUINT16 (cur+6),
 					  fn_col, fn_row, ptgbase == FORMULA_PTG_AREAN);
 				ptg_length = 8;
 			} else {
 				getRefV7 (&first,
-					  GSF_LE_GET_GUINT8(cur+4),
-					  GSF_LE_GET_GUINT16(cur+0),
+					  GSF_LE_GET_GUINT8 (cur+4),
+					  GSF_LE_GET_GUINT16 (cur+0),
 					  fn_col, fn_row, ptgbase == FORMULA_PTG_AREAN);
 				getRefV7 (&last,
-					  GSF_LE_GET_GUINT8(cur+5),
-					  GSF_LE_GET_GUINT16(cur+2),
+					  GSF_LE_GET_GUINT8 (cur+5),
+					  GSF_LE_GET_GUINT16 (cur+2),
 					  fn_col, fn_row, ptgbase == FORMULA_PTG_AREAN);
 				ptg_length = 6;
 			}
@@ -1596,22 +1581,22 @@ excel_parse_formula (MSContainer const *container,
 
 			if (ver >= MS_BIFF_V8) {
 				getRefV8 (&first,
-					  GSF_LE_GET_GUINT16(cur+2),
-					  GSF_LE_GET_GUINT16(cur+6),
+					  GSF_LE_GET_GUINT16 (cur+2),
+					  GSF_LE_GET_GUINT16 (cur+6),
 					  fn_col, fn_row, 0);
 				getRefV8 (&last,
-					  GSF_LE_GET_GUINT16(cur+4),
-					  GSF_LE_GET_GUINT16(cur+8),
+					  GSF_LE_GET_GUINT16 (cur+4),
+					  GSF_LE_GET_GUINT16 (cur+8),
 					  fn_col, fn_row, 0);
 				ptg_length = 10;
 			} else {
 				getRefV7 (&first,
-					  GSF_LE_GET_GUINT8(cur+18),
-					  GSF_LE_GET_GUINT16(cur+14),
+					  GSF_LE_GET_GUINT8 (cur+18),
+					  GSF_LE_GET_GUINT16 (cur+14),
 					  fn_col, fn_row, shared);
 				getRefV7 (&last,
-					  GSF_LE_GET_GUINT8(cur+19),
-					  GSF_LE_GET_GUINT16(cur+16),
+					  GSF_LE_GET_GUINT8 (cur+19),
+					  GSF_LE_GET_GUINT16 (cur+16),
 					  fn_col, fn_row, shared);
 				ptg_length = 20;
 			}
@@ -1649,32 +1634,48 @@ excel_parse_formula (MSContainer const *container,
 	}
 
 	if (error) {
-		fprintf (stderr, "formula data : %s\n", (shared?" (shared)":"(NOT shared)"));
+		g_printerr ("formula data : %s\n", (shared?" (shared)":"(NOT shared)"));
 		gsf_mem_dump (mem, length);
 
 		parse_list_free (&stack);
 		return xl_expr_err (esheet, fn_col, fn_row,
-			"Unknown Formula/Array", "#Unknown!");
+				    "Unknown Formula/Array",
+				    "#Unknown!");
 	}
 
 	if (stack == NULL)
 		return xl_expr_err (esheet, fn_col, fn_row,
-			"Stack too short - unusual", "#ShortStack!");
+				    "Stack too short - unusual",
+				    "#ShortStack!");
 	if (gnm_expr_list_length (stack) > 1) {
 		parse_list_free (&stack);
 		return xl_expr_err (esheet, fn_col, fn_row,
-			"Too much data on stack - probable cause: fixed args function is var-arg",
-			"#LongStack!");
+				    "Too much data on stack - probable cause: fixed args function is var-arg",
+				    "#LongStack!");
 	}
 
 #ifndef NO_DEBUG_EXCEL
 	if (ms_excel_formula_debug > 0 && esheet != NULL) {
 		GnmParsePos pp;
-		GnmExpr const *expr = parse_list_pop (&stack);
+		GnmExpr const *expr = stack->data;
 		parse_pos_init (&pp, NULL, esheet->sheet, fn_col, fn_row);
-		fprintf (stderr, "%s\n",  gnm_expr_as_string (expr, &pp, gnm_expr_conventions_default));
-		return expr;
+		g_printerr ("%s\n",  gnm_expr_as_string (expr, &pp, gnm_expr_conventions_default));
 	}
 #endif
-	return expr_tree_sharer_share (container->importer->expr_sharer, parse_list_pop (&stack));
+	return parse_list_pop (&stack);
+}
+
+GnmExprTop const *
+excel_parse_formula (MSContainer const *container,
+		     ExcelReadSheet const *esheet,
+		     int fn_col, int fn_row,
+		     guint8 const *mem, guint16 length,
+		     gboolean shared,
+		     gboolean *array_element)
+{
+	return gnm_expr_top_new (excel_parse_formula1 (container, esheet,
+						       fn_col, fn_row,
+						       mem, length,
+						       shared,
+						       array_element));
 }
