@@ -4,7 +4,7 @@
  *
  * Author:
  *   Jukka-Pekka Iivonen (iivonen@iki.fi)
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -132,11 +132,11 @@ database_find_values (Sheet *sheet, GnmValue const *database,
 		GnmCell *cell = current->data;
 		GnmValue *value = cell->value;
 
-		if ((flags & COLLECT_IGNORE_STRINGS) && value->type == VALUE_STRING)
+		if ((flags & COLLECT_IGNORE_STRINGS) && VALUE_IS_STRING (value))
 			continue;
-		if ((flags & COLLECT_IGNORE_BOOLS) && value->type == VALUE_BOOLEAN)
+		if ((flags & COLLECT_IGNORE_BOOLS) && VALUE_IS_BOOLEAN (value))
 			continue;
-		if ((flags & COLLECT_IGNORE_BLANKS) && value->type == VALUE_EMPTY)
+		if ((flags & COLLECT_IGNORE_BLANKS) && VALUE_IS_EMPTY (value))
 			continue;
 		if (floats)
 			res1[count++] = value_get_as_float (value);
@@ -812,7 +812,7 @@ static GnmFuncHelp const help_dstdev[] = {
 
            "@DESCRIPTION="
            "DSTDEV function returns the estimate of the standard deviation "
-	   "of a population based on a sample. The populations consists of "
+	   "of a population based on a sample. The population consists of "
 	   "numbers that match conditions specified.\n"
 	   "\n"
 	   "@database is a range of cells in which rows of related " \
@@ -888,7 +888,7 @@ static GnmFuncHelp const help_dstdevp[] = {
 
            "@DESCRIPTION="
            "DSTDEVP function returns the standard deviation of a population "
-	   "based on the entire populations. The populations consists of "
+	   "based on the entire population. The population consists of "
 	   "numbers that match conditions specified.\n"
 	   "\n"
 	   "@database is a range of cells in which rows of related " \
@@ -1040,7 +1040,7 @@ static GnmFuncHelp const help_dvar[] = {
 
            "@DESCRIPTION="
            "DVAR function returns the estimate of variance of a population "
-	   "based on a sample. The populations consists of numbers "
+	   "based on a sample. The population consists of numbers "
 	   "that match conditions specified.\n"
 	   "\n"
 	   "@database is a range of cells in which rows of related " \
@@ -1116,7 +1116,7 @@ static GnmFuncHelp const help_dvarp[] = {
 
            "@DESCRIPTION="
            "DVARP function returns the variance of a population based "
-	   "on the entire populations. The populations consists of numbers "
+	   "on the entire population. The population consists of numbers "
 	   "that match conditions specified.\n"
 	   "\n"
 	   "@database is a range of cells in which rows of related " \
