@@ -6,7 +6,7 @@
  *   Michael Meeks <michael@ximian.com>
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
  *   JP Rosevear <jpr@arcavia.com>
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ find_type_valid (GnmValue const *find)
 	/* Excel does not lookup errors or blanks */
 	if (VALUE_IS_EMPTY (find))
 		return FALSE;
-	return VALUE_IS_NUMBER (find) || find->type == VALUE_STRING;
+	return VALUE_IS_NUMBER (find) || VALUE_IS_STRING (find);
 }
 
 static gboolean
@@ -363,68 +363,61 @@ static GnmFuncHelp const help_address[] = {
 static GnmValue *
 gnumeric_address (FunctionEvalInfo *ei, GnmValue const * const *args)
 {
-        int   row, col, abs_num, a1;
-	gchar *sheet_name, *buf;
-	char const *sheet_quote;
+	GnmExprConventions const *conv = gnm_expr_conventions_default;
+	GString		*res;
+	GnmCellRef	 ref;
+	GnmParsePos	 pp;
+	gboolean	 err;
+	int		 col, row;
 
-	row = value_get_as_int (args[0]);
-	col = value_get_as_int (args[1]);
+	switch (args[2] ? value_get_as_int (args[2]) : 1) {
+	case 1: case 5: ref.col_relative = ref.row_relative = FALSE; break;
+	case 2: case 6:
+		ref.col_relative = TRUE;
+		ref.row_relative = FALSE;
+		break;
+	case 3: case 7:
+		ref.col_relative = FALSE;
+		ref.row_relative = TRUE;
+		break;
+	case 4: case 8: ref.col_relative = ref.row_relative = TRUE; break;
 
-	if (row < 1 || SHEET_MAX_ROWS <= row ||
-	    col < 1 || SHEET_MAX_COLS <= col)
-	        return value_new_error_VALUE (ei->pos);
+	default :
+		return value_new_error_VALUE (ei->pos);
+	}
 
-	abs_num = args[2] ? value_get_as_int (args[2]) : 1;
-
-	if (args[3] == NULL)
-	        a1 = 1;
-	else {
-		gboolean err;
-	        a1 = value_get_as_bool (args[3], &err);
+	ref.sheet = NULL;
+	row = ref.row = value_get_as_int (args[0]) - 1;
+	col = ref.col = value_get_as_int (args[1]) - 1;
+	parse_pos_init_evalpos (&pp, ei->pos);
+	if (NULL != args[3]) {
+		/* MS Excel is ridiculous.  This is a special case */
+		if (!value_get_as_bool (args[3], &err)) {
+			conv = gnm_expr_conventions_r1c1;
+			if (ref.col_relative)
+				col = ei->pos->eval.col + (++ref.col);
+			if (ref.row_relative)
+				row = ei->pos->eval.row + (++ref.row);
+		}
 		if (err)
 		        return value_new_error_VALUE (ei->pos);
 	}
-
-	sheet_name = (args[4] != NULL)
-		? sheet_name_quote (value_peek_string (args[4]))
-		: g_strdup ("");
-	sheet_quote = *sheet_name ? "!" : "";
-
-	buf = g_new (gchar, strlen (sheet_name) + 1 + 50);
-	switch (abs_num) {
-	case 1: case 5:
-	        if (a1)
-		        sprintf (buf, "%s%s$%s$%d", sheet_name, sheet_quote, col_name (col - 1),
-				 row);
-		else
-		        sprintf (buf, "%s%sR%dC%d", sheet_name, sheet_quote, row, col);
-		break;
-	case 2: case 6:
-	        if (a1)
-		        sprintf (buf, "%s%s%s$%d", sheet_name, sheet_quote, col_name (col - 1), row);
-		else
-		        sprintf (buf, "%s%sR%dC[%d]", sheet_name, sheet_quote, row, col);
-		break;
-	case 3: case 7:
-	        if (a1)
-		        sprintf (buf, "%s%s$%s%d", sheet_name, sheet_quote, col_name (col - 1), row);
-		else
-		        sprintf (buf, "%s%sR[%d]C%d", sheet_name, sheet_quote, row, col);
-		break;
-	case 4: case 8:
-	        if (a1)
-		        sprintf (buf, "%s%s%s%d", sheet_name, sheet_quote, col_name (col - 1), row);
-		else
-		        sprintf (buf, "%s%sR[%d]C[%d]", sheet_name, sheet_quote, row, col);
-		break;
-	default:
-	        g_free (sheet_name);
-	        g_free (buf);
+	if (col < 0 || col >= SHEET_MAX_COLS)
+		return value_new_error_VALUE (ei->pos);
+	if (row < 0 || row >= SHEET_MAX_ROWS)
 		return value_new_error_VALUE (ei->pos);
-	}
-	g_free (sheet_name);
 
-	return value_new_string_nocopy (buf);
+	if (NULL != args[4]) {
+		res = gnm_expr_conv_quote (gnm_expr_conventions_default,
+			value_peek_string (args[4]));
+		g_string_append_c (res, '!');
+	} else
+		res = g_string_new (NULL);
+	if (conv != gnm_expr_conventions_r1c1)
+		pp.eval.col = pp.eval.row = 0;
+	cellref_as_string (res, conv, &ref, &pp, TRUE);
+
+	return value_new_string_nocopy (g_string_free (res, FALSE));
 }
 
 /***************************************************************************/
@@ -449,19 +442,20 @@ static GnmFuncHelp const help_areas[] = {
 
 /* TODO : we need to rethink EXPR_SET as an operator vs a value type */
 static GnmValue *
-gnumeric_areas (FunctionEvalInfo *ei, GnmExprList const *l)
+gnumeric_areas (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	GnmExpr const *expr;
 	int res = -1;
-	int argc =  gnm_expr_list_length (l);
 
-	if (argc < 1 || l->data == NULL || argc > 1)
+	if (argc != 1 || argv[0] == NULL)
 		return value_new_error_VALUE (ei->pos);
-	expr = l->data;
+	expr = argv[0];
 
-restart :
-	switch (expr->any.oper) {
+ restart:
+	switch (GNM_EXPR_GET_OPER (expr)) {
 	case GNM_EXPR_OP_CONSTANT:
+		if (VALUE_IS_ERROR (expr->constant.value))
+			return value_dup (expr->constant.value);
 		if (expr->constant.value->type != VALUE_CELLRANGE)
 			break;
 
@@ -472,7 +466,8 @@ restart :
 		break;
 	case GNM_EXPR_OP_ANY_BINARY:
 	case GNM_EXPR_OP_ANY_UNARY:
-	case GNM_EXPR_OP_ARRAY:
+	case GNM_EXPR_OP_ARRAY_CORNER:
+	case GNM_EXPR_OP_ARRAY_ELEM:
 		break;
 
 	case GNM_EXPR_OP_FUNCALL: {
@@ -486,18 +481,18 @@ restart :
 
 	case GNM_EXPR_OP_NAME:
 		if (expr->name.name->active) {
-			expr = expr->name.name->expr;
+			expr = expr->name.name->texpr->expr;
 			goto restart;
 		}
 		break;
 
 	case GNM_EXPR_OP_SET:
-		res = gnm_expr_list_length (expr->set.set);
+		res = expr->set.argc;
 		break;
 
 	default:
 		g_warning ("unknown expr type.");
-	};
+	}
 
 	if (res > 0)
 		return value_new_int (res);
@@ -528,32 +523,30 @@ static GnmFuncHelp const help_choose[] = {
 };
 
 static GnmValue *
-gnumeric_choose (FunctionEvalInfo *ei, GnmExprList const *l)
+gnumeric_choose (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	int     index;
-	int     argc;
 	GnmValue  *v;
+	int i;
 
-	argc =  gnm_expr_list_length (l);
-
-	if (argc < 1 || !l->data)
+	if (argc < 1)
 		return value_new_error_VALUE (ei->pos);
 
-	v = gnm_expr_eval (l->data, ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+	v = gnm_expr_eval (argv[0], ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	if (!v)
 		return NULL;
 
-	if ((v->type != VALUE_INTEGER) && (v->type != VALUE_FLOAT)) {
+	if (!VALUE_IS_FLOAT (v)) {
 		value_release (v);
 		return value_new_error_VALUE (ei->pos);
 	}
 
 	index = value_get_as_int (v);
 	value_release (v);
-	for (l = l->next; l != NULL ; l = l->next) {
+	for (i = 1; i < argc; i++) {
 		index--;
 		if (!index)
-			return gnm_expr_eval (l->data, ei->pos,
+			return gnm_expr_eval (argv[i], ei->pos,
 					      GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 	}
 	return value_new_error_VALUE (ei->pos);
@@ -836,20 +829,20 @@ gnumeric_indirect (FunctionEvalInfo *ei, GnmValue const * const *args)
 {
 	GnmParsePos  pp;
 	GnmValue *res = NULL;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	char const *text = value_peek_string (args[0]);
 	GnmExprConventions const *convs = gnm_expr_conventions_default;
 
 	if (args[1] && !value_get_as_checked_bool (args[1]))
 		convs = gnm_expr_conventions_r1c1;
 
-	expr = gnm_expr_parse_str (text,
+	texpr = gnm_expr_parse_str (text,
 		parse_pos_init_evalpos (&pp, ei->pos),
 		GNM_EXPR_PARSE_DEFAULT, convs, NULL);
 
-	if (expr != NULL) {
-		res = gnm_expr_get_range (expr);
-		gnm_expr_unref (expr);
+	if (texpr != NULL) {
+		res = gnm_expr_top_get_range (texpr);
+		gnm_expr_top_unref (texpr);
 	}
 	return (res != NULL) ? res : value_new_error_REF (ei->pos);
 }
@@ -880,22 +873,21 @@ static GnmFuncHelp const help_index[] = {
 };
 
 static GnmValue *
-gnumeric_index (FunctionEvalInfo *ei, GnmExprList const *l)
+gnumeric_index (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	GnmExpr const *source;
 	int elem[3] = { 0, 0, 0 };
-	unsigned i = 0;
+	int i = 0;
 	gboolean valid;
 	GnmValue *v, *res;
 
-	if (l == NULL)
+	if (argc == 0)
 		return value_new_error_VALUE (ei->pos);
-	source = l->data;
-	l = l->next;
+	source = argv[0];
 
-	for (i = 0; l != NULL && i < G_N_ELEMENTS (elem) ; i++, l = l->next) {
+	for (i = 0; i + 1 < argc && i < (int)G_N_ELEMENTS (elem); i++) {
 		v = value_coerce_to_number (
-			gnm_expr_eval (l->data, ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY),
+			gnm_expr_eval (argv[i + 1], ei->pos, GNM_EXPR_EVAL_SCALAR_NON_EMPTY),
 			&valid, ei->pos);
 		if (!valid)
 			return v;
@@ -903,9 +895,11 @@ gnumeric_index (FunctionEvalInfo *ei, GnmExprList const *l)
 		value_release (v);
 	}
 
-	if (source->any.oper == GNM_EXPR_OP_SET) {
-		source = gnm_expr_list_nth (source->set.set, elem[2]);
-		if (elem[2] < 0 || source == NULL)
+	if (GNM_EXPR_GET_OPER (source) == GNM_EXPR_OP_SET) {
+		source = (elem[2] >= 0 && elem[2] < source->set.argc)
+			? source->set.argv[elem[2]]
+			: NULL;
+		if (source == NULL)
 			return value_new_error_REF (ei->pos);
 	} else if (elem[2] != 0)
 		return value_new_error_REF (ei->pos);
@@ -958,8 +952,11 @@ gnumeric_column (FunctionEvalInfo *ei, GnmValue const * const *args)
 
 	if (ref == NULL) {
 		col   = ei->pos->eval.col + 1; /* user visible counts from 0 */
-		width = ei->pos->cols;
-		height= ei->pos->rows;
+		if (ei->pos->array != NULL) {
+			width = ei->pos->array->cols;
+			height= ei->pos->array->rows;
+		} else
+			return value_new_int (col);
 	} else if (ref->type == VALUE_CELLRANGE) {
 		Sheet    *tmp;
 		GnmRange  r;
@@ -1136,8 +1133,11 @@ gnumeric_row (FunctionEvalInfo *ei, GnmValue const * const *args)
 
 	if (ref == NULL) {
 		row   = ei->pos->eval.row + 1; /* user visible counts from 0 */
-		width = ei->pos->cols;
-		height= ei->pos->rows;
+		if (ei->pos->array != NULL) {
+			width = ei->pos->array->cols;
+			height= ei->pos->array->rows;
+		} else
+			return value_new_int (row);
 	} else if (ref->type == VALUE_CELLRANGE) {
 		Sheet    *tmp;
 		GnmRange  r;
@@ -1265,7 +1265,7 @@ gnumeric_transpose (FunctionEvalInfo *ei, GnmValue const * const *argv)
 /***************************************************************************/
 
 GnmFuncDescriptor const lookup_functions[] = {
-	{ "address",   "ff|ffs", N_("row_num,col_num,abs_num,a1,text"),
+	{ "address",   "ff|fbs", N_("row_num,col_num,abs_num,a1,text"),
 	  help_address,  gnumeric_address, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "areas", NULL,	N_("reference"),
