@@ -5,7 +5,7 @@
  * Authors:
  *   Miguel de Icaza (miguel@gnu.org)
  *   Jukka-Pekka Iivonen (iivonen@iki.fi)
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,38 +52,24 @@ static GnmValue *
 callback_function_sumxy (Sheet *sheet, int col, int row,
 			 GnmCell *cell, void *user_data)
 {
-        math_sums_t *mm = user_data;
-        gnm_float  x;
-	gpointer    p;
-
 	if (cell == NULL)
 	        return NULL;
 	cell_eval (cell);
-        switch (cell->value->type) {
-	case VALUE_ERROR:
-		return VALUE_TERMINATE;
 
-	case VALUE_BOOLEAN:
-	        x = cell->value->v_bool.val ? 1 : 0;
-		break;
-	case VALUE_INTEGER:
-	        x = cell->value->v_int.val;
-		break;
-	case VALUE_FLOAT:
-	        x = cell->value->v_float.val;
-		break;
-	case VALUE_EMPTY:
-	default:
-	        return NULL;
-	}
+	if (VALUE_IS_NUMBER (cell->value)) {
+		math_sums_t *mm = user_data;
+		gnm_float *p = g_new (gnm_float, 1);
+		*p = value_get_as_float (cell->value);
+		mm->list = g_slist_append (mm->list, p);
+		mm->num++;
 
-	p = g_new (gnm_float, 1);
-	*((gnm_float *) p) = x;
-	mm->list = g_slist_append (mm->list, p);
-	mm->num++;
-
-	return NULL;
+		return NULL;
+	} else if (VALUE_IS_ERROR (cell->value))
+		return VALUE_TERMINATE;  /* FIXME: This is probably wrong.  */
+	else
+		return NULL;
 }
+
 /***************************************************************************/
 
 static GnmFuncHelp const help_gcd[] = {
@@ -108,19 +94,35 @@ static GnmFuncHelp const help_gcd[] = {
 	{ GNM_FUNC_HELP_END }
 };
 
+static const double gnm_gcd_max = 1 / GNM_EPSILON;
+
+static gnm_float
+gnm_gcd (gnm_float a, gnm_float b)
+{
+	g_return_val_if_fail (a > 0 && a <= gnm_gcd_max, -1);
+	g_return_val_if_fail (b > 0 && b <= gnm_gcd_max, -1);
+
+	while (gnm_abs (b) > 0.5) {
+		gnm_float r = gnm_fmod (a, b);
+		a = b;
+		b = r;
+	}
+	return a;
+}
+
 static int
 range_gcd (gnm_float const *xs, int n, gnm_float *res)
 {
 	if (n > 0) {
 		int i;
-		int gcd_so_far = 0;
+		gnm_float gcd_so_far = xs[0];
 
 		for (i = 0; i < n; i++) {
 			gnm_float thisx = gnm_fake_floor (xs[i]);
-			if (thisx <= 0 || thisx > INT_MAX)
+			if (thisx <= 0 || thisx > gnm_gcd_max)
 				return 1;
 			else
-				gcd_so_far = gcd ((int)thisx, gcd_so_far);
+				gcd_so_far = gnm_gcd (thisx, gcd_so_far);
 		}
 		*res = gcd_so_far;
 		return 0;
@@ -129,9 +131,9 @@ range_gcd (gnm_float const *xs, int n, gnm_float *res)
 }
 
 static GnmValue *
-gnumeric_gcd (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_gcd (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_gcd,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -167,30 +169,40 @@ static GnmFuncHelp const help_lcm[] = {
 static int
 range_lcm (gnm_float const *xs, int n, gnm_float *res)
 {
-	if (n > 0) {
-		int i;
-		int lcm_so_far = 1;
+	/* This function violates the "const".  */
+	gnm_float *xsuc = (gnm_float *)xs;
 
-		for (i = 0; i < n; i++) {
-			gnm_float x = gnm_fake_floor (xs[i]);
-			if (x <= 0 || x > INT_MAX)
+	if (n > 0) {
+		int i, j;
+		gnm_float gcd_so_far = 1;
+
+		for (i = j = 0; i < n; i++) {
+			int k;
+			gnm_float thisx = gnm_fake_floor (xsuc[i]);
+
+			if (thisx < 1 || thisx > gnm_gcd_max)
 				return 1;
-			else {
-				int xi = (int) x;
-				lcm_so_far /= gcd (lcm_so_far, xi);
-				lcm_so_far *= xi;
-			}
+
+			for (k = 0; k < j; k++)
+				thisx /= gnm_gcd (thisx, xsuc[k]);
+
+			if (thisx == 1)
+				continue;
+
+			xsuc[j++] = thisx;
+			gcd_so_far *= thisx;
 		}
-		*res = lcm_so_far;
+
+		*res = gcd_so_far;
 		return 0;
 	} else
 		return 1;
 }
 
 static GnmValue *
-gnumeric_lcm (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_lcm (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_lcm,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -218,9 +230,9 @@ static GnmFuncHelp const help_hypot[] = {
 };
 
 static GnmValue *
-gnumeric_hypot (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_hypot (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_hypot,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -285,10 +297,9 @@ static GnmFuncHelp const help_acos[] = {
 static GnmValue *
 gnumeric_acos (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float t;
+	gnm_float t = value_get_as_float (argv [0]);
 
-	t = value_get_as_float (argv [0]);
-	if ((t < -1.0) || (t > 1.0))
+	if (t < -1.0 || t > 1.0)
 		return value_new_error_NUM (ei->pos);
 
 	return value_new_float (gnm_acos (t));
@@ -321,9 +332,8 @@ static GnmFuncHelp const help_acosh[] = {
 static GnmValue *
 gnumeric_acosh (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float t;
+	gnm_float t = value_get_as_float (argv [0]);
 
-	t = value_get_as_float (argv [0]);
 	if (t < 1.0)
 		return value_new_error_NUM (ei->pos);
 
@@ -357,10 +367,9 @@ static GnmFuncHelp const help_asin[] = {
 static GnmValue *
 gnumeric_asin (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float t;
+	gnm_float t = value_get_as_float (argv [0]);
 
-	t = value_get_as_float (argv [0]);
-	if ((t < -1.0) || (t > 1.0))
+	if (t < -1.0 || t > 1.0)
 		return value_new_error_NUM (ei->pos);
 
 	return value_new_float (gnm_asin (t));
@@ -448,10 +457,9 @@ static GnmFuncHelp const help_atanh[] = {
 static GnmValue *
 gnumeric_atanh (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float t;
+	gnm_float t = value_get_as_float (argv [0]);
 
-	t = value_get_as_float (argv [0]);
-	if ((t <= -1.0) || (t >= 1.0))
+	if (t <= -1.0 || t >= 1.0)
 		return value_new_error_NUM (ei->pos);
 
 	return value_new_float (gnm_atanh (value_get_as_float (argv [0])));
@@ -546,24 +554,22 @@ static GnmFuncHelp const help_countif[] = {
 
 typedef struct {
         GnmCriteriaFunc  test;
-        GnmValue            *test_value;
-	unsigned int	     count;
+        GnmValue *test_value;
+	int count;
 } CountIfClosure;
 
 static GnmValue *
-cb_countif (Sheet *sheet, int col, int row, GnmCell *cell,
-	    CountIfClosure *res)
+cb_countif (Sheet *sheet, int col, int row, GnmCell *cell, CountIfClosure *res)
 {
 	if (cell == NULL)
 		return NULL;
 	cell_eval (cell);
-	switch (cell->value->type) {
-	case VALUE_BOOLEAN: case VALUE_INTEGER: case VALUE_FLOAT:
-	case VALUE_STRING:
+
+	if (VALUE_IS_NUMBER (cell->value) || VALUE_IS_STRING (cell->value)) {
 		if ((res->test) (cell->value, res->test_value))
 			res->count++;
-	default: break;
 	}
+
 	return NULL;
 }
 
@@ -583,7 +589,7 @@ gnumeric_countif (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	if (r->type != VALUE_CELLRANGE ||
 	    ((sheet = eval_sheet (r->cell.a.sheet, ei->pos->sheet)) != r->cell.b.sheet &&
 	      r->cell.b.sheet != NULL) ||
-	    (!VALUE_IS_NUMBER (argv[1]) && argv[1]->type != VALUE_STRING))
+	    (!VALUE_IS_NUMBER (argv[1]) && !VALUE_IS_STRING (argv[1])))
 	        return value_new_error_VALUE (ei->pos);
 
 	res.count = 0;
@@ -642,9 +648,8 @@ cb_sumif (Sheet *sheet, int col, int row, GnmCell *cell,
 	if (cell == NULL)
 		return NULL;
 	cell_eval (cell);
-	switch (cell->value->type) {
-	case VALUE_BOOLEAN: case VALUE_INTEGER: case VALUE_FLOAT:
-	case VALUE_STRING:
+
+	if (VALUE_IS_NUMBER (cell->value) || VALUE_IS_STRING (cell->value)) {
 		if ((res->test) (cell->value, res->test_value)) {
 			if (NULL != res->target_sheet) {
 				cell = sheet_cell_get (res->target_sheet,
@@ -652,17 +657,20 @@ cb_sumif (Sheet *sheet, int col, int row, GnmCell *cell,
 				if (cell != NULL) {
 					cell_eval (cell);
 					switch (cell->value->type) {
-					case VALUE_INTEGER:	res->sum += cell->value->v_int.val; break;
-					case VALUE_FLOAT:	res->sum += cell->value->v_float.val; break;
-					default : break;
+					case VALUE_FLOAT:
+						/* FIXME: Check bools.  */
+						res->sum += value_get_as_float (cell->value);
+						break;
+					default:
+						break;
 					}
 				}
 			} else
+				/* FIXME: Check bools.  */
 				res->sum += value_get_as_float (cell->value);
 		}
-	default:
-		break;
 	}
+
 	return NULL;
 }
 
@@ -683,7 +691,7 @@ gnumeric_sumif (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	if (r->type != VALUE_CELLRANGE ||
 	    ((sheet = eval_sheet (r->cell.a.sheet, ei->pos->sheet)) != r->cell.b.sheet &&
 	      r->cell.b.sheet != NULL) ||
-	    (!VALUE_IS_NUMBER (argv[1]) && argv[1]->type != VALUE_STRING))
+	    (!VALUE_IS_NUMBER (argv[1]) && !VALUE_IS_STRING (argv[1])))
 	        return value_new_error_VALUE (ei->pos);
 
 	col_end = r->cell.b.col;
@@ -748,21 +756,16 @@ static GnmFuncHelp const help_ceiling[] = {
 static GnmValue *
 gnumeric_ceiling (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float number = value_get_as_float (argv[0]);
-        gnm_float s;
+	gnm_float x = value_get_as_float (argv[0]);
+        gnm_float s = argv[1] ? value_get_as_float (argv[1]) : (x > 0 ? 1 : -1);
 
-	if (argv[1] == NULL)
-	        s = (number >= 0) ? 1.0 : -1.0;
-	else
-	        s = value_get_as_float (argv[1]);
-
-	if (number == 0 || s == 0)
+	if (x == 0 || s == 0)
 		return value_new_int (0);
 
-	if (number / s < 0)
+	if (x / s < 0)
 		return value_new_error_NUM (ei->pos);
 
-	return value_new_float (gnm_fake_ceil (number / s) * s);
+	return value_new_float (gnm_fake_ceil (x / s) * s);
 }
 
 /***************************************************************************/
@@ -1029,8 +1032,8 @@ static GnmFuncHelp const help_combin[] = {
 static GnmValue *
 gnumeric_combin (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int n = value_get_as_int (argv[0]);
-	int k = value_get_as_int (argv[1]);
+	gnm_float n = gnm_floor (value_get_as_float (argv[0]));
+	gnm_float k = gnm_floor (value_get_as_float (argv[1]));
 
 	if (k >= 0 && n >= k)
 		return value_new_float (combin (n ,k));
@@ -1065,24 +1068,19 @@ static GnmFuncHelp const help_floor[] = {
 static GnmValue *
 gnumeric_floor (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-        gnm_float number, s;
+	gnm_float x = value_get_as_float (argv[0]);
+        gnm_float s = argv[1] ? value_get_as_float (argv[1]) : (x > 0 ? 1 : -1);
 
-	number = value_get_as_float (argv[0]);
-	if (argv[1] == NULL)
-	        s = (number >= 0) ? 1.0 : -1.0;
-	else
-	        s = value_get_as_float (argv[1]);
-
-	if (number == 0)
+	if (x == 0)
 		return value_new_int (0);
 
 	if (s == 0)
 		return value_new_error_DIV0 (ei->pos);
 
-	if (number / s < 0)
+	if (x / s < 0)
 		return value_new_error_NUM (ei->pos);
 
-	return value_new_float (gnm_fake_floor (number / s) * s);
+	return value_new_float (gnm_fake_floor (x / s) * s);
 }
 
 /***************************************************************************/
@@ -1276,6 +1274,7 @@ static GnmValue *
 gnumeric_log2 (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float t = value_get_as_float (argv [0]);
+
 	if (t <= 0.0)
 		return value_new_error_NUM (ei->pos);
 
@@ -1305,9 +1304,8 @@ static GnmFuncHelp const help_log10[] = {
 static GnmValue *
 gnumeric_log10 (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float t;
+	gnm_float t = value_get_as_float (argv [0]);
 
-	t = value_get_as_float (argv [0]);
 	if (t <= 0.0)
 		return value_new_error_NUM (ei->pos);
 
@@ -1348,10 +1346,9 @@ static GnmFuncHelp const help_mod[] = {
 static GnmValue *
 gnumeric_mod (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float a, b, babs, r;
-
-	a = value_get_as_float (argv[0]);
-	b = value_get_as_float (argv[1]);
+	gnm_float a = value_get_as_float (argv[0]);
+	gnm_float b = value_get_as_float (argv[1]);
+	gnm_float babs, r;
 
 	if (b == 0)
 		return value_new_error_DIV0 (ei->pos);
@@ -1504,9 +1501,9 @@ static GnmFuncHelp const help_suma[] = {
 };
 
 static GnmValue *
-gnumeric_suma (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_suma (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_sum,
 				     COLLECT_ZERO_STRINGS |
 				     COLLECT_ZEROONE_BOOLS |
@@ -1538,9 +1535,9 @@ static GnmFuncHelp const help_sumsq[] = {
 
 
 static GnmValue *
-gnumeric_sumsq (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_sumsq (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_sumsq,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -1570,9 +1567,9 @@ static GnmFuncHelp const help_multinomial[] = {
 
 
 static GnmValue *
-gnumeric_multinomial (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_multinomial (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_multinomial,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -1601,9 +1598,9 @@ static GnmFuncHelp const help_g_product[] = {
 };
 
 static GnmValue *
-gnumeric_g_product (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_g_product (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_product,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -1772,11 +1769,13 @@ gnumeric_even (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	ceiled = gnm_ceil (number);
 	if (gnm_fmod (ceiled, 2) == 0)
 	        if (number > ceiled)
-		        return value_new_int ((int) (sign * (ceiled + 2)));
+		        number = sign * (ceiled + 2);
 		else
-		        return value_new_int ((int) (sign * ceiled));
+		        number = sign * ceiled;
 	else
-	        return value_new_int ((int) (sign * (ceiled + 1)));
+	        number = sign * (ceiled + 1);
+
+	return value_new_float (number);
 }
 
 /***************************************************************************/
@@ -1814,11 +1813,13 @@ gnumeric_odd (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	ceiled = gnm_ceil (number);
 	if (gnm_fmod (ceiled, 2) == 1)
 	        if (number > ceiled)
-		        return value_new_int ((int) (sign * (ceiled + 2)));
+		        number = sign * (ceiled + 2);
 		else
-		        return value_new_int ((int) (sign * ceiled));
+		        number = sign * ceiled;
 	else
-	        return value_new_int ((int) (sign * (ceiled + 1)));
+		number = sign * (ceiled + 1);
+
+	return value_new_float (number);
 }
 
 /***************************************************************************/
@@ -1896,8 +1897,8 @@ gnumeric_fib (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	static int fibs[47];
 	static int fib_count = G_N_ELEMENTS (fibs);
 	static gboolean inited = FALSE;
+	gnm_float n = gnm_floor (value_get_as_float (argv[0]));
 
-	int n = value_get_as_int (argv[0]);
 	if (n <= 0)
 		return value_new_error_NUM (ei->pos);
 
@@ -1909,7 +1910,7 @@ gnumeric_fib (FunctionEvalInfo *ei, GnmValue const * const *argv)
 				fibs[i] = fibs[i - 1] + fibs[i - 2];
 			inited = TRUE;
 		}
-		return value_new_int (fibs[n]);
+		return value_new_int (fibs[(int)n]);
 	} else {
 		gnm_float s5 = gnm_sqrt (5.0);
 		gnm_float r1 = (1 + s5) / 2;
@@ -1949,7 +1950,7 @@ gnumeric_quotient (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	if (den == 0)
 	        return value_new_error_DIV0 (ei->pos);
 	else
-	        return value_new_int ((int) (num / den));
+	        return value_new_float (gnm_trunc (num / den));
 }
 
 /***************************************************************************/
@@ -1977,9 +1978,7 @@ static GnmFuncHelp const help_sign[] = {
 static GnmValue *
 gnumeric_sign (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-        gnm_float n;
-
-	n = value_get_as_float (argv[0]);
+        gnm_float n = value_get_as_float (argv[0]);
 
 	if (n > 0)
 	      return value_new_int (1);
@@ -2012,9 +2011,8 @@ static GnmFuncHelp const help_sqrtpi[] = {
 static GnmValue *
 gnumeric_sqrtpi (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-        gnm_float n;
+        gnm_float n = value_get_as_float (argv[0]);
 
-	n = value_get_as_float (argv[0]);
 	if (n < 0)
 		return value_new_error_NUM (ei->pos);
 
@@ -2267,52 +2265,48 @@ static GnmFuncHelp const help_roman[] = {
 static GnmValue *
 gnumeric_roman (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	char const letter[] = { 'M', 'D', 'C', 'L', 'X', 'V', 'I' };
-	int  const largest = 1000;
-
-	static char buf[256];
-	char        *p;
-
-	int n, form;
+	static char const letter[] = { 'M', 'D', 'C', 'L', 'X', 'V', 'I' };
+	int const largest = 1000;
+	char buf[256];
+	char *p;
+	gnm_float n = gnm_floor (value_get_as_float (argv[0]));
+	gnm_float form = argv[1] ? gnm_floor (value_get_as_float (argv[1])) : 0;
 	int i, j, dec;
 
 	dec = largest;
-	n = value_get_as_int (argv[0]);
-	form = argv[1] ? value_get_as_int (argv[1]) : 0;
 
 	if (n < 0 || n > 3999)
 		return value_new_error_VALUE (ei->pos);
+	if (form < 0 || form > 4)
+		return value_new_error_VALUE (ei->pos);
 
 	if (n == 0)
 		return value_new_string ("");
 
-	if (form < 0 || form > 4)
-		return value_new_error_NUM (ei->pos);
-
 	for (i = j = 0; dec > 1; dec /= 10, j += 2) {
 	        for (; n > 0; i++) {
 		        if (n >= dec) {
-			        buf[i] = letter [j];
+			        buf[i] = letter[j];
 				n -= dec;
 			} else if (n >= dec - dec / 10) {
-			        buf [i++] = letter [j + 2];
-				buf [i] = letter [j];
+			        buf[i++] = letter[j + 2];
+				buf[i] = letter[j];
 				n -= dec - dec / 10;
 			} else if (n >= dec / 2) {
-			        buf [i] = letter [j + 1];
+			        buf[i] = letter[j + 1];
 				n -= dec / 2;
 			} else if (n >= dec / 2 - dec / 10) {
-			        buf [i++] = letter [j + 2];
-				buf [i] = letter [j + 1];
+			        buf[i++] = letter[j + 2];
+				buf[i] = letter[j + 1];
 				n -= dec / 2 - dec / 10;
 			} else if (dec == 10) {
-			        buf [i] = letter [j + 2];
+			        buf[i] = letter[j + 2];
 				n--;
 			} else
 			        break;
 		}
 	}
-	buf [i] = '\0';
+	buf[i] = '\0';
 
 
 	if (form > 0) {
@@ -2897,9 +2891,9 @@ range_seriessum (gnm_float const *xs, int n, gnm_float *res)
 
 
 static GnmValue *
-gnumeric_seriessum (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_seriessum (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_seriessum,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -3119,7 +3113,7 @@ gnumeric_mmult (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	        res->v_array.vals[c] = g_new (GnmValue *, rows_a);
 	        for (r = 0; r < rows_a; r++)
 		        res->v_array.vals[c][r] =
-			    value_new_float (product [r + c * rows_a]);
+			    value_new_float (product[r + c * rows_a]);
 	}
 	g_free (A);
 	g_free (B);
@@ -3204,24 +3198,22 @@ static GnmFuncHelp const help_sumproduct[] = {
 };
 
 static GnmValue *
-gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList const *args)
+gnumeric_sumproduct (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	gnm_float **data;
 	GnmValue *result;
-	int i, argc;
-	GnmExprList const *l;
+	int i;
 	gboolean size_error = FALSE;
 	int sizex = -1, sizey = -1;
 
-	if (args == NULL)
+	if (argc == 0)
 		return value_new_error_VALUE (ei->pos);
 
-	argc = gnm_expr_list_length (args);
 	data = g_new0 (gnm_float *, argc);
 
-	for (l = args, i = 0; l; l = l->next, i++) {
+	for (i = 0; i < argc; i++) {
 		int thissizex, thissizey, x, y;
-		GnmExpr const *expr = l->data;
+		GnmExpr const *expr = argv[i];
 		GnmValue    *val = gnm_expr_eval (expr, ei->pos,
 					   GNM_EXPR_EVAL_PERMIT_NON_SCALAR |
 					   GNM_EXPR_EVAL_PERMIT_EMPTY);
@@ -3241,7 +3233,7 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList const *args)
 				/* FIXME: efficiency worries?  */
 				GnmValue const *v = value_area_fetch_x_y (val, x, y, ei->pos);
 				switch (v->type) {
-				case VALUE_ERROR :
+				case VALUE_ERROR:
 					/*
 					 * We carefully tranverse the argument
 					 * list and then the arrays in such an
@@ -3258,7 +3250,6 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList const *args)
 					result = value_dup (v);
 					value_release (val);
 					goto done;
-				case VALUE_INTEGER:
 				case VALUE_FLOAT:
 					data[i][y * thissizex + x] = value_get_as_float (v);
 					break;
