@@ -10,6 +10,7 @@
 #include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "lotus.h"
+#include "lotus-formula.h"
 #include "lotus-types.h"
 
 #include <goffice/app/file.h>
@@ -53,11 +54,14 @@ lotus_file_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
 	switch (version) {
 	case LOTUS_VERSION_ORIG_123:
 	case LOTUS_VERSION_SYMPHONY:
+	case LOTUS_VERSION_SYMPHONY2:
 		return len == 2;
 
+	case LOTUS_VERSION_123V4: /* Barely and crudely handled.  */
 	case LOTUS_VERSION_123V6:
+	case LOTUS_VERSION_123V7:
 	case LOTUS_VERSION_123SS98:
-		return TRUE;
+		return len >= 19;
 
 	default:
 		return FALSE;
@@ -73,10 +77,28 @@ lotus_file_open (GOFileOpener const *fo, IOContext *io_context,
 	state.input	 = input;
 	state.io_context = io_context;
 	state.wbv	 = wb_view;
-	state.wb	 = wb_view_workbook (wb_view);
+	state.wb	 = wb_view_get_workbook (wb_view);
 	state.sheet	 = NULL;
 
 	if (!lotus_read (&state))
 		gnumeric_io_error_string (io_context,
 			_("Error while reading lotus workbook."));
 }
+
+
+
+G_MODULE_EXPORT void
+go_plugin_init (G_GNUC_UNUSED GOPlugin *plugin,
+		G_GNUC_UNUSED GOCmdContext *cc)
+{
+	lmbcs_init ();
+	lotus_formula_init ();
+}
+
+G_MODULE_EXPORT void
+go_plugin_shutdown (G_GNUC_UNUSED GOPlugin *plugin,
+		    G_GNUC_UNUSED GOCmdContext *cc)
+{
+	lotus_formula_shutdown ();
+	lmbcs_shutdown ();
+}
