@@ -1,4 +1,4 @@
-/* -*-mode:c; c-style:k&r; c-basic-offset:8; -*- */
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * GNU Oleo input filter for Gnumeric
  *
@@ -154,7 +154,7 @@ oleo_get_gnumeric_expr (char const *o_expr,
 			cellref_as_string (gres, gnm_expr_conventions_default,
 					   &start, cur_pos, TRUE);
 
-			if (!cellref_equal (&start, &end)) {
+			if (!gnm_cellref_equal (&start, &end)) {
 				g_string_append_c (gres, ':');
 				cellref_as_string (gres, gnm_expr_conventions_default,
 						   &end, cur_pos, TRUE);
@@ -173,12 +173,12 @@ oleo_get_gnumeric_expr (char const *o_expr,
 }
 
 
-static GnmExpr const *
+static GnmExprTop const *
 oleo_parse_formula (OleoParseState *state, char const *text, int col, int row)
 {
 	GnmParsePos pos;
 	GnmParseError error;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	char *gnumeric_text;
 
 	GnmCell const *cell = sheet_cell_fetch (state->sheet,
@@ -187,7 +187,7 @@ oleo_parse_formula (OleoParseState *state, char const *text, int col, int row)
 	parse_pos_init_cell (&pos, cell);
 
 	gnumeric_text = oleo_get_gnumeric_expr (text, &pos);
-	expr = gnm_expr_parse_str (gnumeric_text,
+	texpr = gnm_expr_parse_str (gnumeric_text,
 				   &pos, GNM_EXPR_PARSE_DEFAULT,
 				   state->exprconv, parse_error_init (&error));
 
@@ -199,14 +199,14 @@ oleo_parse_formula (OleoParseState *state, char const *text, int col, int row)
 	g_free (gnumeric_text);
 	parse_error_free (&error);
 
-	return expr;
+	return texpr;
 }
 
 static void
 oleo_deal_with_cell (OleoParseState *state, guint8 *str, int *ccol, int *crow, GnmStyle *style)
 {
 	GnmCell *cell;
-	GnmExpr const *expr = NULL;
+	GnmExprTop const *texpr = NULL;
 	char *ptr = str + 1, *cval = NULL, *formula = NULL;
 
 	while (*ptr) {
@@ -252,7 +252,7 @@ oleo_deal_with_cell (OleoParseState *state, guint8 *str, int *ccol, int *crow, G
 		OLEO_TO_GNUMERIC (*ccol), OLEO_TO_GNUMERIC (*crow));
 
 	if (formula != NULL)
-		expr = oleo_parse_formula (state, formula, *ccol, *crow);
+		texpr = oleo_parse_formula (state, formula, *ccol, *crow);
 
 	if (cval != NULL) {
 		GnmValue *val = format_match_simple (cval);
@@ -266,8 +266,8 @@ oleo_deal_with_cell (OleoParseState *state, guint8 *str, int *ccol, int *crow, G
 				val = value_new_string (cval);
 		}
 
-		if (expr != NULL)
-			cell_set_expr_and_value (cell, expr, val, TRUE);
+		if (texpr != NULL)
+			cell_set_expr_and_value (cell, texpr, val, TRUE);
 		else
 			cell_set_value (cell, val);
 
@@ -279,11 +279,11 @@ oleo_deal_with_cell (OleoParseState *state, guint8 *str, int *ccol, int *crow, G
 		g_warning ("oleo: cval is NULL.");
 #endif
 		/* We can still store the expression, even if the value is missing */
-		if (expr != NULL)
-			cell_set_expr (cell, expr);
+		if (texpr != NULL)
+			cell_set_expr (cell, texpr);
 	}
-	if (expr)
-		gnm_expr_unref (expr);
+	if (texpr)
+		gnm_expr_top_unref (texpr);
 }
 
 
@@ -326,7 +326,7 @@ oleo_deal_with_format (OleoParseState *state, guint8 *str, int *ccol, int *crow,
 			break;
 		case 'R':
 			gnm_style_set_align_h (mstyle, HALIGN_RIGHT);
-	}
+		}
 	}
 	if (fmt_string->len)
 		gnm_style_set_format_text (mstyle, fmt_string->str);
@@ -356,11 +356,10 @@ oleo_conventions (void)
 	GnmExprConventions *res = gnm_expr_conventions_new ();
 
 	res->decimal_sep_dot = TRUE;
-	res->ref_parser = gnm_1_0_rangeref_parse;
+	res->ref_parser = rangeref_parse;
 	res->range_sep_colon = TRUE;
 	res->sheet_sep_exclamation = TRUE;
-	res->dots_in_names = TRUE;
-	res->ignore_whitespace = TRUE;
+	res->intersection_char = 0;
 	res->unknown_function_handler = gnm_func_placeholder_factory;
 
 	return res;
