@@ -216,12 +216,13 @@ pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 			case HALIGN_GENERAL:break;
 			case HALIGN_LEFT:	attr |= 0x0100; break;
 			case HALIGN_RIGHT:	attr |= 0x0200; break;
+			case HALIGN_DISTRIBUTED:
 			case HALIGN_CENTER_ACROSS_SELECTION :
 			case HALIGN_CENTER:	attr |= 0x0300; break;
 			}
 		}
 		if ((attr & 0x8000)) {
-			gboolean is_locked = gnm_style_get_content_locked (def);
+			gboolean is_locked = gnm_style_get_contents_locked (def);
 			attr = (attr & 0x3fff) | (is_locked ? 0x4000 : 0);
 		}
 		gnm_style_unref (def);
@@ -239,7 +240,7 @@ pln_get_style (PlanPerfectImport *state, guint8 const* data, gboolean is_cell)
 		};
 		res = gnm_style_new_default ();
 		gnm_style_set_font_italic (res, (attr & 0x0010) ? TRUE : FALSE);
-		gnm_style_set_content_hidden (res, (attr & 0x0020) ? TRUE : FALSE);
+		gnm_style_set_contents_hidden (res, (attr & 0x0020) ? TRUE : FALSE);
 		gnm_style_set_font_uline (res,
 			(attr & 0x1000) ?  UNDERLINE_DOUBLE :
 			((attr & 0x0040) ?  UNDERLINE_SINGLE : UNDERLINE_NONE));
@@ -489,9 +490,7 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 	int max_row = SHEET_MAX_ROWS;
 	int i, rcode, rlength;
 	guint8 const *data;
-	GnmCell    *cell;
 	GnmValue   *v;
-	GnmExpr const *expr;
 	GnmStyle  *style;
 	GnmParsePos pp;
 	GnmRange r;
@@ -557,6 +556,8 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 
 	/* process the CELL information */
 	while (NULL != (data = gsf_input_read (input, 20, NULL))) {
+		GnmExprTop const *texpr = NULL;
+		GnmCell *cell = NULL;
 		unsigned type = GSF_LE_GET_GUINT16 (data + 12);
 		unsigned length = GSF_LE_GET_GUINT16 (data + 18);
 
@@ -576,8 +577,6 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 				pp.eval.col, max_col);
 
 		v = NULL;
-		expr = NULL;
-		cell = NULL;
 		if ((type & 0x7) != 0) {
 			style = pln_get_style (state, data, TRUE);
 			if (style != NULL)
@@ -626,11 +625,11 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 				char *expr_txt = pln_convert_expr (&pp, data);
 
 				if (expr_txt != NULL) {
-					expr = gnm_expr_parse_str (expr_txt, &pp,
+					texpr = gnm_expr_parse_str (expr_txt, &pp,
 								   GNM_EXPR_PARSE_DEFAULT,
 								   gnm_expr_conventions_default,
 								   NULL);
-					if (expr == NULL) {
+					if (texpr == NULL) {
 						if (v != NULL)
 							value_release (v);
 						v = value_new_string_nocopy (expr_txt);
@@ -640,12 +639,12 @@ pln_parse_sheet (GsfInput *input, PlanPerfectImport *state)
 			}
 		}
 
-		if (expr != NULL) {
+		if (texpr != NULL) {
 			if (v != NULL)
-				cell_set_expr_and_value (cell, expr, v, TRUE);
+				cell_set_expr_and_value (cell, texpr, v, TRUE);
 			else
-				cell_set_expr (cell, expr);
-			gnm_expr_unref (expr);
+				cell_set_expr (cell, texpr);
+			gnm_expr_top_unref (texpr);
 		} else if (v != NULL)
 			cell_set_value (cell, v);
 	}
@@ -663,7 +662,7 @@ pln_file_open (GOFileOpener const *fo, IOContext *io_context,
 	ErrorInfo *error;
 	PlanPerfectImport state;
 
-	wb    = wb_view_workbook (wb_view);
+	wb    = wb_view_get_workbook (wb_view);
 	name  = workbook_sheet_get_free_name (wb, "PlanPerfect", FALSE, TRUE);
 	sheet = sheet_new (wb, name);
 	g_free (name);
