@@ -267,7 +267,7 @@ gplp_func_file_open (GOFileOpener const *fo,
 
 	loader_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	SWITCH_TO_PLUGIN (plugin_service_get_plugin (service));
-	sheet = sheet_new (wb_view_workbook (wb_view), _("Some name"));
+	sheet = sheet_new (wb_view_get_workbook (wb_view), _("Some name"));
 	input_wrapper = pygobject_new (G_OBJECT (input));
 	if (input_wrapper != NULL) {
 		 /* wrapping adds a reference */
@@ -280,7 +280,7 @@ gplp_func_file_open (GOFileOpener const *fo,
 	}
 	if (open_result != NULL) {
 		Py_DECREF (open_result);
-		workbook_sheet_attach (wb_view_workbook (wb_view), sheet);
+		workbook_sheet_attach (wb_view_get_workbook (wb_view), sheet);
 	} else {
 		gnumeric_io_error_string (io_context, py_exc_to_string ());
 		gnm_python_clear_error_if_needed (SERVICE_GET_LOADER (service)->py_object);
@@ -371,7 +371,7 @@ gplp_func_file_save (GOFileSaver const *fs, GOPluginService *service,
 
 	saver_data = g_object_get_data (G_OBJECT (service), "loader_data");
 	SWITCH_TO_PLUGIN (plugin_service_get_plugin (service));
-	py_workbook = py_new_Workbook_object (wb_view_workbook (wb_view));
+	py_workbook = py_new_Workbook_object (wb_view_get_workbook (wb_view));
 	output_wrapper = pygobject_new (G_OBJECT (output));
 	if (output_wrapper != NULL) {
 		/* wrapping adds a reference */
@@ -481,15 +481,15 @@ call_python_function_args (FunctionEvalInfo *ei, GnmValue const * const *args)
 }
 
 static GnmValue *
-call_python_function_nodes (FunctionEvalInfo *ei, GnmExprList const *expr_tree_list)
+call_python_function_nodes (FunctionEvalInfo *ei,
+			    int argc, const GnmExprConstPtr *argv)
 {
 	GOPluginService *service;
 	ServiceLoaderDataFunctionGroup *loader_data;
 	PyObject *python_fn;
 	GnmFunc const * fndef;
 	GnmValue **values;
-	gint n_args, i;
-	GnmExprList const *l;
+	gint i;
 	GnmValue *ret_value;
 
 	g_return_val_if_fail (ei != NULL, NULL);
@@ -502,14 +502,13 @@ call_python_function_nodes (FunctionEvalInfo *ei, GnmExprList const *expr_tree_l
 	python_fn = PyDict_GetItemString (loader_data->python_fn_info_dict,
 	                                  (gchar *) gnm_func_get_name (fndef));
 
-	n_args = gnm_expr_list_length (expr_tree_list);
-	values = g_new (GnmValue *, n_args);
-	for (i = 0, l = expr_tree_list; l != NULL; i++, l = l->next) {
-		values[i] = gnm_expr_eval (l->data, ei->pos, GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
+	values = g_new (GnmValue *, argc);
+	for (i = 0; i < argc; i++) {
+		values[i] = gnm_expr_eval (argv[i], ei->pos, GNM_EXPR_EVAL_PERMIT_NON_SCALAR);
 	}
-	ret_value = call_python_function (python_fn, ei->pos, n_args,
+	ret_value = call_python_function (python_fn, ei->pos, argc,
 					  (GnmValue const * const *)values);
-	for (i = 0; i < n_args; i++) {
+	for (i = 0; i < argc; i++) {
 		value_release (values[i]);
 	}
 	g_free (values);
