@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -357,7 +357,7 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 	guint16 ref_offset = GSF_LE_GET_GUINT16 (data + 12);
 	GnmValue   *val;
 	GSList  *stack = NULL;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	guint8 const *refs, *fmla;
 	int len;
 
@@ -377,7 +377,7 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 #endif
 
 	while (fmla < refs && *fmla != QPRO_OP_EOF) {
-		expr = NULL;
+		GnmExpr const *expr = NULL;
 		len = 1;
 #if 0
 		g_print ("Operator %d.\n", *fmla);
@@ -483,17 +483,16 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 		case QPRO_OP_OR: {
 			GnmFunc *f = gnm_func_lookup (*fmla == QPRO_OP_OR ? "or" : "and",
 						      NULL);
-			GnmExprList *arglist = NULL;
-			arglist = g_slist_prepend (arglist, (gpointer)expr_stack_pop (&stack));
-			arglist = g_slist_prepend (arglist, (gpointer)expr_stack_pop (&stack));
-			expr = gnm_expr_new_funcall (f, arglist);
+			GnmExpr const *r = expr_stack_pop (&stack);
+			GnmExpr const *l = expr_stack_pop (&stack);
+			expr = gnm_expr_new_funcall2 (f, l, r);
 			break;
 		}
 
 		case QPRO_OP_NOT: {
 			GnmFunc *f = gnm_func_lookup ("NOT", NULL);
-			GnmExprList *arglist = g_slist_prepend (NULL, (gpointer)expr_stack_pop (&stack));
-			expr = gnm_expr_new_funcall (f, arglist);
+			GnmExpr const *a = expr_stack_pop (&stack);
+			expr = gnm_expr_new_funcall1 (f, a);
 			break;
 		}
 
@@ -585,7 +584,7 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 		g_return_if_fail (stack->next == NULL);
 	}
 
-	expr = stack->data;
+	texpr = gnm_expr_top_new (stack->data);
 	g_slist_free (stack);
 
 	switch (magic) {
@@ -626,9 +625,10 @@ qpro_parse_formula (QProReadState *state, int col, int row,
 		val = value_new_float (gsf_le_get_double (data));
 	}
 
-	cell_set_expr_and_value (sheet_cell_fetch (state->cur_sheet, col, row),
-		expr, val, TRUE);
-	gnm_expr_unref (expr);
+	cell_set_expr_and_value
+		(sheet_cell_fetch (state->cur_sheet, col, row),
+		 texpr, val, TRUE);
+	gnm_expr_top_unref (texpr);
 }
 
 static GnmStyle *
@@ -838,7 +838,7 @@ qpro_file_open (GOFileOpener const *fo, IOContext *context,
 
 	state.io_context = context;
 	state.wbv = new_wb_view;
-	state.wb = wb_view_workbook (new_wb_view);
+	state.wb = wb_view_get_workbook (new_wb_view);
 	state.cur_sheet = NULL;
 	state.converter	 = g_iconv_open ("UTF-8", "ISO-8859-1");
 
