@@ -81,12 +81,22 @@ xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 
 	switch (field->type) {
 	case 'C': {
-		if (file->char_map != (GIConv)-1)
-			val = value_new_string_nocopy (
-				g_convert_with_iconv (g_strchomp (s), -1,
-				      file->char_map, NULL, NULL, NULL));
+		char *sutf8 = g_convert_with_iconv (g_strchomp (s), -1,
+						    file->char_map, NULL, NULL, NULL);
+		if (!sutf8) {
+			char *t;
+			for (t = s; *t; t++)
+				if ((guchar)*t >= 0x7f)
+					*t = '?';
+			sutf8 = s;
+			s = NULL;
+			g_warning ("Unrepresentable characters replaced by '?'");
+		}
+		if (sutf8)
+			val = value_new_string_nocopy (sutf8);
 		else
-			val = value_new_string_nocopy (g_strchomp (s));
+			val = value_new_string ("???");
+		g_free (s);
 		return val;
 	}
 	case 'N':
@@ -129,7 +139,7 @@ xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 		g_free (s);
 		return val;
 	case 'F':
-		g_assert (sizeof (double) == field->len);
+		g_return_val_if_fail (sizeof (double) == field->len, value_new_float (0.));
 		val = value_new_float (XB_GETDOUBLE (s));
 		g_free (s);
 		return val;
@@ -137,7 +147,7 @@ xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 		gint64 tmp = GINT32_FROM_LE (*(gint64 *)s);
 		g_free (s);
 		g_warning ("FIXME: \"BINARY\" field type doesn't work");
-		g_assert (sizeof(tmp) == field->len);
+		g_return_val_if_fail (sizeof(tmp) == field->len, value_new_int (0));
 		return value_new_int (tmp);
 	}
 	default: {
@@ -172,7 +182,7 @@ xbase_file_open (GOFileOpener const *fo, IOContext *io_context,
 		return;
 	}
 
-	wb = wb_view_workbook (wb_view);
+	wb = wb_view_get_workbook (wb_view);
 	name = workbook_sheet_get_free_name (wb, _("Sheet"), FALSE, TRUE);
 	sheet = sheet_new (wb, name);
 	g_free (name);
@@ -185,8 +195,8 @@ xbase_file_open (GOFileOpener const *fo, IOContext *io_context,
 	}
 	{
 		GnmRange r;
-		GnmStyle *bold = mstyle_new ();
-		mstyle_set_font_bold (bold, TRUE);
+		GnmStyle *bold = gnm_style_new ();
+		gnm_style_set_font_bold (bold, TRUE);
 		sheet_style_apply_range	(sheet,
 			range_init (&r, 0, 0, file->fields-1, 0), bold);
 	}
