@@ -17,10 +17,12 @@
 #include "gnm-format.h"
 #include "rendered-value.h"
 #include "parse-util.h"
+#include "sheet-merge.h"
 #include <goffice/utils/go-color.h>
 
 #include <gdk/gdk.h>
 #include <string.h>
+#include <math.h>
 
 static char const hashes[] =
 "################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################################";
@@ -62,6 +64,8 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 	layout = rv->layout;
 	indent = (rv->indent_left + rv->indent_right) * PANGO_SCALE;
 
+	rv->drawn = TRUE;
+
 	if (width <= 0 || height <= 0)
 		return FALSE;
 
@@ -77,16 +81,26 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 	    rv->might_overflow &&
 	    !rv->numeric_overflow) {
 		char const *text = pango_layout_get_text (layout);
-		/* This assumes that hash marks are wider than
+		size_t textlen = strlen (text);
+#if 0
+		g_print ("nat=%d  w=%d  i=%d\n", rv->layout_natural_width, width, indent);
+#endif
+		/* This assumes that two hash marks are wider than
 		   the characters in the number.  Probably ok.  */
 		pango_layout_set_text (layout, hashes,
-				       MIN (sizeof (hashes) - 1, strlen (text)));
+				       MIN (sizeof (hashes) - 1, 2 * textlen));
 		rv->numeric_overflow = TRUE;
 		rv->variable_width = TRUE;
 		rv->hfilled = TRUE;
 	}
 
-	if (rv->wrap_text) {
+	if (rv->rotation && !rv->noborders) {
+		RenderedRotatedValue const *rrv = (RenderedRotatedValue *)rv;
+		double sin_a = rrv->rotmat.xy;
+		if (sin_a < 0) {
+			hoffset += (width - indent) - rv->layout_natural_width;
+		}
+	} else if (!rv->rotation && rv->wrap_text) {
 		int wanted_width = MAX (0, width - indent);
 		if (wanted_width != pango_layout_get_width (layout)) {
 			pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
@@ -98,6 +112,7 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 		case HALIGN_RIGHT:
 			hoffset += (width - indent) - rv->layout_natural_width;
 			break;
+		case HALIGN_DISTRIBUTED:
 		case HALIGN_CENTER:
 			if (h_center == -1)
 				h_center = width / 2;
@@ -158,6 +173,7 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 		break;
 	}
 
+	case VALIGN_DISTRIBUTED: /* dunno what this does yet */
 	case VALIGN_CENTER: {
 		int dh = (height - rv->layout_natural_height) / 2;
 		if (rv->rotation == 0 && dh < 0)
@@ -167,7 +183,6 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 	}
 
 	case VALIGN_JUSTIFY:
-	case VALIGN_DISTRIBUTED: /* dunno what this does yet */
 		text_base = rect_y;
 		if (!rv->vfilled && height > rv->layout_natural_height) {
 			int line_count = pango_layout_get_line_count (layout);
@@ -196,6 +211,52 @@ cell_calc_layout (GnmCell const *cell, RenderedValue *rv, int y_direction,
 	return TRUE;
 }
 
+/*
+ * This finishes a layout by pretending to draw it.  The effect is to
+ * handler numerical overflow, filling, etc.
+ * (Doesn't currently handle vertical fill.)
+ */
+void
+cell_finish_layout (GnmCell const *cell, RenderedValue *rv,
+		    gboolean inhibit_overflow)
+{
+	gint dummy_x, dummy_y;
+	GOColor dummy_fore_color;
+	int dummy_h_center = -1;  /* Affects position only.  */
+	int dummy_height = 1;  /* Unhandled.  */
+	int col_width_pixels;
+	gboolean might_overflow;
+
+	if (!rv)
+		rv = cell->rendered_value;
+	if (rv->drawn)
+		return;
+
+	if (cell_is_merged (cell)) {
+		Sheet const *sheet = cell->base.sheet;
+		GnmRange const *merged =
+			sheet_merge_is_corner (sheet, &cell->pos);
+
+		col_width_pixels = sheet_col_get_distance_pixels
+			(sheet,
+			 merged->start.col, merged->end.col + 1);
+	} else
+		col_width_pixels = cell->col_info->size_pixels;
+	/* This probably isn't right for the merged case */
+	col_width_pixels -= (cell->col_info->margin_a +
+			     cell->col_info->margin_b +
+			     1);
+
+	might_overflow = rv->might_overflow;
+	if (inhibit_overflow) rv->might_overflow = FALSE;
+	cell_calc_layout (cell, rv, -1,
+			  col_width_pixels * PANGO_SCALE,
+			  dummy_height,
+			  dummy_h_center,
+			  &dummy_fore_color, &dummy_x, &dummy_y);
+	rv->might_overflow = might_overflow;
+}
+
 
 void
 cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
@@ -221,8 +282,7 @@ cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
 			      height * PANGO_SCALE,
 			      h_center == -1 ? -1 : (h_center * PANGO_SCALE),
 			      &fore_color, &x, &y)) {
-		PangoContext *context = NULL;
-		GdkColor tmp;
+		GdkColor fore_gdk;
 
 		/* +1 to get past left grid-line.  */
 		GdkRectangle rect;
@@ -241,29 +301,41 @@ cell_draw (GnmCell const *cell, GdkGC *gc, GdkDrawable *drawable,
 					    rect.width, rect.height);
 		}
 #endif
-		gdk_gc_set_clip_rectangle (gc, &rect);
+
+		/*
+		 * HACK -- do not clip rotated cells.  This gives an
+		 * approximation to the right effect.  (The right way
+		 * would be to create a proper cellspan type.)
+		 */
+		gdk_gc_set_clip_rectangle (gc,
+					   rv->rotation ? NULL : &rect);
 
 		/* See http://bugzilla.gnome.org/show_bug.cgi?id=105322 */
-		gdk_gc_set_rgb_fg_color (gc,
-			go_color_to_gdk (fore_color, &tmp));
+		go_color_to_gdk (fore_color, &fore_gdk);
+		gdk_gc_set_rgb_fg_color (gc, &fore_gdk);
 
 		if (rv->rotation) {
-			PangoMatrix rotmat = PANGO_MATRIX_INIT;
+			RenderedRotatedValue *rrv = (RenderedRotatedValue *)rv;
+			PangoContext *context = pango_layout_get_context (rv->layout);
+			struct RenderedRotatedValueInfo const *li = rrv->lines;
+			GSList *lines;
 
-			context = pango_layout_get_context (rv->layout);
-			pango_matrix_rotate (&rotmat, rv->rotation);
-			pango_context_set_matrix (context, &rotmat);
+			pango_context_set_matrix (context, &rrv->rotmat);
 			pango_layout_context_changed (rv->layout);
-		}
-
-		gdk_draw_layout (drawable, gc,
-				 x1 + PANGO_PIXELS (x),
-				 y1 + PANGO_PIXELS (y),
-				 rv->layout);
-
-		if (context) {
+			for (lines = pango_layout_get_lines (rv->layout);
+			     lines;
+			     lines = lines->next, li++) {
+				gdk_draw_layout_line (drawable, gc,
+					x1 + PANGO_PIXELS (x + li->dx),
+					y1 + PANGO_PIXELS (y + li->dy),
+					lines->data);
+			}
 			pango_context_set_matrix (context, NULL);
 			pango_layout_context_changed (rv->layout);
-		}
+		} else
+			gdk_draw_layout (drawable, gc,
+				x1 + PANGO_PIXELS (x),
+				y1 + PANGO_PIXELS (y),
+				rv->layout);
 	}
 }
