@@ -33,35 +33,39 @@ struct _GnmCell {
  * Manage cells
  */
 GnmCell  *cell_new      (void);
-GnmCell	 *cell_copy	(GnmCell const *cell);
 void	  cell_destroy  (GnmCell *cell);
 void	  cell_relocate (GnmCell *cell, GnmExprRewriteInfo const *rwinfo);
 
+/* DEPRECATED  */
+GnmCell	 *cell_dup	(GnmCell const *cell);
+
 /**
  * GnmCell state checking
  */
 #define	    cell_needs_recalc(cell)	((cell)->base.flags & DEPENDENT_NEEDS_RECALC)
 #define	    cell_expr_is_linked(cell)	((cell)->base.flags & DEPENDENT_IS_LINKED)
-#define	    cell_has_expr(cell)		((cell)->base.expression != NULL)
+#define	    cell_has_expr(cell)		((cell)->base.texpr != NULL)
 #define	    cell_is_linked(cell)	((cell)->base.flags & CELL_IN_SHEET_LIST)
 #define	    cell_is_merged(cell)	((cell)->base.flags & CELL_IS_MERGED)
-GnmComment *cell_has_comment_pos  (Sheet const *sheet, GnmCellPos const *pos);
-GnmComment *cell_has_comment	  (GnmCell const *cell);
 gboolean    cell_is_empty	  (GnmCell const *cell);
 gboolean    cell_is_blank	  (GnmCell const *cell); /* empty, or "" */
 GnmValue   *cell_is_error	  (GnmCell const *cell);
 gboolean    cell_is_number	  (GnmCell const *cell);
 gboolean    cell_is_zero	  (GnmCell const *cell);
-gboolean    cell_is_partial_array (GnmCell const *cell);
-GnmExprArray const *cell_is_array (GnmCell const *cell);
 
-#define cell_eval(cell)							\
-{									\
-	if (cell_needs_recalc (cell)) {					\
-		cell_eval_content (cell);				\
-		cell->base.flags &= ~(DEPENDENT_NEEDS_RECALC | CELL_HAS_NEW_EXPR );		\
-	}								\
-}
+gboolean    cell_is_array	  (GnmCell const *cell);
+gboolean    cell_is_nonsingleton_array (GnmCell const *cell);
+GnmExprArrayCorner const *
+	    cell_is_array_corner  (GnmCell const *cell);
+gboolean    cell_array_bound	  (GnmCell const *cell, GnmRange *res);
+
+#define cell_eval(cell)											\
+	do {												\
+		if (cell_needs_recalc (cell)) {								\
+			cell_eval_content (cell);							\
+			cell->base.flags &= ~(DEPENDENT_NEEDS_RECALC | (int)CELL_HAS_NEW_EXPR );	\
+		}											\
+	} while (0)
 
 /**
  * Utilities to assign the contents of a cell
@@ -69,19 +73,20 @@ GnmExprArray const *cell_is_array (GnmCell const *cell);
 void cell_set_text		(GnmCell *c, char const *text);
 void cell_assign_value		(GnmCell *c, GnmValue *v);
 void cell_set_value		(GnmCell *c, GnmValue *v);
-void cell_set_expr_and_value	(GnmCell *c, GnmExpr const *expr, GnmValue *v,
+void cell_set_expr_and_value	(GnmCell *c,
+				 GnmExprTop const *texpr, GnmValue *v,
 				 gboolean link_expr);
-void cell_set_expr		(GnmCell *c, GnmExpr const *expr);
-void cell_set_expr_unsafe 	(GnmCell *cell, GnmExpr const *expr);
+void cell_set_expr		(GnmCell *c, GnmExprTop const *texpr);
+void cell_set_expr_unsafe 	(GnmCell *cell, GnmExprTop const *texpr);
 void cell_set_array_formula	(Sheet *sheet,
 				 int cola, int rowa, int colb, int rowb,
-				 GnmExpr const *expr);
+				 GnmExprTop const *texpr);
 void cell_convert_expr_to_value	(GnmCell *cell);
 
 /**
  * Manipulate GnmCell attributes
  */
-GnmStyle *cell_get_mstyle		(GnmCell const *cell);
+GnmStyle const *cell_get_style	(GnmCell const *cell);
 GOFormat *cell_get_format	(GnmCell const *cell);
 void	cell_set_format		(GnmCell *cell, char const *format);
 
