@@ -183,7 +183,7 @@ cellspan_is_empty (int col, ColRowInfo const *ri, GnmCell const *ok_span_cell)
 	 * because we don't respan expression results.
 	 */
 	return (tmp == NULL || tmp->value == NULL ||
-		(tmp->value->type == VALUE_EMPTY && !cell_has_expr(tmp)));
+		(VALUE_IS_EMPTY (tmp->value) && !cell_has_expr(tmp)));
 }
 
 /*
@@ -201,7 +201,7 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 	int h_align, v_align, left, max_col, min_col;
 	int row, pos, margin;
 	int cell_width_pixel, indented_w;
-	GnmStyle *mstyle;
+	GnmStyle const *style;
 	ColRowInfo const *ri;
 	GnmRange const *merge_left;
 	GnmRange const *merge_right;
@@ -211,6 +211,9 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 	sheet = cell->base.sheet;
 	ri = cell->row_info;
 
+	style = cell_get_style (cell);
+	h_align = style_default_halign (style, cell);
+
         /*
 	 * Report only one column is used if
 	 *	- Cell is in a hidden col
@@ -220,15 +223,14 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 	 * 	- The alignment mode are set to "justify"
 	 */
 	if (sheet != NULL &&
+	    h_align != HALIGN_CENTER_ACROSS_SELECTION &&
 	    (cell_is_merged (cell) ||
 	     (!sheet->display_formulas && cell_is_number (cell)))) {
 		*col1 = *col2 = cell->pos.col;
 		return;
 	}
 
-	mstyle = cell_get_mstyle (cell);
-	h_align = style_default_halign (mstyle, cell);
-	v_align = mstyle_get_align_v (mstyle);
+	v_align = gnm_style_get_align_v (style);
 	row   = cell->pos.row;
 	indented_w = cell_width_pixel = cell_rendered_width (cell);
 	if (h_align == HALIGN_LEFT || h_align == HALIGN_RIGHT) {
@@ -240,10 +242,11 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 	if (cell_is_empty (cell) ||
 	    !cell->col_info->visible ||
 	    (h_align != HALIGN_CENTER_ACROSS_SELECTION &&
-		 (mstyle_get_wrap_text (mstyle) ||
+		 (gnm_style_get_wrap_text (style) ||
 		  indented_w <= COL_INTERNAL_WIDTH (cell->col_info))) ||
 	    h_align == HALIGN_JUSTIFY ||
 	    h_align == HALIGN_FILL ||
+	    h_align == HALIGN_DISTRIBUTED ||
 	    v_align == VALIGN_JUSTIFY ||
 	    v_align == VALIGN_DISTRIBUTED) {
 		*col1 = *col2 = cell->pos.col;
@@ -362,10 +365,10 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 			ColRowInfo const *ci = sheet_col_get_info (sheet, pos_l);
 			if (ci->visible) {
 				if (cellspan_is_empty (pos_l, ri, cell)) {
-					GnmStyle * const mstyle =
+					GnmStyle const * const style =
 						sheet_style_get (cell->base.sheet, pos_l, row);
 
-					if (mstyle_get_align_h (mstyle) != HALIGN_CENTER_ACROSS_SELECTION)
+					if (gnm_style_get_align_h (style) != HALIGN_CENTER_ACROSS_SELECTION)
 						break;
 					*col1 = pos_l;
 				} else
@@ -376,10 +379,10 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 			ColRowInfo const *ci = sheet_col_get_info (sheet, pos_r);
 			if (ci->visible) {
 				if (cellspan_is_empty (pos_r, ri, cell)) {
-					GnmStyle * const mstyle =
+					GnmStyle const * const style =
 						sheet_style_get (cell->base.sheet, pos_r, row);
 
-					if (mstyle_get_align_h (mstyle) != HALIGN_CENTER_ACROSS_SELECTION)
+					if (gnm_style_get_align_h (style) != HALIGN_CENTER_ACROSS_SELECTION)
 						break;
 					*col2 = pos_r;
 				} else
@@ -390,7 +393,7 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 	}
 
 	default:
-		g_warning ("Unknown horizontal alignment type %d.", h_align);
+		g_warning ("Unknown horizontal alignment type %x.", h_align);
 	} /* switch */
 }
 
