@@ -17,6 +17,7 @@
 #include "gnm-datetime.h"
 #include "workbook.h"
 #include "sheet.h"
+#include "number-match.h"
 
 /* ------------------------------------------------------------------------- */
 
@@ -69,7 +70,6 @@ callback_function_collect (GnmEvalPos const *ep, GnmValue const *value,
 			return value_new_error_VALUE (ep);
 		break;
 
-	case VALUE_INTEGER:
 	case VALUE_FLOAT:
 		x = value_get_as_float (value);
 		break;
@@ -79,11 +79,12 @@ callback_function_collect (GnmEvalPos const *ep, GnmValue const *value,
 			GnmValue *vc = format_match_number (value_peek_string (value),
 							    NULL,
 							    cl->date_conv);
-			gboolean bad = !vc || VALUE_TYPE(vc) == VALUE_BOOLEAN;
+			gboolean bad = !vc || VALUE_IS_BOOLEAN (vc);
 			if (vc) {
 				x = value_get_as_float (vc);
 				value_release (vc);
-			}
+			} else
+				x = 0;
 
 			if (bad)
 				return value_new_error_VALUE (ep);
@@ -150,7 +151,8 @@ callback_function_collect (GnmEvalPos const *ep, GnmValue const *value,
  * gnm_float.
  */
 static gnm_float *
-collect_floats (GnmExprList const *exprlist, GnmEvalPos const *ep, CollectFlags flags,
+collect_floats (int argc, const GnmExprConstPtr *argv,
+		GnmEvalPos const *ep, CollectFlags flags,
 		int *n, GnmValue **error, GSList **info)
 {
 	GnmValue * err;
@@ -178,11 +180,11 @@ collect_floats (GnmExprList const *exprlist, GnmEvalPos const *ep, CollectFlags
 
 	err = function_iterate_argument_values
 		(ep, &callback_function_collect, &cl,
-		 exprlist,
+		 argc, argv,
 		 TRUE, iter_flags);
 
 	if (err) {
-		g_assert (err->type == VALUE_ERROR);
+		g_assert (VALUE_IS_ERROR (err));
 		g_free (cl.data);
 		g_slist_free (cl.info);
 		*error = err;
@@ -203,16 +205,11 @@ gnm_float *
 collect_floats_value (GnmValue const *val, GnmEvalPos const *ep,
 		      CollectFlags flags, int *n, GnmValue **error)
 {
-	GnmExprList *exprlist;
-	GnmExprConstant expr_val;
-	gnm_float *res;
+	GnmExpr expr_val;
+	GnmExprConstPtr argv[1] = { &expr_val };
 
-	gnm_expr_constant_init (&expr_val, val);
-	exprlist = gnm_expr_list_prepend (NULL, &expr_val);
-	res = collect_floats (exprlist, ep, flags, n, error, NULL);
-	gnm_expr_list_free (exprlist);
-
-	return res;
+	gnm_expr_constant_init (&expr_val.constant, val);
+	return collect_floats (1, argv, ep, flags, n, error, NULL);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -223,14 +220,12 @@ collect_floats_value_with_info (GnmValue const *val, GnmEvalPos const *ep,
 				CollectFlags flags, int *n, GSList **info,
 				GnmValue **error)
 {
-	GnmExprList *exprlist;
-	GnmExprConstant expr_val;
+	GnmExpr expr_val;
+	GnmExprConstPtr argv[1] = { &expr_val };
 	gnm_float *res;
 
-	gnm_expr_constant_init (&expr_val, val);
-	exprlist = gnm_expr_list_prepend (NULL, &expr_val);
-	res = collect_floats (exprlist, ep, flags, n, error, info);
-	gnm_expr_list_free (exprlist);
+	gnm_expr_constant_init (&expr_val.constant, val);
+	res = collect_floats (1, argv, ep, flags, n, error, info);
 
 	if (info)
 		*info = g_slist_reverse (*info);
@@ -242,7 +237,8 @@ collect_floats_value_with_info (GnmValue const *val, GnmEvalPos const *ep,
 /* ------------------------------------------------------------------------- */
 
 GnmValue *
-float_range_function (GnmExprList const *exprlist, FunctionEvalInfo *ei,
+float_range_function (int argc, const GnmExprConstPtr *argv,
+		      FunctionEvalInfo *ei,
 		      float_range_function_t func,
 		      CollectFlags flags,
 		      GnmStdError func_error)
@@ -251,7 +247,7 @@ float_range_function (GnmExprList const *exprlist, FunctionEvalInfo *ei,
 	gnm_float *vals, res;
 	int n, err;
 
-	vals = collect_floats (exprlist, ei->pos, flags, &n, &error, NULL);
+	vals = collect_floats (argc, argv, ei->pos, flags, &n, &error, NULL);
 	if (!vals)
 		return (error != VALUE_TERMINATE) ? error : NULL;
 
@@ -367,7 +363,8 @@ strip_missing (GArray * data, GSList **missing)
 
 
 GnmValue *
-float_range_function2 (GnmValue const *val0, GnmValue const *val1, FunctionEvalInfo *ei,
+float_range_function2 (GnmValue const *val0, GnmValue const *val1,
+		       FunctionEvalInfo *ei,
 		       float_range_function2_t func,
 		       CollectFlags flags,
 		       GnmStdError func_error)
@@ -430,7 +427,6 @@ float_range_function2 (GnmValue const *val0, GnmValue const *val1, FunctionEvalI
 			}
 		}
 
-
 		if (func (vals0, vals1, n0, &fres))
 			res = value_new_error_std (ei->pos, func_error);
 		else
@@ -455,7 +451,7 @@ callback_function_collect_strings (GnmEvalPos const *ep, GnmValue const *value,
 				   void *closure)
 {
 	char *text = NULL;
-	collect_strings_t *cl = (collect_strings_t *)closure;
+	collect_strings_t *cl = closure;
 
 	if (value == NULL) {
 		if (cl->flags & COLLECT_IGNORE_BLANKS)
@@ -464,7 +460,6 @@ callback_function_collect_strings (GnmEvalPos const *ep, GnmValue const *value,
 	} else switch (value->type) {
 	case VALUE_EMPTY:
 	case VALUE_BOOLEAN:
-	case VALUE_INTEGER:
 	case VALUE_FLOAT:
 	case VALUE_STRING:
 		text = value_get_as_string (value);
@@ -482,8 +477,7 @@ callback_function_collect_strings (GnmEvalPos const *ep, GnmValue const *value,
 			return value_new_error_VALUE (ep);
 		break;
 	default:
-		text = g_strdup_printf ("Trouble in callback_function_collect. (%d)",
-					value->type);
+		g_assert_not_reached ();
 		break;
 	}
 
@@ -507,7 +501,9 @@ callback_function_collect_strings (GnmEvalPos const *ep, GnmValue const *value,
  */
 
 static GSList *
-collect_strings (GnmExprList const *exprlist, GnmEvalPos const *ep, CollectFlags flags, GnmValue **error)
+collect_strings (int argc, const GnmExprConstPtr *argv,
+		 GnmEvalPos const *ep, CollectFlags flags,
+		 GnmValue **error)
 {
 	GnmValue * err;
 	collect_strings_t cl;
@@ -515,12 +511,13 @@ collect_strings (GnmExprList const *exprlist, GnmEvalPos const *ep, CollectFlags
 	cl.data = NULL;
 	cl.flags = flags;
 
-	err = function_iterate_argument_values (ep, &callback_function_collect_strings,
-		&cl, exprlist,
+	err = function_iterate_argument_values
+		(ep, &callback_function_collect_strings, &cl,
+		 argc, argv,
 		TRUE, (flags & COLLECT_IGNORE_BLANKS) ?  CELL_ITER_IGNORE_BLANK : CELL_ITER_ALL);
 
 	if (err) {
-		g_assert (err->type == VALUE_ERROR);
+		g_assert (VALUE_IS_ERROR (err));
 		g_slist_foreach (cl.data, (GFunc)g_free, NULL);
 		g_slist_free (cl.data);
 		*error = err;
@@ -532,7 +529,8 @@ collect_strings (GnmExprList const *exprlist, GnmEvalPos const *ep, CollectFlags
 
 
 GnmValue *
-string_range_function (GnmExprList const *exprlist, FunctionEvalInfo *ei,
+string_range_function (int argc, const GnmExprConstPtr *argv,
+		       FunctionEvalInfo *ei,
 		       string_range_function_t func,
 		       CollectFlags flags,
 		       GnmStdError func_error)
@@ -542,7 +540,7 @@ string_range_function (GnmExprList const *exprlist, FunctionEvalInfo *ei,
 	char *res = NULL;
 	int err;
 
-	vals = collect_strings (exprlist, ei->pos, flags, &error);
+	vals = collect_strings (argc, argv, ei->pos, flags, &error);
 	if (!vals)
 		return (error != VALUE_TERMINATE) ? error : NULL;
 
