@@ -2,7 +2,7 @@
 /*
  * colrow.c: Utilities for Rows and Columns
  *
- * Copyright (C) 1999-2001 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 1999-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 #include <gnumeric-config.h>
@@ -146,7 +146,7 @@ colrow_copy (ColRowInfo *dst, ColRowInfo const *src)
  * Iterates through the existing rows or columns within the range supplied.
  * Currently only support left -> right iteration.  If a callback returns
  * TRUE iteration stops.
- */
+ **/
 gboolean
 colrow_foreach (ColRowCollection const *infos, int first, int last,
 		ColRowHandler callback, void *user_data)
@@ -338,6 +338,7 @@ colrow_get_states (Sheet *sheet, gboolean is_cols, int first, int last)
 {
 	ColRowStateList *list = NULL;
 	ColRowRLEState  *rles;
+	/* gcc is wrong run_state is always initialized */
 	ColRowState	 run_state, cur_state;
 	int              i, run_length = 0;
 
@@ -490,8 +491,8 @@ colrow_set_sizes (Sheet *sheet, gboolean is_cols,
 			if (tmp < 0)
 				/* Fall back to assigning the defaul if it is empty */
 				tmp = (is_cols)
-					? sheet_col_size_fit_pixels (sheet, i)
-					: sheet_row_size_fit_pixels (sheet, i);
+					? sheet_col_size_fit_pixels (sheet, i, 0, SHEET_MAX_ROWS - 1, FALSE)
+					: sheet_row_size_fit_pixels (sheet, i, 0, SHEET_MAX_COLS - 1, FALSE);
 
 			if (tmp > 0) {
 				if (is_cols)
@@ -618,32 +619,6 @@ colrow_restore_state_group (Sheet *sheet, gboolean is_cols,
 	g_slist_free (state_groups);
 }
 
-static gboolean
-cb_autofit_height (ColRowInfo *info, void *sheet)
-{
-	/* If the size was not set by the user then auto resize */
-	if (!info->hard_size) {
-		int const new_size = sheet_row_size_fit_pixels (sheet, info->pos);
-		if (new_size > 0)
-			sheet_row_set_size_pixels (sheet, info->pos, new_size, FALSE);
-	}
-	return FALSE;
-}
-
-static gboolean
-cb_autofit_height_no_shrink (ColRowInfo *info, void *sheet)
-{
-	/* If the size was not set by the user then auto resize */
-	if (!info->hard_size) {
-		int const new_size     = sheet_row_size_fit_pixels (sheet, info->pos);
-		int const default_size = sheet_row_get_default_size_pixels (sheet);
-
-		if (new_size > 0 && new_size > default_size)
-			sheet_row_set_size_pixels (sheet, info->pos, new_size, FALSE);
-	}
-	return FALSE;
-}
-
 /**
  * rows_height_update
  * @sheet:  The sheet,
@@ -657,14 +632,133 @@ void
 rows_height_update (Sheet *sheet, GnmRange const * range, gboolean shrink)
 {
 	/* FIXME : this needs to check font sizes and contents rather than
-	 * just contents.  Empty cells will cause resize also
-	 */
-	if (shrink)
-		colrow_foreach (&sheet->rows, range->start.row, range->end.row,
-				&cb_autofit_height, sheet);
-	else
-		colrow_foreach (&sheet->rows, range->start.row, range->end.row,
-				&cb_autofit_height_no_shrink, sheet);
+	 * just contents.  Empty cells will cause resize also */
+	colrow_autofit (sheet, range, FALSE, FALSE,
+			FALSE, !shrink,
+			NULL, NULL);
+}
+
+/* ------------------------------------------------------------------------- */
+
+struct cb_autofit {
+	Sheet *sheet;
+	const GnmRange *range;
+	gboolean ignore_strings;
+	gboolean min_current;
+	gboolean min_default;
+};
+
+static gboolean
+cb_autofit_col (ColRowInfo *info, void *data_)
+{
+	struct cb_autofit *data = data_;
+	int size, min, max;
+
+	if (info->hard_size)
+		return FALSE;
+
+	size = sheet_col_size_fit_pixels
+		(data->sheet, info->pos,
+		 data->range->start.row, data->range->end.row,
+		 data->ignore_strings);
+	/* FIXME: better idea than this?  */
+	max = 50 * sheet_col_get_default_size_pixels (data->sheet);
+	size = MIN (size, max);
+
+	min = 0;
+	if (data->min_current)
+		min = MAX (min, info->size_pixels);
+	if (data->min_default)
+		min = MAX (min, sheet_col_get_default_size_pixels (data->sheet));
+
+	if (size > min)
+		sheet_col_set_size_pixels (data->sheet, info->pos,
+					   size, FALSE);
+
+	return FALSE;
+}
+
+static gboolean
+cb_autofit_row (ColRowInfo *info, void *data_)
+{
+	struct cb_autofit *data = data_;
+	int size, min, max;
+
+	if (info->hard_size)
+		return FALSE;
+
+	size = sheet_row_size_fit_pixels
+		(data->sheet, info->pos,
+		 data->range->start.col, data->range->end.col,
+		 data->ignore_strings);
+	max = 20 * sheet_row_get_default_size_pixels (data->sheet);
+	size = MIN (size, max);
+
+	min = 0;
+	if (data->min_current)
+		min = MAX (min, info->size_pixels);
+	if (data->min_default)
+		min = MAX (min, sheet_row_get_default_size_pixels (data->sheet));
+		
+	if (size > min)
+		sheet_row_set_size_pixels (data->sheet, info->pos,
+					   size, FALSE);
+
+	return FALSE;
+}
+
+/*
+ * colrow_autofit:
+ * @sheet: the sheet to change
+ * @range: the range to consider
+ * @is_cols: TRUE for columns, FALSE for rows.
+ * @ignore_strings: Don't consider cells with string values.
+ * @min_current: Don't shrink below current size.
+ * @min_default: Don't shrink below default size.
+ * @indices: indices appropriate for colrow_restore_state_group.
+ * @sizes: old sizes appropriate for colrow_restore_state_group.
+ *
+ * This function autofits columns or rows in @range as specified by
+ * @is_cols.  Only cells in @range are considered for the sizing
+ * and the size can be bounded below by current size and/or default
+ * size.
+ */
+void
+colrow_autofit (Sheet *sheet, const GnmRange *range, gboolean is_cols,
+		gboolean ignore_strings,
+		gboolean min_current, gboolean min_default,
+		ColRowIndexList **indices,
+		ColRowStateGroup **sizes)
+{
+	struct cb_autofit data;
+	int a, b;
+	ColRowCollection *crs;
+	ColRowHandler handler;
+
+	data.sheet = sheet;
+	data.range = range;
+	data.ignore_strings = ignore_strings;
+	data.min_current = min_current;
+	data.min_default = min_default;
+
+	if (is_cols) {
+		a = range->start.col;
+		b = range->end.col;
+		crs = &sheet->cols;
+		handler = cb_autofit_col;
+	} else {
+		a = range->start.row;
+		b = range->end.row;
+		crs = &sheet->rows;
+		handler = cb_autofit_row;
+	}
+
+	if (indices)
+		*indices = colrow_get_index_list (a, b, NULL);
+	if (sizes)
+		*sizes = g_slist_prepend (NULL, colrow_get_states (sheet, is_cols, a, b));
+
+	colrow_foreach (crs, a, b, handler, &data);
 }
 
 /*****************************************************************************/
@@ -773,7 +867,7 @@ cb_colrow_visibility (SheetView *sv, GnmRange const *r, void *closure)
  * pairs of row/col ranges that need to be hidden or unhiden.
  *
  * NOTE : leave sheet non-const until we have a const version of
- *        selection_apply.
+ *        sv_selection_apply.
  */
 ColRowVisList *
 colrow_get_visiblity_toggle (SheetView *sv, gboolean is_cols,
@@ -784,7 +878,7 @@ colrow_get_visiblity_toggle (SheetView *sv, gboolean is_cols,
 	closure.visible = visible;
 	closure.elements = NULL;
 
-	selection_apply (sv, &cb_colrow_visibility, FALSE, &closure);
+	sv_selection_apply (sv, &cb_colrow_visibility, FALSE, &closure);
 
 	return closure.elements;
 }
