@@ -2,7 +2,7 @@
 /*
  * dependent.c:  Manage calculation dependencies between objects
  *
- * Copyright (C) 2000-2005
+ * Copyright (C) 2000-2006
  *  Jody Goldberg   (jody@gnome.org)
  *  Morten Welinder (terra@gnome.org)
  *
@@ -36,6 +36,7 @@
 #include "rendered-value.h" /* FIXME : should not be needed with JIT-R */
 #include "ranges.h"
 #include "gutils.h"
+#include "sheet-view.h"
 
 #include <string.h>
 #include <goffice/utils/go-glib-extras.h>
@@ -50,12 +51,12 @@ static void name_dep_debug_name	   (GnmDependent const *dep, GString *target);
 
 static GnmCellPos const dummy = { 0, 0 };
 static GPtrArray *dep_classes = NULL;
-static DependentClass dynamic_dep_class = {
+static GnmDependentClass dynamic_dep_class = {
 	dynamic_dep_eval,
 	NULL,
 	dynamic_dep_debug_name,
 };
-static DependentClass name_dep_class = {
+static GnmDependentClass name_dep_class = {
 	name_dep_eval,
 	NULL,
 	name_dep_debug_name,
@@ -96,7 +97,7 @@ dependent_types_shutdown (void)
  * of dependents.
  */
 guint32
-dependent_type_register (DependentClass const *klass)
+dependent_type_register (GnmDependentClass const *klass)
 {
 	guint32 res;
 
@@ -147,7 +148,7 @@ dependent_changed (GnmDependent *dep)
  * NOTE : it does NOT relink dependents in case they are going to move later.
  **/
 void
-dependent_set_expr (GnmDependent *dep, GnmExpr const *new_expr)
+dependent_set_expr (GnmDependent *dep, GnmExprTop const *new_texpr)
 {
 	int const t = dependent_type (dep);
 
@@ -159,26 +160,26 @@ dependent_set_expr (GnmDependent *dep, GnmExpr const *new_expr)
 		 * Explicitly do not check for array subdivision, we may be
 		 * replacing the corner of an array.
 		 */
-		cell_set_expr_unsafe (DEP_TO_CELL (dep), new_expr);
+		cell_set_expr_unsafe (DEP_TO_CELL (dep), new_texpr);
 	} else {
-		DependentClass *klass = g_ptr_array_index (dep_classes, t);
+		GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
 
 		g_return_if_fail (klass);
-		if (new_expr != NULL)
-			gnm_expr_ref (new_expr);
-		if (klass->set_expr != NULL)
-			(*klass->set_expr) (dep, new_expr);
+		if (new_texpr)
+			gnm_expr_top_ref (new_texpr);
+		if (klass->set_expr)
+			(*klass->set_expr) (dep, new_texpr);
 
-		if (dep->expression != NULL)
-			gnm_expr_unref (dep->expression);
-		dep->expression = new_expr;
-		if (new_expr != NULL)
+		if (dep->texpr)
+			gnm_expr_top_unref (dep->texpr);
+		dep->texpr = new_texpr;
+		if (new_texpr)
 			dependent_changed (dep);
 	}
 }
 
-static inline const GnmCellPos *
-dependent_pos (const GnmDependent *dep)
+static inline GnmCellPos const *
+dependent_pos (GnmDependent const *dep)
 {
 	return dependent_is_cell (dep) ? &DEP_TO_CELL (dep)->pos : &dummy;
 }
@@ -197,7 +198,7 @@ dependent_set_sheet (GnmDependent *dep, Sheet *sheet)
 	g_return_if_fail (!dependent_is_linked (dep));
 
 	dep->sheet = sheet;
-	if (dep->expression != NULL) {
+	if (dep->texpr) {
 		dependent_link (dep);
 		dependent_changed (dep);
 	}
@@ -439,7 +440,7 @@ micro_hash_init (MicroHash *hash_table, gpointer key)
 }
 
 static inline gboolean
-micro_hash_is_empty (const MicroHash *hash_table)
+micro_hash_is_empty (MicroHash const *hash_table)
 {
 	return hash_table->num_elements == 0;
 }
@@ -551,7 +552,7 @@ link_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *ref
 		deps = dep->sheet->deps;
 
 	/* Inserts if it is not already there */
-	cellref_get_abs_pos (ref, pos, &lookup.pos);
+	gnm_cellpos_init_cellref (&lookup.pos, ref, pos);
 	single = g_hash_table_lookup (deps->single_hash, &lookup);
 	if (single == NULL) {
 		single = go_mem_chunk_alloc (deps->single_pool);
@@ -574,7 +575,7 @@ unlink_single_dep (GnmDependent *dep, GnmCellPos const *pos, GnmCellRef const *a
 	if (!deps)
 		return;
 
-	cellref_get_abs_pos (a, pos, &lookup.pos);
+	gnm_cellpos_init_cellref (&lookup.pos, a, pos);
 	single = g_hash_table_lookup (deps->single_hash, &lookup);
 	if (single != NULL) {
 		micro_hash_remove (&single->deps, dep);
@@ -649,8 +650,8 @@ link_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 	DependencyRange range;
 	DependentFlags flag = DEPENDENT_NO_FLAG;
 
-	cellref_get_abs_pos (a, pos, &range.range.start);
-	cellref_get_abs_pos (b, pos, &range.range.end);
+	gnm_cellpos_init_cellref (&range.range.start, a, pos);
+	gnm_cellpos_init_cellref (&range.range.end, b, pos);
 	range_normalize (&range.range);
 
 	if (a->sheet != NULL) {
@@ -686,8 +687,8 @@ unlink_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 {
 	DependencyRange range;
 
-	cellref_get_abs_pos (a, pos, &range.range.start);
-	cellref_get_abs_pos (b, pos, &range.range.end);
+	gnm_cellpos_init_cellref (&range.range.start, a, pos);
+	gnm_cellpos_init_cellref (&range.range.end, b, pos);
 	range_normalize (&range.range);
 
 	if (a->sheet != NULL) {
@@ -711,81 +712,83 @@ unlink_cellrange_dep (GnmDependent *dep, GnmCellPos const *pos,
 }
 
 static DependentFlags
-link_expr_dep (GnmDependent *dep, GnmExpr const *tree)
+link_expr_dep (GnmEvalPos *ep, GnmExpr const *tree)
 {
 	g_return_val_if_fail (tree != NULL, DEPENDENT_NO_FLAG);
 
-	switch (tree->any.oper) {
+	switch (GNM_EXPR_GET_OPER (tree)) {
 	case GNM_EXPR_OP_ANY_BINARY:
-		return  link_expr_dep (dep, tree->binary.value_a) |
-			link_expr_dep (dep, tree->binary.value_b);
+		return  link_expr_dep (ep, tree->binary.value_a) |
+			link_expr_dep (ep, tree->binary.value_b);
 	case GNM_EXPR_OP_ANY_UNARY:
-		return link_expr_dep (dep, tree->unary.value);
+		return link_expr_dep (ep, tree->unary.value);
 	case GNM_EXPR_OP_CELLREF:
-		return link_single_dep (dep, dependent_pos (dep), &tree->cellref.ref);
+		return link_single_dep (ep->dep, dependent_pos (ep->dep), &tree->cellref.ref);
 
 	case GNM_EXPR_OP_CONSTANT:
 		/* TODO: pass in eval flags so that we can use implicit
 		 * intersection
 		 */
 		if (VALUE_CELLRANGE == tree->constant.value->type)
-			return link_cellrange_dep
-				(dep,
-				 dependent_pos (dep),
+			return link_cellrange_dep (ep->dep, dependent_pos (ep->dep),
 				 &tree->constant.value->v_range.cell.a,
 				 &tree->constant.value->v_range.cell.b);
 		return DEPENDENT_NO_FLAG;
 
 	/* TODO : Can we use argument types to be smarter here ? */
 	case GNM_EXPR_OP_FUNCALL: {
-		GnmExprList *l;
+		int i;
 		DependentFlags flag = DEPENDENT_NO_FLAG;
 		if (tree->func.func->fn_type == GNM_FUNC_TYPE_STUB)
 			gnm_func_load_stub (tree->func.func);
 		if (tree->func.func->linker) {
-			GnmEvalPos		 ep;
 			FunctionEvalInfo fei;
-			fei.pos = eval_pos_init_dep (&ep, dep);
+			fei.pos = ep;
 			fei.func_call = &tree->func;
 			flag = tree->func.func->linker (&fei);
 		}
-		for (l = tree->func.arg_list; l; l = l->next)
-			flag |= link_expr_dep (dep, l->data);
+		if (!(flag & DEPENDENT_IGNORE_ARGS))
+			for (i = 0; i < tree->func.argc; i++)
+				flag |= link_expr_dep (ep, tree->func.argv[i]);
 		return flag;
 	}
 
 	case GNM_EXPR_OP_NAME:
-		expr_name_add_dep (tree->name.name, dep);
+		expr_name_add_dep (tree->name.name, ep->dep);
 		if (tree->name.name->active)
-			return link_expr_dep (dep, tree->name.name->expr) | DEPENDENT_USES_NAME;
+			return link_expr_dep (ep, tree->name.name->texpr->expr) | DEPENDENT_USES_NAME;
 		return DEPENDENT_USES_NAME;
 
-	case GNM_EXPR_OP_ARRAY:
-		if (tree->array.x != 0 || tree->array.y != 0) {
-			/* Non-corner cells depend on the corner */
-			GnmCellRef a;
-			GnmCellPos const *pos = dependent_pos (dep);
-			/* We cannot support array expressions unless
-			 * we have a position.
-			 */
-			g_return_val_if_fail (pos != NULL, DEPENDENT_NO_FLAG);
+	case GNM_EXPR_OP_ARRAY_ELEM: {
+		/* Non-corner cells depend on the corner */
+		GnmCellRef a;
+		GnmCellPos const *pos = dependent_pos (ep->dep);
+		/* We cannot support array expressions unless
+		 * we have a position.
+		 */
+		g_return_val_if_fail (pos != NULL, DEPENDENT_NO_FLAG);
 
-			a.col_relative = a.row_relative = FALSE;
-			a.sheet = dep->sheet;
-			a.col   = pos->col - tree->array.x;
-			a.row   = pos->row - tree->array.y;
+		a.col_relative = a.row_relative = FALSE;
+		a.sheet = ep->dep->sheet;
+		a.col   = pos->col - tree->array_elem.x;
+		a.row   = pos->row - tree->array_elem.y;
 
-			return link_single_dep (dep, pos, &a);
-		} else
-			/* Corner cell depends on the contents of the expr */
-			return link_expr_dep (dep, tree->array.corner.expr);
+		return link_single_dep (ep->dep, pos, &a);
+	}
+
+	case GNM_EXPR_OP_ARRAY_CORNER: {
+		GnmEvalPos pos = *ep;
+		pos.array = &tree->array_corner;
+		/* Corner cell depends on the contents of the expr */
+		return link_expr_dep (&pos, tree->array_corner.expr);
+	}
 
 	case GNM_EXPR_OP_SET: {
-		GnmExprList *l;
+		int i;
 		DependentFlags res = DEPENDENT_NO_FLAG;
 
-		for (l = tree->set.set; l; l = l->next)
-			res |= link_expr_dep (dep, l->data);
+		for (i = 0; i < tree->set.argc; i++)
+			res |= link_expr_dep (ep, tree->set.argv[i]);
 		return res;
 	}
 	case GNM_EXPR_OP_RANGE_CTOR:
@@ -803,7 +806,7 @@ link_expr_dep (GnmDependent *dep, GnmExpr const *tree)
 static void
 unlink_expr_dep (GnmDependent *dep, GnmExpr const *tree)
 {
-	switch (tree->any.oper) {
+	switch (GNM_EXPR_GET_OPER (tree)) {
 	case GNM_EXPR_OP_ANY_BINARY:
 		unlink_expr_dep (dep, tree->binary.value_a);
 		unlink_expr_dep (dep, tree->binary.value_b);
@@ -828,7 +831,7 @@ unlink_expr_dep (GnmDependent *dep, GnmExpr const *tree)
 	 * more cunning handling of argument type matching.
 	 */
 	case GNM_EXPR_OP_FUNCALL: {
-		GnmExprList *l;
+		int i;
 		if (tree->func.func->unlinker) {
 			GnmEvalPos ep;
 			FunctionEvalInfo fei;
@@ -836,44 +839,43 @@ unlink_expr_dep (GnmDependent *dep, GnmExpr const *tree)
 			fei.func_call = &tree->func;
 			tree->func.func->unlinker (&fei);
 		}
-		for (l = tree->func.arg_list; l; l = l->next)
-			unlink_expr_dep (dep, l->data);
+		for (i = 0; i < tree->func.argc; i++)
+			unlink_expr_dep (dep, tree->func.argv[i]);
 		return;
 	}
 
 	case GNM_EXPR_OP_NAME:
 		expr_name_remove_dep (tree->name.name, dep);
 		if (tree->name.name->active)
-			unlink_expr_dep (dep, tree->name.name->expr);
+			unlink_expr_dep (dep, tree->name.name->texpr->expr);
 		return;
 
-	case GNM_EXPR_OP_ARRAY:
-		if (tree->array.x != 0 || tree->array.y != 0) {
-			/* Non-corner cells depend on the corner */
-			GnmCellRef a;
-			GnmCellPos const *pos = dependent_pos (dep);
+	case GNM_EXPR_OP_ARRAY_ELEM: {
+		GnmCellRef a;
+		GnmCellPos const *pos = dependent_pos (dep);
 
-			/* We cannot support array expressions unless
-			 * we have a position.
-			 */
-			g_return_if_fail (pos != NULL);
+		g_return_if_fail (pos != NULL);
 
-			a.col_relative = a.row_relative = FALSE;
-			a.sheet = dep->sheet;
-			a.col   = pos->col - tree->array.x;
-			a.row   = pos->row - tree->array.y;
+		/* Non-corner cells depend on the corner */
+		a.col_relative = a.row_relative = FALSE;
+		a.sheet = dep->sheet;
+		a.col   = pos->col - tree->array_elem.x;
+		a.row   = pos->row - tree->array_elem.y;
 
-			unlink_single_dep (dep, pos, &a);
-		} else
-			/* Corner cell depends on the contents of the expr */
-			unlink_expr_dep (dep, tree->array.corner.expr);
+		unlink_single_dep (dep, pos, &a);
+		return;
+	}
+
+	case GNM_EXPR_OP_ARRAY_CORNER:
+		/* Corner depends on the contents of the expr */
+		unlink_expr_dep (dep, tree->array_corner.expr);
 		return;
 
 	case GNM_EXPR_OP_SET: {
-		GnmExprList *l;
+		int i;
 
-		for (l = tree->set.set; l; l = l->next)
-			unlink_expr_dep (dep, l->data);
+		for (i = 0; i < tree->set.argc; i++)
+			unlink_expr_dep (dep, tree->set.argv[i]);
 		return;
 	}
 
@@ -942,12 +944,12 @@ dynamic_dep_debug_name (GnmDependent const *dep, GString *target)
 }
 
 void
-dependent_add_dynamic_dep (GnmDependent *dep, GnmValueRange const *v)
+dependent_add_dynamic_dep (GnmDependent *dep, GnmRangeRef const *rr)
 {
-	DependentFlags   flags;
-	DynamicDep	*dyn;
-	GnmCellPos const	*pos;
-	DependencyRange  range;
+	DependentFlags    flags;
+	DynamicDep	 *dyn;
+	GnmCellPos const *pos;
+	DependencyRange   range;
 
 	g_return_if_fail (dep != NULL);
 
@@ -960,21 +962,21 @@ dependent_add_dynamic_dep (GnmDependent *dep, GnmValueRange const *v)
 		dyn = g_new (DynamicDep, 1);
 		dyn->base.flags		= DEPENDENT_DYNAMIC_DEP;
 		dyn->base.sheet		= dep->sheet;
-		dyn->base.expression	= NULL;
+		dyn->base.texpr		= NULL;
 		dyn->container		= dep;
 		dyn->ranges		= NULL;
 		dyn->singles		= NULL;
 		g_hash_table_insert (dep->sheet->deps->dynamic_deps, dep, dyn);
 	}
 
-	cellref_get_abs_pos (&v->cell.a, pos, &range.range.start);
-	cellref_get_abs_pos (&v->cell.b, pos, &range.range.end);
+	gnm_cellpos_init_cellref (&range.range.start, &rr->a, pos);
+	gnm_cellpos_init_cellref (&range.range.end, &rr->b, pos);
 	if (range_is_singleton (&range.range)) {
-		flags = link_single_dep (&dyn->base, pos, &v->cell.a);
-		dyn->singles = g_slist_prepend (dyn->singles, value_dup ((GnmValue *)v));
+		flags = link_single_dep (&dyn->base, pos, &rr->a);
+		dyn->singles = g_slist_prepend (dyn->singles, gnm_rangeref_dup (rr));
 	} else {
-		flags = link_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
-		dyn->ranges = g_slist_prepend (dyn->ranges, value_dup ((GnmValue *)v));
+		flags = link_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b);
+		dyn->ranges = g_slist_prepend (dyn->ranges, gnm_rangeref_dup (rr));
 	}
 	if (flags & DEPENDENT_HAS_3D)
 		workbook_link_3d_dep (dep);
@@ -997,10 +999,11 @@ dependent_clear_dynamic_deps (GnmDependent *dep)
 void
 dependent_link (GnmDependent *dep)
 {
-	Sheet *sheet;
+	Sheet	   *sheet;
+	GnmEvalPos  ep;
 
 	g_return_if_fail (dep != NULL);
-	g_return_if_fail (dep->expression != NULL);
+	g_return_if_fail (dep->texpr != NULL);
 	g_return_if_fail (!(dep->flags & DEPENDENT_IS_LINKED));
 	g_return_if_fail (IS_SHEET (dep->sheet));
 	g_return_if_fail (dep->sheet->deps != NULL);
@@ -1015,9 +1018,8 @@ dependent_link (GnmDependent *dep)
 	else
 		sheet->deps->head = dep; /* first element */
 	sheet->deps->tail = dep;
-	dep->flags |=
-		DEPENDENT_IS_LINKED |
-		link_expr_dep (dep, dep->expression);
+	dep->flags |= DEPENDENT_IS_LINKED |
+		link_expr_dep (eval_pos_init_dep (&ep, dep), dep->texpr->expr);
 
 	if (dep->flags & DEPENDENT_HAS_3D)
 		workbook_link_3d_dep (dep);
@@ -1037,10 +1039,10 @@ dependent_unlink (GnmDependent *dep)
 
 	g_return_if_fail (dep != NULL);
 	g_return_if_fail (dependent_is_linked (dep));
-	g_return_if_fail (dep->expression != NULL);
+	g_return_if_fail (dep->texpr != NULL);
 	g_return_if_fail (IS_SHEET (dep->sheet));
 
-	unlink_expr_dep (dep, dep->expression);
+	unlink_expr_dep (dep, dep->texpr->expr);
 	contain = dep->sheet->deps;
 	if (contain != NULL) {
 		if (contain->head == dep)
@@ -1092,7 +1094,7 @@ cell_eval_content (GnmCell *cell)
 #ifdef DEBUG_EVALUATION
 	{
 		GnmParsePos pp;
-		char *str = gnm_expr_as_string (cell->base.expression,
+		char *str = gnm_expr_top_as_string (cell->base.texpr,
 			parse_pos_init_cell (&pp, cell), gnm_expr_conventions_default);
 		g_print ("{\nEvaluating %s: %s;\n", cell_name (cell), str);
 		g_free (str);
@@ -1141,8 +1143,8 @@ cell_eval_content (GnmCell *cell)
 	max_iteration = cell->base.sheet->workbook->iteration.max_number;
 
 iterate :
-	v = gnm_expr_eval (cell->base.expression, &pos,
-		GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+	v = gnm_expr_top_eval (cell->base.texpr, &pos,
+			       GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	if (v == NULL)
 		v = value_new_error (&pos, "Internal error");
 
@@ -1218,7 +1220,7 @@ dependent_eval (GnmDependent *dep)
 		int const t = dependent_type (dep);
 
 		if (t != DEPENDENT_CELL) {
-			DependentClass *klass = g_ptr_array_index (dep_classes, t);
+			GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
 
 			g_return_val_if_fail (klass, FALSE);
 
@@ -1441,7 +1443,7 @@ typedef struct
 		GnmEvalPos    pos;
 		GnmDependent *dep;
 	} u;
-	GnmExpr const *oldtree;
+	GnmExprTop const *oldtree;
 } ExprRelocateStorage;
 
 /**
@@ -1456,7 +1458,7 @@ dependents_unrelocate_free (GSList *info)
 	GSList *ptr = info;
 	for (; ptr != NULL ; ptr = ptr->next) {
 		ExprRelocateStorage *tmp = ptr->data;
-		gnm_expr_unref (tmp->oldtree);
+		gnm_expr_top_unref (tmp->oldtree);
 		g_free (tmp);
 	}
 	g_slist_free (info);
@@ -1496,7 +1498,7 @@ dependents_unrelocate (GSList *info)
 			dependent_flag_recalc (tmp->u.dep);
 			dependent_link (tmp->u.dep);
 		}
-		gnm_expr_unref (tmp->oldtree);
+		gnm_expr_top_unref (tmp->oldtree);
 		g_free (tmp);
 	}
 	g_slist_free (info);
@@ -1540,7 +1542,8 @@ cb_range_contained_collect (DependencyRange const *deprange,
 
 	if (range_overlap (user->target, range))
 		micro_hash_foreach_dep (deprange->deps, dep, {
-			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE))) {
+			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE)) &&
+			    dependent_type (dep) != DEPENDENT_DYNAMIC_DEP) {
 				dep->flags |= DEPENDENT_FLAGGED;
 				user->list = g_slist_prepend (user->list, dep);
 			}});
@@ -1553,7 +1556,8 @@ cb_single_contained_collect (DependencySingle const *depsingle,
 {
 	if (range_contains (user->target, depsingle->pos.col, depsingle->pos.row))
 		micro_hash_foreach_dep (depsingle->deps, dep, {
-			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE))) {
+			if (!(dep->flags & (DEPENDENT_FLAGGED | DEPENDENT_CAN_RELOCATE)) &&
+			    dependent_type (dep) != DEPENDENT_DYNAMIC_DEP) {
 				dep->flags |= DEPENDENT_FLAGGED;
 				user->list = g_slist_prepend (user->list, dep);
 			}});
@@ -1561,24 +1565,22 @@ cb_single_contained_collect (DependencySingle const *depsingle,
 
 /**
  * dependents_relocate:
- * Fixes references to or from a region that is going to be moved.
- *
  * @info : the descriptor record for what is being moved where.
  *
+ * Fixes references to or from a region that is going to be moved.
  * Returns a list of the locations and expressions that were changed outside of
  * the region.
+ *
  * NOTE : Does not queue the changed elemenents or their recursive dependents
- * for recalc
- */
+ * 	for recalc
+ **/
 GSList *
 dependents_relocate (GnmExprRelocateInfo const *info)
 {
 	GnmExprRewriteInfo rwinfo;
-	GnmDependent *dep;
 	GSList    *l, *dependents = NULL, *undo_info = NULL;
 	Sheet	  *sheet;
 	GnmRange const   *r;
-	GnmExpr const *newtree;
 	int i;
 	CollectClosure collect;
 	GHashTable *names;
@@ -1622,11 +1624,13 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 	}
 	dependents = collect.list;
 
-	rwinfo.type = GNM_EXPR_REWRITE_RELOCATE;
+	rwinfo.rw_type = GNM_EXPR_REWRITE_EXPR;
 	memcpy (&rwinfo.u.relocate, info, sizeof (GnmExprRelocateInfo));
 
 	for (l = dependents; l; l = l->next) {
-		dep = l->data;
+		GnmExprTop const *newtree;
+		GnmDependent *dep = l->data;
+
 		dep->flags &= ~DEPENDENT_FLAGGED;
 		sheet_flag_status_update_range (dep->sheet, NULL);
 
@@ -1634,7 +1638,7 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 
 		/* it is possible nothing changed for contained deps
 		 * using absolute references */
-		newtree = gnm_expr_rewrite (dep->expression, &rwinfo);
+		newtree = gnm_expr_top_rewrite (dep->texpr, &rwinfo);
 		if (newtree != NULL) {
 			int const t = dependent_type (dep);
 			ExprRelocateStorage *tmp =
@@ -1647,12 +1651,12 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 					tmp->u.pos = rwinfo.u.relocate.pos;
 				else
 					tmp->u.dep = dep;
-				tmp->oldtree = dep->expression;
-				gnm_expr_ref (tmp->oldtree);
+				tmp->oldtree = dep->texpr;
+				gnm_expr_top_ref (tmp->oldtree);
 				undo_info = g_slist_prepend (undo_info, tmp);
 
 				dependent_set_expr (dep, newtree); /* unlinks */
-				gnm_expr_unref (newtree);
+				gnm_expr_top_unref (newtree);
 
 				/* queue the things that depend on the changed dep
 				 * even if it is going to move.
@@ -1683,6 +1687,7 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 
 	names = info->origin_sheet->deps->referencing_names;
 	if (names != NULL) {
+		rwinfo.rw_type = GNM_EXPR_REWRITE_NAME;
 	}
 
 	g_slist_free (dependents);
@@ -1704,8 +1709,9 @@ cb_collect_range (G_GNUC_UNUSED gpointer key,
 static void
 dep_hash_destroy (GHashTable *hash, GSList **dyn_deps, Sheet *sheet, gboolean destroy)
 {
-	GSList *deps = NULL, *l2;
+	GSList *deps = NULL, *l;
 	GnmExprRewriteInfo rwinfo;
+	GSList *deplist = NULL;
 
 	/* We collect first because we will be changing the hash.  */
 	if (destroy) {
@@ -1717,10 +1723,8 @@ dep_hash_destroy (GHashTable *hash, GSList **dyn_deps, Sheet *sheet, gboolean de
 		g_hash_table_foreach (hash, (GHFunc)cb_collect_range, &deps);
 	}
 
-	rwinfo.type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
-	for (l2 = deps; l2; l2 = l2->next) {
-		DependencyAny *depany = l2->data;
-		GSList *deplist = NULL, *l;
+	for (l = deps; l; l = l->next) {
+		DependencyAny *depany = l->data;
 
 		micro_hash_foreach_dep (depany->deps, dep, {
 			if (dependent_type (dep) == DEPENDENT_DYNAMIC_DEP) {
@@ -1738,34 +1742,43 @@ dep_hash_destroy (GHashTable *hash, GSList **dyn_deps, Sheet *sheet, gboolean de
 			}
 		});
 
-		for (l = deplist; l; l = l->next) {
-			GnmDependent *dep = l->data;
-			GnmExpr *e = (GnmExpr *)dep->expression;
-			/* We are told this dependent depends on this region, hence if
-			 * newtree is null then either
-			 * 1) we did not depend on it (ie., serious breakage )
-			 * 2) we had a duplicate reference and we have already removed it.
-			 * 3) We depended on things via a name which will be
-			 *    invalidated elsewhere */
-			GnmExpr const *newtree = gnm_expr_rewrite (e, &rwinfo);
-			if (newtree != NULL) {
-				if (!destroy) {
-					gnm_expr_ref (e);
-					sheet->revive.dep_exprs =
-						g_slist_prepend
-						(g_slist_prepend (sheet->revive.dep_exprs, e),
-						 dep);
-				}
-				dependent_set_expr (dep, newtree);
-				gnm_expr_unref (newtree);
-			}
-		}
-		g_slist_free (deplist);
-
 		if (destroy)
 			micro_hash_release (&depany->deps);
 	}
 	g_slist_free (deps);
+
+	/*
+	 * We do this after the above loop as this loop will
+	 * invalidate some of the DependencyAny pointers from
+	 * above.  The testcase for that is 314207, deleting
+	 * all but the first sheet in one go.
+	 */
+	rwinfo.rw_type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
+	for (l = deplist; l; l = l->next) {
+		GnmDependent *dep = l->data;
+		GnmExprTop const *te = dep->texpr;
+		/* We are told this dependent depends on this region, hence if
+		 * newtree is null then either
+		 * 1) we did not depend on it (ie., serious breakage )
+		 * 2) we had a duplicate reference and we have already removed it.
+		 * 3) We depended on things via a name which will be
+		 *    invalidated elsewhere */
+		GnmExprTop const *newtree = gnm_expr_top_rewrite (te, &rwinfo);
+		if (newtree != NULL) {
+			if (!destroy) {
+				gnm_expr_top_ref (te);
+				sheet->revive.dep_texprs =
+					g_slist_prepend
+					(g_slist_prepend (sheet->revive.dep_texprs, (gpointer)te),
+					 dep);
+			}
+			dependent_set_expr (dep, newtree);
+			gnm_expr_top_unref (newtree);
+		}
+	}
+	g_slist_free (deplist);
+
+
 }
 
 static void
@@ -1799,8 +1812,8 @@ cb_collect_deps_of_names (GnmNamedExpr *nexpr,
 static void
 invalidate_name (GnmNamedExpr *nexpr, Sheet *sheet, gboolean destroy)
 {
-	GnmExpr *old_expr = (GnmExpr *)nexpr->expr;
-	GnmExpr const *new_expr = NULL;
+	GnmExprTop const *old_expr = nexpr->texpr;
+	GnmExprTop const *new_expr = NULL;
 	gboolean scope_being_killed =
 		nexpr->pos.sheet
 		? nexpr->pos.sheet->being_invalidated
@@ -1808,8 +1821,8 @@ invalidate_name (GnmNamedExpr *nexpr, Sheet *sheet, gboolean destroy)
 
 	if (!scope_being_killed) {
 		GnmExprRewriteInfo rwinfo;
-		rwinfo.type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
-		new_expr = gnm_expr_rewrite (old_expr, &rwinfo);
+		rwinfo.rw_type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
+		new_expr = gnm_expr_top_rewrite (old_expr, &rwinfo);
 		g_return_if_fail (new_expr != NULL);
 	}
 
@@ -1817,12 +1830,14 @@ invalidate_name (GnmNamedExpr *nexpr, Sheet *sheet, gboolean destroy)
 		g_warning ("Left-over name dependencies:\n");
 
 	if (!destroy) {
-		gnm_expr_ref (old_expr);
-		sheet->revive.name_exprs =
-			g_slist_prepend (sheet->revive.name_exprs, old_expr);
+		gnm_expr_top_ref (old_expr);
+		sheet->revive.name_texprs =
+			g_slist_prepend (sheet->revive.name_texprs,
+					 (gpointer)old_expr);
 		expr_name_ref (nexpr);
-		sheet->revive.name_exprs =
-			g_slist_prepend (sheet->revive.name_exprs, nexpr);
+		sheet->revive.name_texprs =
+			g_slist_prepend (sheet->revive.name_texprs,
+					 nexpr);
 	}
 
 	expr_name_set_expr (nexpr, new_expr);
@@ -1912,24 +1927,24 @@ clear_revive_info (Sheet *sheet)
 {
 	GSList *l;
 
-	for (l = sheet->revive.name_exprs; l; l = l->next->next) {
+	for (l = sheet->revive.name_texprs; l; l = l->next->next) {
 		GnmNamedExpr *nexpr = l->data;
-		GnmExpr *expr = l->next->data;
+		GnmExprTop const *texpr = l->next->data;
 
 		expr_name_unref (nexpr);
-		gnm_expr_unref (expr);
+		gnm_expr_top_unref (texpr);
 	}
-	g_slist_free (sheet->revive.name_exprs);
-	sheet->revive.name_exprs = NULL;
+	g_slist_free (sheet->revive.name_texprs);
+	sheet->revive.name_texprs = NULL;
 
-	for (l = sheet->revive.dep_exprs; l; l = l->next->next) {
+	for (l = sheet->revive.dep_texprs; l; l = l->next->next) {
 		GnmDependent *dep = l->data;
-		GnmExpr *expr = l->next->data;
+		GnmExprTop *texpr = l->next->data;
 		(void)dep;
-		gnm_expr_unref (expr);
+		gnm_expr_top_unref (texpr);
 	}
-	g_slist_free (sheet->revive.dep_exprs);
-	sheet->revive.dep_exprs = NULL;
+	g_slist_free (sheet->revive.dep_texprs);
+	sheet->revive.dep_texprs = NULL;
 
 	g_slist_free (sheet->revive.relink);
 	sheet->revive.relink = NULL;
@@ -2068,22 +2083,22 @@ tweak_3d (Sheet *sheet, gboolean destroy)
 			      (GHFunc)cb_tweak_3d,
 			      &deps);
 
-	rwinfo.type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
+	rwinfo.rw_type = GNM_EXPR_REWRITE_INVALIDATE_SHEETS;
 	for (l = deps; l; l = l->next) {
 		GnmDependent *dep = l->data;
-		GnmExpr *e = (GnmExpr *)dep->expression;
-		const GnmExpr *newtree = gnm_expr_rewrite (e, &rwinfo);
+		GnmExprTop const *te = dep->texpr;
+		GnmExprTop const *newtree = gnm_expr_top_rewrite (te, &rwinfo);
 
 		if (newtree != NULL) {
 			if (!destroy) {
-				gnm_expr_ref (e);
-				sheet->revive.dep_exprs =
+				gnm_expr_top_ref (te);
+				sheet->revive.dep_texprs =
 					g_slist_prepend
-					(g_slist_prepend (sheet->revive.dep_exprs, e),
+					(g_slist_prepend (sheet->revive.dep_texprs, (gpointer)te),
 					 dep);
 			}
 			dependent_set_expr (dep, newtree);
-			gnm_expr_unref (newtree);
+			gnm_expr_top_unref (newtree);
 			dependent_link (dep);
 			dependent_changed (dep);
 		}
@@ -2181,18 +2196,18 @@ dependents_revive_sheet (Sheet *sheet)
 	GSList *l;
 
 	/* Restore the expressions of names that got changed.  */
-	for (l = sheet->revive.name_exprs; l; l = l->next->next) {
+	for (l = sheet->revive.name_texprs; l; l = l->next->next) {
 		GnmNamedExpr *nexpr = l->data;
-		GnmExpr *expr = l->next->data;
-		gnm_expr_ref (expr);
-		expr_name_set_expr (nexpr, expr);
+		GnmExprTop const *texpr = l->next->data;
+		gnm_expr_top_ref (texpr);
+		expr_name_set_expr (nexpr, texpr);
 	}
 
 	/* Restore the expressions of deps that got changed.  */
-	for (l = sheet->revive.dep_exprs; l; l = l->next->next) {
+	for (l = sheet->revive.dep_texprs; l; l = l->next->next) {
 		GnmDependent *dep = l->data;
-		GnmExpr *expr = l->next->data;
-		dependent_set_expr (dep, expr);
+		GnmExprTop const *texpr = l->next->data;
+		dependent_set_expr (dep, texpr);
 		dependent_link (dep);
 		dependent_changed (dep);
 	}
@@ -2232,7 +2247,9 @@ workbook_recalc (Workbook *wb)
 
 	WORKBOOK_FOREACH_DEPENDENT (wb, dep, redraw |= dependent_eval (dep););
 	if (redraw) {
-		WORKBOOK_FOREACH_SHEET (wb, sheet, sheet_redraw_all (sheet, FALSE););
+		WORKBOOK_FOREACH_SHEET (wb, sheet, {
+			SHEET_FOREACH_VIEW (sheet, sv, sv_flag_selection_change (sv););
+			sheet_redraw_all (sheet, FALSE);});
 	}
 }
 
@@ -2256,21 +2273,21 @@ dynamic_dep_free (DynamicDep *dyn)
 {
 	GnmDependent *dep = dyn->container;
 	GnmCellPos const *pos = dependent_pos (dep);
-	GnmValueRange *v;
+	GnmRangeRef *rr;
 	GSList *ptr;
 
 	for (ptr = dyn->singles ; ptr != NULL ; ptr = ptr->next) {
-		v = ptr->data;
-		unlink_single_dep (&dyn->base, pos, &v->cell.a);
-		value_release ((GnmValue *)v);
+		rr = ptr->data;
+		unlink_single_dep (&dyn->base, pos, &rr->a);
+		g_free (rr);
 	}
 	g_slist_free (dyn->singles);
 	dyn->singles = NULL;
 
 	for (ptr = dyn->ranges ; ptr != NULL ; ptr = ptr->next) {
-		v = ptr->data;
-		unlink_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
-		value_release ((GnmValue *)v);
+		rr = ptr->data;
+		unlink_cellrange_dep (&dyn->base, pos, &rr->a, &rr->b);
+		g_free (rr);
 	}
 	g_slist_free (dyn->ranges);
 	dyn->ranges = NULL;
@@ -2332,7 +2349,7 @@ dump_range_dep (gpointer key, G_GNUC_UNUSED gpointer value,
 	GString *target = g_string_new (NULL);
 
 	g_string_append (target, "    ");
-	g_string_append (target, range_name (range));
+	g_string_append (target, range_as_string (range));
 	g_string_append (target, " -> ");
 
 	micro_hash_foreach_list (deprange->deps, list,
@@ -2382,16 +2399,14 @@ dump_dynamic_dep (gpointer key, G_GNUC_UNUSED gpointer value,
 
 	g_string_append (target, ", s=[");
 	for (l = dyn->singles; l; l = l->next) {
-		GnmValueRange const *v = l->data;
-		rangeref_as_string (target, gnm_expr_conventions_default, &v->cell, &pp);
+		rangeref_as_string (target, gnm_expr_conventions_default, l->data, &pp);
 		if (l->next)
 			g_string_append (target, ", ");
 	}
 
 	g_string_append (target, "], r=[");
 	for (l = dyn->ranges; l; l = l->next) {
-		GnmValueRange const *v = l->data;
-		rangeref_as_string (target, gnm_expr_conventions_default, &v->cell, &pp);
+		rangeref_as_string (target, gnm_expr_conventions_default, l->data, &pp);
 		if (l->next)
 			g_string_append (target, ", ");
 	}
@@ -2483,7 +2498,7 @@ gnm_dep_container_dump (GnmDepContainer const *deps)
 void
 gnm_dep_container_sanity_check (GnmDepContainer const *deps)
 {
-	const GnmDependent *dep;
+	GnmDependent const *dep;
 	GHashTable *seenb4;
 
 	if (deps->head && !deps->tail)
@@ -2536,7 +2551,7 @@ dependent_debug_name (GnmDependent const *dep, GString *target)
 
 	t = dependent_type (dep);
 	if (t != DEPENDENT_CELL) {
-		DependentClass *klass = g_ptr_array_index (dep_classes, t);
+		GnmDependentClass *klass = g_ptr_array_index (dep_classes, t);
 
 		g_return_if_fail (klass);
 		(*klass->debug_name) (dep, target);
