@@ -9,7 +9,7 @@ struct _GnmDependent
 {
 	guint	  flags;
 	Sheet	 *sheet;
-	GnmExpr const *expression;
+	GnmExprTop const *texpr;
 
 	/* Double-linked list.  */
 	GnmDependent *next_dep, *prev_dep;
@@ -17,9 +17,9 @@ struct _GnmDependent
 
 typedef struct {
 	void (*eval) (GnmDependent *dep);
-	void (*set_expr) (GnmDependent *dep, GnmExpr const *new_expr);
+	void (*set_expr) (GnmDependent *dep, GnmExprTop const *new_texpr);
 	void (*debug_name) (GnmDependent const *dep, GString *target);
-} DependentClass;
+} GnmDependentClass;
 
 typedef enum {
 	DEPENDENT_NO_FLAG	   = 0,
@@ -41,9 +41,10 @@ typedef enum {
 	DEPENDENT_GOES_INTERBOOK   = 0x00020000,
 	DEPENDENT_USES_NAME	   = 0x00040000,
 	DEPENDENT_HAS_3D	   = 0x00080000,
-	DEPENDENT_ALWAYS_UNLINK    = 0x00100000,
+	DEPENDENT_ALWAYS_UNLINK    = 0x00100000,	/* what should this do ? */
 	DEPENDENT_HAS_DYNAMIC_DEPS = 0x00200000,
-	DEPENDENT_LINK_FLAGS	   = 0x003ff000,
+	DEPENDENT_IGNORE_ARGS	   = 0x00400000,
+	DEPENDENT_LINK_FLAGS	   = 0x007ff000,
 
 	/* An internal utility flag */
 	DEPENDENT_FLAGGED	   = 0x01000000,
@@ -79,17 +80,17 @@ struct _GnmDepContainer {
 
 typedef void (*DepFunc) (GnmDependent *dep, gpointer user);
 
-guint32	 dependent_type_register   (DependentClass const *klass);
+guint32	 dependent_type_register   (GnmDependentClass const *klass);
 void	 dependent_types_init	   (void);
 void	 dependent_types_shutdown  (void);
 
-void	 dependent_set_expr	   (GnmDependent *dep, GnmExpr const *new_expr);
+void	 dependent_set_expr	   (GnmDependent *dep, GnmExprTop const *new_texpr);
 void	 dependent_set_sheet	   (GnmDependent *dep, Sheet *sheet);
 void	 dependent_link		   (GnmDependent *dep);
 void	 dependent_unlink	   (GnmDependent *dep);
 gboolean dependent_eval		   (GnmDependent *dep);
 void	 dependent_queue_recalc	   (GnmDependent *dep);
-void	 dependent_add_dynamic_dep (GnmDependent *dep, GnmValueRange const *v);
+void	 dependent_add_dynamic_dep (GnmDependent *dep, GnmRangeRef const *rr);
 
 GSList  *dependents_relocate	    (GnmExprRelocateInfo const *info);
 void     dependents_unrelocate      (GSList *info);
@@ -127,7 +128,7 @@ t ## _get_dep_type (void)					\
 {								\
 	static guint32 type = 0;				\
 	if (type == 0) {					\
-		static DependentClass klass;			\
+		static GnmDependentClass klass;			\
 		klass.eval	 = &t ## _eval;			\
 		klass.set_expr	 = set_expr_handler;		\
 		klass.debug_name = &t ## _debug_name;		\
