@@ -40,12 +40,12 @@
 #include <gtk/gtktable.h>
 #include <gtk/gtkbox.h>
 
-char const *output_group[] = {
+char const * const output_group[] = {
 	"newsheet-button",
 	"newworkbook-button",
 	"outputrange-button",
 	"inplace-button",          /* used only in advanced filter  */
-	0
+	NULL
 };
 
 /**
