@@ -212,7 +212,7 @@ error_in_entry (GenericToolState *state, GtkWidget *entry, const char *err_str)
  * Destroy the dialog and associated data structures.
  *
  **/
-gboolean
+static gboolean
 tool_destroy (GtkObject *w, GenericToolState  *state)
 {
 	gboolean res = FALSE;
@@ -333,7 +333,7 @@ dialog_tool_init (GenericToolState *state,
 	GtkWidget *widget;
 
 	state->wbcg  = wbcg;
-	state->wb    = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb    = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sheet;
 	state->sv    = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	state->warning_dialog = NULL;
