@@ -107,10 +107,10 @@ map_op (AutoFilterState *state, GnmFilterOp *op,
 	default :
 		g_warning ("huh?");
 		return NULL;
-	};
+	}
 
 	if (v == NULL) {
-		Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (state->wbcg));
+		Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (state->wbcg));
 		v = format_match (txt, NULL, workbook_date_conv (wb));
 	}
 	if (v == NULL)
@@ -200,9 +200,9 @@ init_operator (AutoFilterState *state, GnmFilterOp op, GnmValue const *v,
 	case GNM_FILTER_OP_NOT_EQUAL:	i = 2; break;
 	default :
 		return;
-	};
+	}
 
-	if (v != NULL && v->type == VALUE_STRING && (i == 1 || i == 2)) {
+	if (v != NULL && VALUE_IS_STRING (v) && (i == 1 || i == 2)) {
 		unsigned const len = strlen (str);
 
 		/* there needs to be at least 1 letter */
