@@ -45,14 +45,9 @@
 #include <selection.h>
 #include <ranges.h>
 
-#include <libfoocanvas/foo-canvas-rect-ellipse.h>
+#include <goffice/cut-n-paste/foocanvas/foo-canvas-rect-ellipse.h>
 #include <glade/glade.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkcelllayout.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkvscrollbar.h>
-#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtk.h>
 #include <gsf/gsf-impl-utils.h>
 #include <string.h>
 
@@ -72,7 +67,7 @@
 /* Keep these strings very short.
    They are used as a sample data for a sheet, so you can put anything here
    ("One", "Two", "Three" for example) */
-static char const *
+static char const *const
 demotable[PREVIEW_ROWS][PREVIEW_COLS] = {
 	{ N_(" ")    , N_("Jan"), N_("Feb"), N_("Mar"), N_("Total") },
 	{ N_("North"),   N_("6"),  N_("13"),  N_("20"),    N_("39") },
@@ -220,13 +215,13 @@ templates_load (AutoFormatState *state)
 	GSList *l;
 	gint n_templates;
 
-	g_assert (state != NULL);
+	g_return_val_if_fail (state != NULL, FALSE);
 
 	if (state->category_groups == NULL)
 		return FALSE;
 
 	state->templates = category_group_get_templates_list (
-		state->current_category_group, GNM_CMD_CONTEXT (state->wbcg));
+		state->current_category_group, GO_CMD_CONTEXT (state->wbcg));
 	for (l = state->templates; l != NULL; l = l->next) {
 		FormatTemplate *ft = l->data;
 		range_init (&ft->dimension,
@@ -341,9 +336,9 @@ previews_load (AutoFormatState *state, int topindex)
 					"y1", (double)(-INNER_BORDER),
 					"x2", (double)(TOTAL_WIDTH + INNER_BORDER),
 					"y2", (double)(TOTAL_HEIGHT + INNER_BORDER),
-					"width_pixels", (int) 3,
-					"outline_color", "red",
-					"fill_color", NULL,
+					"width-pixels", (int) 3,
+					"outline-color", "red",
+					"fill-color", NULL,
 					NULL);
 				gtk_frame_set_shadow_type (state->frame[i], GTK_SHADOW_IN);
 			} else
@@ -358,7 +353,7 @@ previews_load (AutoFormatState *state, int topindex)
 
 			gtk_tooltips_set_tip (state->tooltips,
 				GTK_WIDGET (state->canvas[i]),
-				ft->name, "");
+				_(ft->name), "");
 
 			gtk_widget_show (GTK_WIDGET (state->canvas[i]));
 			start = g_slist_next (start);
@@ -437,12 +432,12 @@ cb_canvas_button_press (FooCanvas *canvas,
 
 	ft = ptr->data;
 	state->selected_template = ft;
-	gtk_entry_set_text (state->info_name, ft->name);
+	gtk_entry_set_text (state->info_name,   _(ft->name));
 	gtk_entry_set_text (state->info_author, ft->author);
 	gnumeric_textview_set_text (GTK_TEXT_VIEW (state->info_descr),
-		ft->description);
+		_(ft->description));
 
-	gtk_entry_set_text (state->info_cat, ft->category->name);
+	gtk_entry_set_text (state->info_cat, _(ft->category->name));
 
 	return TRUE;
 }
@@ -488,9 +483,9 @@ cb_category_changed (AutoFormatState *state)
 		g_warning ("Error while loading templates!");
 
 	gtk_tooltips_set_tip (state->tooltips, GTK_WIDGET (state->category),
-		(state->current_category_group->description != NULL)
+		_((state->current_category_group->description != NULL)
 			? state->current_category_group->description
-			: state->current_category_group->name,
+			: state->current_category_group->name),
 		"");
 
 	previews_load (state, 0);
@@ -525,7 +520,7 @@ category_group_cmp (gconstpointer a, gconstpointer b)
 {
 	FormatTemplateCategoryGroup const *group_a = a;
 	FormatTemplateCategoryGroup const *group_b = b;
-	return g_utf8_collate (group_a->name, group_b->name);
+	return g_utf8_collate (_(group_a->name), _(group_b->name));
 }
 
 static gboolean
@@ -555,13 +550,13 @@ dialog_autoformat (WorkbookControlGUI *wbcg)
 	AutoFormatState *state;
 	int i;
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"autoformat.glade", NULL, NULL);
 	if (gui == NULL)
 		return;
 
 	state = g_new0 (AutoFormatState, 1);
-	state->wb              = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb              = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->wbcg            = wbcg;
 	state->gui             = gui;
 	state->templates       = NULL;
@@ -577,8 +572,12 @@ dialog_autoformat (WorkbookControlGUI *wbcg)
 	state->more_down         = FALSE;
 	state->selected_template = NULL;
 	state->tooltips          = gtk_tooltips_new ();
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+	g_object_ref_sink (state->tooltips);
+#else
 	g_object_ref (state->tooltips);
 	gtk_object_sink (GTK_OBJECT (state->tooltips));
+#endif
 
 	state->dialog     = GTK_DIALOG (glade_xml_get_widget (gui, "dialog"));
 	state->category   = GTK_COMBO_BOX (glade_xml_get_widget (gui, "format_category"));
@@ -666,14 +665,11 @@ dialog_autoformat (WorkbookControlGUI *wbcg)
 
 		for (i = 0 ; ptr != NULL ; ptr = ptr->next, i++) {
 			FormatTemplateCategoryGroup *group = ptr->data;
-			/* This is a name of the "General" autoformat template category.
-			   Please use the same translation as in General.category XML file */
-			if (!strcmp (group->name, _("General")) ||
-			    !strcmp (group->name,   "General" ))
+			if (!strcmp (group->name,   "General" ))
 				select = i;
 			gtk_list_store_append (store, &iter);
 			gtk_list_store_set (store, &iter,
-						0, group->name,
+						0, _(group->name),
 						-1);
 		}
 
@@ -693,7 +689,7 @@ dialog_autoformat (WorkbookControlGUI *wbcg)
 	/* a candidate for merging into attach guru */
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify)cb_autoformat_destroy);
-	gnumeric_non_modal_dialog (wbcg_toplevel (state->wbcg),
+	go_gtk_nonmodal_dialog (wbcg_toplevel (state->wbcg),
 				   GTK_WINDOW (state->dialog));
 	wbcg_edit_attach_guru (state->wbcg, GTK_WIDGET (state->dialog));
 	/* not show all or the scrollbars will appear */
