@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * dialog-autosave.c:
  *
@@ -22,7 +23,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  **/
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -30,6 +30,8 @@
 #include <workbook.h>
 #include <workbook-control-gui-priv.h>
 #include <gui-util.h>
+#include <goffice/app/go-doc.h>
+#include <glib/gi18n.h>
 
 #include <glade/glade.h>
 #include <gtk/gtktogglebutton.h>
@@ -54,38 +56,33 @@ autosave_set_sensitivity (G_GNUC_UNUSED GtkWidget *widget,
 {
         gboolean active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (state->autosave_on_off));
 	gint minutes;
-	gint minutes_valid = entry_to_int (GTK_ENTRY (state->minutes_entry), &minutes, FALSE);
+	gboolean minutes_err = entry_to_int (GTK_ENTRY (state->minutes_entry), &minutes, FALSE);
 
 	gtk_widget_set_sensitive (state->minutes_entry, active);
 	gtk_widget_set_sensitive (state->prompt_cb, active);
 
-	gtk_widget_set_sensitive (state->ok_button, !active ||
-				  ((minutes_valid == 0) && (minutes > 0)));
-
+	gtk_widget_set_sensitive (state->ok_button,
+				  !active || (!minutes_err && minutes > 0));
 }
 
 gboolean
 dialog_autosave_prompt (WorkbookControlGUI *wbcg)
 {
-	gint result;
-	GtkWidget *dialog;
-	const char *uri;
-
-	uri = workbook_get_uri (wb_control_workbook (WORKBOOK_CONTROL (wbcg)));
-	dialog = gtk_message_dialog_new (wbcg_toplevel (wbcg),
+	char const *uri   = go_doc_get_uri (
+		wb_control_get_doc (WORKBOOK_CONTROL (wbcg)));
+	GtkWidget *dialog = gtk_message_dialog_new (wbcg_toplevel (wbcg),
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_YES_NO,
 					 _("Do you want to save the workbook %s ?"),
 					 uri);
-	result = gtk_dialog_run (GTK_DIALOG (dialog));
+	gint result = gtk_dialog_run (GTK_DIALOG (dialog));
+
 	gtk_widget_destroy (dialog);
 
 	return result == GTK_RESPONSE_YES;
 }
 
-
-
 /**
  * dialog_autosave:
  * @window:
@@ -135,19 +132,19 @@ cb_autosave_cancel (G_GNUC_UNUSED GtkWidget *button,
 static void
 cb_autosave_ok (G_GNUC_UNUSED GtkWidget *button, autosave_t *state)
 {
-		if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (state->autosave_on_off))) {
-			int minutes;
-			int minutes_valid = entry_to_int (GTK_ENTRY (state->minutes_entry),
-							  &minutes, TRUE);
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (state->autosave_on_off))) {
+		int minutes;
+		gboolean minutes_err = entry_to_int (GTK_ENTRY (state->minutes_entry),
+						     &minutes, TRUE);
 
-			g_return_if_fail (minutes_valid == 0); /* Why is ok active? */
+		g_return_if_fail (!minutes_err); /* Why is ok active? */
 
-		        wbcg_autosave_set (state->wbcg, minutes,
-					   gtk_toggle_button_get_active (
-						   GTK_TOGGLE_BUTTON (state->prompt_cb)));
-		} else
-			wbcg_autosave_set (state->wbcg, 0, FALSE);
-		gtk_widget_destroy (state->dialog);
+		wbcg_autosave_set (state->wbcg, minutes,
+				   gtk_toggle_button_get_active (
+					   GTK_TOGGLE_BUTTON (state->prompt_cb)));
+	} else
+		wbcg_autosave_set (state->wbcg, 0, FALSE);
+	gtk_widget_destroy (state->dialog);
 }
 
 void
@@ -160,7 +157,7 @@ dialog_autosave (WorkbookControlGUI *wbcg)
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, AUTOSAVE_KEY))
 		return;
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"autosave.glade", NULL, NULL);
         if (gui == NULL)
                 return;
@@ -168,7 +165,7 @@ dialog_autosave (WorkbookControlGUI *wbcg)
 
 	state = g_new (autosave_t, 1);
 	state->wbcg = wbcg;
-	state->wb   = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb   = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->gui  = gui;
 
 	state->dialog = glade_xml_get_widget (state->gui, "AutoSave");
@@ -180,7 +177,7 @@ dialog_autosave (WorkbookControlGUI *wbcg)
 
 	if (!state->dialog || !state->minutes_entry || !state->prompt_cb ||
 	    !state->autosave_on_off) {
-		gnumeric_notice (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the autosave dialog."));
 		g_free (state);
 		return;
@@ -222,4 +219,3 @@ dialog_autosave (WorkbookControlGUI *wbcg)
 	gtk_widget_show (state->dialog);
 
 }
-
