@@ -445,10 +445,6 @@ dialog_formula_guru_load_expr (GtkTreePath const *parent_path, gint child_num,
 			       GnmExpr const *expr, FormulaGuruState *state)
 {
 	GtkTreePath *path;
-	char *text;
-	GSList *args;
-	GtkTreeIter iter;
-	int i;
 
 	if (parent_path == NULL)
 		path = gtk_tree_path_new_first ();
@@ -458,27 +454,33 @@ dialog_formula_guru_load_expr (GtkTreePath const *parent_path, gint child_num,
 		gtk_tree_path_append_index (path, child_num);
 	}
 
-	switch (expr->any.oper) {
-	case GNM_EXPR_OP_FUNCALL:
+	switch (GNM_EXPR_GET_OPER (expr)) {
+	case GNM_EXPR_OP_FUNCALL: {
+		int i;
+		GtkTreeIter iter;
+
 		dialog_formula_guru_load_fd (path, expr->func.func, state);
-		for (args = expr->func.arg_list, i = 0; args; args = args->next, i++)
+		for (i = 0; i < expr->func.argc; i++)
 			dialog_formula_guru_load_expr (path, i,
-						       (GnmExpr const *) args->data,
+						       expr->func.argv[i],
 						       state);
-		gtk_tree_path_append_index (path, i - 1);
+		gtk_tree_path_append_index (path, MAX (0, i - 1));
 		if (gtk_tree_model_get_iter (GTK_TREE_MODEL (state->model),
                                              &iter, path))
 			dialog_formula_guru_adjust_varargs (&iter, state);
 
 		break;
+	}
+
 	case GNM_EXPR_OP_ANY_BINARY:
 	case GNM_EXPR_OP_UNARY_NEG:
-	default:
-		text = gnm_expr_as_string (expr, state->pos,
+	default: {
+		char *text = gnm_expr_as_string (expr, state->pos,
 			gnm_expr_conventions_default);
 		dialog_formula_guru_load_string (path, text, state);
 		g_free (text);
 		break;
+	}
 
 	}
 	gtk_tree_path_free (path);
@@ -947,7 +949,7 @@ dialog_formula_guru (WorkbookControlGUI *wbcg, GnmFunc const *fd)
 
 	state = g_new (FormulaGuruState, 1);
 	state->wbcg  = wbcg;
-	state->wb    = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb    = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->gui   = gui;
 	state->active_path = NULL;
 	state->pos = NULL;
@@ -955,7 +957,7 @@ dialog_formula_guru (WorkbookControlGUI *wbcg, GnmFunc const *fd)
 	sv = wb_control_cur_sheet_view (WORKBOOK_CONTROL (wbcg));
 	cell = sheet_cell_get (sv_sheet (sv), sv->edit_pos.col, sv->edit_pos.row);
 	if (cell != NULL && cell_has_expr (cell))
-		expr = gnm_expr_first_func (cell->base.expression);
+		expr = gnm_expr_top_first_funcall (cell->base.texpr);
 
 	if (expr == NULL) {
 		wbcg_edit_start (wbcg, TRUE, TRUE);
