@@ -137,7 +137,7 @@ static void
 cb_dialog_goto_update_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 				   GotoState *state)
 {
-	GnmValue *val = global_range_parse (wb_control_cur_sheet (WORKBOOK_CONTROL (state->wbcg)),
+	GnmValue *val = value_new_cellrange_str (wb_control_cur_sheet (WORKBOOK_CONTROL (state->wbcg)),
 					    gtk_entry_get_text (state->goto_text));
 	if (val != NULL) {
 		gtk_widget_set_sensitive (state->go_button, TRUE);
@@ -341,19 +341,19 @@ dialog_goto_cell (WorkbookControlGUI *wbcg)
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, GOTO_KEY))
 		return;
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"goto.glade", NULL, NULL);
         if (gui == NULL)
                 return;
 
 	state = g_new (GotoState, 1);
 	state->wbcg   = wbcg;
-	state->wb     = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb     = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->gui    = gui;
         state->dialog = glade_xml_get_widget (state->gui, "goto_dialog");
 
 	if (dialog_goto_init (state)) {
-		gnumeric_notice (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR,
 				 _("Could not create the goto dialog."));
 		g_free (state);
 		return;
