@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -132,7 +132,7 @@ dhl_get_target_cur_wb (HyperlinkState *state, gboolean *success)
 			parse_pos_init_sheet (&pp, sheet);
 			nexpr = expr_name_lookup (&pp, target);
 			if (nexpr != NULL)
-				val = gnm_expr_get_range (nexpr->expr);
+				val = gnm_expr_top_get_range (nexpr->texpr);
 		}
 		if (val) {
 			*success = TRUE;
@@ -216,7 +216,7 @@ dhl_get_target_email (HyperlinkState *state, gboolean *success)
 		return g_strconcat ("mailto:", address, NULL);
 	}
 
-	encoded = go_url_encode ((char*)subject);
+	encoded = go_url_encode (subject, 0);
 
 	result = g_strconcat ("mailto:", address, "?subject=", encoded, NULL);
 	g_free (encoded);
@@ -333,11 +333,10 @@ dhl_cb_ok (G_GNUC_UNUSED GtkWidget *button, HyperlinkState *state)
 	if (target) {
 		gnm_hlink_set_target (state->link, target);
 		gnm_hlink_set_tip (state->link, dhl_get_tip (state));
-		style = mstyle_new ();
-		mstyle_set_hlink (style, g_object_ref (state->link));
-		mstyle_set_font_uline (style, UNDERLINE_SINGLE);
-		mstyle_set_color (style, MSTYLE_COLOR_FORE,
-			style_color_new_name ("blue"));
+		style = gnm_style_new ();
+		gnm_style_set_hlink (style, g_object_ref (state->link));
+		gnm_style_set_font_uline (style, UNDERLINE_SINGLE);
+		gnm_style_set_font_color (style, style_color_new_name ("blue"));
 
 		if (state->is_new)
 			cmdname = _("Add Hyperlink");
@@ -346,8 +345,8 @@ dhl_cb_ok (G_GNUC_UNUSED GtkWidget *button, HyperlinkState *state)
 		cmd_selection_format (WORKBOOK_CONTROL (state->wbcg), style,
 				      NULL, cmdname);
 	} else if (!state->is_new) {
-		style = mstyle_new ();
-		mstyle_set_hlink (style, NULL);
+		style = gnm_style_new ();
+		gnm_style_set_hlink (style, NULL);
 		cmdname = _("Remove Hyperlink");
 		cmd_selection_format (WORKBOOK_CONTROL (state->wbcg), style,
 				      NULL, cmdname);
@@ -505,7 +504,7 @@ dialog_hyperlink (WorkbookControlGUI *wbcg, SheetControl *sc)
 	HyperlinkState* state;
 	GnmHLink	*link = NULL;
 	Sheet		*sheet;
-	GList		*ptr;
+	GSList		*ptr;
 
 	g_return_if_fail (wbcg != NULL);
 
@@ -519,7 +518,7 @@ dialog_hyperlink (WorkbookControlGUI *wbcg, SheetControl *sc)
 
 	state = g_new (HyperlinkState, 1);
 	state->wbcg  = wbcg;
-	state->wb   = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb   = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sc   = sc;
 	state->gui  = gui;
         state->dialog = glade_xml_get_widget (state->gui, "hyperlink-dialog");
