@@ -145,7 +145,7 @@ cb_paste_special_response (GtkWidget *dialog,
 	if (response_id == GTK_RESPONSE_OK) {
 		switch (state->type) {
 		case 0: result = PASTE_ALL_TYPES;	break;
-		case 1: result = PASTE_CONTENT;		break;
+		case 1: result = PASTE_CONTENTS;	break;
 		case 2: result = PASTE_AS_VALUES;	break;
 		case 3: result = PASTE_FORMATS;		break;
 		case 4: result = PASTE_COMMENTS;	break;
@@ -194,7 +194,7 @@ dialog_paste_special (WorkbookControlGUI *wbcg)
 	state = g_new0 (PasteSpecialState, 1);
 	state->wbcg   = wbcg;
 	state->dialog = GTK_DIALOG (tmp);
-	gtk_dialog_set_default_response (state->dialog, GTK_RESPONSE_CANCEL);
+	gtk_dialog_set_default_response (state->dialog, GTK_RESPONSE_OK);
 
 	f1  = gtk_frame_new (_("Paste type"));
 	f1v = gtk_vbox_new (TRUE, 0);
