@@ -3,7 +3,7 @@
  *   Dialog for entering a search-and-replace query.
  *
  * Author:
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,9 +33,8 @@
 #include <selection.h>
 
 #include <glade/glade.h>
-#include <gtk/gtktogglebutton.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkstock.h>
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtk.h>
 #include <string.h>
 
 #define SEARCH_REPLACE_KEY "search-replace-dialog"
@@ -50,32 +49,32 @@ typedef struct {
 	SearchDialogCallback cb;
 } DialogState;
 
-static const char *error_group[] = {
+static const char * const error_group[] = {
 	"error_fail",
 	"error_skip",
 	"error_query",
 	"error_error",
 	"error_string",
-	0
+	NULL
 };
 
-static const char *search_type_group[] = {
+static const char * const search_type_group[] = {
 	"search_type_text",
 	"search_type_regexp",
-	0
+	NULL
 };
 
-static const char *scope_group[] = {
+static const char * const scope_group[] = {
 	"scope_workbook",
 	"scope_sheet",
 	"scope_range",
-	0
+	NULL
 };
 
-static const char *direction_group[] = {
+static const char * const direction_group[] = {
 	"row_major",
 	"column_major",
-	0
+	NULL
 };
 
 static gboolean
@@ -102,59 +101,48 @@ ok_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 	GnmSearchReplace *sr;
 	char *err;
 	int i;
-
-	sr = search_replace_new ();
-
-	sr->search_text = g_strdup (gtk_entry_get_text (dd->search_text));
-	sr->replace_text = g_strdup (gtk_entry_get_text (dd->replace_text));
-
-#if 0
-	/* Save the contents of both gnome-entry's. */
-	gnome_entry_append_history (dd->search_text, TRUE, sr->search_text);
-	gnome_entry_append_history (dd->replace_text, TRUE, sr->replace_text);
-#endif
-
-	i = gnumeric_glade_group_value (gui, search_type_group);
-	sr->is_regexp = (i == 1);
+	GnmSearchReplaceScope scope;
 
 	i = gnumeric_glade_group_value (gui, scope_group);
-	sr->scope = (i == -1) ? SRS_sheet : (SearchReplaceScope)i;
+	scope = (i == -1) ? GNM_SRS_SHEET : (GnmSearchReplaceScope)i;
 
-	/* FIXME: parsing of an gnm_expr_entry should happen by the gee */
-	sr->range_text = g_strdup (gnm_expr_entry_get_text (dd->rangetext));
-	sr->curr_sheet = wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg));
-
-	sr->query = is_checked (gui, "query");
-	sr->preserve_case = is_checked (gui, "preserve_case");
-	sr->ignore_case = is_checked (gui, "ignore_case");
-	sr->match_words = is_checked (gui, "match_words");
-
-	sr->search_strings = is_checked (gui, "search_string");
-	sr->search_other_values = is_checked (gui, "search_other");
-	sr->search_expressions = is_checked (gui, "search_expr");
-	sr->search_expression_results = FALSE;
-	sr->search_comments = is_checked (gui, "search_comments");
+	sr = g_object_new (GNM_SEARCH_REPLACE_TYPE,
+			   "sheet", wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg)),
+			   "scope", scope,
+			   "range-text", gnm_expr_entry_get_text (dd->rangetext),
+			   "search-text", gtk_entry_get_text (dd->search_text),
+			   "replace-text", gtk_entry_get_text (dd->replace_text),
+			   "is-regexp", gnumeric_glade_group_value (gui, search_type_group) == 1,
+			   "ignore-case", is_checked (gui, "ignore_case"),
+			   "match-words", is_checked (gui, "match_words"),
+			   "preserve-case", is_checked (gui, "preserve_case"),
+			   "query", is_checked (gui, "query"),
+			   "replace-keep-strings", is_checked (gui, "keep_strings"),
+			   "search-strings", is_checked (gui, "search_string"),
+			   "search-other-values", is_checked (gui, "search_other"),
+			   "search-expressions", is_checked (gui, "search_expr"),
+			   "search-expression-results", FALSE,
+			   "search-comments", is_checked (gui, "search_comments"),
+			   "by-row", gnumeric_glade_group_value (gui, direction_group) == 0,
+			   NULL);
 
 	i = gnumeric_glade_group_value (gui, error_group);
 	sr->error_behaviour = (i == -1) ? SRE_fail : (SearchReplaceError)i;
 
-	i = gnumeric_glade_group_value (gui, direction_group);
-	sr->by_row = (i == 0);
-
-	err = search_replace_verify (sr, TRUE);
+	err = gnm_search_replace_verify (sr, TRUE);
 	if (err) {
-		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR, err);
+		go_gtk_notice_dialog (GTK_WINDOW (dialog), GTK_MESSAGE_ERROR, err);
 		g_free (err);
-		search_replace_free (sr);
+		g_object_unref (sr);
 		return;
 	} else if (!sr->search_strings &&
 		   !sr->search_other_values &&
 		   !sr->search_expressions &&
 		   !sr->search_expression_results &&
 		   !sr->search_comments) {
-		gnumeric_notice (wbcg, GTK_MESSAGE_ERROR,
+		go_gtk_notice_dialog (GTK_WINDOW (dialog), GTK_MESSAGE_ERROR,
 				 _("You must select some cell types to search."));
-		search_replace_free (sr);
+		g_object_unref (sr);
 		return;
 	}
 
@@ -162,7 +150,7 @@ ok_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 	dd = NULL;  /* Destroyed */
 
 	cb (wbcg, sr);
-	search_replace_free (sr);
+	g_object_unref (sr);
 }
 
 static void
@@ -224,7 +212,7 @@ dialog_search_replace (WorkbookControlGUI *wbcg,
 	if (gnumeric_dialog_raise_if_exists (wbcg, SEARCH_REPLACE_KEY))
 		return;
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"search-replace.glade", NULL, NULL);
         if (gui == NULL)
                 return;
@@ -302,11 +290,10 @@ dialog_search_replace_query (WorkbookControlGUI *wbcg,
 	GladeXML *gui;
 	GtkDialog *dialog;
 	int res;
-	GtkWindow *toplevel;
 
 	g_return_val_if_fail (wbcg != NULL, 0);
 
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"search-replace.glade", NULL, NULL);
         if (gui == NULL)
                 return 0;
@@ -321,42 +308,23 @@ dialog_search_replace_query (WorkbookControlGUI *wbcg,
 			    new_text);
 	set_checked (gui, "qd_query", sr->query);
 
-
-	toplevel = wbcg_toplevel (wbcg);
-	if (GTK_WINDOW (dialog)->transient_parent != toplevel)
-		gtk_window_set_transient_for (GTK_WINDOW (dialog), toplevel);
-
-	gtk_tooltips_set_tip (gtk_tooltips_new (),
-			      gtk_dialog_add_button (dialog, GTK_STOCK_CANCEL,
-						     GTK_RESPONSE_CANCEL),
-			      _("Perform no more replacements"), NULL);
-	gtk_tooltips_set_tip (gtk_tooltips_new (),
-			      gtk_dialog_add_button (dialog, GTK_STOCK_NO,
-						     GTK_RESPONSE_NO),
-			      _("Do not perform this replacement"), NULL);
-	gtk_tooltips_set_tip (gtk_tooltips_new (),
-			      gtk_dialog_add_button (dialog, GTK_STOCK_YES,
-						     GTK_RESPONSE_YES),
-			      _("Perform this replacement"), NULL);
-
+	wbcg_set_transient_for (wbcg, GTK_WINDOW (dialog));
+	go_dialog_guess_alternative_button_order (dialog);
 	gtk_widget_show_all (GTK_WIDGET (dialog));
-	res = gtk_dialog_run (dialog);
 
-	/* Unless cancel is pressed, propagate the query setting back.  */
-	if (res != GTK_RESPONSE_CANCEL && res != GTK_RESPONSE_NONE &&
-	    res != GTK_RESPONSE_DELETE_EVENT)
+	res = gtk_dialog_run (dialog);
+	switch (res) {
+	case GTK_RESPONSE_YES:
+	case GTK_RESPONSE_NO:
 		sr->query = is_checked (gui, "qd_query");
+		break;
+	default:
+		res = GTK_RESPONSE_CANCEL;
+	}
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
-/*FIXME: rather than recoding the result value we should change the tests down stream */
-
-	if (res == GTK_RESPONSE_YES)
-		return 0;
-	if (res == GTK_RESPONSE_NO)
-		return 1;
-
-	return -1;
+	return res;
 }
 
 /* ------------------------------------------------------------------------- */
