@@ -3,7 +3,7 @@
  *   Dialog for entering a search query.
  *
  * Author:
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,8 +52,6 @@
 #include <gtk/gtktable.h>
 #include <string.h>
 
-#define SEARCH_KEY "search-dialog"
-
 enum {
 	COL_SHEET = 0,
 	COL_CELL,
@@ -71,27 +69,26 @@ typedef struct {
 	GtkWidget *prev_button, *next_button;
 	GtkNotebook *notebook;
 	int notebook_matches_page;
+	gulong workbook_sheet_deleted_signal;
 
 	GtkTreeView *matches_table;
-	GtkTreeModel *matches_model;
-
 	GPtrArray *matches;
 } DialogState;
 
-static const char *search_type_group[] = {
+static const char * const search_type_group[] = {
 	"search_type_text",
 	"search_type_regexp",
 	NULL
 };
 
-static const char *scope_group[] = {
+static const char * const scope_group[] = {
 	"scope_workbook",
 	"scope_sheet",
 	"scope_range",
 	NULL
 };
 
-static const char *direction_group[] = {
+static const char * const direction_group[] = {
 	"row_major",
 	"column_major",
 	NULL
@@ -106,9 +103,20 @@ static void
 search_get_value (gint row, gint column, gpointer _dd, GValue *value)
 {
 	DialogState *dd = (DialogState *)_dd;
-	GnumericLazyList *ll = GNUMERIC_LAZY_LIST (dd->matches_model);
+	GnumericLazyList *ll = GNUMERIC_LAZY_LIST (gtk_tree_view_get_model (dd->matches_table));
 	SearchFilterResult *item = g_ptr_array_index (dd->matches, row);
-	GnmCell *cell = item->cell;
+	GnmCell *cell;
+	GnmComment *comment;
+
+	if (item->locus == SRL_commment) {
+		cell = NULL;
+		comment = sheet_get_comment (item->ep.sheet, &item->ep.eval);
+	} else {
+		cell = sheet_cell_get (item->ep.sheet,
+				       item->ep.eval.col,
+				       item->ep.eval.row);
+		comment = NULL;
+	}
 
 	g_value_init (value, ll->column_headers[column]);
 
@@ -132,19 +140,19 @@ search_get_value (gint row, gint column, gpointer _dd, GValue *value)
 			g_value_set_static_string (value, _("Result"));
 			return;
 		case SRL_contents: {
-			GnmValue *v = cell->value;
+			GnmValue *v = cell ? cell->value : NULL;
 			char const *type;
 
-			gboolean is_expr = cell_has_expr (cell);
+			gboolean is_expr = cell && cell_has_expr (cell);
 			gboolean is_value = !is_expr && !cell_is_empty (cell) && v;
 
-			if (is_expr)
+			if (!cell)
+				type = _("Deleted");
+			else if (is_expr)
 				type = _("Expression");
-			else if (is_value && v->type == VALUE_STRING)
+			else if (is_value && VALUE_IS_STRING (v))
 				type = _("String");
-			else if (is_value && v->type == VALUE_INTEGER)
-				type = _("Integer");
-			else if (is_value && v->type == VALUE_FLOAT)
+			else if (is_value && VALUE_IS_FLOAT (v))
 				type = _("Number");
 			else
 				type = _("Other value");
@@ -162,15 +170,22 @@ search_get_value (gint row, gint column, gpointer _dd, GValue *value)
 	case COL_CONTENTS:
 		switch (item->locus) {
 		case SRL_commment:
-			g_value_set_string (value, cell_comment_text_get (item->comment));
+			if (comment)
+				g_value_set_string (value, cell_comment_text_get (comment));
+			else
+				g_value_set_static_string (value, _("Deleted"));
 			return;
 		case SRL_value:
-			g_value_take_string (value,
-				value_get_as_string (cell->value));
+			if (cell && cell->value)
+				g_value_take_string (value, value_get_as_string (cell->value));
+			else
+				g_value_set_static_string (value, _("Deleted"));
 			return;
 		case SRL_contents:
-			g_value_take_string (value,
-				cell_get_entered_text (cell));
+			if (cell)
+				g_value_take_string (value, cell_get_entered_text (cell));
+			else
+				g_value_set_static_string (value, _("Deleted"));
 			return;
 #ifndef DEBUG_SWITCH_ENUM
 	default:
@@ -187,12 +202,27 @@ search_get_value (gint row, gint column, gpointer _dd, GValue *value)
 
 /* ------------------------------------------------------------------------- */
 
+static GnumericLazyList *
+make_matches_model (DialogState *dd, int rows)
+{
+	return gnumeric_lazy_list_new (search_get_value,
+				       dd,
+				       rows,
+				       4,
+				       G_TYPE_STRING,
+				       G_TYPE_STRING,
+				       G_TYPE_STRING,
+				       G_TYPE_STRING);
+}
+
 static void
 free_state (DialogState *dd)
 {
+	g_signal_handler_disconnect
+		(G_OBJECT (wb_control_get_workbook (WORKBOOK_CONTROL (dd->wbcg))),
+		 dd->workbook_sheet_deleted_signal);
 	search_filter_matching_free (dd->matches);
-	g_object_unref (G_OBJECT (dd->gui));
-	g_object_unref (G_OBJECT (dd->matches_model));
+	g_object_unref (dd->gui);
 	memset (dd, 0, sizeof (*dd));
 	g_free (dd);
 }
@@ -210,12 +240,14 @@ range_focused (G_GNUC_UNUSED GtkWidget *widget,
 static void
 dialog_destroy (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 {
+#ifdef USE_GURU
 	wbcg_edit_detach_guru (dd->wbcg);
+#endif
 	free_state (dd);
 }
 
 static void
-close_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
+close_clicked (G_GNUC_UNUSED GObject *dummy, DialogState *dd)
 {
 	GtkDialog *dialog = dd->dialog;
 	gtk_widget_destroy (GTK_WIDGET (dialog));
@@ -275,76 +307,60 @@ search_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 	GnmSearchReplace *sr;
 	char *err;
 	int i;
-
-	sr = search_replace_new ();
-
-	sr->search_text = g_strdup (gtk_entry_get_text (dd->gentry));
-	sr->replace_text = NULL;
-
-	i = gnumeric_glade_group_value (gui, search_type_group);
-	sr->is_regexp = (i == 1);
+	GnmSearchReplaceScope scope;
 
 	i = gnumeric_glade_group_value (gui, scope_group);
-	sr->scope = (i == -1) ? SRS_sheet : (SearchReplaceScope)i;
+	scope = (i == -1) ? GNM_SRS_SHEET : (GnmSearchReplaceScope)i;
 
-	/* FIXME: parsing of an gnm_expr_entry should happen by the gee */
-	sr->range_text = g_strdup (gnm_expr_entry_get_text (dd->rangetext));
-	sr->curr_sheet = wb_control_cur_sheet (wbc);
+	sr = g_object_new (GNM_SEARCH_REPLACE_TYPE,
+			   "sheet", wb_control_cur_sheet (wbc),
+			   "scope", scope,
+			   "range-text", gnm_expr_entry_get_text (dd->rangetext),
+			   "search-text", gtk_entry_get_text (dd->gentry),
+			   "is-regexp", gnumeric_glade_group_value (gui, search_type_group) == 1,
+			   "ignore-case", is_checked (gui, "ignore_case"),
+			   "match-words", is_checked (gui, "match_words"),
+			   "search-strings", is_checked (gui, "search_string"),
+			   "search-other-values", is_checked (gui, "search_other"),
+			   "search-expressions", is_checked (gui, "search_expr"),
+			   "search-expression-results", is_checked (gui, "search_expr_results"),
+			   "search-comments", is_checked (gui, "search_comments"),
+			   "by-row", gnumeric_glade_group_value (gui, direction_group) == 0,
+			   NULL);
 
-#if 0
-	if (dd->repl) {
-		sr->query = is_checked (gui, "query");
-		sr->preserve_case = is_checked (gui, "preserve_case");
-	}
-#endif
-	sr->ignore_case = is_checked (gui, "ignore_case");
-	sr->match_words = is_checked (gui, "match_words");
-
-	sr->search_strings = is_checked (gui, "search_string");
-	sr->search_other_values = is_checked (gui, "search_other");
-	sr->search_expressions = is_checked (gui, "search_expr");
-	sr->search_expression_results = is_checked (gui, "search_expr_results");
-	sr->search_comments = is_checked (gui, "search_comments");
-
-#if 0
-	if (dd->repl) {
-		i = gnumeric_glade_group_value (gui, error_group);
-		sr->error_behaviour = (i == -1) ? SRE_fail : (SearchReplaceError)i;
-	}
-#endif
-
-	i = gnumeric_glade_group_value (gui, direction_group);
-	sr->by_row = (i == 0);
-
-	err = search_replace_verify (sr, FALSE);
+	err = gnm_search_replace_verify (sr, FALSE);
 	if (err) {
-		gnumeric_notice (GTK_WINDOW (dd->dialog),
-				 GTK_MESSAGE_ERROR, err);
+		go_gtk_notice_dialog (GTK_WINDOW (dd->dialog),
+				      GTK_MESSAGE_ERROR, err);
 		g_free (err);
-		search_replace_free (sr);
+		g_object_unref (sr);
 		return;
 	} else if (!sr->search_strings &&
 		   !sr->search_other_values &&
 		   !sr->search_expressions &&
 		   !sr->search_expression_results &&
 		   !sr->search_comments) {
-		gnumeric_notice (GTK_WINDOW (dd->dialog), GTK_MESSAGE_ERROR,
-				 _("You must select some cell types to search."));
-		search_replace_free (sr);
+		go_gtk_notice_dialog (GTK_WINDOW (dd->dialog), GTK_MESSAGE_ERROR,
+				      _("You must select some cell types to search."));
+		g_object_unref (sr);
 		return;
 	}
 
 	{
-		GnumericLazyList *ll = GNUMERIC_LAZY_LIST (dd->matches_model);
-		GPtrArray *cells = search_collect_cells (sr, wb_control_cur_sheet (wbc));
-
-		gnumeric_lazy_list_set_rows (ll, 0);
+		GnumericLazyList *ll;
+		GPtrArray *cells;
 
+		/* Clear current table.  */
+		gtk_tree_view_set_model (dd->matches_table, NULL);
 		search_filter_matching_free (dd->matches);
+
+		cells = search_collect_cells (sr);
 		dd->matches = search_filter_matching (sr, cells);
 		search_collect_cells_free (cells);
 
-		gnumeric_lazy_list_set_rows (ll, dd->matches->len);
+		ll = make_matches_model (dd, dd->matches->len);
+		gtk_tree_view_set_model (dd->matches_table, GTK_TREE_MODEL (ll));
+		g_object_unref (ll);
 
 		/* Set sensitivity of buttons.  */
 		cursor_change (dd->matches_table, dd);
@@ -353,12 +369,7 @@ search_clicked (G_GNUC_UNUSED GtkWidget *widget, DialogState *dd)
 	gtk_notebook_set_current_page (dd->notebook, dd->notebook_matches_page);
 	gtk_widget_grab_focus (GTK_WIDGET (dd->matches_table));
 
-#if 0
-	/* Save the contents of the search in the gnome-entry. */
-	gnome_entry_append_history (dd->gentry, TRUE, sr->search_text);
-#endif
-
-	search_replace_free (sr);
+	g_object_unref (sr);
 }
 
 static void
@@ -413,15 +424,16 @@ static const struct {
 };
 
 static GtkTreeView *
-make_matches_table (GtkTreeModel *model)
+make_matches_table (DialogState *dd)
 {
 	GtkTreeView *tree_view;
+	GtkTreeModel *model = GTK_TREE_MODEL (make_matches_model (dd, 0));
 	int i;
 
 	tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (model));
 #ifdef NOT_YET
 	/* Gtk+ isn't ready yet -- 20031224.  */
-	g_object_set (tree_view, "fixed_height_mode", TRUE, NULL);
+	g_object_set (tree_view, "fixed-height-mode", TRUE, NULL);
 #endif
 
 	for (i = 0; i < (int)G_N_ELEMENTS (columns); i++) {
@@ -431,17 +443,17 @@ make_matches_table (GtkTreeModel *model)
 								  columns[i].type, i,
 								  NULL);
 		/* Set single_paragraph_mode to ensure fixed height.  */
-		g_object_set (cell, "single_paragraph_mode", TRUE, NULL);
+		g_object_set (cell, "single-paragraph-mode", TRUE, NULL);
 		if (i == COL_CONTENTS)
 			g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
 		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
 		gtk_tree_view_append_column (tree_view, column);
 	}
 
+	g_object_unref (model);
 	return tree_view;
 }
 
-
 void
 dialog_search (WorkbookControlGUI *wbcg)
 {
@@ -452,14 +464,13 @@ dialog_search (WorkbookControlGUI *wbcg)
 
 	g_return_if_fail (wbcg != NULL);
 
+#ifdef USE_GURU
 	/* Only one guru per workbook. */
 	if (wbcg_edit_get_guru (wbcg))
 		return;
+#endif
 
-	if (gnumeric_dialog_raise_if_exists (wbcg, SEARCH_KEY))
-		return;
-
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"search.glade", NULL, NULL);
         if (gui == NULL)
                 return;
@@ -504,15 +515,7 @@ dialog_search (WorkbookControlGUI *wbcg)
 	gtk_widget_grab_focus (GTK_WIDGET (dd->gentry));
 	gnumeric_editable_enters (GTK_WINDOW (dialog), GTK_WIDGET (dd->gentry));
 
-	dd->matches_model = GTK_TREE_MODEL
-		(gnumeric_lazy_list_new (search_get_value,
-					 dd,
-					 4,
-					 G_TYPE_STRING,
-					 G_TYPE_STRING,
-					 G_TYPE_STRING,
-					 G_TYPE_STRING));
-	dd->matches_table = make_matches_table (dd->matches_model);
+	dd->matches_table = make_matches_table (dd);
 
 	{
 		GtkWidget *scrolled_window =
@@ -559,13 +562,21 @@ dialog_search (WorkbookControlGUI *wbcg)
 		"toggled",
 		G_CALLBACK (cb_focus_on_entry), dd->rangetext);
 
+	dd->workbook_sheet_deleted_signal =
+		g_signal_connect (G_OBJECT (wb_control_get_workbook (WORKBOOK_CONTROL (wbcg))),
+				  "sheet_deleted",
+				  G_CALLBACK (close_clicked),
+				  dd);
+
 	gnumeric_init_help_button (
 		glade_xml_get_widget (gui, "help_button"),
 		GNUMERIC_HELP_LINK_SEARCH);
 
+#ifdef USE_GURU
 	wbcg_edit_attach_guru_with_unfocused_rs (wbcg, GTK_WIDGET (dialog), dd->rangetext);
+#endif
 
-	gnumeric_keyed_dialog (wbcg, GTK_WINDOW (dialog), SEARCH_KEY);
+	go_gtk_nonmodal_dialog (wbcg_toplevel (wbcg), GTK_WINDOW (dialog));
 	gtk_widget_show_all (GTK_WIDGET (dialog));
 }
 
