@@ -27,10 +27,8 @@
 #include <gnm-format.h>
 #include <workbook.h>
 #include <gui-util.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkcombobox.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkvbox.h>
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 /*************************************************************************************************
  * MISC UTILITY FUNCTIONS
@@ -87,11 +85,33 @@ format_page_trim_menu_changed (G_GNUC_UNUSED GtkMenu *menu,
 	format_page_update_preview (data);
 }
 
+/*
+ * More or less a copy of gtk_tree_view_clamp_column_visible.
+ */
+static void
+tree_view_clamp_column_visible (GtkTreeView       *tree_view,
+				GtkTreeViewColumn *column)
+{
+	GtkAdjustment *hadjustment = gtk_tree_view_get_hadjustment (tree_view);
+	GtkWidget *button = column->button;
+
+	if ((hadjustment->value + hadjustment->page_size) <
+	    (button->allocation.x + button->allocation.width))
+		gtk_adjustment_set_value (hadjustment,
+					  button->allocation.x +
+					  button->allocation.width -
+					  hadjustment->page_size);
+	else if (hadjustment->value > button->allocation.x)
+		gtk_adjustment_set_value (hadjustment,
+					  button->allocation.x);
+}
+
 static void
 activate_column (StfDialogData *pagedata, int i)
 {
 	GOFormat *colformat;
 	GtkCellRenderer *cell;
+	GtkTreeViewColumn *column;
 	RenderData_t *renderdata = pagedata->format.renderdata;
 
 	cell = stf_preview_get_cell_renderer (renderdata,
@@ -103,8 +123,13 @@ activate_column (StfDialogData *pagedata, int i)
 	}
 
 	pagedata->format.index = i;
-	cell = stf_preview_get_cell_renderer (renderdata,
-					      pagedata->format.index);
+
+	column = stf_preview_get_column (renderdata, i);
+	if (column) {
+		tree_view_clamp_column_visible (renderdata->tree_view, column);
+	}
+
+	cell = stf_preview_get_cell_renderer (renderdata, i);
 	if (cell) {
 		g_object_set (G_OBJECT (cell),
 			      "background", "lightgrey",
@@ -133,8 +158,8 @@ cb_col_check_clicked (GtkToggleButton *togglebutton, gpointer _i)
 	StfDialogData *pagedata =
 		g_object_get_data (G_OBJECT (togglebutton), "pagedata");
 	gboolean active = gtk_toggle_button_get_active (togglebutton);
-	GtkCellRenderer   *renderer;
-	
+	GtkCellRenderer *renderer;
+
 	g_return_if_fail (i < pagedata->format.col_import_array_len);
 
 	if (pagedata->format.col_import_array[i] == active)
@@ -230,7 +255,7 @@ cb_popup_menu_check_right (GtkWidget *widget, gpointer data)
 	StfDialogData *pagedata = data;
 	
 	check_columns_for_import (pagedata, pagedata->format.index + 1,
-				    pagedata->format.renderdata->colcount - 1);
+				  pagedata->format.renderdata->colcount - 1);
 }
 
 static void
@@ -378,6 +403,48 @@ cb_treeview_button_press (GtkWidget *treeview,
 	return FALSE;
 }
 
+static gint
+cb_treeview_key_press (GtkWidget *treeview,
+		       GdkEventKey *event,
+		       StfDialogData *pagedata)
+{
+	if (event->type == GDK_KEY_PRESS) {
+		switch (event->keyval) {
+		case GDK_Left:
+		case GDK_KP_Left:
+			if (pagedata->format.index > 0)
+				activate_column (pagedata,
+						 pagedata->format.index - 1);
+			return TRUE;
+
+		case GDK_Right:
+		case GDK_KP_Right:
+			if (pagedata->format.index + 1 < (int)pagedata->format.formats->len)
+				activate_column (pagedata,
+						 pagedata->format.index + 1);
+			return TRUE;
+
+		case GDK_space:
+		case GDK_Return: {
+			GtkTreeViewColumn *column = stf_preview_get_column
+				(pagedata->format.renderdata,
+				 pagedata->format.index);
+			GtkToggleButton *button =
+				g_object_get_data (G_OBJECT (column),
+						   "checkbox");
+			gtk_toggle_button_set_active
+				(button,
+				 !gtk_toggle_button_get_active (button));
+			return TRUE;
+		}
+
+		default:
+			; /*  Nothing.  */
+		}
+	}
+
+	return FALSE;
+}
 
 /**
  * format_page_update_preview
@@ -647,11 +714,16 @@ stf_dialog_format_page_init (GladeXML *gui, StfDialogData *pagedata)
 	/* Set properties */
 	pagedata->format.renderdata =
 		stf_preview_new (pagedata->format.format_data_container,
-				 workbook_date_conv (wb_control_workbook (WORKBOOK_CONTROL (pagedata->wbcg))));
+				 workbook_date_conv (wb_control_get_workbook (WORKBOOK_CONTROL (pagedata->wbcg))));
 	pagedata->format.formats          = g_ptr_array_new ();
 	pagedata->format.index         = -1;
 	pagedata->format.manual_change = FALSE;
 
+	/* Update widgets before connecting signals, see #333407.  */
+	gtk_combo_box_set_active (GTK_COMBO_BOX (pagedata->format.format_trim),
+				  0);
+	format_page_update_column_selection (pagedata);
+
 	/* Connect signals */
 	pagedata->format.format_changed_handler_id = 
 	     g_signal_connect (G_OBJECT (pagedata->format.format_selector),
@@ -664,12 +736,12 @@ stf_dialog_format_page_init (GladeXML *gui, StfDialogData *pagedata)
         g_signal_connect (G_OBJECT (pagedata->format.format_trim),
 			  "changed",
 			  G_CALLBACK (format_page_trim_menu_changed), pagedata);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (pagedata->format.format_trim), 0);
-
 	g_signal_connect (G_OBJECT (pagedata->format.renderdata->tree_view),
 			  "button_press_event",
 			  G_CALLBACK (cb_treeview_button_press),
 			  pagedata);
-
-	format_page_update_column_selection (pagedata);
+	g_signal_connect (G_OBJECT (pagedata->format.renderdata->tree_view),
+			  "key_press_event",
+			  G_CALLBACK (cb_treeview_key_press),
+			  pagedata);
 }
