@@ -20,7 +20,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -63,7 +63,7 @@ typedef struct {
 	GtkTreeSelection   *sheet_list_selection;
 } ZoomState;
 
-static struct {
+static const struct {
 	char const * const name;
 	gint const factor;
 } buttons[] = {
@@ -176,7 +176,7 @@ dialog_zoom (WorkbookControlGUI *wbcg, Sheet *sheet)
 
 	if (gnumeric_dialog_raise_if_exists (wbcg, ZOOM_DIALOG_KEY))
 		return;
-	gui = gnm_glade_xml_new (GNM_CMD_CONTEXT (wbcg),
+	gui = gnm_glade_xml_new (GO_CMD_CONTEXT (wbcg),
 		"dialog-zoom.glade", NULL, NULL);
 	if (gui == NULL)
 		return;
@@ -202,7 +202,7 @@ dialog_zoom (WorkbookControlGUI *wbcg, Sheet *sheet)
 	gtk_tree_view_column_set_sort_column_id (column, COL_SHEET_NAME);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (state->sheet_list), column);
 
-	sheets = workbook_sheets (wb_control_workbook (WORKBOOK_CONTROL (wbcg)));
+	sheets = workbook_sheets (wb_control_get_workbook (WORKBOOK_CONTROL (wbcg)));
 	cur_row = row = 0;
 	for (l = sheets; l; l = l->next) {
 		GtkTreeIter iter;
