@@ -56,9 +56,9 @@ static GnmFuncHelp const help_sum[] = {
 };
 
 GnmValue *
-gnumeric_sum (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_sum (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_sum,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -97,9 +97,9 @@ range_bogusproduct (const gnm_float *xs, int n, gnm_float *res)
 }
 
 GnmValue *
-gnumeric_product (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_product (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_bogusproduct,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -132,8 +132,35 @@ gnumeric_version (FunctionEvalInfo *ei, GnmValue const * const *argv)
 
 /***************************************************************************/
 
+static DependentFlags
+gnumeric_table_link (FunctionEvalInfo *ei)
+{
+	GnmDependent *dep = ei->pos->dep;
+	GnmRangeRef rr;
+
+	rr.a.col_relative = rr.a.row_relative = 
+	rr.b.col_relative = rr.b.row_relative = FALSE;
+	rr.a.sheet = rr.b.sheet = dep->sheet;
+
+	g_return_val_if_fail (ei->pos->array != NULL, DEPENDENT_IGNORE_ARGS);
+
+	g_return_val_if_fail (ei->pos->eval.col > 0, DEPENDENT_IGNORE_ARGS);
+	rr.a.col = rr.b.col = ei->pos->eval.col - 1;
+	rr.a.row = ei->pos->eval.row;
+	rr.b.row = rr.a.row + ei->pos->array->rows - 1;
+	dependent_add_dynamic_dep (dep, &rr);
+
+	g_return_val_if_fail (ei->pos->eval.row > 0, DEPENDENT_IGNORE_ARGS);
+	rr.a.row = rr.b.row = ei->pos->eval.row - 1;
+	rr.a.col = ei->pos->eval.col;
+	rr.b.col = rr.a.col + ei->pos->array->cols - 1;
+	dependent_add_dynamic_dep (dep, &rr);
+
+	return DEPENDENT_IGNORE_ARGS;
+}
+
 static GnmValue *
-gnumeric_table (FunctionEvalInfo *ei, GnmExprList const *args)
+gnumeric_table (FunctionEvalInfo *ei, int argc, const GnmExprConstPtr *argv)
 {
 	GnmExpr const *arg;
 	GnmCell       *in[3], *x_iter, *y_iter;
@@ -141,15 +168,20 @@ gnumeric_table (FunctionEvalInfo *ei, GnmExprList const *args)
 	GnmCellPos     pos;
 	int x, y;
 
-	if (gnm_expr_list_length (args) != 2 &&
+	/* evaluation clears the dynamic deps */
+	gnumeric_table_link (ei);
+
+	/* FIXME: This looks b-o-g-u-s!  */
+	if (argc != 2 &&
 	    ei->pos->eval.col > 0 && ei->pos->eval.row > 0)
 		return value_new_error_REF (ei->pos);
 
 	for (x = 0; x < 2 ; x++) {
-		arg = gnm_expr_list_nth (args, x);
+		arg = (x < argc) ? argv[x] : NULL;
 		val[x] = NULL;
-		if (NULL != arg && arg->any.oper == GNM_EXPR_OP_CELLREF) {
-			cellref_get_abs_pos (&arg->cellref.ref, &ei->pos->eval, &pos);
+		if (NULL != arg && GNM_EXPR_GET_OPER (arg) == GNM_EXPR_OP_CELLREF) {
+			gnm_cellpos_init_cellref (&pos,
+				&arg->cellref.ref, &ei->pos->eval);
 			in[x] = sheet_cell_get (ei->pos->sheet, pos.col, pos.row);
 			if (NULL == in[x])
 				in[x] = sheet_cell_fetch (ei->pos->sheet, pos.col, pos.row);
@@ -168,34 +200,45 @@ gnumeric_table (FunctionEvalInfo *ei, GnmExprList const *args)
 				ei->pos->eval.col - 1, ei->pos->eval.row - 1);
 		else
 			val[2] = value_dup (in[2]->value);
-	}
+	} else
+		in[2] = NULL;
 
-	res = value_new_array (ei->pos->cols, ei->pos->rows);
-	for (x = ei->pos->cols ; x-- > 0 ; ) {
+	res = value_new_array (ei->pos->array->cols, ei->pos->array->rows);
+	for (x = ei->pos->array->cols ; x-- > 0 ; ) {
 		x_iter = sheet_cell_get (ei->pos->sheet,
 			x + ei->pos->eval.col, ei->pos->eval.row-1);
 		if (NULL == x_iter)
 			continue;
-		cell_eval (x_iter);
-		in[0]->value = value_dup (x_iter->value);
-		dependent_queue_recalc (&in[0]->base);
-		for (y = ei->pos->rows ; y-- > 0 ; ) {
+		if (NULL != in[0]) {
+			cell_eval (x_iter);
+			in[0]->value = value_dup (x_iter->value);
+			dependent_queue_recalc (&in[0]->base);
+		}
+		for (y = ei->pos->array->rows ; y-- > 0 ; ) {
 			y_iter = sheet_cell_get (ei->pos->sheet,
 				ei->pos->eval.col-1, y + ei->pos->eval.row);
 			if (NULL == y_iter)
 				continue;
 			cell_eval (y_iter);
-			in[1]->value = value_dup (y_iter->value);
-			dependent_queue_recalc (&in[1]->base);
-
-			cell_eval (in[2]);
-			value_array_set (res, x, y, value_dup (in[2]->value));
-
-			value_release (in[1]->value);
+			if (NULL != in[1]) {
+				in[1]->value = value_dup (y_iter->value);
+				dependent_queue_recalc (&in[1]->base);
+				if (NULL != in[0]) {
+					cell_eval (in[2]);
+					value_array_set (res, x, y, value_dup (in[2]->value));
+					value_release (in[1]->value);
+				} else {
+					cell_eval (x_iter);
+					value_array_set (res, x, y, value_dup (x_iter->value));
+				}
+			} else
+				value_array_set (res, x, y, value_dup (y_iter->value));
 		}
-		value_release (in[0]->value);
+		if (NULL != in[0])
+			value_release (in[0]->value);
 	}
-	value_release (in[2]->value);
+	if (NULL != in[2])
+		value_release (in[2]->value);
 	for (x = 0 ; x < 3 ; x++)
 		if (in[x]) {
 			if (val[x])
@@ -239,7 +282,8 @@ func_builtin_init (void)
 		},
 		{	"table",	"",	"",
 			NULL,		NULL,	gnumeric_table,
-			NULL, NULL, NULL, GNM_FUNC_SIMPLE + GNM_FUNC_INTERNAL,
+			gnumeric_table_link, NULL,
+			NULL, GNM_FUNC_SIMPLE + GNM_FUNC_INTERNAL,
 			GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC,
 			GNM_FUNC_TEST_STATUS_EXHAUSTIVE
 		},
