@@ -41,26 +41,24 @@
 #define GNUMERIC_ITEM "GnmPane"
 #include "item-debug.h"
 
-#define DEBUG_DND
+#undef DEBUG_DND
 
 /**
  * For now, application/x-gnumeric is disabled. It handles neither
  * images nor graphs correctly.
  */
-static const GtkTargetEntry drag_types_in[] =
-	{
-		{(char *) "GNUMERIC_SAME_PROC", GTK_TARGET_SAME_APP, 0},
- 		/* {(char *) "application/x-gnumeric", 0, 0}, */
-	};
+static GtkTargetEntry const drag_types_in[] = {
+	{(char *) "GNUMERIC_SAME_PROC", GTK_TARGET_SAME_APP, 0},
+	/* {(char *) "application/x-gnumeric", 0, 0}, */
+};
 
-static const GtkTargetEntry drag_types_out[] =
-	{
-		{(char *) "GNUMERIC_SAME_PROC", GTK_TARGET_SAME_APP, 0},
-		{(char *) "application/x-gnumeric", 0, 0},
-	};
+static GtkTargetEntry const drag_types_out[] = {
+	{(char *) "GNUMERIC_SAME_PROC", GTK_TARGET_SAME_APP, 0},
+	{(char *) "application/x-gnumeric", 0, 0},
+};
 
 static void
-gnumeric_pane_realized (GtkWidget *widget, gpointer ignored)
+gnm_pane_header_realized (GtkWidget *widget)
 {
 	gdk_window_set_back_pixmap (GTK_LAYOUT (widget)->bin_window, NULL, FALSE);
 }
@@ -90,18 +88,17 @@ gnumeric_pane_header_init (GnmPane *pane, SheetControlGUI *scg,
 		pane->row.canvas = canvas;
 		pane->row.item = ITEM_BAR (item);
 	}
-	pane->colrow_resize.points = NULL;
-	pane->colrow_resize.start  = NULL;
-	pane->colrow_resize.guide  = NULL;
+	pane->size_guide.points = NULL;
+	pane->size_guide.start  = NULL;
+	pane->size_guide.guide  = NULL;
 
 	if (NULL != scg &&
 	    NULL != (sheet = sc_sheet (SHEET_CONTROL (scg))) &&
 	    fabs (1. - sheet->last_zoom_factor_used) > 1e-6)
 		foo_canvas_set_pixels_per_unit (canvas, sheet->last_zoom_factor_used);
 
-	g_signal_connect (G_OBJECT (canvas),
-		"realize",
-		G_CALLBACK (gnumeric_pane_realized), NULL);
+	g_signal_connect (G_OBJECT (canvas), "realize",
+		G_CALLBACK (gnm_pane_header_realized), NULL);
 }
 
 static void
@@ -142,10 +139,10 @@ gnm_pane_display_obj_size_tip (GnmPane *pane, SheetObject const *so)
 	scg_object_coords_to_anchor (scg, coords, &anchor);
 	sheet_object_anchor_to_pts (&anchor, sc_sheet (SHEET_CONTROL (scg)), pts);
 	msg = g_strdup_printf (_("%.1f x %.1f pts\n%d x %d pixels"),
-		MAX (fabs (pts[3] - pts[1]) + 1., 0),
-		MAX (fabs (pts[2] - pts[0]) + 1., 0),
-		MAX ((int)floor (fabs (coords [3] - coords [1]) + 1.5), 0),
-		MAX ((int)floor (fabs (coords [2] - coords [0]) + 1.5), 0));
+		MAX (fabs (pts[2] - pts[0]), 0),
+		MAX (fabs (pts[3] - pts[1]), 0),
+		MAX ((int)floor (fabs (coords [2] - coords [0]) + 0.5), 0),
+		MAX ((int)floor (fabs (coords [3] - coords [1]) + 0.5), 0));
 	gtk_label_set_text (GTK_LABEL (pane->size_tip), msg);
 	g_free (msg);
 }
@@ -163,8 +160,10 @@ cb_pane_popup_menu (GnmPane *pane)
 		NULL, NULL);
 
 	if (gdk_win != NULL) {
+		gpointer gtk_win_void = NULL;
 		GtkWindow *gtk_win = NULL;
-		gdk_window_get_user_data (gdk_win, (gpointer *) &gtk_win);
+		gdk_window_get_user_data (gdk_win, &gtk_win_void);
+		gtk_win = gtk_win_void;
 		if (gtk_win != NULL) {
 			if (gtk_win == (GtkWindow *)pane->col.canvas)
 				is_col = TRUE;
@@ -208,10 +207,10 @@ cb_pane_drag_data_received (GtkWidget *widget, GdkDragContext *context,
 }
 
 static void
-cb_gnm_pane_drag_data_get (GtkWidget *widget, GdkDragContext *context,
-			   GtkSelectionData *selection_data,
-			   guint info, guint time,
-			   SheetControlGUI *scg)
+cb_pane_drag_data_get (GtkWidget *widget, GdkDragContext *context,
+		       GtkSelectionData *selection_data,
+		       guint info, guint time,
+		       SheetControlGUI *scg)
 {
 #ifdef DEBUG_DND
 	gchar *target_name = gdk_atom_name (selection_data->target);
@@ -223,8 +222,8 @@ cb_gnm_pane_drag_data_get (GtkWidget *widget, GdkDragContext *context,
 
 /* Move the rubber bands if we are the source */
 static gboolean
-cb_gnm_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
-			 int x, int y, guint32 time, GnmPane *pane)
+cb_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
+		     int x, int y, guint32 time, GnmPane *pane)
 {
 	GtkWidget *source_widget = gtk_drag_get_source_widget (context);
 	SheetControlGUI *scg = GNM_CANVAS (widget)->simple.scg;
@@ -236,7 +235,8 @@ cb_gnm_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
 		GdkModifierType mask;
 		double wx, wy;
 
-		g_object_set_data (&context->parent_instance, "gnm-pane", pane);
+		g_object_set_data (&context->parent_instance,
+			"wbcg", scg_get_wbcg (scg));
 		gnm_canvas_window_to_coord (gcanvas, x, y, &wx, &wy);
 
 		gdk_window_get_pointer (gtk_widget_get_parent_window (widget),
@@ -248,6 +248,18 @@ cb_gnm_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
 	return TRUE;
 }
 
+static void
+cb_pane_drag_end (GtkWidget *widget, GdkDragContext *context,
+		  GnmPane *source_pane)
+{
+	/* sync the ctrl-pts with the object in case the drag was canceled. */
+	gnm_pane_objects_drag (source_pane, NULL,
+		source_pane->drag.origin_x,
+		source_pane->drag.origin_y,
+		8, FALSE, FALSE);
+	source_pane->drag.had_motion = FALSE;
+}
+
 /**
  * Move the rubber bands back to original position when curser leaves
  * the scg, but not when it moves to another pane. We use object data,
@@ -255,28 +267,27 @@ cb_gnm_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
  * drag_leave to the old one.
  */
 static void
-cb_gnm_pane_drag_leave (GtkWidget *widget, GdkDragContext *context,
-		       guint32 time, GnmPane *pane)
+cb_pane_drag_leave (GtkWidget *widget, GdkDragContext *context,
+		    guint32 time, GnmPane *pane)
 {
 	GtkWidget *source_widget = gtk_drag_get_source_widget (context);
 	GnmPane *source_pane;
-	GnmPane *ctxt_pane;
-
+	WorkbookControlGUI *wbcg;
+  
 	if (!source_widget) return;
 
 	g_return_if_fail (IS_GNM_CANVAS (source_widget));
 
 	source_pane = GNM_CANVAS (source_widget)->pane;
 
-	ctxt_pane = g_object_get_data (&context->parent_instance, "gnm-pane");
-	if (pane != ctxt_pane)
+	wbcg = scg_get_wbcg (source_pane->gcanvas->simple.scg);
+	if (wbcg == g_object_get_data (&context->parent_instance, "wbcg"))
 		return;
-	g_object_set_data (&context->parent_instance, "gnm-pane", NULL);
 
 	gnm_pane_objects_drag (source_pane, NULL,
-			       source_pane->drag.origin_x,
-			       source_pane->drag.origin_y,
-			       8, FALSE, FALSE);
+		source_pane->drag.origin_x,
+		source_pane->drag.origin_y,
+		8, FALSE, FALSE);
 	source_pane->drag.had_motion = FALSE;
 }
 
@@ -292,14 +303,26 @@ gnm_pane_drag_dest_init (GnmPane *pane, SheetControlGUI *scg)
 	gtk_drag_dest_add_image_targets (widget);
 	gtk_drag_dest_add_text_targets (widget);
 
-	g_signal_connect (widget, "drag-data-received",
-		G_CALLBACK (cb_pane_drag_data_received), pane);
-	g_signal_connect (widget, "drag-data-get",
-		G_CALLBACK (cb_gnm_pane_drag_data_get), scg);
-	g_signal_connect (widget, "drag-motion",
-		G_CALLBACK (cb_gnm_pane_drag_motion), pane);
- 	g_signal_connect (widget, "drag-leave",
-		G_CALLBACK (cb_gnm_pane_drag_leave), pane);
+	g_object_connect (G_OBJECT (widget),
+		"signal::drag-data-received",	G_CALLBACK (cb_pane_drag_data_received), pane,
+		"signal::drag-data-get",	G_CALLBACK (cb_pane_drag_data_get),	scg,
+		"signal::drag-motion",		G_CALLBACK (cb_pane_drag_motion),	pane,
+		"signal::drag-leave",		G_CALLBACK (cb_pane_drag_leave),	pane,
+		"signal::drag-end",		G_CALLBACK (cb_pane_drag_end),		pane,
+		NULL);
+}
+
+static void
+cb_pane_init_objs (GnmPane *pane)
+{
+	/* create views for the sheet objects now that we exist */
+	Sheet *sheet = sc_sheet (SHEET_CONTROL (pane->gcanvas->simple.scg));
+	GSList *ptr;
+
+	if (sheet != NULL)
+		for (ptr = sheet->sheet_objects; ptr != NULL ; ptr = ptr->next)
+			sheet_object_new_view (ptr->data,
+				(SheetObjectViewContainer *)pane);
 }
 
 void
@@ -317,6 +340,8 @@ gnm_pane_init (GnmPane *pane, SheetControlGUI *scg,
 	g_signal_connect_swapped (pane->gcanvas,
 		"popup-menu",
 		G_CALLBACK (cb_pane_popup_menu), pane);
+	g_signal_connect_swapped (G_OBJECT (pane->gcanvas), "realize",
+		G_CALLBACK (cb_pane_init_objs), pane);
 
 	if (NULL != scg &&
 	    NULL != (sheet = sc_sheet (SHEET_CONTROL (scg))) &&
@@ -356,16 +381,6 @@ gnm_pane_init (GnmPane *pane, SheetControlGUI *scg,
 	pane->drag.button = 0;
 	pane->drag.ctrl_pts = g_hash_table_new_full (g_direct_hash, g_direct_equal,
 		NULL, (GDestroyNotify) cb_ctrl_pts_free);
-
-	/* create views for the sheet objects */
-	sheet = sc_sheet (SHEET_CONTROL (scg));
-	if (sheet != NULL) {
-		GSList *ptr;
-		for (ptr = sheet->sheet_objects; ptr != NULL ; ptr = ptr->next)
-			sheet_object_new_view (ptr->data,
-				(SheetObjectViewContainer *)pane);
-	}
-
 	gnm_pane_drag_dest_init (pane, scg);
 	pane->mouse_cursor = NULL;
 }
@@ -411,9 +426,9 @@ gnm_pane_release (GnmPane *pane)
 	pane->grid = NULL;
 	pane->editor = NULL;
 	pane->cursor.std = pane->cursor.rangesel = pane->cursor.special = pane->cursor.rangehighlight = NULL;
-	pane->colrow_resize.guide = NULL;
-	pane->colrow_resize.start = NULL;
-	pane->colrow_resize.points = NULL;
+	pane->size_guide.guide = NULL;
+	pane->size_guide.start = NULL;
+	pane->size_guide.points = NULL;
 }
 
 void
@@ -435,30 +450,28 @@ gnm_pane_bound_set (GnmPane *pane,
 /****************************************************************************/
 
 void
-gnm_pane_colrow_resize_start (GnmPane *pane,
-			      gboolean is_cols, int resize_pos)
+gnm_pane_size_guide_start (GnmPane *pane, gboolean vert, int colrow, int width)
 {
 	SheetControlGUI const *scg;
 	GnmCanvas const *gcanvas;
 	FooCanvasPoints *points;
-	FooCanvasItem *item;
 	double zoom;
 	gboolean text_is_rtl;
 
 	g_return_if_fail (pane != NULL);
-	g_return_if_fail (pane->colrow_resize.guide  == NULL);
-	g_return_if_fail (pane->colrow_resize.start  == NULL);
-	g_return_if_fail (pane->colrow_resize.points == NULL);
+	g_return_if_fail (pane->size_guide.guide  == NULL);
+	g_return_if_fail (pane->size_guide.start  == NULL);
+	g_return_if_fail (pane->size_guide.points == NULL);
 
 	gcanvas = pane->gcanvas;
 	scg = gcanvas->simple.scg;
 	text_is_rtl = scg->sheet_control.sheet->text_is_rtl;
 	zoom = FOO_CANVAS (gcanvas)->pixels_per_unit;
 
-	points = pane->colrow_resize.points = foo_canvas_points_new (2);
-	if (is_cols) {
+	points = pane->size_guide.points = foo_canvas_points_new (2);
+	if (vert) {
 		double x = scg_colrow_distance_get (scg, TRUE,
-					0, resize_pos) / zoom;
+					0, colrow) / zoom;
 		if (text_is_rtl)
 			x = -x;
 		points->coords [0] = x;
@@ -469,7 +482,7 @@ gnm_pane_colrow_resize_start (GnmPane *pane,
 					0, gcanvas->last_visible.row+1) / zoom;
 	} else {
 		double const y = scg_colrow_distance_get (scg, FALSE,
-					0, resize_pos) / zoom;
+					0, colrow) / zoom;
 		points->coords [0] = scg_colrow_distance_get (scg, TRUE,
 					0, gcanvas->first.col) / zoom;
 		points->coords [1] = y;
@@ -483,55 +496,73 @@ gnm_pane_colrow_resize_start (GnmPane *pane,
 		}
 	}
 
-	/* Position the stationary only.  Guide line is handled elsewhere. */
-	item = foo_canvas_item_new (gcanvas->action_items,
+	/* Guideline positioning is done in gnm_pane_size_guide_motion */
+	pane->size_guide.guide = foo_canvas_item_new (gcanvas->action_items,
 		FOO_TYPE_CANVAS_LINE,
-		"points", points,
 		"fill-color", "black",
-		"width-pixels", 1,
+		"width-pixels", width,
 		NULL);
-	pane->colrow_resize.start = GTK_OBJECT (item);
 
-	item = foo_canvas_item_new (gcanvas->action_items,
-		FOO_TYPE_CANVAS_LINE,
-		"fill-color", "black",
-		"width-pixels", 1,
-		NULL);
-	pane->colrow_resize.guide = GTK_OBJECT (item);
+	/* cheat for now and differentiate between col/row resize and frozen panes
+	 * using the width.  Frozen pane guides do not require a start line */
+	if (width == 1)
+		pane->size_guide.start = foo_canvas_item_new (gcanvas->action_items,
+			FOO_TYPE_CANVAS_LINE,
+			"points", points,
+			"fill-color", "black",
+			"width-pixels", 1,
+			NULL);
+	else {
+		static char const dat [] = { 0x22, 0x88, 0x22, 0x88, 0x22, 0x88, 0x22, 0x88 };
+		GdkBitmap *stipple = gdk_bitmap_create_from_data (
+			GTK_WIDGET (pane->gcanvas)->window, dat, 8, 8);
+		foo_canvas_item_set (pane->size_guide.guide, "fill-stipple", stipple, NULL);
+		g_object_unref (stipple);
+	}
 }
 
 void
-gnm_pane_colrow_resize_stop (GnmPane *pane)
+gnm_pane_size_guide_stop (GnmPane *pane)
 {
 	g_return_if_fail (pane != NULL);
 
-	if (pane->colrow_resize.points != NULL) {
-		foo_canvas_points_free (pane->colrow_resize.points);
-		pane->colrow_resize.points = NULL;
+	if (pane->size_guide.points != NULL) {
+		foo_canvas_points_free (pane->size_guide.points);
+		pane->size_guide.points = NULL;
 	}
-	if (pane->colrow_resize.start != NULL) {
-		gtk_object_destroy (pane->colrow_resize.start);
-		pane->colrow_resize.start = NULL;
+	if (pane->size_guide.start != NULL) {
+		gtk_object_destroy (GTK_OBJECT (pane->size_guide.start));
+		pane->size_guide.start = NULL;
 	}
-	if (pane->colrow_resize.guide != NULL) {
-		gtk_object_destroy (pane->colrow_resize.guide);
-		pane->colrow_resize.guide = NULL;
+	if (pane->size_guide.guide != NULL) {
+		gtk_object_destroy (GTK_OBJECT (pane->size_guide.guide));
+		pane->size_guide.guide = NULL;
 	}
 }
 
+/**
+ * gnm_pane_size_guide_motion
+ * @pane : #GnmPane
+ * @vert : TRUE for a vertical guide, FALSE for horizontal
+ * @guide_pos : in unscaled sheet pixel coords
+ *
+ * Moves the guide line to @guide_pos.
+ * NOTE : gnm_pane_size_guide_start must be called before any calls to
+ *	gnm_pane_size_guide_motion
+ **/
 void
-gnm_pane_colrow_resize_move (GnmPane *pane, gboolean is_cols, int resize_pos)
+gnm_pane_size_guide_motion (GnmPane *pane, gboolean vert, int guide_pos)
 {
-	FooCanvasItem *resize_guide = FOO_CANVAS_ITEM (pane->colrow_resize.guide);
-	FooCanvasPoints *points     = pane->colrow_resize.points;
+	FooCanvasItem *resize_guide = FOO_CANVAS_ITEM (pane->size_guide.guide);
+	FooCanvasPoints *points     = pane->size_guide.points;
 	double const	 scale	    = 1. / resize_guide->canvas->pixels_per_unit;
 
-	if (is_cols) {
+	if (vert) {
 		gboolean text_is_rtl = pane->gcanvas->simple.scg->sheet_control.sheet->text_is_rtl;
 		points->coords [0] = points->coords [2] = scale *
-			(text_is_rtl ? -resize_pos : resize_pos);
+			(text_is_rtl ? -guide_pos : guide_pos);
 	} else
-		points->coords [1] = points->coords [3] = scale * resize_pos;
+		points->coords [1] = points->coords [3] = scale * guide_pos;
 
 	foo_canvas_item_set (resize_guide, "points",  points, NULL);
 }
@@ -748,21 +779,22 @@ gnm_pane_object_move (GnmPane *pane, GObject *ctrl_pt,
 }
 
 static gboolean
-cb_slide_handler (GnmCanvas *gcanvas, int col, int row, gpointer ctrl_pt)
+cb_slide_handler (GnmCanvas *gcanvas, GnmCanvasSlideInfo const *info)
 {
 	int x, y;
 	SheetControlGUI const *scg = gcanvas->simple.scg;
 	double const scale = 1. / FOO_CANVAS (gcanvas)->pixels_per_unit;
 
-	x = scg_colrow_distance_get (scg, TRUE, gcanvas->first.col, col);
+	x = scg_colrow_distance_get (scg, TRUE, gcanvas->first.col, info->col);
 	x += gcanvas->first_offset.col;
-	y = scg_colrow_distance_get (scg, FALSE, gcanvas->first.row, row);
+	y = scg_colrow_distance_get (scg, FALSE, gcanvas->first.row, info->row);
 	y += gcanvas->first_offset.row;
 
 	if (scg->sheet_control.sheet->text_is_rtl)
 		x *= -1;
 
-	gnm_pane_object_move (gcanvas->pane, ctrl_pt, x * scale, y * scale, FALSE, FALSE);
+	gnm_pane_object_move (gcanvas->pane, info->user_data,
+		x * scale, y * scale, FALSE, FALSE);
 
 	return TRUE;
 }
@@ -792,13 +824,13 @@ build_so_menu (GnmPane *pane, SheetObjectView *view,
 			break;
 		if (a->icon != NULL) {
 			if (a->label != NULL) {
-				item = gtk_image_menu_item_new_with_mnemonic (a->label);
+				item = gtk_image_menu_item_new_with_mnemonic (_(a->label));
 				gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
 					gtk_image_new_from_stock (a->icon, GTK_ICON_SIZE_MENU));
 			} else
 				item = gtk_image_menu_item_new_from_stock (a->icon, NULL);
 		} else if (a->label != NULL)
-			item = gtk_menu_item_new_with_mnemonic (a->label);
+			item = gtk_menu_item_new_with_mnemonic (_(a->label));
 		else
 			item = gtk_separator_menu_item_new ();
 		if (a->submenu > 0)
@@ -1056,12 +1088,14 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 
 	case GDK_BUTTON_RELEASE:
 		if (pane->drag.button != (int)event->button.button)
-			return FALSE;
+			break;
 		pane->drag.button = 0;
 		gnm_simple_canvas_ungrab (ctrl_pt, event->button.time);
 		gnm_canvas_slide_stop (gcanvas);
 		control_point_set_cursor (scg, ctrl_pt);
-		if (pane->drag.had_motion)
+		if (idx == 8)
+			; /* ignore fake event generated by the dnd code */
+		else if (pane->drag.had_motion)
 			scg_objects_drag_commit	(scg, idx,
 				pane->drag.created_objects);
 		else if (pane->drag.created_objects && idx == 7) {
@@ -1075,7 +1109,7 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 
 	case GDK_BUTTON_PRESS:
 		if (0 != pane->drag.button)
-			return FALSE;
+			break;
 		switch (event->button.button) {
 		case 1:
 		case 2: gnm_pane_object_start_resize (pane, &event->button, so,  idx, FALSE);
@@ -1101,7 +1135,7 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 		 * the semantics of the col,row args in the callback.  However,
 		 * that is more work than I want to do right now.
 		 */
-		if (idx == 8 &&  !pane->drag.created_objects)
+		if (idx == 8) 
 			gnm_pane_drag_begin (pane, so, event);
 		else if (gnm_canvas_handle_motion (GNM_CANVAS (ctrl_pt->canvas),
 						   ctrl_pt->canvas, &event->motion,
@@ -1360,9 +1394,12 @@ cb_sheet_object_canvas_event (FooCanvasItem *view, GdkEvent *event,
 static void
 cb_sheet_object_view_destroyed (FooCanvasItem *view, SheetObject *so)
 {
-	GnmPane *pane = GNM_CANVAS (view->canvas)->pane;
-	if (pane != NULL && g_hash_table_lookup (pane->drag.ctrl_pts, so) != NULL)
-		scg_object_unselect (GNM_SIMPLE_CANVAS (view->canvas)->scg, so);
+	if (NULL != view->canvas) {
+		GnmPane *pane = GNM_CANVAS (view->canvas)->pane;
+		if (pane != NULL &&
+		    g_hash_table_lookup (pane->drag.ctrl_pts, so) != NULL)
+			scg_object_unselect (GNM_SIMPLE_CANVAS (view->canvas)->scg, so);
+	}
 }
 
 static int
