@@ -36,7 +36,6 @@ typedef struct _GnmComment		GnmComment;
 
 typedef union  _GnmValue		GnmValue;
 typedef struct _GnmValueBool		GnmValueBool;
-typedef struct _GnmValueInt		GnmValueInt;
 typedef struct _GnmValueFloat		GnmValueFloat;
 typedef struct _GnmValueErr		GnmValueErr;
 typedef struct _GnmValueStr		GnmValueStr;
@@ -51,7 +50,6 @@ typedef enum {
 	GNM_ERROR_NAME,
 	GNM_ERROR_NUM,
 	GNM_ERROR_NA,
-	GNM_ERROR_RECALC,
 	GNM_ERROR_UNKNOWN
 } GnmStdError;
 
@@ -66,8 +64,12 @@ typedef struct _GnmExprUnary		GnmExprUnary;
 typedef struct _GnmExprBinary		GnmExprBinary;
 typedef struct _GnmExprName		GnmExprName;
 typedef struct _GnmExprCellRef		GnmExprCellRef;
-typedef struct _GnmExprArray		GnmExprArray;
+typedef struct _GnmExprArrayCorner	GnmExprArrayCorner;
+typedef struct _GnmExprArrayElem	GnmExprArrayElem;
 typedef struct _GnmExprSet		GnmExprSet;
+typedef const GnmExpr *			GnmExprConstPtr;
+
+typedef struct _GnmExprTop		GnmExprTop;
 
 typedef struct _GnmExprRelocateInfo	GnmExprRelocateInfo;
 typedef struct _GnmExprRewriteInfo 	GnmExprRewriteInfo;
@@ -90,6 +92,7 @@ typedef GList   			ColRowIndexList;
 typedef struct _ColRowIndexSet          ColRowIndexSet;
 
 typedef struct _GnmFont	        	GnmFont;
+typedef struct _GnmFontMetrics        	GnmFontMetrics;
 typedef struct _GnmColor	        GnmColor;
 typedef struct _GnmBorder	        GnmBorder;
 typedef struct _GnmStyle		GnmStyle;
