@@ -30,9 +30,11 @@
 #include "ranges.h"
 #include "parse-util.h"
 #include "gui-util.h"
+#include "sheet-autofill.h"
 #include <gsf/gsf-impl-utils.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkstock.h>
+#include <goffice/app/go-cmd-context-impl.h>
 #define GNUMERIC_ITEM "CURSOR"
 #include "item-debug.h"
 
@@ -47,13 +49,14 @@ struct _ItemCursor {
 
 	SheetControlGUI *scg;
 	gboolean	 pos_initialized;
-	GnmRange		 pos;
+	GnmRange	 pos;
 
 	/* Offset of dragging cell from top left of pos */
 	int col_delta, row_delta;
 
 	/* Tip for movement */
 	GtkWidget        *tip;
+	GnmCellPos last_tip_pos;
 
 	ItemCursorStyle style;
 	GdkGC    *gc;
@@ -73,7 +76,8 @@ struct _ItemCursor {
 	struct { /* cursor outline in canvas coords (bounding box is larger) */
 		int x1, x2, y1, y2;
 	} outline;
-	int	 drag_button;
+	int drag_button;
+	guint drag_button_state;
 
 	gboolean visible;
 	gboolean use_color;
@@ -106,7 +110,7 @@ cb_item_cursor_animation (ItemCursor *ic)
 }
 
 static void
-item_cursor_finalize (GObject *obj)
+item_cursor_dispose (GObject *obj)
 {
 	ItemCursor *ic = ITEM_CURSOR (obj);
 
@@ -115,7 +119,7 @@ item_cursor_finalize (GObject *obj)
 		ic->tip = NULL;
 	}
 
-	(*G_OBJECT_CLASS (parent_class)->finalize) (obj);
+	G_OBJECT_CLASS (parent_class)->dispose (obj);
 }
 
 static void
@@ -269,7 +273,7 @@ item_cursor_draw (FooCanvasItem *item, GdkDrawable *drawable,
 		draw_center   = TRUE;
 		draw_thick    = 3;
 		draw_stippled = TRUE;
-		fore          = &gs_black;
+		fore          = &gs_white;
 		back          = &gs_white;
 		break;
 
@@ -335,9 +339,9 @@ item_cursor_draw (FooCanvasItem *item, GdkDrawable *drawable,
 
 	gdk_gc_set_line_attributes (ic->gc, 1,
 		GDK_LINE_SOLID, GDK_CAP_BUTT, GDK_JOIN_MITER);
-	gdk_gc_set_rgb_fg_color (ic->gc, &gs_black);
+	gdk_gc_set_rgb_fg_color (ic->gc, &gs_white);
 	gdk_gc_set_rgb_bg_color (ic->gc, &gs_white);
-	values.function = GDK_EQUIV;
+	values.function = GDK_XOR;
 	gdk_gc_set_values (ic->gc, &values, GDK_GC_FUNCTION);
 
 	if (draw_external) {
@@ -632,6 +636,7 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 
 		scg_special_cursor_start (ic->scg, style, button);
 		special_cursor = gcanvas->pane->cursor.special;
+		special_cursor->drag_button_state = ic->drag_button_state;
 		if (style == ITEM_CURSOR_AUTOFILL)
 			item_cursor_setup_auto_fill (
 				special_cursor, ic, x, y);
@@ -854,12 +859,25 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 			return TRUE;
 
 		if (event->button.button != 3) {
+			int x, y;
+
 			/* prepare to create fill or drag cursors, but dont until we
 			 * move.  If we did create them here there would be problems
 			 * with race conditions when the new cursors pop into existence
 			 * during a double-click
 			 */
+
+			foo_canvas_w2c (canvas,
+					event->button.x, event->button.y, &x, &y);
+			if (item_cursor_in_drag_handle (ic, x, y))
+				go_cmd_context_progress_message_set (GO_CMD_CONTEXT (ic->scg->wbcg),
+								     _("Drag to autofill"));
+			else
+				go_cmd_context_progress_message_set (GO_CMD_CONTEXT (ic->scg->wbcg),
+								     _("Drag to move"));
+
 			ic->drag_button = event->button.button;
+			ic->drag_button_state = event->button.state;
 			gnm_simple_canvas_grab (item,
 				GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK,
 				NULL, event->button.time);
@@ -876,6 +894,8 @@ item_cursor_selection_event (FooCanvasItem *item, GdkEvent *event)
 			gnm_simple_canvas_ungrab (item, event->button.time);
 			ic->drag_button = -1;
 		}
+		go_cmd_context_progress_message_set (GO_CMD_CONTEXT (ic->scg->wbcg),
+						     " ");
 		return TRUE;
 
 	default:
@@ -1096,7 +1116,7 @@ item_cursor_set_visibility (ItemCursor *ic, gboolean visible)
 }
 
 static void
-item_cursor_tip_setlabel (ItemCursor *ic)
+item_cursor_tip_setlabel (ItemCursor *ic, const char *text)
 {
 	if (ic->tip == NULL) {
 		ic->tip = gnumeric_create_tooltip ();
@@ -1105,7 +1125,7 @@ item_cursor_tip_setlabel (ItemCursor *ic)
 	}
 
 	g_return_if_fail (ic->tip != NULL);
-	gtk_label_set_text (GTK_LABEL (ic->tip), range_name (&ic->pos));
+	gtk_label_set_text (GTK_LABEL (ic->tip), text);
 }
 
 static gboolean
@@ -1128,7 +1148,7 @@ cb_move_cursor (GnmCanvas *gcanvas, GnmCanvasSlideInfo const *info)
 	else if (corner.row >= (SHEET_MAX_ROWS - h))
 		corner.row = SHEET_MAX_ROWS - h - 1;
 
-	item_cursor_tip_setlabel (ic);
+	item_cursor_tip_setlabel (ic, range_as_string (&ic->pos));
 
 	/* Make target cell visible, and adjust the cursor size */
 	item_cursor_set_bounds_visibly (ic, info->col, info->row, &corner,
@@ -1167,7 +1187,7 @@ item_cursor_drag_event (FooCanvasItem *item, GdkEvent *event)
 		/* This kind of cursor is created and grabbed.  Then destroyed
 		 * when the button is released.  If we are seeing a press it
 		 * means that someone has pressed another button WHILE THE
-		 * FIRST IS STILL DOWN.  Ignore ths event.
+		 * FIRST IS STILL DOWN.  Ignore this event.
 		 */
 		return TRUE;
 
@@ -1186,6 +1206,7 @@ cb_autofill_scroll (GnmCanvas *gcanvas, GnmCanvasSlideInfo const *info)
 	ItemCursor *ic = info->user_data;
 	GnmRange r = ic->autofill_src;
 	int col = info->col, row = info->row;
+	int h, w;
 
 	/* compass offsets are distances (in cells) from the edges of the
 	 * selected area to the mouse cursor */
@@ -1215,9 +1236,50 @@ cb_autofill_scroll (GnmCanvas *gcanvas, GnmCanvasSlideInfo const *info)
 			row = r.end.row;
 	}
 
+	/* Check if we have moved to a new cell.  */
+	if (col == ic->last_tip_pos.col && row == ic->last_tip_pos.row)
+		return FALSE;
+	ic->last_tip_pos.col = col;
+	ic->last_tip_pos.row = row;
+
 	item_cursor_set_bounds_visibly (ic, col, row,
 		&r.start, r.end.col, r.end.row);
 
+	w = range_width (&ic->autofill_src);
+	h = range_height (&ic->autofill_src);
+	if (ic->pos.start.col + w - 1 == ic->pos.end.col &&
+	    ic->pos.start.row + h - 1 == ic->pos.end.row)
+		item_cursor_tip_setlabel (ic, _("Autofill"));
+	else {
+		gboolean inverse_autofill =
+			(ic->pos.start.col < ic->autofill_src.start.col ||
+			 ic->pos.start.row < ic->autofill_src.start.row);
+		gboolean default_increment =
+			ic->drag_button_state & GDK_CONTROL_MASK;
+		SheetControl *sc = (SheetControl *) ic->scg;
+		Sheet *sheet = sc->sheet;
+		char *hint;
+
+		if (inverse_autofill)
+			hint = sheet_autofill_hint
+				(sheet, default_increment,
+				 ic->pos.end.col, ic->pos.end.row,
+				 w, h,
+				 ic->pos.start.col, ic->pos.start.row);
+		else
+			hint = sheet_autofill_hint
+				(sheet, default_increment,
+				 ic->pos.start.col, ic->pos.start.row,
+				 w, h,
+				 ic->pos.end.col, ic->pos.end.row);
+
+		if (hint) {
+			item_cursor_tip_setlabel (ic, hint);
+			g_free (hint);
+		} else
+			item_cursor_tip_setlabel (ic, "");
+	}
+
 	return FALSE;
 }
 
@@ -1226,19 +1288,19 @@ item_cursor_autofill_event (FooCanvasItem *item, GdkEvent *event)
 {
 	ItemCursor *ic = ITEM_CURSOR (item);
 	SheetControl *sc = (SheetControl *) ic->scg;
-	gboolean inverse_autofill;
 
 	switch (event->type) {
-	case GDK_BUTTON_RELEASE:
-		gnm_canvas_slide_stop (GNM_CANVAS (item->canvas));
+	case GDK_BUTTON_RELEASE: {
+		gboolean inverse_autofill =
+			(ic->pos.start.col < ic->autofill_src.start.col ||
+			 ic->pos.start.row < ic->autofill_src.start.row);
+		gboolean default_increment =
+			ic->drag_button_state & GDK_CONTROL_MASK;
 
+		gnm_canvas_slide_stop (GNM_CANVAS (item->canvas));
 		gnm_simple_canvas_ungrab (item, event->button.time);
 
-		inverse_autofill = (ic->pos.start.col < ic->autofill_src.start.col ||
-				    ic->pos.start.row < ic->autofill_src.start.row);
-
-		cmd_autofill (sc->wbc, sc->sheet,
-			      event->button.state & GDK_CONTROL_MASK,
+		cmd_autofill (sc->wbc, sc->sheet, default_increment,
 			      ic->pos.start.col, ic->pos.start.row,
 			      range_width (&ic->autofill_src),
 			      range_height (&ic->autofill_src),
@@ -1247,6 +1309,7 @@ item_cursor_autofill_event (FooCanvasItem *item, GdkEvent *event)
 
 		scg_special_cursor_stop	(ic->scg);
 		return TRUE;
+	}
 
 	case GDK_MOTION_NOTIFY:
 		item_cursor_handle_motion (ic, event, &cb_autofill_scroll);
@@ -1335,7 +1398,7 @@ item_cursor_class_init (GObjectClass *gobject_klass)
 	parent_class = g_type_class_peek_parent (gobject_klass);
 
 	gobject_klass->set_property = item_cursor_set_property;
-	gobject_klass->finalize     = item_cursor_finalize;
+	gobject_klass->dispose = item_cursor_dispose;
 	g_object_class_install_property (gobject_klass, ITEM_CURSOR_PROP_SHEET_CONTROL_GUI,
 		g_param_spec_object ("SheetControlGUI", "SheetControlGUI",
 			"the sheet control gui controlling the item",
@@ -1385,6 +1448,8 @@ item_cursor_init (ItemCursor *ic)
 	ic->row_delta = 0;
 
 	ic->tip = NULL;
+	ic->last_tip_pos.col = -1;
+	ic->last_tip_pos.row = -1;
 
 	ic->style = ITEM_CURSOR_SELECTION;
 	ic->gc = NULL;
