@@ -7,6 +7,10 @@
  * Authors:
  *     Miguel de Icaza (miguel@kernel.org)
  *     Jody Goldberg (jody@gnome.org)
+ *
+ * Port to Maemo:
+ * 	Eduardo Lima  (eduardo.lima@indt.org.br)
+ * 	Renato Araujo (renato.filho@indt.org.br)
  */
 
 #include <gnumeric-config.h>
@@ -314,10 +318,10 @@ item_grid_draw_merged_range (GdkDrawable *drawable, ItemGrid *ig,
 				   l - r - (ci->margin_b + ci->margin_a + 1),
 				   b - t - (ri->margin_b + ri->margin_a + 1), -1);
 		else
-		cell_draw (cell, ig->gc.cell, drawable,
-			   l, t,
-			   r - l - (ci->margin_b + ci->margin_a + 1),
-			   b - t - (ri->margin_b + ri->margin_a + 1), -1);
+			cell_draw (cell, ig->gc.cell, drawable,
+				   l, t,
+				   r - l - (ci->margin_b + ci->margin_a + 1),
+				   b - t - (ri->margin_b + ri->margin_a + 1), -1);
 	}
 	style_border_draw_diag (style, drawable, l, t, r, b);
 }
@@ -892,8 +896,12 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 				      NULL);
 
 			if ((ig->last_click_time + double_click_time) > event->time &&
-			    wbcg_edit_start (scg->wbcg, FALSE, FALSE))
+			    wbcg_edit_start (scg->wbcg, FALSE, FALSE)) {
+#ifdef USE_HILDON
+				gtk_im_context_focus_in (GNM_CANVAS (canvas)->im_context);
+#endif
 				break;
+			}
 		}
 
 		ig->last_click_time = event->time;
@@ -1014,7 +1022,7 @@ item_grid_event (FooCanvasItem *item, GdkEvent *event)
 
 		default:
 			g_assert_not_reached ();
-		};
+		}
 
 		ig->selecting = ITEM_GRID_NO_SELECTION;
 		gnm_simple_canvas_ungrab (item, event->button.time);
@@ -1058,7 +1066,7 @@ item_grid_event (FooCanvasItem *item, GdkEvent *event)
 			break;
 		default:
 			g_assert_not_reached ();
-		};
+		}
 
 		gnm_canvas_handle_motion (gcanvas, canvas, &event->motion,
 			GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y |
