@@ -14,8 +14,7 @@
 
 #include "application.h"
 #include "stf.h"
-#include "plugin.h"
-#include "format.h"
+#include "gnm-format.h"
 #include "command-context.h"
 #include "command-context-stderr.h"
 #include "workbook.h"
@@ -23,9 +22,7 @@
 #include "number-match.h"
 #include "expr-name.h"
 #include "func.h"
-#include "application.h"
 #include "print-info.h"
-#include "global-gnome-font.h"
 #include "style.h"
 #include "mstyle.h"
 #include "style-color.h"
@@ -34,16 +31,20 @@
 #include "sheet-autofill.h"
 #include "xml-io.h"
 #include "cell.h"
+#include "clipboard.h"
 #include "value.h"
 #include "expr.h"
 #include "parse-util.h"
 #include "rendered-value.h"
 #include "gnumeric-gconf.h"
-#include "plugin-service.h"
+#include "gnm-plugin.h"
 #include "mathfunc.h"
 #include "hlink.h"
-#include "gnumeric-paths.h"
 #include <goffice/goffice.h>
+#include <goffice/app/go-plugin.h>
+#include <goffice/app/go-plugin-service.h>
+#include <goffice/app/go-cmd-context.h>
+#include <goffice/app/go-plugin-loader-module.h>
 
 #include <locale.h>
 #include <glade/glade.h>
@@ -52,14 +53,11 @@
 int gnumeric_debugging = 0;
 int dependency_debugging = 0;
 int expression_sharing_debugging = 0;
-int immediate_exit_flag = 0;
 int print_debugging = 0;
 gboolean initial_workbook_open_complete = FALSE;
-char	   *x_geometry;
-char const *gnumeric_lib_dir = GNUMERIC_LIBDIR;
-char const *gnumeric_data_dir = GNUMERIC_DATADIR;
-char const *gnumeric_icon_dir = GNUMERIC_ICONDIR;
-char const *gnumeric_locale_dir = GNUMERIC_LOCALEDIR;
+gboolean gnumeric_no_splash = FALSE;
+gboolean  gnumeric_no_warnings = FALSE;
+char *x_geometry;
 
 /**
  * gnm_pre_parse_init :
@@ -75,22 +73,10 @@ gnm_pre_parse_init (char const* gnumeric_binary)
 	/* Make stdout line buffered - we only use it for debug info */
 	setvbuf (stdout, NULL, _IOLBF, 0);
 
-#ifdef G_OS_WIN32
-{
-	char *dir;
-	dir = g_win32_get_package_installation_directory (NULL, NULL);
-	gnumeric_data_dir = g_build_filename (dir,
-		"share", "gnumeric", GNUMERIC_VERSION, NULL);
-	gnumeric_lib_dir = g_build_filename (dir,
-		"lib", "gnumeric", GNUMERIC_VERSION, NULL);
-	gnumeric_icon_dir = g_build_filename (dir,
-		"lib", "pixmaps", "gnumeric", NULL);
-	gnumeric_locale_dir = g_build_filename (dir,
-		"lib", "locale", NULL);
-}
-#endif
+	gutils_init ();
 
-	bindtextdomain (GETTEXT_PACKAGE, gnumeric_locale_dir);
+	bindtextdomain (GETTEXT_PACKAGE, gnm_locale_dir ());
+	bindtextdomain (GETTEXT_PACKAGE "-functions", gnm_locale_dir ());
 	textdomain (GETTEXT_PACKAGE);
 
 	/* Force all of the locale segments to update from the environment.
@@ -121,25 +107,27 @@ extern void libgoffice_init (void);
 void
 gnm_common_init (gboolean fast)
 {
-	plugin_services_init ();
 	libgoffice_init ();
+	plugin_service_define ("function_group",
+		&plugin_service_function_group_get_type);
+	plugin_service_define ("ui",
+		&plugin_service_ui_get_type);
+	go_plugin_loader_module_register_version ("gnumeric", GNUMERIC_VERSION);
 
 	g_object_new (GNM_APP_TYPE, NULL);
 	mathfunc_init ();
 	gnm_string_init ();
-	mstyle_init ();
+	gnm_style_init ();
 	gnm_conf_init (fast);
 	value_init ();
 	parse_util_init ();
 	expr_init ();
 	cell_init ();
+	clipboard_init ();
 	dependent_types_init ();
 	rendered_value_init ();
 	gnumeric_color_init ();
 	style_init ();
-	format_color_init ();
-	format_match_init ();
-	/* e_cursors_init (); */
 	functions_init ();
 	print_init ();
 	autofill_init ();
@@ -158,15 +146,19 @@ gnm_common_init (gboolean fast)
 	glade_init ();
 }
 
+/**
+ * TODO : do we really want this here ?
+ * seems like a better fit in main-application.c
+ **/
 int
-gnm_dump_func_defs (char const* filename, gboolean def_or_state)
+gnm_dump_func_defs (char const* filename, int dump_type)
 {
 	int retval;
-	GnmCmdContext *cc = cmd_context_stderr_new ();
+	GOCmdContext *cc = cmd_context_stderr_new ();
 
-	plugins_init (cc);
+	gnm_plugins_init (cc);
 	if ((retval = cmd_context_stderr_get_status (COMMAND_CONTEXT_STDERR (cc))) == 0)
-		function_dump_defs (filename, def_or_state);
+		function_dump_defs (filename, dump_type);
 
 	return retval;
 }
@@ -174,27 +166,34 @@ gnm_dump_func_defs (char const* filename, gboolean def_or_state)
 void
 gnm_shutdown (void)
 {
+	GSList *plugin_states;
+
 	gnm_app_release_pref_dialog ();
 	gnm_app_clipboard_clear (TRUE);
 
-	plugins_shutdown ();
+	plugin_states = go_plugins_shutdown ();
+	if (NULL != plugin_states) {
+		gnm_gconf_set_plugin_file_states (plugin_states);
+		go_conf_sync (NULL);
+	}
+
+	autofill_shutdown ();
 	print_shutdown ();
 	functions_shutdown ();
-	/* e_cursors_shutdown (); */
-	format_match_finish ();
-	format_color_shutdown ();
 	style_shutdown ();
 	gnumeric_color_shutdown ();
 	rendered_value_shutdown ();
 	dependent_types_shutdown ();
+	clipboard_shutdown ();
 	cell_shutdown ();
 	expr_shutdown ();
 	parse_util_shutdown ();
 	value_shutdown ();
 	gnm_conf_shutdown ();
-	mstyle_shutdown ();
+	gnm_style_shutdown ();
 	gnm_string_shutdown ();
 	libgoffice_shutdown ();
 	plugin_services_shutdown ();
 	g_object_unref (gnm_app_get_app ());
+	gutils_shutdown ();
 }
