@@ -10,7 +10,6 @@
  *	Miguel de Icaza (miguel@gnu.org)
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "print-info.h"
 
@@ -24,10 +23,12 @@
 #include "gnumeric-gconf.h"
 #include "gnumeric-gconf-priv.h"
 
+#include <goffice/app/go-doc.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/utils/datetime.h>
 #include <goffice/utils/go-glib-extras.h>
 
+#include <glib/gi18n.h>
 #include <string.h>
 #include <locale.h>
 #include <time.h>
@@ -35,7 +36,7 @@
 #define MAX_SAVED_CUSTOM_HF_FORMATS 9
 
 GList *hf_formats = NULL;
-gint   hf_formats_base_num = 0;
+static gint hf_formats_base_num = 0;
 
 PrintHF *
 print_hf_new (char const *left_side_format,
@@ -343,9 +344,9 @@ print_info_save (PrintInformation const *pi)
 	gnm_gconf_set_print_order_across_then_down (pi->print_across_then_down);
 	
 	go_conf_set_string (node, PRINTSETUP_GCONF_REPEAT_TOP,
-		pi->repeat_top.use ? range_name (&pi->repeat_top.range) : "");
+		pi->repeat_top.use ? range_as_string (&pi->repeat_top.range) : "");
 	go_conf_set_string (node, PRINTSETUP_GCONF_REPEAT_LEFT,
-		pi->repeat_left.use ? range_name (&pi->repeat_left.range) : "");
+		pi->repeat_left.use ? range_as_string (&pi->repeat_left.range) : "");
 
 	save_formats ();
 
@@ -452,7 +453,8 @@ static void
 render_file (GString *target, HFRenderInfo *info, char const *args)
 {
 	if (info->sheet != NULL && info->sheet->workbook != NULL) {
-		char *name = go_basename_from_uri (workbook_get_uri (info->sheet->workbook));
+		char *name = go_basename_from_uri (
+			go_doc_get_uri (GO_DOC (info->sheet->workbook)));
 		g_string_append (target, name);
 		g_free (name);
 	} else 
@@ -463,7 +465,8 @@ static void
 render_path (GString *target, HFRenderInfo *info, char const *args)
 {
 	if (info->sheet != NULL && info->sheet->workbook != NULL) {
-		char *path = go_dirname_from_uri (workbook_get_uri (info->sheet->workbook), TRUE);
+		char *path = go_dirname_from_uri (
+			go_doc_get_uri (GO_DOC (info->sheet->workbook)), TRUE);
 		g_string_append (target, path);
 		g_free (path);
 	} else 
