@@ -38,6 +38,7 @@ enum {
 	PROP_SEARCH_COMMENTS,
 	PROP_BY_ROW,
 	PROP_QUERY,
+	PROP_REPLACE_KEEP_STRINGS,
 	PROP_SHEET,
 	PROP_SCOPE,
 	PROP_RANGE_TEXT
@@ -259,7 +260,7 @@ gnm_search_replace_comment (GnmSearchReplace *sr,
 
 	if (!sr->search_comments) return FALSE;
 
-	res->comment = cell_has_comment_pos (ep->sheet, &ep->eval);
+	res->comment = sheet_get_comment (ep->sheet, &ep->eval);
 	if (!res->comment) return FALSE;
 
 	res->old_text = cell_comment_text_get (res->comment);
@@ -300,7 +301,7 @@ gnm_search_replace_cell (GnmSearchReplace *sr,
 
 	is_expr = cell_has_expr (cell);
 	is_value = !is_expr && !cell_is_empty (cell) && v;
-	is_string = is_value && (v->type == VALUE_STRING);
+	is_string = is_value && (VALUE_IS_STRING (v));
 	is_other = is_value && !is_string;
 
 	if ((is_expr && sr->search_expressions) ||
@@ -310,7 +311,7 @@ gnm_search_replace_cell (GnmSearchReplace *sr,
 		gboolean initial_quote;
 
 		res->old_text = cell_get_entered_text (cell);
-		initial_quote = (is_value && res->old_text[0] == '\'');
+		initial_quote = (is_string && res->old_text[0] == '\'');
 
 		actual_src = res->old_text + (initial_quote ? 1 : 0);
 
@@ -318,7 +319,7 @@ gnm_search_replace_cell (GnmSearchReplace *sr,
 			res->new_text = go_search_replace_string (GO_SEARCH_REPLACE (sr),
 								  actual_src);
 			if (res->new_text) {
-				if (initial_quote) {
+				if (sr->replace_keep_strings && is_string) {
 					/*
 					 * The initial quote was not part of the s-a-r,
 					 * so tack it back on.
@@ -424,6 +425,9 @@ gnm_search_replace_get_property (GObject     *object,
 	case PROP_QUERY:
 		g_value_set_boolean (value, sr->query);
 		break;
+	case PROP_REPLACE_KEEP_STRINGS:
+		g_value_set_boolean (value, sr->replace_keep_strings);
+		break;
 	case PROP_SHEET:
 		g_value_set_object (value, sr->sheet);
 		break;
@@ -489,6 +493,9 @@ gnm_search_replace_set_property (GObject      *object,
 	case PROP_QUERY:
 		sr->query = g_value_get_boolean (value);
 		break;
+	case PROP_REPLACE_KEEP_STRINGS:
+		sr->replace_keep_strings = g_value_get_boolean (value);
+		break;
 	case PROP_SHEET:
 		gnm_search_replace_set_sheet (sr, g_value_get_object (value));
 		break;
@@ -591,11 +598,21 @@ gnm_search_replace_class_init (GObjectClass *gobject_class)
 				       FALSE,
 				       GSF_PARAM_STATIC |
 				       G_PARAM_READWRITE));
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_REPLACE_KEEP_STRINGS,
+		 g_param_spec_boolean ("replace-keep-strings",
+				       _("Keep Strings"),
+				       _("Should replacement keep strings as strings?"),
+				       FALSE,
+				       GSF_PARAM_STATIC |
+				       G_PARAM_READWRITE));
 	g_object_class_install_property
 		(gobject_class,
 		 PROP_SHEET,
-		 g_param_spec_object ("sheet", "Sheet",
-				      "The sheet in which to search.",
+		 g_param_spec_object ("sheet",
+				      _("Sheet"),
+				      _("The sheet in which to search."),
 				      GNM_SHEET_TYPE,
 				      GSF_PARAM_STATIC |
 				      G_PARAM_READWRITE));
