@@ -44,7 +44,7 @@
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-data-set.h>
 #include <goffice/graph/gog-renderer-gnome-print.h>
-#include <goffice/graph/gog-renderer-pixbuf.h>
+#include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-renderer-svg.h>
 #include <goffice/graph/gog-control-foocanvas.h>
 #include <goffice/utils/go-file.h>
@@ -113,7 +113,7 @@ static GSF_CLASS_FULL (SOGraphFooView, so_graph_foo_view,
 typedef struct {
 	SheetObject  base;
 	GogGraph	*graph;
-	GObject		*renderer;
+	GogRenderer	*renderer;
 	gulong		 add_sig, remove_sig;
 } SheetObjectGraph;
 typedef SheetObjectClass SheetObjectGraphClass;
@@ -147,7 +147,7 @@ sog_datas_set_sheet (SheetObjectGraph *sog, Sheet *sheet)
 }
 
 static void
-sheet_object_graph_finalize (GObject *obj)
+gnm_sog_finalize (GObject *obj)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (obj);
 
@@ -164,7 +164,7 @@ sheet_object_graph_finalize (GObject *obj)
 }
 
 static SheetObjectView *
-sheet_object_graph_new_view (SheetObject *so, SheetObjectViewContainer *container)
+gnm_sog_new_view (SheetObject *so, SheetObjectViewContainer *container)
 {
 	GnmCanvas *gcanvas = ((GnmPane *)container)->gcanvas;
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
@@ -176,7 +176,7 @@ sheet_object_graph_new_view (SheetObject *so, SheetObjectViewContainer *containe
 }
 
 static GtkTargetList *
-sheet_object_graph_get_target_list (SheetObject const *so)
+gnm_sog_get_target_list (SheetObject const *so)
 {
 	GtkTargetList *tl = gtk_target_list_new (NULL, 0);
 	char *mime_str = go_image_format_to_mime ("svg");
@@ -200,7 +200,7 @@ sheet_object_graph_get_target_list (SheetObject const *so)
 }
 
 static GtkTargetList *
-sheet_object_graph_get_object_target_list (SheetObject const *so)
+gnm_sog_get_object_target_list (SheetObject const *so)
 {
 	GtkTargetList *tl, *tl2;
 	GtkTargetPair *pair;
@@ -221,24 +221,9 @@ sheet_object_graph_get_object_target_list (SheetObject const *so)
 	return tl;
 }
 
-static gboolean
-sog_gsf_gdk_pixbuf_save (const gchar *buf,
-			 gsize count,
-			 GError **error,
-			 gpointer data)
-{
-	GsfOutput *output = GSF_OUTPUT (data);
-	gboolean ok = gsf_output_write (output, count, buf);
-
-	if (!ok && error)
-		*error = g_error_copy (gsf_output_error (output));
-
-	return ok;
-}
-
 static void
-sheet_object_graph_write_image (SheetObject const *so, const char *format,
-				GsfOutput *output, GError **err)
+gnm_sog_write_image (SheetObject const *so, const char *format, double resolution,
+		     GsfOutput *output, GError **err)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	gboolean res = FALSE;
@@ -258,95 +243,74 @@ sheet_object_graph_write_image (SheetObject const *so, const char *format,
 
 	g_return_if_fail (w > 0 && h > 0);
 
-	if (strcmp (format, "svg") == 0) {
-		res = gog_graph_export_to_svg (sog->graph, output, w, h, 1.0);
-	} else {
-		GogRendererPixbuf *prend = GOG_RENDERER_PIXBUF (sog->renderer);
-		GdkPixbuf *pixbuf = gog_renderer_pixbuf_get (prend);
-
-		if (!pixbuf) {
-			gog_renderer_pixbuf_update (prend, w, h, 1.);
-			pixbuf = gog_renderer_pixbuf_get (prend);
-		}
-		res = gdk_pixbuf_save_to_callback (pixbuf,
-						   sog_gsf_gdk_pixbuf_save,
-						   output, format,
-						   err, NULL);
-	}
+	res = gog_graph_export_image (sog->graph, go_image_get_format_from_name (format),
+				      output, resolution, resolution);
+	
 	if (!res && err && *err == NULL)
 		*err = g_error_new (gsf_output_error_id (), 0,
 				    _("Unknown failure while saving image"));
 }
 
 static void
-sheet_object_graph_write_object (SheetObject const *so, const char *format,
-				GsfOutput *output, GError **err)
+gnm_sog_write_object (SheetObject const *so, const char *format,
+				 GsfOutput *output, GError **err)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
-	xmlDocPtr pDoc = xmlNewDoc ((xmlChar const*)"1.0");
-	xmlChar *mem;
-	int size;
+	GsfXMLOut *xout;
 	char *old_num_locale, *old_monetary_locale;
-	GogObject *graph = gog_object_dup (GOG_OBJECT (sog->graph), NULL, gog_dataset_dup_to_simple);
+	GogObject *graph;
+
 	g_return_if_fail (strcmp (format, "application/x-goffice-graph") == 0);
+
 	old_num_locale = g_strdup (go_setlocale (LC_NUMERIC, NULL));
 	go_setlocale (LC_NUMERIC, "C");
 	old_monetary_locale = g_strdup (go_setlocale (LC_MONETARY, NULL));
-	go_setlocale(LC_MONETARY, "C");
-	pDoc->children = gog_object_write_xml (graph, pDoc);
+	go_setlocale (LC_MONETARY, "C");
+	go_set_untranslated_bools ();
+
+	graph = gog_object_dup (GOG_OBJECT (sog->graph),
+		NULL, gog_dataset_dup_to_simple);
+	xout = gsf_xml_out_new (output);
+	gog_object_write_xml_sax (GOG_OBJECT (graph), xout);
+	g_object_unref (xout);
 	g_object_unref (graph);
-	xmlDocDumpMemory (pDoc, &mem, &size);
+
+	/* go_setlocale restores bools to locale translation */
 	go_setlocale (LC_MONETARY, old_monetary_locale);
 	g_free (old_monetary_locale);
 	go_setlocale (LC_NUMERIC, old_num_locale);
 	g_free (old_num_locale);
-	gsf_output_write (output, size, (guint8 const*) mem);
 }
 
-/* 
- * The following are useful formats to save in:
- *  png
- *  svg
- *  eps
- *
- * TODO: Possibly add an eps renderer.  We may also use a new instance of
- * pixbufrenderer to save as png. This would allow the user to specify size of
- * the saved image, if that's wanted.
- */
 static void
 sog_cb_save_as (SheetObject *so, SheetControl *sc)
 {
-	static GOImageType const fmts[] = {
-		{(char *) "svg",  (char *) N_("SVG (vector graphics)"), (char *) "svg", FALSE},
-		{(char *) "png",  (char *) N_("PNG (raster graphics)"), (char *) "png", TRUE},
-		{(char *) "jpeg", (char *) N_("JPEG (photograph)"),     (char *) "jpg", TRUE}
-	};
-
 	WorkbookControlGUI *wbcg;
 	char *uri;
 	GError *err = NULL;
 	GsfOutput *output;
-	GSList *l = NULL;
-	GOImageType const *sel_fmt = &fmts[0];
-	guint i;
+	GSList *l;
+	GOImageFormat selected_format;
+	GOImageFormatInfo const *format_info;
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
+	double resolution;
 
 	g_return_if_fail (sog != NULL);
 
-	for (i = 0; i < G_N_ELEMENTS (fmts); i++)
-		l = g_slist_prepend (l, (gpointer) (fmts + i));
-	l = g_slist_reverse (l);
-
-	wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
+	l = gog_graph_get_supported_image_formats ();
+	g_return_if_fail (l != NULL);
+	selected_format = GPOINTER_TO_UINT (l->data);
 
 #warning "This violates model gui barrier"
-	uri = gui_get_image_save_info (wbcg_toplevel (wbcg), l, &sel_fmt);
+	wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
+	uri = gui_get_image_save_info (wbcg_toplevel (wbcg), l, &selected_format, &resolution);
 	if (!uri)
 		goto out;
 	output = go_file_create (uri, &err);
 	if (!output)
 		goto out;
-	sheet_object_write_image (so, sel_fmt->name, output, &err);
+	format_info = go_image_get_format_info (selected_format);
+	sheet_object_write_image (so, format_info->name, resolution, output, &err);
 	g_object_unref (output);
 		
 	if (err != NULL)
@@ -358,7 +322,7 @@ out:
 }
 
 static void
-sheet_object_graph_populate_menu (SheetObject *so, GPtrArray *actions)
+gnm_sog_populate_menu (SheetObject *so, GPtrArray *actions)
 {
 	static SheetObjectAction const sog_action =
 		{ GTK_STOCK_SAVE_AS, N_("_Save as image"), NULL, 0, sog_cb_save_as };
@@ -367,7 +331,7 @@ sheet_object_graph_populate_menu (SheetObject *so, GPtrArray *actions)
 }
 
 static gboolean
-sheet_object_graph_read_xml_dom (SheetObject *so, char const *typename,
+gnm_sog_read_xml_dom (SheetObject *so, char const *typename,
 				 XmlParseContext const *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr child = e_xml_get_child_by_name (tree, "GogObject");
@@ -380,25 +344,29 @@ sheet_object_graph_read_xml_dom (SheetObject *so, char const *typename,
 	return FALSE;
 }
 
-static gboolean
-sheet_object_graph_write_xml_dom (SheetObject const *so,
-				  XmlParseContext const *ctxt, xmlNode *parent)
-{
-	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
-	xmlNode *res = gog_object_write_xml (GOG_OBJECT (sog->graph), ctxt->doc);
-	if (res != NULL)
-		xmlAddChild (parent, res);
-	return FALSE;
-}
 static void
-sheet_object_graph_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
+gnm_sog_write_xml_sax (SheetObject const *so, GsfXMLOut *output)
 {
 	SheetObjectGraph const *sog = SHEET_OBJECT_GRAPH (so);
 	gog_object_write_xml_sax (GOG_OBJECT (sog->graph), output);
 }
 
 static void
-sheet_object_graph_copy (SheetObject *dst, SheetObject const *src)
+sog_xml_finish (GogObject *graph, SheetObject *so)
+{
+	sheet_object_graph_set_gog (so, GOG_GRAPH (graph));
+	g_object_unref (graph);
+}
+
+static void
+gnm_sog_prep_sax_parser (SheetObject *so, GsfXMLIn *xin, xmlChar const **attrs)
+{
+	gog_object_sax_push_parser (xin, attrs,
+		(GogObjectSaxHandler) sog_xml_finish, so);
+}
+
+static void
+gnm_sog_copy (SheetObject *dst, SheetObject const *src)
 {
 	SheetObjectGraph const *sog = SHEET_OBJECT_GRAPH (src);
 	GogGraph *graph   = gog_graph_dup (sog->graph);
@@ -407,7 +375,7 @@ sheet_object_graph_copy (SheetObject *dst, SheetObject const *src)
 }
 
 static void
-sheet_object_graph_print (SheetObject const *so, GnomePrintContext *ctx,
+gnm_sog_print (SheetObject const *so, GnomePrintContext *ctx,
 			  double width, double height)
 {
 	gog_graph_print_to_gnome_print (
@@ -417,10 +385,10 @@ sheet_object_graph_print (SheetObject const *so, GnomePrintContext *ctx,
 typedef struct {
 	SheetObject *so;
 	WorkbookControl *wbc;
-} sheet_object_graph_user_config_t;
+} gnm_sog_user_config_t;
 
 static void
-sheet_object_graph_user_config_free_data (gpointer data,
+gnm_sog_user_config_free_data (gpointer data,
 					  GClosure *closure)
 {
 	g_free (data);
@@ -428,18 +396,18 @@ sheet_object_graph_user_config_free_data (gpointer data,
 }
 
 static void
-cb_update_graph (GogGraph *graph, sheet_object_graph_user_config_t *data)
+cb_update_graph (GogGraph *graph, gnm_sog_user_config_t *data)
 {
 	cmd_so_graph_config (data->wbc, data->so, G_OBJECT (graph), 
 			     G_OBJECT (SHEET_OBJECT_GRAPH (data->so)->graph));
 }
 
 static void
-sheet_object_graph_user_config (SheetObject *so, SheetControl *sc)
+gnm_sog_user_config (SheetObject *so, SheetControl *sc)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	WorkbookControlGUI *wbcg;
-	sheet_object_graph_user_config_t *data;
+	gnm_sog_user_config_t *data;
 	GClosure *closure;
 
 	g_return_if_fail (sog != NULL);
@@ -447,19 +415,19 @@ sheet_object_graph_user_config (SheetObject *so, SheetControl *sc)
 
 	wbcg = scg_get_wbcg (SHEET_CONTROL_GUI (sc));
 	
-	data = g_new0 (sheet_object_graph_user_config_t, 1);
+	data = g_new0 (gnm_sog_user_config_t, 1);
 	data->so = so;
 	data->wbc = WORKBOOK_CONTROL (wbcg);
  
 	closure = g_cclosure_new (G_CALLBACK (cb_update_graph), data,
-		(GClosureNotify) sheet_object_graph_user_config_free_data);
+		(GClosureNotify) gnm_sog_user_config_free_data);
 	
  	sheet_object_graph_guru (wbcg, sog->graph, closure);
 	g_closure_sink (closure);
 }
 
 static gboolean
-sheet_object_graph_set_sheet (SheetObject *so, Sheet *sheet)
+gnm_sog_set_sheet (SheetObject *so, Sheet *sheet)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	if (sog->graph != NULL)
@@ -468,7 +436,7 @@ sheet_object_graph_set_sheet (SheetObject *so, Sheet *sheet)
 }
 
 static gboolean
-sheet_object_graph_remove_from_sheet (SheetObject *so)
+gnm_sog_remove_from_sheet (SheetObject *so)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 	if (sog->graph != NULL)
@@ -477,14 +445,14 @@ sheet_object_graph_remove_from_sheet (SheetObject *so)
 }
 
 static void
-sheet_object_graph_default_size (SheetObject const *so, double *w, double *h)
+gnm_sog_default_size (SheetObject const *so, double *w, double *h)
 {
 	*w = GO_CM_TO_PT ((double)12);
 	*h = GO_CM_TO_PT ((double)8);
 }
 
 static void
-sheet_object_graph_bounds_changed (SheetObject *so)
+gnm_sog_bounds_changed (SheetObject *so)
 {
 	SheetObjectGraph *sog = SHEET_OBJECT_GRAPH (so);
 
@@ -498,34 +466,34 @@ sheet_object_graph_bounds_changed (SheetObject *so)
 }
 
 static void
-sheet_object_graph_class_init (GObjectClass *klass)
+gnm_sog_class_init (GObjectClass *klass)
 {
 	SheetObjectClass	*so_class  = SHEET_OBJECT_CLASS (klass);
 
 	parent_klass = g_type_class_peek_parent (klass);
 
 	/* Object class method overrides */
-	klass->finalize = sheet_object_graph_finalize;
+	klass->finalize = gnm_sog_finalize;
 
 	/* SheetObject class method overrides */
-	so_class->new_view	     = sheet_object_graph_new_view;
-	so_class->bounds_changed     = sheet_object_graph_bounds_changed;
-	so_class->populate_menu	     = sheet_object_graph_populate_menu;
-	so_class->read_xml_dom	     = sheet_object_graph_read_xml_dom;
-	so_class->write_xml_dom	     = sheet_object_graph_write_xml_dom;
-	so_class->write_xml_sax	     = sheet_object_graph_write_xml_sax;
-	so_class->copy               = sheet_object_graph_copy;
-	so_class->user_config        = sheet_object_graph_user_config;
-	so_class->assign_to_sheet    = sheet_object_graph_set_sheet;
-	so_class->remove_from_sheet  = sheet_object_graph_remove_from_sheet;
-	so_class->print		     = sheet_object_graph_print;
-	so_class->default_size	     = sheet_object_graph_default_size;
+	so_class->new_view		= gnm_sog_new_view;
+	so_class->bounds_changed	= gnm_sog_bounds_changed;
+	so_class->populate_menu		= gnm_sog_populate_menu;
+	so_class->read_xml_dom		= gnm_sog_read_xml_dom;
+	so_class->write_xml_sax		= gnm_sog_write_xml_sax;
+	so_class->prep_sax_parser	= gnm_sog_prep_sax_parser;
+	so_class->copy			= gnm_sog_copy;
+	so_class->user_config		= gnm_sog_user_config;
+	so_class->assign_to_sheet	= gnm_sog_set_sheet;
+	so_class->remove_from_sheet	= gnm_sog_remove_from_sheet;
+	so_class->print			= gnm_sog_print;
+	so_class->default_size		= gnm_sog_default_size;
 
 	so_class->rubber_band_directly = FALSE;
 }
 
 static void
-sheet_object_graph_init (GObject *obj)
+gnm_sog_init (GObject *obj)
 {
 	SheetObject *so = SHEET_OBJECT (obj);
 	so->anchor.direction = SO_DIR_DOWN_RIGHT;
@@ -534,20 +502,20 @@ sheet_object_graph_init (GObject *obj)
 static void
 sog_imageable_init (SheetObjectImageableIface *soi_iface)
 {
-	soi_iface->get_target_list = sheet_object_graph_get_target_list;
-	soi_iface->write_image	   = sheet_object_graph_write_image;
+	soi_iface->get_target_list = gnm_sog_get_target_list;
+	soi_iface->write_image	   = gnm_sog_write_image;
 }
 
 static void
 sog_exportable_init (SheetObjectExportableIface *soe_iface)
 {
-	soe_iface->get_target_list = sheet_object_graph_get_object_target_list;
-	soe_iface->write_object	   = sheet_object_graph_write_object;
+	soe_iface->get_target_list = gnm_sog_get_object_target_list;
+	soe_iface->write_object	   = gnm_sog_write_object;
 }
 
 GSF_CLASS_FULL (SheetObjectGraph, sheet_object_graph,
-		NULL, NULL, sheet_object_graph_class_init,NULL,
-		sheet_object_graph_init, SHEET_OBJECT_TYPE, 0,
+		NULL, NULL, gnm_sog_class_init,NULL,
+		gnm_sog_init, SHEET_OBJECT_TYPE, 0,
 		GSF_INTERFACE (sog_imageable_init, SHEET_OBJECT_IMAGEABLE_TYPE) \
 		GSF_INTERFACE (sog_exportable_init, SHEET_OBJECT_EXPORTABLE_TYPE));
 
@@ -618,9 +586,7 @@ sheet_object_graph_set_gog (SheetObject *so, GogGraph *graph)
 	if (sog->renderer != NULL)
 		g_object_set (sog->renderer, "model", graph, NULL);
 	else
-		sog->renderer = g_object_new (GOG_RENDERER_PIXBUF_TYPE,
-					      "model", sog->graph,
-					      NULL);
+		sog->renderer = gog_renderer_new_for_pixbuf (sog->graph);
 }
 
 static void
