@@ -3,7 +3,7 @@
 /*
  * sheet-style.c: storage mechanism for styles and eventually cells.
  *
- * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as published
@@ -25,7 +25,6 @@
 #include "ranges.h"
 #include "sheet.h"
 #include "expr.h"
-#include "gnm-format.h"
 #include "style.h"
 #include "style-border.h"
 #include "style-color.h"
@@ -89,8 +88,9 @@ static void
 pstyle_set_border (GnmStyle *st, GnmBorder *border,
 		   StyleBorderLocation side)
 {
-	gnm_style_set_border (st, MSTYLE_BORDER_TOP + side,
-			   style_border_ref (border));
+	gnm_style_set_border (st,
+			      STYLE_BORDER_LOCATION_TO_STYLE_ELEMENT (side),
+			      style_border_ref (border));
 }
 
 /* Amortize the cost of applying a partial style over a large region
@@ -162,7 +162,7 @@ rstyle_apply (GnmStyle **old, ReplacementStyle *rs)
 		 */
 		s = (GnmStyle *)g_hash_table_lookup (rs->cache, *old);
 		if (s == NULL) {
-			GnmStyle *tmp = gnm_style_merge (*old, rs->pstyle);
+			GnmStyle *tmp = gnm_style_new_merged (*old, rs->pstyle);
 			s = sheet_style_find (rs->sheet, tmp);
 			gnm_style_link (*old);
 			g_hash_table_insert (rs->cache, *old, s);
@@ -327,7 +327,7 @@ cell_tile_style_new (GnmStyle *style, CellTileType t)
 			   break;
 	default : g_return_val_if_fail (FALSE, NULL);
 		return NULL;
-	};
+	}
 #endif
 
 	*((CellTileType *)&(res->type)) = t;
@@ -391,7 +391,7 @@ cell_tile_ptr_matrix_new (CellTile *t)
 		break;
 	}
 	default : ;
-	};
+	}
 
 	return (CellTile *)res;
 }
@@ -435,7 +435,7 @@ cell_tile_matrix_set (CellTile *t, GnmRange const *indic, ReplacementStyle *rs)
 	case TILE_MATRIX :
 	default :
 		break;
-	};
+	}
 
 	if (t->type != TILE_MATRIX)
 		cell_tile_dtor (t);
@@ -514,7 +514,7 @@ sheet_style_init (Sheet *sheet)
 	if (sheet->sheet_type == GNM_SHEET_OBJECT) {
 		gnm_style_set_back_color (default_style,
 			style_color_new_i8 (0x50, 0x50, 0x50));
-		kstyle_set_pattern (default_style, 1);
+		gnm_style_set_pattern (default_style, 1);
 	}
 #endif
 	sheet->style_data->default_style =
@@ -597,13 +597,13 @@ sheet_style_shutdown (Sheet *sheet)
 }
 
 /**
- * sheet_style_set_auto_pattern_color
- *
+ * sheet_style_set_auto_pattern_color :
  * @sheet:         The sheet
  * @pattern_color: The color
  *
  * Set the color for rendering auto colored patterns in this sheet.
- */
+ * Absorbs a reference to @pattern_color;
+ **/
 void
 sheet_style_set_auto_pattern_color (Sheet  *sheet, GnmColor *pattern_color)
 {
@@ -623,11 +623,11 @@ sheet_style_set_auto_pattern_color (Sheet  *sheet, GnmColor *pattern_color)
 
 /**
  * sheet_style_get_auto_pattern_color:
- *
  * @sheet: the sheet
  *
+ * Caller receives a reference to the result.
  * Returns the color for rendering auto colored patterns in this sheet.
- */
+ **/
 GnmColor *
 sheet_style_get_auto_pattern_color (Sheet const *sheet)
 {
@@ -1105,14 +1105,14 @@ sheet_style_default (Sheet const *sheet)
 
 /**
  * sheet_style_get :
- *
- * @sheet :
+ * @sheet : #Sheet
  * @col   :
  * @row   :
  *
  * Find the fully qualified style applicable to the specified cellpos.
- */
-GnmStyle *
+ * Does _not_ add a reference.
+ **/
+GnmStyle const *
 sheet_style_get (Sheet const *sheet, int col, int row)
 {
 	int width = TILE_SIZE_COL*TILE_SIZE_COL*TILE_SIZE_COL;
@@ -1147,7 +1147,7 @@ tail_recursion :
 
 	default :
 		break;
-	};
+	}
 
 	g_warning ("Adaptive Quad Tree corruption !");
 	return NULL;
@@ -1608,12 +1608,14 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 		      (ForeachTileFunc)cb_find_conflicts, &user);
 
 	/* copy over the diagonals */
-	for (i = STYLE_BORDER_REV_DIAG ; i <= STYLE_BORDER_DIAG ; i++)
-		if (!(user.conflicts & (1 << (MSTYLE_BORDER_TOP+i))))
-			borders [i] = style_border_ref (
-				gnm_style_get_border (*style, MSTYLE_BORDER_TOP+i));
+	for (i = STYLE_BORDER_REV_DIAG ; i <= STYLE_BORDER_DIAG ; i++) {
+		GnmStyleElement se = STYLE_BORDER_LOCATION_TO_STYLE_ELEMENT (i);
+		if (user.conflicts & (1 << se))
+			borders[i] = NULL;
 		else
-			borders [i] = NULL;
+			borders[i] = style_border_ref (
+				gnm_style_get_border (*style, se));
+	}
 
 	start_col = r->start.col;
 	if (r->start.col > 0)
@@ -2015,8 +2017,8 @@ sheet_style_get_list (Sheet const *sheet, GnmRange const *r)
 	StyleListMerge mi;
 
 	mi.style_equal = gnm_style_equal;
-	mi.cache = g_hash_table_new ((GHashFunc)&cellpos_hash,
-				     (GCompareFunc)&cellpos_equal);
+	mi.cache = g_hash_table_new ((GHashFunc)&gnm_cellpos_hash,
+				     (GCompareFunc)&gnm_cellpos_equal);
 
 	foreach_tile (sheet->style_data->styles,
 		      TILE_TOP_LEVEL, 0, 0, r,
@@ -2065,8 +2067,8 @@ sheet_style_collect_conditions (Sheet const *sheet, GnmRange const *r)
 	GnmStyleList *res = NULL;
 	StyleListMerge mi;
 	mi.style_equal = style_conditions_equal;
-	mi.cache = g_hash_table_new ((GHashFunc)&cellpos_hash,
-				     (GCompareFunc)&cellpos_equal);
+	mi.cache = g_hash_table_new ((GHashFunc)&gnm_cellpos_hash,
+				     (GCompareFunc)&gnm_cellpos_equal);
 
 	foreach_tile (sheet->style_data->styles,
 		      TILE_TOP_LEVEL, 0, 0, r,
@@ -2117,8 +2119,8 @@ sheet_style_collect_validations (Sheet const *sheet, GnmRange const *r)
 	StyleListMerge mi;
 
 	mi.style_equal = style_validation_equal;
-	mi.cache = g_hash_table_new ((GHashFunc)&cellpos_hash,
-				     (GCompareFunc)&cellpos_equal);
+	mi.cache = g_hash_table_new ((GHashFunc)&gnm_cellpos_hash,
+				     (GCompareFunc)&gnm_cellpos_equal);
 
 	foreach_tile (sheet->style_data->styles,
 		      TILE_TOP_LEVEL, 0, 0, r,
@@ -2322,7 +2324,7 @@ cb_validate (GnmStyle *style,
 			   corner_col, corner_row,
 			   corner_col+width -1, corner_row+height-1);
 		g_warning ("%s!%s", sheet->name_unquoted,
-			   range_name (&r));
+			   range_as_string (&r));
 	}
 }
 
