@@ -39,9 +39,9 @@
 #include "number-match.h"
 #include "gutils.h"
 #include "parse-util.h"
-#include "gnm-format.h"
 #include <goffice/utils/datetime.h>
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-format.h>
 
 #include <stdlib.h>
 #include <locale.h>
@@ -56,8 +56,6 @@ go_setlocale(LC_ALL, parseoptions->locale);}
 go_setlocale(LC_ALL, oldlocale);\
 g_free (oldlocale);}
 
-#define WARN_TOO_MANY_ROWS _("Too many rows in data to parse: %d")
-
 /* Source_t struct, used for interchanging parsing information between the low level parse functions */
 typedef struct {
 	GStringChunk *chunk;
@@ -158,9 +156,10 @@ stf_parse_options_new (void)
 	parseoptions->splitpositions = NULL;
 	stf_parse_options_fixed_splitpositions_clear (parseoptions);
 
-	parseoptions->stringindicator        = '"';
+	parseoptions->stringindicator = '"';
 	parseoptions->indicator_2x_is_single = TRUE;
-	parseoptions->duplicates             = FALSE;
+	parseoptions->duplicates = FALSE;
+	parseoptions->trim_seps = FALSE;
 
 	parseoptions->sep.str = NULL;
 	parseoptions->sep.chr = NULL;
@@ -266,30 +265,6 @@ stf_parse_options_add_line_terminator (StfParseOptions_t *parseoptions, char con
 	compile_terminators (parseoptions);
 }
 
-/**
- * stf_parse_options_remove_line_terminator:
- *
- * This will remove from the line terminators, in both the Fixed width and CSV delimited importers
- * this indicates the end of a row.
- *
- **/
-void
-stf_parse_options_remove_line_terminator (StfParseOptions_t *parseoptions, char const *terminator)
-{
-	GSList*    in_list;
-
-	g_return_if_fail (parseoptions != NULL);
-
-	in_list = g_slist_find_custom (parseoptions->terminator, terminator, go_str_compare);
-
-	if (in_list) {
-		char *s = in_list->data;
-		GO_SLIST_REMOVE (parseoptions->terminator, in_list->data);
-		g_free (s);
-		compile_terminators (parseoptions);
-	}
-}
-
 /**
  * stf_parse_options_clear_line_terminator:
  *
@@ -376,6 +351,19 @@ stf_parse_options_csv_set_duplicates (StfParseOptions_t *parseoptions, gboolean
 	parseoptions->duplicates = duplicates;
 }
 
+/**
+ * stf_parse_options_csv_set_trim_seps:
+ * @trim_seps : a boolean value indicating whether we want to ignore
+ *               separators at the beginning of lines
+ **/
+void
+stf_parse_options_csv_set_trim_seps (StfParseOptions_t *parseoptions, gboolean const trim_seps)
+{
+	g_return_if_fail (parseoptions != NULL);
+
+	parseoptions->trim_seps = trim_seps;
+}
+
 /**
  * stf_parse_options_fixed_splitpositions_clear:
  *
@@ -667,6 +655,8 @@ stf_parse_csv_line (Source_t *src, StfParseOptions_t *parseoptions)
 	g_return_val_if_fail (parseoptions != NULL, NULL);
 
 	line = g_ptr_array_new ();
+	if (parseoptions->trim_seps)
+		stf_parse_eat_separators (src, parseoptions);
 	while (*src->position != '\0' && !compare_terminator (src->position, parseoptions)) {
 		char *field = stf_parse_csv_cell (src, parseoptions);
 		if (parseoptions->duplicates)
@@ -776,8 +766,7 @@ stf_parse_general_free (GPtrArray *lines)
 GPtrArray *
 stf_parse_general (StfParseOptions_t *parseoptions,
 		   GStringChunk *lines_chunk,
-		   char const *data, char const *data_end,
-		   int maxlines)
+		   char const *data, char const *data_end)
 {
 	GPtrArray *lines;
 	Source_t src;
@@ -797,11 +786,6 @@ stf_parse_general (StfParseOptions_t *parseoptions,
 	while (*src.position != '\0' && src.position < data_end) {
 		GPtrArray *line;
 
-		if (++row >= SHEET_MAX_ROWS) {
-			g_warning (WARN_TOO_MANY_ROWS, row);
-			break;
-		}
-
 		line = parseoptions->parsetype == PARSE_TYPE_CSV
 			? stf_parse_csv_line (&src, parseoptions)
 			: stf_parse_fixed_line (&src, parseoptions);
@@ -809,7 +793,7 @@ stf_parse_general (StfParseOptions_t *parseoptions,
 		g_ptr_array_add (lines, line);
 		src.position += compare_terminator (src.position, parseoptions);
 
-		if (row >= maxlines)
+		if (++row == SHEET_MAX_ROWS)
 			break;
 	}
 
@@ -1162,8 +1146,7 @@ stf_parse_sheet (StfParseOptions_t *parseoptions,
 	if (!data_end)
 		data_end = data + strlen (data);
 	lines_chunk = g_string_chunk_new (100 * 1024);
-	lines = stf_parse_general (parseoptions, lines_chunk, data, data_end,
-				   SHEET_MAX_ROWS);
+	lines = stf_parse_general (parseoptions, lines_chunk, data, data_end);
 	if (lines == NULL)
 		return FALSE;
 	for (row = start_row, lrow = 0; lrow < lines->len ; row++, lrow++) {
@@ -1206,7 +1189,7 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 	GODateConventions const *date_conv = wb ? workbook_date_conv (wb) : &default_conv;
 
 	GnmCellRegion *cr;
-	GSList	 *content = NULL;
+	GSList	 *contents = NULL;
 	unsigned int row, colhigh = 0;
 	GStringChunk *lines_chunk;
 	GPtrArray *lines;
@@ -1220,8 +1203,7 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 	if (!data_end)
 		data_end = data + strlen (data);
 	lines_chunk = g_string_chunk_new (100 * 1024);
-	lines = stf_parse_general (parseoptions, lines_chunk, data, data_end,
-				   SHEET_MAX_ROWS);
+	lines = stf_parse_general (parseoptions, lines_chunk, data, data_end);
 	for (row = 0; row < lines->len; row++) {
 		GPtrArray *line = g_ptr_array_index (lines, row);
 		unsigned int col, targetcol = 0;
@@ -1244,8 +1226,8 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 
 					ccopy = gnm_cell_copy_new (targetcol, row);
 					ccopy->val  = v;
-					ccopy->expr = NULL;
-					content = g_slist_prepend (content, ccopy);
+					ccopy->texpr = NULL;
+					contents = g_slist_prepend (contents, ccopy);
 					targetcol++;
 					if (targetcol > colhigh)
 						colhigh = targetcol;
@@ -1259,7 +1241,7 @@ stf_parse_region (StfParseOptions_t *parseoptions, char const *data, char const
 	END_LOCALE_SWITCH;
 
 	cr = cellregion_new (NULL);
-	cr->content = content;
+	cr->contents = contents;
 	cr->cols    = (colhigh > 0) ? colhigh : 1;
 	cr->rows    = row;
 
@@ -1424,6 +1406,8 @@ stf_parse_options_guess (const char *data)
 		    count_character (lines, (c = ' '), 0.5) > 0) {
 			char sep[7];
 			sep[g_unichar_to_utf8 (c, sep)] = 0;
+			if (c == ' ')
+				strcat (sep, "\t");
 			stf_parse_options_csv_set_separators (res, sep, NULL);
 		}
 	}
@@ -1434,7 +1418,10 @@ stf_parse_options_guess (const char *data)
 		stf_parse_options_set_type (res, PARSE_TYPE_CSV);
 		stf_parse_options_set_trim_spaces (res, TRIM_TYPE_LEFT | TRIM_TYPE_RIGHT);
 		stf_parse_options_csv_set_indicator_2x_is_single (res, TRUE);
-		stf_parse_options_csv_set_duplicates (res, FALSE);
+		stf_parse_options_csv_set_duplicates
+			(res, strchr (res->sep.chr, ' ') != NULL);
+		stf_parse_options_csv_set_trim_seps
+			(res, strchr (res->sep.chr, ' ') != NULL);
 
 		stf_parse_options_csv_set_stringindicator (res, '"');
 	} else {
