@@ -78,7 +78,7 @@ solver_param_destroy (SolverParameters *sp)
 			 (GFunc)solver_constraint_destroy,
 			 NULL);
 	g_slist_free (sp->constraints);
-	g_list_free (sp->input_cells);
+	g_slist_free (sp->input_cells);
 	g_free (sp->input_entry_str);
 	g_free (sp->options.scenario_name);
 	g_free (sp);
@@ -470,7 +470,7 @@ lp_qp_solver_init (Sheet *sheet, const SolverParameters *param,
 		g_get_current_time (&cur_time);
 		if (cur_time.tv_sec - start.tv_sec >
 		    param->options.max_time_sec) {
-			*errmsg = solver_max_time_err;
+			*errmsg = SOLVER_MAX_TIME_ERR;
 			solver_results_free (res);
 			return NULL;
 		}
@@ -760,14 +760,10 @@ solver_lp_copy (const SolverParameters *src_param, Sheet *new_sheet)
 
 	/* Copy the input cell list */
 	for (inputs = src_param->input_cells; inputs ; inputs = inputs->next) {
-		GnmCell *cell = inputs->data;
-		GnmCell *new_cell;
-
-		new_cell = cell_copy (cell);
+		GnmCell *new_cell = cell_dup (inputs->data);
 		new_cell->base.sheet = new_sheet;
-		dst_param->input_cells =
-		        g_slist_prepend (dst_param->input_cells,
-					 (gpointer) new_cell);
+		dst_param->input_cells = g_slist_prepend (
+			dst_param->input_cells, (gpointer) new_cell);
 	}
 	dst_param->input_cells = g_slist_reverse (dst_param->input_cells);
 
