@@ -40,12 +40,8 @@ value_dump (GnmValue const *value)
 		printf ("STRING: %s\n", value->v_str.val->str);
 		break;
 
-	case VALUE_INTEGER:
-		printf ("NUM: %d\n", value->v_int.val);
-		break;
-
 	case VALUE_FLOAT:
-		printf ("Float: %" GNM_FORMAT_f "\n", value->v_float.val);
+		printf ("NUMBER: %" GNM_FORMAT_f "\n", value_get_as_float (value));
 		break;
 
 	case VALUE_ARRAY: {
@@ -134,20 +130,40 @@ value_area_get_height (GnmValue const *v, GnmEvalPos const *ep)
 	return 1;
 }
 
+/**
+ * value_area_fetch_x_y:
+ * @v : const #GnmValue *
+ * @x : column
+ * @y : row
+ * @ep : const #GnmEvalPos *
+ * 
+ * An internal routine to get a cell from an array or range.
+ * Ensures that elements of CELLRANGE are evaluated
+ *
+ * Returns the element if it exists and is non-empty otherwise returns 0
+ **/
 GnmValue const *
 value_area_fetch_x_y (GnmValue const *v, int x, int y, GnmEvalPos const *ep)
 {
 	GnmValue const * const res = value_area_get_x_y (v, x, y, ep);
-	if (res && res->type != VALUE_EMPTY)
+	if (VALUE_IS_EMPTY (res))
+		return value_zero;
+	else
 		return res;
-
-	return value_zero;
 }
 
-/*
- * An internal routine to get a cell from an array or range.  If any
- * problems occur a NULL is returned.
- */
+/**
+ * value_area_get_x_y:
+ * @v : const #GnmValue *
+ * @x : column
+ * @y : row
+ * @ep : const #GnmEvalPos *
+ * 
+ * An internal routine to get a cell from an array or range.
+ * Ensures that elements of CELLRANGE are evaluated
+ *
+ * If any problems occur a NULL is returned.
+ **/
 GnmValue const *
 value_area_get_x_y (GnmValue const *v, int x, int y, GnmEvalPos const *ep)
 {
