@@ -4,6 +4,7 @@
  * wbc-gtk.c: A raw gtk based WorkbookControl
  *
  * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2006 Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -49,24 +50,13 @@
 #include <goffice/utils/go-file.h>
 #include <goffice/app/go-cmd-context-impl.h>
 #include <gsf/gsf-impl-utils.h>
-#include <gtk/gtkactiongroup.h>
-#include <gtk/gtkuimanager.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkaccellabel.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtktoggleaction.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkhandlebox.h>
-#include <gtk/gtkcheckmenuitem.h>
+#include <gtk/gtk.h>
 #include "gdk/gdkkeysyms.h"
 #include <glib/gi18n.h>
 #include <errno.h>
 #include <string.h>
 
 #ifdef USE_HILDON
-	#include <gtk/gtktable.h>
-	#include <gtk/gtktoolbutton.h>
 	#include <goffice/gtk/go-combo-text.h>
 	#include <hildon-lgpl/hildon-widgets/hildon-app.h>
 	#include <hildon-lgpl/hildon-widgets/hildon-appview.h>
@@ -85,7 +75,8 @@ struct _WBCgtk {
 		guint		  merge_id;
 	} file_history, toolbar, windows;
 
-	GOActionComboStack	*undo_action, *redo_action;
+	GOActionComboStack	*undo_haction, *redo_haction;
+	GtkAction		*undo_vaction, *redo_vaction;
 	GOActionComboColor	*fore_color, *back_color;
 	GOActionComboText	*font_name, *font_size, *zoom;
 	GOActionComboPixmaps	*borders, *halignment, *valignment;
@@ -100,7 +91,7 @@ struct _WBCgtk {
 		GtkToggleAction	 *top, *center, *bottom;
 	} v_align;
 
-	GtkWidget *menu_zone, *toolbar_zone, *everything;
+	GtkWidget *menu_zone, *everything, *toolbar_zones[4];
 	GHashTable *custom_uis;
 };
 typedef WorkbookControlGUIClass WBCgtkClass;
@@ -182,6 +173,9 @@ wbc_gtk_init_zoom (WBCgtk *gtk)
 	gtk->zoom = g_object_new (go_action_combo_text_get_type (),
 				  "name", "Zoom",
 				  "label", _("_Zoom"),
+				  "visible-vertical", FALSE,
+				  "tooltip", _("Zoom"),
+				  "stock-id", GTK_STOCK_ZOOM_IN,
 				  NULL);
 #ifdef USE_HILDON
 	go_action_combo_text_set_width (gtk->zoom,  "100000000%");
@@ -323,7 +317,9 @@ wbc_gtk_init_borders (WBCgtk *gtk)
 	g_object_set (G_OBJECT (gtk->borders),
 		      "label", _("Borders"),
 		      "tooltip", _("Borders"),
+		      "visible-vertical", FALSE,
 		      NULL);
+	/* TODO: Create vertical version.  */
 #if 0
 	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Foreground"));
 	go_combo_pixmaps_select (gtk->borders, 1); /* default to none */
@@ -373,11 +369,11 @@ wbc_gtk_init_alignments (WBCgtk *gtk)
 	gtk->halignment = go_action_combo_pixmaps_new ("HAlignmentSelector",
 						       halignment_combo_info, 3, 1);
 	g_object_set (G_OBJECT (gtk->halignment),
-		      "label", _("Horzontal Alignment"),
-		      "tooltip", _("Horzontal Alignment"),
+		      "label", _("Horizontal Alignment"),
+		      "tooltip", _("Horizontal Alignment"),
 		      NULL);
 #if 0
-	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horzontal Alignment"));
+	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horizontal Alignment"));
 	go_combo_pixmaps_select (gtk->halignment, 1); /* default to none */
 #endif
 	g_signal_connect (G_OBJECT (gtk->halignment),
@@ -392,7 +388,7 @@ wbc_gtk_init_alignments (WBCgtk *gtk)
 		      "tooltip", _("Vertical Alignment"),
 		      NULL);
 #if 0
-	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horzontal Alignment"));
+	gnm_combo_box_set_title (GO_COMBO_BOX (fore_combo), _("Horizontal Alignment"));
 	go_combo_pixmaps_select (gtk->valignment, 1); /* default to none */
 #endif
 	g_signal_connect (G_OBJECT (gtk->valignment),
@@ -407,7 +403,7 @@ static GOActionComboStack *
 ur_stack (WorkbookControl *wbc, gboolean is_undo)
 {
 	WBCgtk *gtk = (WBCgtk *)wbc;
-	return is_undo ? gtk->undo_action : gtk->redo_action;
+	return is_undo ? gtk->undo_haction : gtk->redo_haction;
 }
 
 static void
@@ -429,45 +425,6 @@ wbc_gtk_undo_redo_push (WorkbookControl *wbc, gboolean is_undo,
 	go_action_combo_stack_push (ur_stack (wbc, is_undo), text, key);
 }
 
-static GOActionComboStack *
-create_undo_redo (WBCgtk *gtk, char const *name, char const *tooltip,
-		  char const *stock_id, char const *accel)
-{
-	GOActionComboStack *res = g_object_new (go_action_combo_stack_get_type (),
-		"name",		name,
-		"tooltip",  	_(tooltip),
-		"stock-id",	stock_id,
-		"sensitive",	FALSE,
-		NULL);
-	
-#ifdef USE_HILDON
-	gtk_action_group_add_action (gtk->actions, GTK_ACTION (res));
-#else
-	gtk_action_group_add_action_with_accel (gtk->actions,
-		GTK_ACTION (res), accel);
-#endif
-	return res;
-}
-
-
-static void
-cb_undo_activated (GOActionComboStack *a, WorkbookControl *wbc)
-{
-	unsigned n = workbook_find_command (wb_control_workbook (wbc), TRUE,
-		go_action_combo_stack_selection (a));
-	while (n-- > 0)
-		command_undo (wbc);
-}
-
-static void
-cb_redo_activated (GOActionComboStack *a, WorkbookControl *wbc)
-{
-	unsigned n = workbook_find_command (wb_control_workbook (wbc), FALSE,
-		go_action_combo_stack_selection (a));
-	while (n-- > 0)
-		command_redo (wbc);
-}
-
 static void
 cb_chain_sensitivity (GtkAction *src, G_GNUC_UNUSED GParamSpec *pspec,
 		      GtkAction *action)
@@ -486,24 +443,89 @@ cb_chain_sensitivity (GtkAction *src, G_GNUC_UNUSED GParamSpec *pspec,
 	g_object_set (action, "sensitive", new_val, NULL);
 }
 
+static void
+create_undo_redo (GOActionComboStack **haction, char const *hname,
+		  GCallback hcb, 
+		  GtkAction **vaction, char const *vname,
+		  GCallback vcb, 
+		  WBCgtk *gtk,
+		  char const *tooltip,
+		  char const *stock_id, char const *accel)
+{
+	*haction = g_object_new
+		(go_action_combo_stack_get_type (),
+		 "name", hname,
+		 "tooltip", tooltip,
+		 "stock-id", stock_id,
+		 "sensitive", FALSE,
+		 "visible-vertical", FALSE,
+		 NULL);
+#ifdef USE_HILDON
+	gtk_action_group_add_action (gtk->actions, GTK_ACTION (*haction));
+#else
+	gtk_action_group_add_action_with_accel (gtk->actions,
+		GTK_ACTION (*haction), accel);
+#endif
+	g_signal_connect (G_OBJECT (*haction), "activate", hcb, gtk);
+
+	*vaction = gtk_action_new (vname, NULL, tooltip, stock_id);
+	g_object_set (G_OBJECT (*vaction),
+		      "sensitive", FALSE,
+		      "visible-horizontal", FALSE,
+		      NULL);
+	gtk_action_group_add_action (gtk->actions, GTK_ACTION (*vaction));
+	g_signal_connect_swapped (G_OBJECT (*vaction), "activate", vcb, gtk);
+
+	g_signal_connect (G_OBJECT (*haction),
+			  "notify::sensitive",
+			  G_CALLBACK (cb_chain_sensitivity),
+			  *vaction);
+}
+
+
+static void
+cb_undo_activated (GOActionComboStack *a, WorkbookControl *wbc)
+{
+	unsigned n = workbook_find_command (wb_control_get_workbook (wbc), TRUE,
+		go_action_combo_stack_selection (a));
+	while (n-- > 0)
+		command_undo (wbc);
+}
+
+static void
+cb_redo_activated (GOActionComboStack *a, WorkbookControl *wbc)
+{
+	unsigned n = workbook_find_command (wb_control_get_workbook (wbc), FALSE,
+		go_action_combo_stack_selection (a));
+	while (n-- > 0)
+		command_redo (wbc);
+}
+
 static void
 wbc_gtk_init_undo_redo (WBCgtk *gtk)
 {
-	gtk->undo_action = create_undo_redo (gtk, N_("Undo"),
-		N_("Undo the last action"), GTK_STOCK_UNDO, "<control>z");
-	g_signal_connect (G_OBJECT (gtk->undo_action),
-		"activate",
-		G_CALLBACK (cb_undo_activated), gtk);
-	g_signal_connect (G_OBJECT (gtk->undo_action),
-		"notify::sensitive",
-		G_CALLBACK (cb_chain_sensitivity),
-		gtk_action_group_get_action (gtk->permanent_actions, "Repeat"));
+	GtkAction *repeat =
+		gtk_action_group_get_action (gtk->permanent_actions, "Repeat");
 
-	gtk->redo_action = create_undo_redo (gtk, N_("Redo"),
-		N_("Redo the undone action"), GTK_STOCK_REDO, "<control>y");
-	g_signal_connect (G_OBJECT (gtk->redo_action),
-		"activate",
-		G_CALLBACK (cb_redo_activated), gtk);
+	create_undo_redo (&gtk->undo_haction, "Undo",
+			  G_CALLBACK (cb_undo_activated),
+			  &gtk->undo_vaction, "VUndo",
+			  G_CALLBACK (command_undo),
+			  gtk,
+			  _("Undo the last action"),
+			  GTK_STOCK_UNDO, "<control>z");
+	g_signal_connect (G_OBJECT (gtk->undo_haction),
+			  "notify::sensitive",
+			  G_CALLBACK (cb_chain_sensitivity),
+			  repeat);
+
+	create_undo_redo (&gtk->redo_haction, "Redo",
+			  G_CALLBACK (cb_redo_activated),
+			  &gtk->redo_vaction, "VRedo",
+			  G_CALLBACK (command_redo),
+			  gtk,
+			  _("Redo the undone action"),
+			  GTK_STOCK_REDO, "<control>y");
 }
 
 /****************************************************************************/
@@ -557,7 +579,9 @@ wbc_gtk_init_color_fore (WBCgtk *gtk)
 	g_object_set (G_OBJECT (gtk->fore_color),
 		      "label", _("Foreground"),
 		      "tooltip", _("Foreground"),
+		      "visible-vertical", FALSE,
 		      NULL);
+	/* TODO: Create vertical version.  */
 	g_signal_connect (G_OBJECT (gtk->fore_color),
 		"activate",
 		G_CALLBACK (cb_fore_color_changed), gtk);
@@ -606,7 +630,9 @@ wbc_gtk_init_color_back (WBCgtk *gtk)
 	g_object_set (G_OBJECT (gtk->back_color),
 		      "label", _("Background"),
 		      "tooltip", _("Background"),
+		      "visible-vertical", FALSE,
 		      NULL);
+	/* TODO: Create vertical version.  */
 	g_object_connect (G_OBJECT (gtk->back_color),
 		"signal::activate", G_CALLBACK (cb_back_color_changed), gtk,
 		"signal::display-custom-dialog", G_CALLBACK (cb_custom_color_created), gtk,
@@ -653,8 +679,11 @@ wbc_gtk_init_font_name (WBCgtk *gtk)
 				       "case-sensitive", FALSE,
 				       "stock-id", GTK_STOCK_SELECT_FONT,
 				       "visible-vertical", FALSE,
+				       "tooltip", _("Font"),
 				       NULL);
 
+	/* TODO: Create vertical version of this.  */
+
 	context = gtk_widget_get_pango_context
 		(GTK_WIDGET (wbcg_toplevel (WORKBOOK_CONTROL_GUI (gtk))));
 	families = go_fonts_list_families (context);
@@ -710,12 +739,12 @@ wbc_gtk_init_font_size (WBCgtk *gtk)
 				       "stock-id", GTK_STOCK_SELECT_FONT,
 				       "visible-vertical", FALSE,
 				       "label", _("Font Size"),
+				       "tooltip", _("Font Size"),
 				       NULL);
 
 	/* TODO: Create vertical version of this.  */
 
 	font_sizes = go_fonts_list_sizes ();
-
 	for (ptr = font_sizes; ptr != NULL ; ptr = ptr->next) {
 		int psize = GPOINTER_TO_INT (ptr->data);
 		char *size_text = g_strdup_printf ("%g", psize / (double)PANGO_SCALE);
@@ -1100,7 +1129,7 @@ regenerate_window_menu (WBCgtk *gtk, Workbook *wb, unsigned i)
 		if (i >= 20)
 			return i;
 		if (IS_WORKBOOK_CONTROL_GUI (wbc) &&
-			(basename = go_basename_from_uri (wb->uri)) != NULL) {
+			(basename = go_basename_from_uri (GO_DOC (wb)->uri)) != NULL) {
 			GString *label = g_string_new (NULL);
 			char *name;
 			const char *s;
@@ -1139,10 +1168,14 @@ regenerate_window_menu (WBCgtk *gtk, Workbook *wb, unsigned i)
 static void
 cb_regenerate_window_menu (WBCgtk *gtk)
 {
-	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (gtk));
+	Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (gtk));
 	GList const *ptr;
 	unsigned i;
 
+	/* This can happen during exit.  */
+	if (!wb)
+		return;
+
 	if (gtk->windows.merge_id != 0)
 		gtk_ui_manager_remove_ui (gtk->ui, gtk->windows.merge_id);
 	gtk->windows.merge_id = gtk_ui_manager_new_merge_id (gtk->ui);
@@ -1241,6 +1274,158 @@ cb_init_extra_ui (GnmAppExtraUI *extra_ui, WBCgtk *gtk)
 /****************************************************************************/
 /* Toolbar menu */
 
+static void
+set_toolbar_style_for_position (GtkToolbar *tb, GtkPositionType pos)
+{
+	GtkHandleBox *hdlbox = GTK_HANDLE_BOX (GTK_WIDGET (tb)->parent);
+
+	static const GtkPositionType hdlpos[] = {
+		GTK_POS_TOP, GTK_POS_TOP,
+		GTK_POS_LEFT, GTK_POS_LEFT
+	};
+	static const GtkOrientation orientations[] = {
+		GTK_ORIENTATION_VERTICAL, GTK_ORIENTATION_VERTICAL,
+		GTK_ORIENTATION_HORIZONTAL, GTK_ORIENTATION_HORIZONTAL
+	};
+
+	gtk_toolbar_set_orientation (tb, orientations[pos]);
+	gtk_handle_box_set_handle_position (hdlbox, hdlpos[pos]);
+}
+
+static void
+set_toolbar_position (GtkToolbar *tb, GtkPositionType pos, WBCgtk *gtk)
+{
+	GtkWidget *hdlbox = GTK_WIDGET (tb)->parent;
+	GtkContainer *zone = GTK_CONTAINER (GTK_WIDGET (hdlbox)->parent);
+	GtkContainer *new_zone = GTK_CONTAINER (gtk->toolbar_zones[pos]);
+	const char *name = g_object_get_data (G_OBJECT (hdlbox), "name");
+
+	if (zone == new_zone)
+		return;
+
+	g_object_ref (hdlbox);
+	if (zone)
+		gtk_container_remove (zone, hdlbox);
+	set_toolbar_style_for_position (tb, pos);
+	gtk_container_add (new_zone, hdlbox);
+	g_object_unref (hdlbox);
+
+	if (zone)
+		gnm_gconf_set_toolbar_position (name, pos);
+}
+
+static void
+cb_set_toolbar_position (GtkMenuItem *item, WBCgtk *gtk)
+{
+	GtkToolbar *tb = g_object_get_data (G_OBJECT (item), "toolbar");
+	GtkPositionType side = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "side"));
+
+	set_toolbar_position (tb, side, gtk);
+}
+
+static void
+cb_tcm_reattach (GtkWidget *widget, GtkHandleBox *hdlbox)
+{
+	GdkEvent *event = gdk_event_new (GDK_DELETE);
+	event->any.type = GDK_DELETE;
+	event->any.window = g_object_ref (hdlbox->float_window);
+	event->any.send_event = TRUE;
+	gtk_main_do_event (event);
+	gdk_event_free (event);
+}
+
+static void
+cb_tcm_hide (GtkWidget *widget, GtkHandleBox *hdlbox)
+{
+	if (hdlbox->child_detached)
+		cb_tcm_reattach (widget, hdlbox);
+	gtk_widget_hide (GTK_WIDGET (hdlbox));
+}
+
+static void
+toolbar_context_menu (GtkToolbar *tb, WBCgtk *gtk, GdkEventButton *event_button)
+{
+	GtkHandleBox *hdlbox = GTK_HANDLE_BOX (GTK_WIDGET (tb)->parent);
+	GtkWidget *zone = GTK_WIDGET (hdlbox)->parent;
+	GtkWidget *menu = gtk_menu_new ();
+	GtkWidget *item;
+
+	static const struct {
+		const char *text;
+		GtkPositionType pos;
+	} pos_items[] = {
+		{ N_("Display above sheets"), GTK_POS_TOP },
+		{ N_("Display to the left of sheets"), GTK_POS_LEFT },
+		{ N_("Display to the right of sheets"), GTK_POS_RIGHT }
+	};
+
+	if (hdlbox->child_detached) {
+		item = gtk_menu_item_new_with_label (_("Reattach to main window"));
+		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+		g_signal_connect (G_OBJECT (item), "activate",
+				  G_CALLBACK (cb_tcm_reattach),
+				  hdlbox);
+	} else {
+		size_t ui;
+		GSList *group = NULL;
+
+		for (ui = 0; ui < G_N_ELEMENTS (pos_items); ui++) {
+			const char *text = _(pos_items[ui].text);
+			GtkPositionType pos = pos_items[ui].pos;
+
+			item = gtk_radio_menu_item_new_with_label (group, text);
+			group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (item));
+
+			GTK_CHECK_MENU_ITEM (item)->active =
+				(zone == gtk->toolbar_zones[pos]);
+
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			g_object_set_data (G_OBJECT (item), "toolbar", tb);
+			g_object_set_data (G_OBJECT (item), "side", GINT_TO_POINTER (pos));
+			g_signal_connect (G_OBJECT (item), "activate",
+					  G_CALLBACK (cb_set_toolbar_position),
+					  gtk);
+		}
+	}
+
+	item = gtk_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	gtk_widget_set_sensitive (item, FALSE);
+
+	item = gtk_menu_item_new_with_label (_("Hide"));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	g_signal_connect (G_OBJECT (item), "activate",
+			  G_CALLBACK (cb_tcm_hide),
+			  hdlbox);
+
+	gtk_widget_show_all (menu);
+	gnumeric_popup_menu (GTK_MENU (menu), event_button);
+}
+
+static gboolean
+cb_toolbar_button_press (GtkToolbar *tb, GdkEventButton *event, WBCgtk *gtk)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		toolbar_context_menu (tb, gtk, event);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static gboolean
+cb_handlebox_button_press (GtkHandleBox *hdlbox, GdkEventButton *event, WBCgtk *gtk)
+{
+	if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
+		GtkToolbar *tb = GTK_TOOLBAR (gtk_bin_get_child (GTK_BIN (hdlbox)));
+		toolbar_context_menu (tb, gtk, event);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+
 static void
 cb_toolbar_activate (GtkToggleAction *action, WorkbookControlGUI *wbcg)
 {
@@ -1281,6 +1466,16 @@ cb_add_menus_toolbars (G_GNUC_UNUSED GtkUIManager *ui,
 		char *toggle_name = g_strdup_printf ("ViewMenuToolbar%s", name);
 		char *tooltip = g_strdup_printf (_("Show/Hide toolbar %s"), _(name));
 		gboolean visible = gnm_gconf_get_toolbar_visible (name);
+		GtkPositionType pos = gnm_gconf_get_toolbar_position (name);
+
+		g_signal_connect (G_OBJECT (w),
+				  "button_press_event",
+				  G_CALLBACK (cb_toolbar_button_press),
+				  gtk);
+		g_signal_connect (G_OBJECT (box),
+				  "button_press_event",
+				  G_CALLBACK (cb_handlebox_button_press),
+				  gtk);
 
 		gtk_container_add (GTK_CONTAINER (box), w);
 		gtk_toolbar_set_show_arrow (GTK_TOOLBAR (w), TRUE);
@@ -1288,7 +1483,8 @@ cb_add_menus_toolbars (G_GNUC_UNUSED GtkUIManager *ui,
 		gtk_widget_show_all (box);
 		if (!visible)
 			gtk_widget_hide (box);
-		gtk_box_pack_start (GTK_BOX (gtk->toolbar_zone), box, FALSE, FALSE, 0);
+		set_toolbar_position (GTK_TOOLBAR (w), pos, gtk);
+
 		g_object_connect (box,
 			"signal::notify::visible", G_CALLBACK (cb_handlebox_visible), wbcg,
 			"signal::child_attached", G_CALLBACK (cb_handlebox_dock_status), GINT_TO_POINTER (TRUE),
@@ -1444,11 +1640,16 @@ wbc_gtk_init (GObject *obj)
 	char		   *uifile;
 	unsigned	    i;
 	GError *error = NULL;
+	GtkWidget *hbox;
 
 	gtk->menu_zone = gtk_vbox_new (TRUE, 0);
-	gtk->toolbar_zone = gtk_vbox_new (FALSE, 0);
 	gtk->everything = gtk_vbox_new (FALSE, 0);
 
+	gtk->toolbar_zones[GTK_POS_TOP] = gtk_vbox_new (FALSE, 0);
+	gtk->toolbar_zones[GTK_POS_BOTTOM] = NULL;
+	gtk->toolbar_zones[GTK_POS_LEFT] = gtk_hbox_new (FALSE, 0);
+	gtk->toolbar_zones[GTK_POS_RIGHT] = gtk_hbox_new (FALSE, 0);
+
 #ifdef USE_HILDON
 	{ GtkWidget *appview = hildon_appview_new ("Gnumeric");
 	wbcg->hildon_app = hildon_app_new_with_appview (HILDON_APPVIEW (appview));
@@ -1461,7 +1662,7 @@ wbc_gtk_init (GObject *obj)
 	gtk_box_pack_start (GTK_BOX (gtk->everything),
 		gtk->menu_zone, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (gtk->everything),
-		gtk->toolbar_zone, FALSE, TRUE, 0);
+		gtk->toolbar_zones[GTK_POS_TOP], FALSE, TRUE, 0);
 #endif
 
 	gtk_window_set_title (wbcg_toplevel (wbcg), "Gnumeric");
@@ -1473,8 +1674,13 @@ wbc_gtk_init (GObject *obj)
 #if 0
 	bonobo_dock_set_client_area (BONOBO_DOCK (gtk->dock), wbcg->table);
 #endif
-	gtk_box_pack_start (GTK_BOX (gtk->everything),
-		wbcg->table, TRUE, TRUE, 0);
+
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk->toolbar_zones[GTK_POS_LEFT], FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), wbcg->table, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk->toolbar_zones[GTK_POS_RIGHT], FALSE, TRUE, 0);
+
+	gtk_box_pack_start (GTK_BOX (gtk->everything), hbox, TRUE, TRUE, 0);
 	gtk_widget_show_all (gtk->everything);
 
 #warning "TODO split into smaller chunks"
@@ -1567,6 +1773,14 @@ wbc_gtk_init (GObject *obj)
 
 	gtk_container_add (GTK_CONTAINER (wbcg->toplevel), gtk->everything);
 
+	/*
+	 * The following line updates the menus before the check so we
+	 * avoid problems like #324692.  We could move it inside the
+	 * conditional, but I don't like doing active things like this
+	 * in there.
+	 */
+	wb_control_undo_redo_labels (WORKBOOK_CONTROL (wbcg), "", "");
+
 #ifdef CHECK_MENU_UNDERLINES
 	gtk_container_foreach (GTK_CONTAINER (gtk->menu_zone),
 			       (GtkCallback)check_underlines,
