@@ -3,7 +3,7 @@
 /*
  * wbcg-actions.c: The callbacks and tables for all the menus and stock toolbars
  *
- * Copyright (C) 2003-2005 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -95,10 +95,11 @@ static GNM_ACTION_DEF (cb_file_new)
 		(gnm_app_prefs->initial_sheet_number);
 
 #ifdef USE_HILDON
-	WorkbookControl * wbc = (WorkbookControl *) wbcg;
-	Workbook *tmp_wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	WorkbookControl * wbc = WORKBOOK_CONTROL (wbcg);
+	Workbook *tmp_wb = wb_control_get_workbook (wbc);
 
-	if (workbook_is_dirty (tmp_wb)) {
+	/* Ask if the user wants to save the current workbook */
+	if (go_doc_is_dirty (GO_DOC (tmp_wb))) {
 		switch (wbcg_show_save_dialog (wbcg, tmp_wb, FALSE)) {
 
 			case GTK_RESPONSE_YES:
@@ -116,8 +117,9 @@ static GNM_ACTION_DEF (cb_file_new)
 		}
 	}
 
+	/* Show the new workbook in the same window */
 	g_object_ref (wbcg);
-	g_object_unref (wb_control_workbook (WORKBOOK_CONTROL (wbcg)));
+	g_object_unref (tmp_wb);
 	wb_control_set_view (wbc, NULL, wb);
 	wb_control_init_state (wbc);
 	sheet_update (wb_view_cur_sheet (wb_control_view (wbc)));
@@ -138,7 +140,7 @@ static GNM_ACTION_DEF (cb_file_print)	{
 	sheet_print (wbcg, wbcg_cur_sheet (wbcg), FALSE, PRINT_ACTIVE_SHEET); }
 static GNM_ACTION_DEF (cb_file_print_preview) {
 	sheet_print (wbcg, wbcg_cur_sheet (wbcg), TRUE, PRINT_ACTIVE_SHEET); }
-static GNM_ACTION_DEF (cb_file_summary)		{ dialog_summary_update (wbcg, TRUE); }
+static GNM_ACTION_DEF (cb_doc_meta_data)	{ dialog_doc_metadata_new (wbcg); }
 static GNM_ACTION_DEF (cb_file_preferences)	{ dialog_preferences (wbcg, 0); }
 static GNM_ACTION_DEF (cb_file_close)		{ wbcg_close_control (wbcg); }
 
@@ -170,7 +172,7 @@ static GNM_ACTION_DEF (cb_file_quit)
 		g_return_if_fail (IS_WORKBOOK (wb));
 		g_return_if_fail (wb->wb_views != NULL);
 
-		if (wb_control_workbook (wbc) == wb)
+		if (wb_control_get_workbook (wbc) == wb)
 			continue;
 		if (discard_all) {
 
@@ -191,25 +193,24 @@ static GNM_ACTION_DEF (cb_file_quit)
 				discard_all = TRUE;
 				old_ptr = ptr;
 				for (ptr = ptr->next; ptr != NULL ; ptr = ptr->next) {
-					Workbook *wba = ptr->data;
+					GODoc *wb = ptr->data;
 					old_ptr = ptr;
-					if (wb_control_workbook (wbc) == wba)
+					if (wb_control_get_doc (wbc) == wb)
 						continue;
-					workbook_set_dirty (wba, FALSE);
-					g_object_unref (wba);
+					go_doc_set_dirty (wb, FALSE);
+					g_object_unref (wb);
 				}
 				ptr = old_ptr;
 				break;
-			};
+			}
 		}
 	}
 
 	if (discard_all) {
-		workbook_set_dirty (wb_control_workbook (wbc), FALSE);
-		g_object_unref (wb_control_workbook (wbc));
-		for (ptr = clean_no_closed; ptr != NULL ; ptr = ptr->next) {
+		go_doc_set_dirty (wb_control_get_doc (wbc), FALSE);
+		g_object_unref (wb_control_get_doc (wbc));
+		for (ptr = clean_no_closed; ptr != NULL ; ptr = ptr->next)
 			g_object_unref (ptr->data);
-		}
 	} else if (ok && wbcg_close_if_user_permits (wbcg, wb_control_view (wbc),
 						     TRUE, TRUE, ask_user) > 0)
 		/* only close pristine books if nothing was canceled. */
@@ -219,11 +220,6 @@ static GNM_ACTION_DEF (cb_file_quit)
 	g_list_free (workbooks);
 	g_list_free (clean_no_closed);
 }
-static GNM_ACTION_DEF (cb_new_doc_metadata)
-{
-	dialog_doc_metadata_new (wbcg);
-}
-
 /****************************************************************************/
 
 static GNM_ACTION_DEF (cb_edit_clear_all)
@@ -345,16 +341,9 @@ static GNM_ACTION_DEF (cb_edit_paste_special)
 
 static GNM_ACTION_DEF (cb_sheet_remove)
 {
-	SheetControlGUI *res;
-	if (wbcg_sheet_to_page_index (wbcg, wbcg_cur_sheet (wbcg), &res) >= 0)
-		scg_delete_sheet_if_possible (NULL, res);
-}
-
-static GNM_ACTION_DEF (cb_edit_duplicate_sheet)
-{
-	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
-	Sheet *old_sheet = wb_control_cur_sheet (wbc);
-	cmd_clone_sheet (wbc, old_sheet);
+	SheetControlGUI *scg = wbcg_cur_scg (wbcg);
+	if (scg)
+		scg_delete_sheet_if_possible (NULL, scg);
 }
 
 #ifdef USE_HILDON
@@ -378,7 +367,7 @@ common_cell_goto (WorkbookControlGUI *wbcg, Sheet *sheet, GnmCellPos const *pos)
 static int
 cb_edit_search_replace_query (SearchReplaceQuery q, GnmSearchReplace *sr, ...)
 {
-	int res = 0;
+	int res;
 	va_list pvar;
 	WorkbookControlGUI *wbcg = sr->user_data;
 
@@ -389,10 +378,7 @@ cb_edit_search_replace_query (SearchReplaceQuery q, GnmSearchReplace *sr, ...)
 		GnmCell *cell = va_arg (pvar, GnmCell *);
 		char const *old_text = va_arg (pvar, const char *);
 		char const *new_text = va_arg (pvar, const char *);
-
-		char *err;
-
-		err = g_strdup_printf
+		char *err = g_strdup_printf
 			(_("In cell %s, the current contents\n"
 			   "        %s\n"
 			   "would have been replaced by\n"
@@ -406,6 +392,7 @@ cb_edit_search_replace_query (SearchReplaceQuery q, GnmSearchReplace *sr, ...)
 
 		go_gtk_notice_dialog (wbcg_toplevel (wbcg), GTK_MESSAGE_ERROR, err);
 		g_free (err);
+		res = GTK_RESPONSE_NO;
 		break;
 	}
 
@@ -441,6 +428,9 @@ cb_edit_search_replace_query (SearchReplaceQuery q, GnmSearchReplace *sr, ...)
 		break;
 	}
 
+	default:
+		/* Shouldn't really happen.  */
+		res = GTK_RESPONSE_CANCEL;
 	}
 
 	va_end (pvar);
@@ -529,7 +519,7 @@ static GNM_ACTION_DEF (cb_edit_recalc)
 	 * ctrl-alt-f9 -  force a full recalc across all sheets
 	 * ctrl-alt-shift-f9  -  a full-monty super recalc
 	 */
-	workbook_recalc_all (wb_control_workbook (WORKBOOK_CONTROL (wbcg)));
+	workbook_recalc_all (wb_control_get_workbook (WORKBOOK_CONTROL (wbcg)));
 }
 
 static GNM_ACTION_DEF (cb_repeat)	{ command_repeat (WORKBOOK_CONTROL (wbcg)); }
@@ -635,7 +625,7 @@ static GNM_ACTION_DEF (cb_view_freeze_panes)
 static GNM_ACTION_DEF (cb_insert_current_date_time)
 {
 	if (wbcg_edit_start (wbcg, FALSE, FALSE)) {
-		Workbook const *wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+		Workbook const *wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 		GnmValue *v = value_new_float (
 			datetime_timet_to_serial_raw (time (NULL), workbook_date_conv (wb)));
 		char *txt = format_value (go_format_default_date_time (), v, NULL, -1,
@@ -648,7 +638,7 @@ static GNM_ACTION_DEF (cb_insert_current_date_time)
 static GNM_ACTION_DEF (cb_insert_current_date)
 {
 	if (wbcg_edit_start (wbcg, FALSE, FALSE)) {
-		Workbook const *wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+		Workbook const *wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 		GnmValue *v = value_new_int (
 			datetime_timet_to_serial (time (NULL), workbook_date_conv (wb)));
 		char *txt = format_value (go_format_default_date (), v, NULL, -1,
@@ -662,7 +652,7 @@ static GNM_ACTION_DEF (cb_insert_current_date)
 static GNM_ACTION_DEF (cb_insert_current_time)
 {
 	if (wbcg_edit_start (wbcg, FALSE, FALSE)) {
-		Workbook const *wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+		Workbook const *wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 		GnmValue *v = value_new_float (
 			datetime_timet_to_seconds (time (NULL)) / (24.0 * 60 * 60));
 		char *txt = format_value (go_format_default_time (), v, NULL, -1,
@@ -808,6 +798,7 @@ static GNM_ACTION_DEF (cb_auto_filter)
 		gnm_filter_remove (filter);
 		gnm_filter_free (filter);
 	}
+	sheet_mark_dirty (sv->sheet);
 	sheet_update (sv->sheet);
 }
 
@@ -906,24 +897,25 @@ static GNM_ACTION_DEF (cb_help_docs)
 	char   *argv[] = { NULL, NULL, NULL };
 	GError *error = NULL;
 
-#warning "handle translations when we generate them"
 #ifndef G_OS_WIN32
 	argv[0] = (char *)"yelp";
-	argv[1] = g_build_filename (gnm_sys_data_dir (), "doc", "C",
-			"gnumeric.xml", NULL);
+	argv[1] = (char *)"help:gnumeric";
+	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+		NULL, NULL, NULL, &error);
 #else
+	/* TODO : handle translations when we generate them */
 	argv[0] = (char *)"hh";
 	argv[1] = g_build_filename (gnm_sys_data_dir (), "doc", "C",
 			"gnumeric.chm", NULL);
-#endif
 	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
 		NULL, NULL, NULL, &error);
 	g_free (argv[1]);
+#endif
 }
 
 static GNM_ACTION_DEF (cb_help_web)
 {
-	GError *err = go_url_show ("http://www.gnome.org/projects/gnumeric/");
+	GError *err = go_url_show ("http://www.gnumeric.org/");
 	if (err != NULL) {
 		go_cmd_context_error (GO_CMD_CONTEXT (wbcg), err);
 		g_error_free (err);
@@ -1022,6 +1014,12 @@ sort_by_rows (WorkbookControlGUI *wbcg, gboolean descending)
 	sel = range_dup (tmp);
 	range_clip_to_finite (sel, sv_sheet (sv));
 
+	if (gnm_app_prefs->sort_default_has_header) {
+		sel->start.row++;
+		if (sel->start.row > sel->end.row)
+			return;
+	}
+
 	numclause = range_width (sel);
 	clause = g_new0 (GnmSortClause, numclause);
 	for (i=0; i < numclause; i++) {
@@ -1162,7 +1160,7 @@ wbcg_set_selection_valign (WorkbookControlGUI *wbcg, GnmVAlign valign)
 	 * then revert to general */
 	wb_view = wb_control_view (wbc);
 	style = wb_view->current_format;
-	if (gnm_style_get_align_h (style) == valign) {
+	if (gnm_style_get_align_v (style) == valign) {
 		if (valign == VALIGN_BOTTOM)
 			return;
 		valign = VALIGN_BOTTOM;
@@ -1261,7 +1259,7 @@ toggle_font_attr (WorkbookControlGUI *wbcg, GtkToggleAction *act,
 	case MSTYLE_FONT_UNDERLINE:	gnm_style_set_font_uline (new_style, val); break;
 	case MSTYLE_FONT_STRIKETHROUGH: gnm_style_set_font_strike (new_style, val); break;
 	case MSTYLE_FONT_SCRIPT:	gnm_style_set_font_script (new_style, val); break;
-	};
+	}
 
 	cmd_selection_format (wbc, new_style, NULL, _("Set Font Style"));
 }
@@ -1567,6 +1565,16 @@ static GtkActionEntry const permanent_actions[] = {
 		G_CALLBACK (cb_repeat) }
 };
 
+#ifdef USE_HILDON
+#define FULLSCREEN_ACCEL "F6"
+#define ZOOM_IN_ACCEL "F7"
+#define ZOOM_OUT_ACCEL "F8"
+#else
+#define FULLSCREEN_ACCEL "F11"
+#define ZOOM_IN_ACCEL NULL
+#define ZOOM_OUT_ACCEL NULL
+#endif
+
 static GtkActionEntry const actions[] = {
 /* File */
 	{ "FileNew", GTK_STOCK_NEW, NULL,
@@ -1593,18 +1601,13 @@ static GtkActionEntry const actions[] = {
 	{ "FilePrint", GTK_STOCK_PRINT, NULL,
 		"<control>p", N_("Print the current file"),
 		G_CALLBACK (cb_file_print) },
-	{ "FileSummary", GTK_STOCK_PROPERTIES, N_("Proper_ties..."),
+	{ "FileMetaData", GTK_STOCK_PROPERTIES, N_("Proper_ties..."),
 		NULL, N_("Edit descriptive information"),
-		G_CALLBACK (cb_file_summary) },
+		G_CALLBACK (cb_doc_meta_data) },
 	{ "FilePreferences", GTK_STOCK_PREFERENCES, N_("Pre_ferences..."),
 		NULL, N_("Change Gnumeric Preferences"),
 		G_CALLBACK (cb_file_preferences) },
 
-/* new hook put here to avoid patch conflict */
-	{ "FileSummary_new", GTK_STOCK_PROPERTIES, N_("NEW Proper_ties..."),
-		NULL, N_("Edit descriptive information"),
-		G_CALLBACK (cb_new_doc_metadata) },
-
 /* Edit -> Clear */
 	{ "EditClearAll", NULL, N_("_All"),
 		NULL, N_("Clear the selected cells' formats, comments, and contents"),
@@ -1632,15 +1635,22 @@ static GtkActionEntry const actions[] = {
 
 
 /* Edit -> Select */
+
+	/* Note : The accelerators involving space are just for display
+	 * 	purposes.  We actually handle this in
+	 * 		gnumeric-canvas.c:gnm_canvas_key_mode_sheet
+	 *	with the rest of the key movement and rangeselection.
+	 *	Otherwise input methods would steal them */
 	{ "EditSelectAll", NULL, N_("Select _All"),
-		"<control>a", N_("Select all cells in the spreadsheet"),
+		"<control><shift>space", N_("Select all cells in the spreadsheet"),
 		G_CALLBACK (cb_edit_select_all) },
 	{ "EditSelectColumn", NULL, N_("Select _Column"),
 		"<control>space", N_("Select an entire column"),
 		G_CALLBACK (cb_edit_select_col) },
 	{ "EditSelectRow", NULL, N_("Select _Row"),
-		"<alt>space", N_("Select an entire row"),
+		"<shift>space", N_("Select an entire row"),
 		G_CALLBACK (cb_edit_select_row) },
+
 	{ "EditSelectArray", NULL, N_("Select Arra_y"),
 		"<control>slash", N_("Select an array of cells"),
 		G_CALLBACK (cb_edit_select_array) },
@@ -1650,9 +1660,6 @@ static GtkActionEntry const actions[] = {
 	{ "EditSelectInputs", NULL, N_("Select _Inputs"),
 		"<control>bracketleft", N_("Select all the cells are used by the current edit cell"),
 		G_CALLBACK (cb_edit_select_inputs) },
-	{ "EditMetadata", NULL, N_("Edit Metadata"),
-		NULL, N_("Edit the document's extra information (metadata)"),
-		G_CALLBACK (cb_new_doc_metadata) },
 
 /* Edit -> Fill */
 	{ "EditFillAutofill", NULL, N_("Auto_fill"),
@@ -1687,7 +1694,7 @@ static GtkActionEntry const actions[] = {
 		G_CALLBACK (wbcg_append_sheet) },
 	{ "EditDuplicateSheet", NULL, N_("_Duplicate"),
 		NULL, N_("Make a copy of the current sheet"),
-		G_CALLBACK (cb_edit_duplicate_sheet) },
+		G_CALLBACK (wbcg_clone_sheet) },
 	{ "SheetRemove", NULL, N_("_Remove"),
 		NULL, N_("Irrevocably remove an entire sheet"),
 		G_CALLBACK (cb_sheet_remove) },
@@ -1729,10 +1736,10 @@ static GtkActionEntry const actions[] = {
 		NULL, N_("Zoom the spreadsheet in or out"),
 		G_CALLBACK (cb_view_zoom) },
 	{ "ViewZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom _In"),
-		NULL, N_("Increase the zoom to make things larger"),
+		ZOOM_IN_ACCEL, N_("Increase the zoom to make things larger"),
 		G_CALLBACK (cb_view_zoom_in) },
 	{ "ViewZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"),
-		NULL, N_("Decrease the zoom to make things smaller"),
+		ZOOM_OUT_ACCEL, N_("Decrease the zoom to make things smaller"),
 		G_CALLBACK (cb_view_zoom_out) },
 	{ "Direction", GTK_STOCK_GO_FORWARD, N_("Direction"),
 		NULL, N_("Toggle sheet direction, left to right vs right to left"),
@@ -2068,7 +2075,7 @@ static GtkActionEntry const actions[] = {
 		"<control>at", N_("Format the selection as time"),
 		G_CALLBACK (cb_format_as_time) },
 	{ "FormatAddBorders", NULL, N_("AddBorders"),
-		"<control>ampersand", N_("Add a border the the selection"),
+		"<control>ampersand", N_("Add a border around the selection"),
 		G_CALLBACK (cb_format_add_borders) },
 	{ "FormatClearBorders", NULL, N_("ClearBorders"),
 		"<control>underscore", N_("Clear the border around the selection"),
@@ -2183,8 +2190,9 @@ static GtkToggleActionEntry const toggle_actions[] = {
 	        G_CALLBACK (cb_view_statusbar), TRUE },
 
 	{ "ViewFullScreen", GTK_STOCK_ZOOM_FIT,
-		N_("F_ull Screen"), "F11",
-		N_("in or out"), G_CALLBACK (cb_view_fullscreen), FALSE }
+		N_("F_ull Screen"), FULLSCREEN_ACCEL,
+		N_("Switch to or from full screen mode"),
+		G_CALLBACK (cb_view_fullscreen), FALSE }
 };
 
 static GtkToggleActionEntry const font_toggle_actions[] = {
