@@ -3,7 +3,7 @@
 /*
  * workbook-control-gui.c: GUI specific routines for a workbook-control.
  *
- * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as published
@@ -61,7 +61,6 @@
 #include "style-color.h"
 #include "str.h"
 #include "cell.h"
-#include "gnm-format.h"
 #include "gui-file.h"
 #include "search.h"
 #include "gui-util.h"
@@ -82,18 +81,7 @@
 #include <gsf/gsf-impl-utils.h>
 
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkseparatormenuitem.h>
-#include <gtk/gtkcheckmenuitem.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkdnd.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkprogressbar.h>
-#include <gtk/gtkstatusbar.h>
-#include <gtk/gtkicontheme.h>
+#include <gtk/gtk.h>
 
 #include <string.h>
 #include <errno.h>
@@ -105,7 +93,7 @@
 
 #define WBCG_CLASS(o) WORKBOOK_CONTROL_GUI_CLASS (G_OBJECT_GET_CLASS (o))
 #define WBCG_VIRTUAL_FULL(func, handle, arglist, call)		\
-void wbcg_ ## func arglist				\
+void wbcg_ ## func arglist					\
 {								\
 	WorkbookControlGUIClass *wbcg_class;			\
 								\
@@ -124,6 +112,8 @@ void wbcg_ ## func arglist				\
 #define TABLE_COLUMNS  1	
 #endif
 
+#define	SHEET_CONTROL_KEY	"SheetControl"
+
 enum {
 	TARGET_URI_LIST,
 	TARGET_SHEET
@@ -164,53 +154,18 @@ wbcg_ui_update_end (WorkbookControlGUI *wbcg)
 	wbcg->updating_ui = FALSE;
 }
 
-static Sheet *
-wbcg_page_index_to_sheet (WorkbookControlGUI *wbcg, int page,
-			  SheetControlGUI **res)
+static SheetControlGUI *
+wbcg_get_scg (WorkbookControlGUI *wbcg, Sheet *sheet)
 {
 	GtkWidget *w;
 
-	if (res)
-		*res = NULL;
-
-	w = gtk_notebook_get_nth_page (wbcg->notebook, page);
-	if (w) {
-		SheetControlGUI *scg = g_object_get_data (G_OBJECT (w), SHEET_CONTROL_KEY);
-		if (scg) {
-			if (res) *res = scg;
-			return ((SheetControl *)scg)->sheet;
-		}
-	}
-
-	return NULL;
-}
-
-int
-wbcg_sheet_to_page_index (WorkbookControlGUI *wbcg, Sheet *sheet,
-			  SheetControlGUI **res)
-{
-	int i = 0;
-	GtkWidget *w;
-
-	if (res)
-		*res = NULL;
-
 	if (sheet == NULL || wbcg->notebook == NULL)
-		return -1;
+		return NULL;
 
-	g_return_val_if_fail (IS_SHEET (sheet), -1);
+	g_return_val_if_fail (IS_SHEET (sheet), NULL);
 
-	for ( ; NULL != (w = gtk_notebook_get_nth_page (wbcg->notebook, i)) ; i++) {
-		SheetControlGUI *scg = g_object_get_data (G_OBJECT (w), SHEET_CONTROL_KEY);
-		SheetControl *sc = (SheetControl *) scg;
-
-		if (scg != NULL && sc->sheet == sheet) {
-			if (res)
-				*res = scg;
-			return i;
-		}
-	}
-	return -1;
+	w = gtk_notebook_get_nth_page (wbcg->notebook, sheet->index_in_wb);
+	return g_object_get_data (G_OBJECT (w), SHEET_CONTROL_KEY);
 }
 
 GtkWindow *
@@ -273,13 +228,7 @@ wbcg_cur_sheet (WorkbookControlGUI *wbcg)
 SheetControlGUI *
 wbcg_cur_scg (WorkbookControlGUI *wbcg)
 {
-	SheetControlGUI *scg;
-
-	wbcg_sheet_to_page_index (
-		wbcg,
-		wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg)), &scg);
-
-	return scg;
+	return wbcg_get_scg (wbcg, wbcg_cur_sheet (wbcg));
 }
 
 /****************************************************************************/
@@ -297,7 +246,8 @@ cb_autosave (gpointer *data)
 	if (wb_view == NULL)
 		return FALSE;
 
-	if (wbcg->autosave && workbook_is_dirty (wb_view_workbook (wb_view))) {
+	if (wbcg->autosave &&
+	    go_doc_is_dirty (wb_view_get_doc (wb_view))) {
 	        if (wbcg->autosave_prompt && !dialog_autosave_prompt (wbcg))
 			return TRUE;
 		gui_file_save (wbcg, wb_view);
@@ -312,7 +262,7 @@ cb_autosave (gpointer *data)
  * Returns true if the cursor keys should be used to select
  * a cell range (if the cursor is in a spot in the expression
  * where it makes sense to have a cell reference), false if not.
- */
+ **/
 gboolean
 wbcg_rangesel_possible (WorkbookControlGUI const *wbcg)
 {
@@ -364,21 +314,6 @@ wbcg_autosave_set (WorkbookControlGUI *wbcg, int minutes, gboolean prompt)
 }
 /****************************************************************************/
 
-static void
-wbcg_set_title (WorkbookControl *wbc, char const *title)
-{
-	WorkbookControlGUI *wbcg = (WorkbookControlGUI *)wbc;
-	char *full_title;
-
-	g_return_if_fail (wbcg != NULL);
-	g_return_if_fail (title != NULL);
-
-	full_title = g_strconcat (title, _(" : Gnumeric"), NULL);
-
- 	gtk_window_set_title (wbcg_toplevel (wbcg), full_title);
-	g_free (full_title);
-}
-
 static void
 wbcg_edit_line_set (WorkbookControl *wbc, char const *text)
 {
@@ -428,8 +363,8 @@ wbcg_update_action_sensitivity (WorkbookControl *wbc)
 		int i;
 		for (i = 0; i < gtk_notebook_get_n_pages (wbcg->notebook); i++) {
 			GtkWidget *page = gtk_notebook_get_nth_page (wbcg->notebook, i);
-			SheetControlGUI *scg = g_object_get_data (G_OBJECT (page), SHEET_CONTROL_KEY);
-			editable_label_set_editable (EDITABLE_LABEL (scg->label), enable_actions);
+			GtkWidget *label = gtk_notebook_get_tab_label (wbcg->notebook, page);
+			editable_label_set_editable (EDITABLE_LABEL (label), enable_actions);
 		}
 	}
 
@@ -443,7 +378,7 @@ cb_sheet_label_edit_finished (EditableLabel *el, char const *new_name,
 	gboolean reject = FALSE;
 	if (new_name != NULL) {
 		const char *old_name = editable_label_get_text (el);
-		Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+		Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 		Sheet *sheet = workbook_sheet_by_name (wb, old_name);
 		reject = cmd_rename_sheet (WORKBOOK_CONTROL (wbcg),
 					   sheet,
@@ -456,24 +391,41 @@ cb_sheet_label_edit_finished (EditableLabel *el, char const *new_name,
 void
 wbcg_insert_sheet (GtkWidget *unused, WorkbookControlGUI *wbcg)
 {
-	Sheet *sheet = wb_control_cur_sheet (WORKBOOK_CONTROL (wbcg));
 	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
-	Workbook *wb = wb_control_workbook (wbc);
+	Sheet *sheet = wb_control_cur_sheet (wbc);
+	Workbook *wb = sheet->workbook;
 	WorkbookSheetState *old_state = workbook_sheet_state_new (wb);
-	workbook_sheet_add (wb, sheet->index_in_wb, FALSE);
-	cmd_reorganize_sheets2 (wbc, old_state);
+	workbook_sheet_add (wb, sheet->index_in_wb);
+	cmd_reorganize_sheets (wbc, old_state, sheet);
 }
 
 void
 wbcg_append_sheet (GtkWidget *unused, WorkbookControlGUI *wbcg)
 {
 	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
-	Workbook *wb = wb_control_workbook (wbc);
+	Sheet *sheet = wb_control_cur_sheet (wbc);
+	Workbook *wb = sheet->workbook;
 	WorkbookSheetState *old_state = workbook_sheet_state_new (wb);
-	workbook_sheet_add (wb, -1, FALSE);
-	cmd_reorganize_sheets2 (wbc, old_state);
+	workbook_sheet_add (wb, -1);
+	cmd_reorganize_sheets (wbc, old_state, sheet);
 }
 
+void
+wbcg_clone_sheet (GtkWidget *unused, WorkbookControlGUI *wbcg)
+{
+	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
+	Sheet *sheet = wb_control_cur_sheet (wbc);
+	Workbook *wb = sheet->workbook;
+	WorkbookSheetState *old_state = workbook_sheet_state_new (wb);
+	Sheet *new_sheet = sheet_dup (sheet);
+	workbook_sheet_attach_at_pos (wb, new_sheet, sheet->index_in_wb + 1);
+	/* See workbook_sheet_add:  */
+	g_signal_emit_by_name (G_OBJECT (wb), "sheet_added", 0);
+	cmd_reorganize_sheets (wbc, old_state, sheet);
+	g_object_unref (new_sheet);
+}
+
+
 static void
 sheet_action_add_sheet (GtkWidget *widget, SheetControlGUI *scg)
 {
@@ -500,8 +452,7 @@ scg_delete_sheet_if_possible (G_GNUC_UNUSED GtkWidget *ignored,
 		WorkbookControl *wbc = sc->wbc;
 		workbook_sheet_delete (sheet);
 		/* Careful: sc just ceased to be valid.  */
-		workbook_set_dirty (wb, TRUE);
-		cmd_reorganize_sheets2 (wbc, old_state);
+		cmd_reorganize_sheets (wbc, old_state, sheet);
 	}
 }
 
@@ -514,9 +465,7 @@ sheet_action_rename_sheet (GtkWidget *widget, SheetControlGUI *scg)
 static void
 sheet_action_clone_sheet (GtkWidget *widget, SheetControlGUI *scg)
 {
-	SheetControl *sc = (SheetControl *) scg;
-
-	cmd_clone_sheet (WORKBOOK_CONTROL (scg->wbcg), sc->sheet);
+	wbcg_clone_sheet (NULL, scg->wbcg);
 }
 
 static void
@@ -586,21 +535,19 @@ sheet_menu_label_run (SheetControlGUI *scg, GdkEventButton *event)
  */
 static gboolean
 cb_sheet_label_button_press (GtkWidget *widget, GdkEventButton *event,
-			     GtkWidget *child)
+			     SheetControlGUI *scg)
 {
-	GtkWidget *notebook;
+	GtkNotebook *notebook;
 	gint page_number;
-	SheetControlGUI *scg = g_object_get_data (G_OBJECT (child), SHEET_CONTROL_KEY);
-
-	g_return_val_if_fail (scg != NULL, FALSE);
+	GtkWidget *table = GTK_WIDGET (scg->table);
 
 	if (event->type != GDK_BUTTON_PRESS)
 		return FALSE;
 
-	notebook = child->parent;
-	page_number = gtk_notebook_page_num (GTK_NOTEBOOK (notebook), child);
+	notebook = GTK_NOTEBOOK (table->parent);
+	page_number = gtk_notebook_page_num (notebook, table);
 
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), page_number);
+	gtk_notebook_set_current_page (notebook, page_number);
 
 	if (event->button == 1 || NULL != scg->wbcg->rangesel)
 		return TRUE;
@@ -677,16 +624,18 @@ cb_sheet_label_drag_data_received (GtkWidget *widget, GdkDragContext *context,
 	 * the sheets.
 	 */
 	if (p_src >= 0) {
-		Sheet *s_src = wbcg_page_index_to_sheet (wbcg, p_src, NULL);
-		int p_dst =
-			gnm_notebook_page_num_by_label (wbcg->notebook,
-							widget);
-		Sheet *s_dst = wbcg_page_index_to_sheet (wbcg, p_dst, NULL);
+		Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
+		Sheet *s_src = workbook_sheet_by_index (wb, p_src);
+		int p_dst = gnm_notebook_page_num_by_label (wbcg->notebook,
+							    widget);
+		Sheet *s_dst = workbook_sheet_by_index (wb, p_dst);
 
 		if (s_src && s_dst && s_src != s_dst) {
-			WorkbookSheetState *old_state = workbook_sheet_state_new (s_src->workbook);
-			workbook_sheet_move (s_src, s_dst->index_in_wb - s_src->index_in_wb);
-			cmd_reorganize_sheets2 (WORKBOOK_CONTROL (wbcg), old_state);
+			WorkbookSheetState *old_state = workbook_sheet_state_new (wb);
+			workbook_sheet_move (s_src, p_dst - p_src);
+			cmd_reorganize_sheets (WORKBOOK_CONTROL (wbcg),
+					       old_state,
+					       s_src);
 		}
 	} else {
 
@@ -721,8 +670,12 @@ cb_sheet_label_drag_begin (GtkWidget *widget, GdkDragContext *context,
 	g_object_unref (pixbuf);
 	gtk_widget_shape_combine_mask (arrow, bitmap, 0, 0);
 	g_object_unref (bitmap);
-	g_object_ref (G_OBJECT (arrow));
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+	g_object_ref_sink (arrow);
+#else
+	g_object_ref (arrow);
 	gtk_object_sink (GTK_OBJECT (arrow));
+#endif
 	g_object_set_data (G_OBJECT (widget), "arrow", arrow);
 }
 
@@ -901,22 +854,31 @@ cb_zoom_change (Sheet *sheet,
 }
 
 static void
-disconnect_sheet_signals (WorkbookControlGUI *wbcg, Sheet *sheet)
+cb_sheet_visibility_change (Sheet *sheet,
+			    G_GNUC_UNUSED GParamSpec *pspec,
+			    GtkWidget *w)
 {
-	int i;
-	SheetControlGUI *scg;
+	if (sheet_is_visible (sheet))
+		gtk_widget_show (w);
+	else
+		gtk_widget_hide (w);
+}
 
-	if (!sheet)
-		return;
+static void
+disconnect_sheet_signals (WorkbookControlGUI *wbcg, Sheet *sheet, gboolean focus_signals_only)
+{
+	SheetControlGUI *scg = wbcg_get_scg (wbcg, sheet);
 
-	i = wbcg_sheet_to_page_index (wbcg, sheet, &scg);
-	if (i < 0)
-		return;
+	if (scg) {
+		g_signal_handlers_disconnect_by_func (sheet, cb_toggle_menu_item_changed, wbcg);
+		g_signal_handlers_disconnect_by_func (sheet, cb_direction_change, scg);
+		g_signal_handlers_disconnect_by_func (sheet, cb_zoom_change, wbcg);
 
-	g_signal_handlers_disconnect_by_func (sheet, cb_sheet_tab_change, scg->label);
-	g_signal_handlers_disconnect_by_func (sheet, cb_toggle_menu_item_changed, wbcg);
-	g_signal_handlers_disconnect_by_func (sheet, cb_direction_change, scg);
-	g_signal_handlers_disconnect_by_func (sheet, cb_zoom_change, wbcg);
+		if (!focus_signals_only) {
+			g_signal_handlers_disconnect_by_func (sheet, cb_sheet_tab_change, scg->label);
+			g_signal_handlers_disconnect_by_func (sheet, cb_sheet_visibility_change, scg->table);
+		}
+	}
 }
 
 static void
@@ -931,17 +893,18 @@ wbcg_sheet_add (WorkbookControl *wbc, SheetView *sv)
 	SheetControl	*sc;
 	Sheet		*sheet;
 	GSList		*ptr;
+	gboolean visible;
 
 	g_return_if_fail (wbcg != NULL);
 
 	sheet = sv_sheet (sv);
-	if (sheet->visibility != GNM_SHEET_VISIBILITY_VISIBLE)
-		return;
+	visible = sheet_is_visible (sheet);
 
 	if (wbcg->notebook == NULL)
 		workbook_setup_sheets (wbcg);
 
 	scg = sheet_control_gui_new (sv, wbcg);
+	g_object_set_data (G_OBJECT (scg->table), SHEET_CONTROL_KEY, scg);
 
 	/*
 	 * NB. this is so we can use editable_label_set_text since
@@ -957,7 +920,7 @@ wbcg_sheet_add (WorkbookControl *wbc, SheetView *sv)
 	/* do not preempt the editable label handler */
 	g_signal_connect_after (G_OBJECT (scg->label),
 		"button_press_event",
-		G_CALLBACK (cb_sheet_label_button_press), scg->table);
+		G_CALLBACK (cb_sheet_label_button_press), scg);
 
 	/* Drag & Drop */
 	gtk_drag_source_set (scg->label, GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
@@ -977,6 +940,14 @@ wbcg_sheet_add (WorkbookControl *wbc, SheetView *sv)
 
 	gtk_widget_show (scg->label);
 	gtk_widget_show_all (GTK_WIDGET (scg->table));
+	if (!visible)
+		gtk_widget_hide (GTK_WIDGET (scg->table));
+	g_object_connect (G_OBJECT (sheet),
+			  "signal::notify::visibility", cb_sheet_visibility_change, scg->table,
+			  "signal::notify::name", cb_sheet_tab_change, scg->label,
+			  "signal::notify::tab-foreground", cb_sheet_tab_change, scg->label,
+			  "signal::notify::tab-background", cb_sheet_tab_change, scg->label,
+			  NULL);
 
 	if (wbcg_ui_update_begin (wbcg)) {
 		gtk_notebook_insert_page (wbcg->notebook,
@@ -999,18 +970,13 @@ static void
 wbcg_sheet_remove (WorkbookControl *wbc, Sheet *sheet)
 {
 	WorkbookControlGUI *wbcg = (WorkbookControlGUI *)wbc;
-	SheetControlGUI *scg;
-	int i;
 
 	/* During destruction we may have already removed the notebook */
 	if (wbcg->notebook == NULL)
 		return;
 
-	i = wbcg_sheet_to_page_index (wbcg, sheet, &scg);
-	g_return_if_fail (i >= 0);
-
-	disconnect_sheet_signals (wbcg, sheet);
-	gtk_notebook_remove_page (wbcg->notebook, i);
+	disconnect_sheet_signals (wbcg, sheet, FALSE);
+	gtk_notebook_remove_page (wbcg->notebook, sheet->index_in_wb);
 
 	wbcg_menu_state_sheet_count (wbcg);
 }
@@ -1019,48 +985,60 @@ static void
 wbcg_sheet_focus (WorkbookControl *wbc, Sheet *sheet)
 {
 	WorkbookControlGUI *wbcg = (WorkbookControlGUI *)wbc;
-	SheetControlGUI *scg;
-	int i = wbcg_sheet_to_page_index (wbcg, sheet, &scg);
+	SheetControlGUI *scg = wbcg_get_scg (wbcg, sheet);
 
-	/* A sheet added in another view may not yet have a view */
-	if (i >= 0) {
-		gtk_notebook_set_current_page (wbcg->notebook, i);
-		if (wbcg->rangesel == NULL)
-			gnm_expr_entry_set_scg (wbcg->edit_line.entry, scg);
+	gtk_notebook_set_current_page (wbcg->notebook, sheet->index_in_wb);
+	if (wbcg->rangesel == NULL)
+		gnm_expr_entry_set_scg (wbcg->edit_line.entry, scg);
 
-		disconnect_sheet_signals (wbcg, wbcg_cur_sheet (wbcg));
+	disconnect_sheet_signals (wbcg, wbcg_cur_sheet (wbcg), TRUE);
 
-		g_object_connect
-			(G_OBJECT (sheet),
-			 "signal::notify::name", cb_sheet_tab_change, scg->label,
-			 "signal::notify::tab-foreground", cb_sheet_tab_change, scg->label,
-			 "signal::notify::tab-background", cb_sheet_tab_change, scg->label,
-			 "signal::notify::display-formulas", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-zeros", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-grid", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-column-header", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-row-header", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-outlines", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-outlines-below", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::display-outlines-right", cb_toggle_menu_item_changed, wbcg,
-			 "signal::notify::text-is-rtl", cb_direction_change, scg,
-			 "signal::notify::zoom-factor", cb_zoom_change, wbcg,
-			 NULL);
+	wbcg_update_menu_feedback (wbcg, sheet);
+
+	g_object_connect
+		(G_OBJECT (sheet),
+		 "signal::notify::display-formulas", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-zeros", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-grid", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-column-header", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-row-header", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-outlines", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-outlines-below", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::display-outlines-right", cb_toggle_menu_item_changed, wbcg,
+		 "signal::notify::text-is-rtl", cb_direction_change, scg,
+		 "signal::notify::zoom-factor", cb_zoom_change, wbcg,
+		 NULL);
+}
+
+static void
+wbcg_sheet_order_changed (WorkbookControlGUI *wbcg)
+{
+	GtkNotebook *nb = wbcg->notebook;
+	int i, n = gtk_notebook_get_n_pages (nb);
+
+	for (i = 0 ; i < n; i++) {
+		GtkWidget *w = gtk_notebook_get_nth_page (nb, i);
+		SheetControlGUI *scg = g_object_get_data (G_OBJECT (w), SHEET_CONTROL_KEY);
+		Sheet *sheet = ((SheetControl *)scg)->sheet;
+		gtk_notebook_reorder_child (nb,
+					    GTK_WIDGET (scg->table),
+					    sheet->index_in_wb);
 	}
 }
 
 static void
-wbcg_sheet_order_changed (WorkbookControlGUI *wbcg, Workbook const *wb)
+wbcg_update_title (WorkbookControlGUI *wbcg)
 {
-	int i;
-	Sheet const *sheet;
-	SheetControlGUI *scg;
-
-	for (i = 0 ; i < workbook_sheet_count (wb); i++)
-		if (NULL != (sheet = wbcg_page_index_to_sheet (wbcg, i, &scg)) &&
-		    sheet->index_in_wb != i)
-			gtk_notebook_reorder_child (wbcg->notebook,
-				GTK_WIDGET (scg->table), sheet->index_in_wb);
+	GODoc *doc = wb_control_get_doc (WORKBOOK_CONTROL (wbcg));
+	char *basename = go_basename_from_uri (doc->uri);
+	char *title = g_strconcat
+		(go_doc_is_dirty (doc) ? "*" : "",
+		 basename ? basename : doc->uri,
+		 _(" : Gnumeric"),
+		 NULL);
+ 	gtk_window_set_title (wbcg_toplevel (wbcg), title);
+	g_free (title);
+	g_free (basename);
 }
 
 static void
@@ -1070,16 +1048,20 @@ wbcg_sheet_remove_all (WorkbookControl *wbc)
 
 	if (wbcg->notebook != NULL) {
 		GtkWidget *tmp = GTK_WIDGET (wbcg->notebook);
-
-		/* Be sure we are no longer editing */
-		wbcg_edit_finish (wbcg, WBC_EDIT_REJECT, NULL);
-
-		disconnect_sheet_signals (wbcg, wbcg_cur_sheet (wbcg));
+		Workbook *wb = wb_control_get_workbook (wbc);
+		int i;
 
 		/* Clear notebook to disable updates as focus changes for pages
 		 * during destruction */
 		wbcg->notebook = NULL;
-		gtk_container_remove (GTK_CONTAINER (wbcg->table), tmp);
+
+		/* Be sure we are no longer editing */
+		wbcg_edit_finish (wbcg, WBC_EDIT_REJECT, NULL);
+
+		for (i = workbook_sheet_count (wb) - 1; i >= 0; i--)
+			disconnect_sheet_signals (wbcg, workbook_sheet_by_index (wb, i), FALSE);
+
+		gtk_widget_destroy (tmp);
 	}
 }
 
@@ -1223,11 +1205,11 @@ wbcg_error_error_info (GOCmdContext *cc, ErrorInfo *error)
 
 int
 wbcg_show_save_dialog (WorkbookControlGUI *wbcg, 
-		       Workbook * wb, gboolean exiting)
+		       Workbook *wb, gboolean exiting)
 {
 	GtkWidget *d;
 	char *msg;
-	const char *wb_uri = workbook_get_uri (wb);
+	char const *wb_uri = go_doc_get_uri (GO_DOC (wb));
 	int ret = 0;
 
 	if (wb_uri) {
@@ -1299,12 +1281,12 @@ wbcg_close_if_user_permits (WorkbookControlGUI *wbcg,
 	gboolean   done      = FALSE;
 	int        iteration = 0;
 	int        button = 0;
-	Workbook  *wb = wb_view_workbook (wb_view);
+	Workbook  *wb = wb_view_get_workbook (wb_view);
 	static int in_can_close;
 
 	g_return_val_if_fail (IS_WORKBOOK (wb), 0);
 
-	if (!close_clean && !workbook_is_dirty (wb))
+	if (!close_clean && !go_doc_is_dirty (GO_DOC (wb)))
 		return 2;
 
 	if (in_can_close)
@@ -1318,7 +1300,7 @@ wbcg_close_if_user_permits (WorkbookControlGUI *wbcg,
 			return 3;
 		}
 	}
-	while (workbook_is_dirty (wb) && !done) {
+	while (go_doc_is_dirty (GO_DOC (wb)) && !done) {
 		iteration++;
 		button = wbcg_show_save_dialog(wbcg, wb, exiting); 
 
@@ -1333,12 +1315,12 @@ wbcg_close_if_user_permits (WorkbookControlGUI *wbcg,
 
 		case GTK_RESPONSE_NO:
 			done      = TRUE;
-			workbook_set_dirty (wb, FALSE);
+			go_doc_set_dirty (GO_DOC (wb), FALSE);
 			break;
 
 		case GNM_RESPONSE_DISCARD_ALL:
 			done      = TRUE;
-			workbook_set_dirty (wb, FALSE);
+			go_doc_set_dirty (GO_DOC (wb), FALSE);
 			break;
 
 		default:  /* CANCEL */
@@ -1390,7 +1372,7 @@ wbcg_close_control (WorkbookControlGUI *wbcg)
 
 	/* This is the last control */
 	if (wb_view->wb_controls->len <= 1) {
-		Workbook *wb = wb_view_workbook (wb_view);
+		Workbook *wb = wb_view_get_workbook (wb_view);
 
 		g_return_val_if_fail (IS_WORKBOOK (wb), TRUE);
 		g_return_val_if_fail (wb->wb_views != NULL, TRUE);
@@ -1451,11 +1433,13 @@ cb_statusbox_focus (GtkEntry *entry, GdkEventFocus *event,
 static GnmValue *
 cb_share_a_cell (Sheet *sheet, int col, int row, GnmCell *cell, gpointer _es)
 {
+#if 0
 	if (cell && cell_has_expr (cell)) {
 		ExprTreeSharer *es = _es;
 		cell->base.expression =
 			expr_tree_sharer_share (es, cell->base.expression);
 	}
+#endif
 
 	return NULL;
 }
@@ -1464,10 +1448,7 @@ cb_share_a_cell (Sheet *sheet, int col, int row, GnmCell *cell, gpointer _es)
 static void
 cb_workbook_debug_info (WorkbookControlGUI *wbcg)
 {
-	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
-
-	if (gnumeric_debugging > 3)
-		summary_info_dump (wb->summary_info);
+	Workbook *wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 
 	if (dependency_debugging > 0) {
 		WORKBOOK_FOREACH_SHEET (wb, sheet,
@@ -1522,22 +1503,22 @@ cb_autofunction (WorkbookControlGUI *wbcg)
 }
 
 static GtkWidget *
-edit_area_button (WorkbookControlGUI *wbcg, gboolean sensitive,
-		  GCallback func, char const *stock_id)
+edit_area_button (WorkbookControlGUI *wbcg, GtkToolbar *tb,
+		  gboolean sensitive,
+		  GCallback func, char const *stock_id,
+		  GtkTooltips *tips, const char *tip)
 {
-	GtkWidget *button = gtk_button_new ();
-	gtk_container_add (GTK_CONTAINER (button),
-		gtk_image_new_from_stock (stock_id,
-					  GTK_ICON_SIZE_BUTTON));
-	GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
-	if (!sensitive)
-		gtk_widget_set_sensitive (button, FALSE);
+	GObject *button =
+		g_object_new (GTK_TYPE_TOOL_BUTTON,
+			      "stock-id", stock_id,
+			      "sensitive", sensitive,
+			      "can-focus", FALSE,
+			      NULL);
+	gtk_tool_item_set_tooltip (GTK_TOOL_ITEM (button), tips, tip, NULL);
+	g_signal_connect_swapped (button, "clicked", func, wbcg);
+	gtk_toolbar_insert (tb, GTK_TOOL_ITEM (button), -1);
 
-	g_signal_connect_swapped (G_OBJECT (button),
-		"clicked",
-		G_CALLBACK (func), wbcg);
-
-	return button;
+	return GTK_WIDGET (button);
 }
 
 /*
@@ -1627,6 +1608,8 @@ wbcg_finalize (GObject *obj)
 {
 	WorkbookControlGUI *wbcg = WORKBOOK_CONTROL_GUI (obj);
 
+	x_store_clipboard_if_needed (wbcg);
+
 	/* Disconnect signals that would attempt to change things during
 	 * destruction.
 	 */
@@ -1662,7 +1645,7 @@ wbcg_finalize (GObject *obj)
 
 #ifdef USE_HILDON	
 	if (wbcg->hildon_app != NULL)
-		gtk_object_destroy (GTK_OBJECT (wbcg->app));
+		gtk_object_destroy (GTK_OBJECT (wbcg->hildon_app));
 #endif	
 	(*parent_class->finalize) (obj);
 }
@@ -1835,26 +1818,15 @@ wbcg_copy_toolbar_visibility (WorkbookControlGUI *new_wbcg,
 }
 
 
-void
-wbcg_toggle_end_mode (WorkbookControlGUI *wbcg)
-{
-	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
-	wbcg_set_end_mode (wbcg, !wbcg->last_key_was_end);
-}
 void
 wbcg_set_end_mode (WorkbookControlGUI *wbcg, gboolean flag)
 {
 	g_return_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg));
 
-	if (wbcg->last_key_was_end == flag)
-		return;
-
-	if (flag == TRUE) {
-		wbcg->last_key_was_end = TRUE;
-		wbcg_set_status_text (wbcg, "END");
-	} else {
-		wbcg->last_key_was_end = FALSE;
-		wbcg_set_status_text (wbcg, "");
+	if ((!wbcg->last_key_was_end) != (!flag)) {
+		wbcg_set_status_text (wbcg,
+			(wbcg->last_key_was_end = flag)
+			? "END" : "");
 	}
 }
 
@@ -1974,15 +1946,15 @@ cb_select_auto_expr (GtkWidget *widget, GdkEventButton *event, WorkbookControlGU
 	for (i = 0; quick_compute_routines [i].displayed_name; i++) {
 		GnmParsePos pp;
 		const char *expr = quick_compute_routines [i].function;
-		const GnmExpr *new_auto_expr;
+		const GnmExprTop *new_auto_expr;
 		GtkWidget *item;
 
 		/* Test the expression...  */
-		parse_pos_init (&pp, wb_control_workbook (WORKBOOK_CONTROL (wbcg)), NULL, 0, 0);
+		parse_pos_init (&pp, wb_control_get_workbook (WORKBOOK_CONTROL (wbcg)), NULL, 0, 0);
 		new_auto_expr = gnm_expr_parse_str_simple (expr, &pp);
 		if (!new_auto_expr)
 			continue;
-		gnm_expr_unref (new_auto_expr);
+		gnm_expr_top_unref (new_auto_expr);
 
 		item = gtk_menu_item_new_with_label (
 			_(quick_compute_routines [i].displayed_name));
@@ -2100,7 +2072,7 @@ wbcg_drag_data_get (GtkWidget          *widget,
 		    guint               time,
 		    WorkbookControl    *wbcg)
 {
-	Workbook *wb    = wb_control_workbook (wbc);
+	Workbook *wb    = wb_control_get_workbook (wbc);
 	Sheet	 *sheet = wb_control_cur_sheet (wbc);
 	BonoboMoniker *moniker;
 	char *s;
@@ -2214,16 +2186,30 @@ cb_wbcg_drag_data_received (GtkWidget *widget, GdkDragContext *context,
 static void
 wbcg_create_edit_area (WorkbookControlGUI *wbcg)
 {
-	GtkWidget *box, *box2;
-	GtkTooltips *tooltips;
+	GtkToolItem *item;
 	GtkEntry *entry;
 	int len;
+	GtkToolbar *tb;
+	GtkTooltips *tooltips;
 
 	wbcg->selection_descriptor = gtk_entry_new ();
 	wbcg_edit_ctor (wbcg);
 	entry = wbcg_get_entry (wbcg);
-	box   = gtk_hbox_new (FALSE, 0);
-	box2  = gtk_hbox_new (FALSE, 0);
+
+	tb = (GtkToolbar *)gtk_toolbar_new ();
+	gtk_toolbar_set_show_arrow (tb, FALSE);
+	gtk_toolbar_set_style (tb, GTK_TOOLBAR_ICONS);
+
+	tooltips = gtk_tooltips_new ();
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+	g_object_ref_sink (tooltips);
+#else
+	g_object_ref (tooltips);
+	gtk_object_sink (GTK_OBJECT (tooltips));
+#endif
+	g_object_set_data_full (G_OBJECT (tb),
+				"tooltips", tooltips,
+				(GDestroyNotify)g_object_unref);
 
 	/* Set a reasonable width for the selection box. */
 	len = go_pango_measure_string (
@@ -2236,46 +2222,40 @@ wbcg_create_edit_area (WorkbookControlGUI *wbcg)
 	 */
 	len = len * 3 / 2;
 	gtk_widget_set_size_request (wbcg->selection_descriptor, len, -1);
+	item = gtk_tool_item_new ();
+	gtk_container_add (GTK_CONTAINER (item), wbcg->selection_descriptor);
+	gtk_toolbar_insert (tb, item, -1);
 
-	tooltips = gtk_tooltips_new ();
-	g_object_ref (tooltips);
-	gtk_object_sink (GTK_OBJECT (tooltips));
-	g_object_set_data_full (G_OBJECT (box),
-				"tooltips", tooltips,
-				(GDestroyNotify)g_object_unref);
-
-	wbcg->cancel_button = edit_area_button (wbcg, FALSE,
-		G_CALLBACK (cb_cancel_input), GTK_STOCK_CANCEL);
-	gtk_tooltips_set_tip (tooltips, wbcg->cancel_button,
-			      _("Cancel change"), "");
-	wbcg->ok_button = edit_area_button (wbcg, FALSE,
-		G_CALLBACK (cb_accept_input), GTK_STOCK_OK);
-	gtk_tooltips_set_tip (tooltips, wbcg->ok_button,
-			      _("Accept change"), "");
-	wbcg->func_button = edit_area_button (wbcg, TRUE,
-		G_CALLBACK (cb_autofunction), "Gnumeric_Equal");
-	gtk_tooltips_set_tip (tooltips, wbcg->func_button,
-			      _("Enter formula..."), "");
-
-	gtk_box_pack_start (GTK_BOX (box2), wbcg->selection_descriptor, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), wbcg->cancel_button, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), wbcg->ok_button, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (box), wbcg->func_button, FALSE, FALSE, 0);
+	wbcg->cancel_button = edit_area_button
+		(wbcg, tb, FALSE,
+		 G_CALLBACK (cb_cancel_input), GTK_STOCK_CANCEL,
+		 tooltips, _("Cancel change"));
+	wbcg->ok_button = edit_area_button
+		(wbcg, tb, FALSE,
+		 G_CALLBACK (cb_accept_input), GTK_STOCK_OK,
+		 tooltips, _("Accept change"));
+	wbcg->func_button = edit_area_button
+		(wbcg, tb, TRUE,
+		 G_CALLBACK (cb_autofunction), "Gnumeric_Equal",
+		 tooltips, _("Enter formula..."));
 
 	/* Dependency debugger */
 	if (gnumeric_debugging > 9 ||
 	    dependency_debugging > 0 ||
 	    expression_sharing_debugging > 0) {
-		GtkWidget *deps_button = edit_area_button (wbcg, TRUE,
-			G_CALLBACK (cb_workbook_debug_info),
-			GTK_STOCK_DIALOG_INFO);
-		gtk_box_pack_start (GTK_BOX (box), deps_button, FALSE, FALSE, 0);
+		(void)edit_area_button (wbcg, tb, TRUE,
+					G_CALLBACK (cb_workbook_debug_info),
+					GTK_STOCK_DIALOG_INFO,
+					NULL, NULL);
 	}
 
-	gtk_box_pack_start (GTK_BOX (box2), box, FALSE, FALSE, 0);
-	gtk_box_pack_end   (GTK_BOX (box2), GTK_WIDGET (wbcg->edit_line.entry), TRUE, TRUE, 0);
+	item = gtk_tool_item_new ();
+	gtk_tool_item_set_expand (item, TRUE);
+	gtk_container_add (GTK_CONTAINER (item),
+			   GTK_WIDGET (wbcg->edit_line.entry));
+	gtk_toolbar_insert (tb, item, -1);
 
-	gtk_table_attach (GTK_TABLE (wbcg->table), box2,
+	gtk_table_attach (GTK_TABLE (wbcg->table), GTK_WIDGET (tb),
 			  0, 1, 0, 1,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, 0, 0, 0);
 
@@ -2292,7 +2272,7 @@ wbcg_create_edit_area (WorkbookControlGUI *wbcg)
 		"focus-out-event",
 		G_CALLBACK (cb_statusbox_focus), wbcg);
 
-	gtk_widget_show_all (box2);
+	gtk_widget_show_all (GTK_WIDGET (tb));
 }
 
 static void
@@ -2447,6 +2427,65 @@ wbcg_progress_message_set (GOCmdContext *cc, gchar const *msg)
 	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (wbcg->progress_bar), msg);
 }
 
+#define DISCONNECT(field)						\
+	if (wbcg->field) {						\
+		g_signal_handler_disconnect (old_wb, wbcg->field);	\
+		wbcg->field = 0;					\
+	}
+
+static void
+wbcg_view_changed (WorkbookControlGUI *wbcg,
+		   G_GNUC_UNUSED GParamSpec *pspec,
+		   Workbook *old_wb)
+{
+	WorkbookControl *wbc = WORKBOOK_CONTROL (wbcg);
+	Workbook *wb = wb_control_get_workbook (wbc);
+
+	/* Reconnect self because we need to change data.  */
+	if (wbcg->sig_view_changed)
+		g_signal_handler_disconnect (wbc, wbcg->sig_view_changed);
+	wbcg->sig_view_changed =
+		g_signal_connect_object
+		(G_OBJECT (wbc),
+		 "notify::view",
+		 G_CALLBACK (wbcg_view_changed),
+		 wb,
+		 0);
+
+	if (old_wb) {
+		DISCONNECT (sig_sheet_order);
+		DISCONNECT (sig_notify_uri);
+		DISCONNECT (sig_notify_dirty);
+	}
+
+	if (wb) {
+		wbcg->sig_sheet_order =
+			g_signal_connect_object
+			(G_OBJECT (wb),
+			 "sheet-order-changed",
+			 G_CALLBACK (wbcg_sheet_order_changed),
+			 wbcg, G_CONNECT_SWAPPED);
+
+		wbcg->sig_notify_uri =
+			g_signal_connect_object
+			(G_OBJECT (wb),
+			 "notify::uri",
+			 G_CALLBACK (wbcg_update_title),
+			 wbcg, G_CONNECT_SWAPPED);
+
+		wbcg->sig_notify_dirty =
+			g_signal_connect_object
+			(G_OBJECT (wb),
+			 "notify::dirty",
+			 G_CALLBACK (wbcg_update_title),
+			 wbcg, G_CONNECT_SWAPPED);
+
+		wbcg_update_title (wbcg);
+	}
+}
+
+#undef DISCONNECT
+
 /***************************************************************************/
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-series.h>
@@ -2490,19 +2529,21 @@ cb_graph_dim_editor_update (GnmExprEntry *gee,
 	if (!gnm_expr_entry_is_blank (editor->entry)) {
 		GnmParsePos pos;
 		GnmParseError  perr;
-		GnmExpr const *expr;
+		GnmExprTop const *texpr;
 
 		parse_error_init (&perr);
-		expr = gnm_expr_entry_parse (editor->entry,
+		texpr = gnm_expr_entry_parse (editor->entry,
 			parse_pos_init_sheet (&pos, sheet),
 			&perr, TRUE, GNM_EXPR_PARSE_UNKNOWN_NAMES_ARE_STRINGS);
 
 		/* TODO : add some error dialogs split out
 		 * the code in workbok_edit to add parens.  */
-		if (expr == NULL) {
+		if (texpr == NULL) {
 			if (editor->data_type == GOG_DATA_SCALAR)
-				expr = gnm_expr_new_constant (value_new_string (
-					gnm_expr_entry_get_text	(editor->entry)));
+				texpr = gnm_expr_top_new_constant
+					(value_new_string
+					 (gnm_expr_entry_get_text
+					  (editor->entry)));
 			else {
 				g_return_if_fail (perr.err != NULL);
 
@@ -2515,13 +2556,13 @@ cb_graph_dim_editor_update (GnmExprEntry *gee,
 
 		switch (editor->data_type) {
 		case GOG_DATA_SCALAR:
-			data = gnm_go_data_scalar_new_expr (sheet, expr);
+			data = gnm_go_data_scalar_new_expr (sheet, texpr);
 			break;
 		case GOG_DATA_VECTOR:
-			data = gnm_go_data_vector_new_expr (sheet, expr);
+			data = gnm_go_data_vector_new_expr (sheet, texpr);
 			break;
 		case GOG_DATA_MATRIX:
-			data = gnm_go_data_matrix_new_expr (sheet, expr);
+			data = gnm_go_data_matrix_new_expr (sheet, texpr);
 		}
 	}
 
@@ -2634,7 +2675,6 @@ workbook_control_gui_class_init (GObjectClass *object_class)
 	parent_class = g_type_class_peek_parent (object_class);
 	object_class->finalize = wbcg_finalize;
 
-	wbc_class->set_title		= wbcg_set_title;
 	wbc_class->edit_line_set	= wbcg_edit_line_set;
 	wbc_class->selection_descr_set	= wbcg_edit_selection_descr_set;
 	wbc_class->auto_expr_value	= wbcg_auto_expr_value;
@@ -2733,9 +2773,8 @@ wbcg_create (WorkbookControlGUI *wbcg,
 		wb_control_style_feedback (wbc, NULL);
 		cb_zoom_change (sheet, NULL, wbcg);
 	}
-	g_signal_connect_object (G_OBJECT (wbv->wb),
-		"sheet-order-changed",
-		G_CALLBACK (wbcg_sheet_order_changed), wbcg, G_CONNECT_SWAPPED);
+
+	wbcg_view_changed (wbcg, NULL, NULL);
 
 	if (optional_screen)
 		gtk_window_set_screen (wbcg_toplevel (wbcg), optional_screen);
