@@ -6,7 +6,7 @@
  *   Miguel de Icaza (miguel@ximian.com)
  *   Jody Goldberg (jody@gnome.org)
  *
- * (C) 2000-2001 Ximian, Inc.
+ * (C) 2000-2005 Ximian, Inc.
  */
 #include <gnumeric-config.h>
 #include <glib/gi18n.h>
@@ -37,6 +37,9 @@
 
 #include <gtk/gtk.h>
 #include <string.h>
+#include <goffice/utils/format-impl.h>
+
+#define GNM_RESPONSE_REMOVE -1000
 
 /*
  * Shuts down the auto completion engine
@@ -116,7 +119,7 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, WBCEditResult result,
 		ValidationStatus valid;
 		char *free_txt = NULL;
 		char const *txt = wbcg_edit_get_display_text (wbcg);
-		GnmStyle *mstyle = sheet_style_get (sheet, sv->edit_pos.col, sv->edit_pos.row);
+		GnmStyle const *mstyle = sheet_style_get (sheet, sv->edit_pos.col, sv->edit_pos.row);
 		char const *expr_txt = NULL;
 
 		/* BE CAREFUL the standard fmts must not NOT include '@' */
@@ -131,39 +134,40 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, WBCEditResult result,
 		 * it _can_ start an expression, which is required for rangesel
 		 * it just isn't an expression. */
 		if (expr_txt != NULL && *expr_txt != '\0' && strcmp (expr_txt, "-")) {
-			GnmExpr const *expr = NULL;
+			GnmExprTop const *texpr = NULL;
 			GnmParsePos    pp;
 			GnmParseError  perr;
 
 			parse_pos_init_editpos (&pp, sv);
 			parse_error_init (&perr);
-			expr = gnm_expr_parse_str (expr_txt,
+			texpr = gnm_expr_parse_str (expr_txt,
 				&pp, GNM_EXPR_PARSE_DEFAULT, gnm_expr_conventions_default, &perr);
 			/* Try adding a single extra closing paren to see if it helps */
-			if (expr == NULL && perr.err != NULL &&
+			if (texpr == NULL && perr.err != NULL &&
 			    perr.err->code == PERR_MISSING_PAREN_CLOSE) {
 				GnmParseError tmp_err;
 				char *tmp = g_strconcat (txt, ")", NULL);
 				parse_error_init (&tmp_err);
-				expr = gnm_expr_parse_str (gnm_expr_char_start_p (tmp),
+				texpr = gnm_expr_parse_str (gnm_expr_char_start_p (tmp),
 					&pp, GNM_EXPR_PARSE_DEFAULT,
 					gnm_expr_conventions_default, &tmp_err);
 				parse_error_free (&tmp_err);
 
-				if (expr != NULL)
+				if (texpr != NULL)
 					txt = free_txt = tmp;
 				else
 					g_free (tmp);
 			}
 
-			if (expr == NULL && perr.err != NULL) {
+			if (texpr == NULL && perr.err != NULL) {
 				ValidationStatus reedit;
 
 				/* set focus _before_ selection.  gtk2 seems to
 				 * screw with selection in gtk_entry_grab_focus
-				 */
-				gtk_window_set_focus (GTK_WINDOW (wbcg->toplevel),
-						      GTK_WIDGET (wbcg_get_entry (wbcg)));
+				 * (no longer required now that we clear
+				 * gtk-entry-select-on-focus) */
+				gtk_window_set_focus (wbcg_toplevel (wbcg),
+					(GtkWidget *) wbcg_get_entry (wbcg));
 
 				if (perr.begin_char != 0 || perr.end_char != 0) {
 					int offset = expr_txt - txt;
@@ -188,8 +192,8 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, WBCEditResult result,
 				 * focused on the edit line (eg hit F2) */
 				wbcg_focus_cur_scg (wbcg);
 			}
-			if (expr != NULL)
-				gnm_expr_unref (expr);
+			if (texpr != NULL)
+				gnm_expr_top_unref (texpr);
 		}
 
 		/* NOTE we assign the value BEFORE validating in case
@@ -217,9 +221,9 @@ wbcg_edit_finish (WorkbookControlGUI *wbcg, WBCEditResult result,
 			result = WBC_EDIT_REJECT;
 			command_undo (wbc);
 			if (valid == VALIDATION_STATUS_INVALID_EDIT) {
-				gtk_window_set_focus (GTK_WINDOW (wbcg->toplevel),
-						      GTK_WIDGET (wbcg_get_entry (wbcg)));
-				  return FALSE;
+				gtk_window_set_focus (wbcg_toplevel (wbcg),
+					(GtkWidget *) wbcg_get_entry (wbcg));
+				return FALSE;
 			}
 		}
 	} else {
@@ -649,6 +653,44 @@ wbcg_edit_get_markup (WorkbookControlGUI *wbcg, gboolean full)
 	return full ? wbcg->edit_line.full_content : wbcg->edit_line.markup;
 }
 
+static gboolean
+close_to_int (gnm_float x, gnm_float eps)
+{
+	return (x - gnm_fake_round (x)) < eps;
+}
+
+
+static void
+guess_time_format (GString *res, gnm_float f)
+{
+	int decs;
+	gnm_float eps = 1e-6;
+
+	g_string_append (res, "hh:mm");
+	f *= 24 * 60;
+	if (close_to_int (f, eps / 60))
+		return;
+
+	g_string_append (res, ":ss");
+	f *= 60;
+	if (close_to_int (f, eps))
+		return;
+
+	g_string_append_c (res, '.');
+	for (decs = 0; decs < 6; decs++) {
+		g_string_append_c (res, '0');
+		f *= 10;
+		if (close_to_int (f, eps))
+			break;
+	}
+}
+
+static void
+cb_warn_toggled (GtkToggleButton *button, gboolean *b)
+{
+	*b = gtk_toggle_button_get_active (button);
+}
+
 /**
  * wbcg_edit_start:
  *
@@ -671,13 +713,19 @@ gboolean
 wbcg_edit_start (WorkbookControlGUI *wbcg,
 		 gboolean blankp, gboolean cursorp)
 {
+	/*
+	 * FIXME!  static?  At worst this should sit in wbcg.
+	 */
 	static gboolean inside_editing = FALSE;
+	/* We could save this, but the situation is rare, if confusing.  */
+	static gboolean warn_on_text_format = TRUE;
 	SheetView *sv;
 	SheetControlGUI *scg;
 	GnmCell *cell;
 	char *text = NULL;
 	int col, row;
 	WorkbookView *wbv;
+	int cursor_pos = -1;
 
 	g_return_val_if_fail (IS_WORKBOOK_CONTROL_GUI (wbcg), FALSE);
 
@@ -702,7 +750,7 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 	 * information if we look at the selection.
 	 */
 	if (wb_view_is_protected (wbv, TRUE) &&
-	    gnm_style_get_content_locked (sheet_style_get (sv->sheet, col, row))) {
+	    gnm_style_get_contents_locked (sheet_style_get (sv->sheet, col, row))) {
 		char *pos =  g_strdup_printf ( _("%s!%s is locked"),
 			sv->sheet->name_quoted, cell_coord_name (col, row));
 		go_cmd_context_error_invalid (GO_CMD_CONTEXT (wbcg), pos,
@@ -714,24 +762,152 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 		return FALSE;
 	}
 
+	cell = sheet_cell_get (sv->sheet, col, row);
+	if (cell &&
+	    warn_on_text_format &&
+	    go_format_is_text (cell_get_format (cell)) &&
+	    (cell_has_expr (cell) || !VALUE_IS_STRING (cell->value))) {
+		GtkResponseType res;
+		GtkWidget *check;
+		GtkWidget *align;
+
+		GtkWidget *d = gnumeric_message_dialog_new
+			(wbcg_toplevel (wbcg),
+			 GTK_DIALOG_DESTROY_WITH_PARENT,
+			 GTK_MESSAGE_WARNING,
+			 _("You are about to edit a cell with \"text\" format."),
+			 _("The cell does not currently contain text, though, so if "
+			   "you go on editing then the contents will be turned into "
+			   "text."));
+		gtk_dialog_add_button (GTK_DIALOG (d), GTK_STOCK_EDIT, GTK_RESPONSE_OK);
+		go_gtk_dialog_add_button
+			(GTK_DIALOG (d), _("Remove format"), GTK_STOCK_REMOVE,
+			 GNM_RESPONSE_REMOVE);
+		gtk_dialog_add_button (GTK_DIALOG (d), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+		gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_CANCEL);
+		
+		check = gtk_check_button_new_with_label (_("Show this dialog next time."));
+		g_signal_connect (check, "toggled", G_CALLBACK (cb_warn_toggled), &warn_on_text_format);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
+		align = gtk_alignment_new (0.5, 0.5, 0, 0);
+		gtk_container_add (GTK_CONTAINER (align), check);
+		gtk_widget_show_all (align);
+		gtk_box_pack_end_defaults (GTK_BOX (GTK_DIALOG (d)->vbox), align);
+		res = go_gtk_dialog_run (GTK_DIALOG (d), wbcg_toplevel (wbcg));
+
+		switch (res) {
+		case GNM_RESPONSE_REMOVE: {
+			GnmStyle *style = gnm_style_new ();
+			gnm_style_set_format (style, go_format_general ());
+			if (!cmd_selection_format (WORKBOOK_CONTROL (wbcg),
+						   style, NULL, NULL))
+				break;
+			/* Fall through.  */
+		}
+		default:
+		case GTK_RESPONSE_CANCEL:
+			inside_editing = FALSE;
+			return FALSE;
+		case GTK_RESPONSE_OK:
+			break;
+		}
+	}
+
 	gnm_app_clipboard_unant ();
 
-	cell = sheet_cell_get (sv->sheet, col, row);
 	if (blankp)
 		gtk_entry_set_text (wbcg_get_entry (wbcg), "");
 	else if (cell != NULL) {
-		text = cell_get_entered_text (cell);
+		gboolean set_text = FALSE;
 
-		/* If this is part of an array we need to remove the
-		 * '{' '}' and the size information from the display.
-		 * That is not actually part of the parsable expression.
-		 */
-		if (NULL != cell_is_array (cell))
+		if (cell_is_array (cell)) {
+			/* If this is part of an array we need to remove the
+			 * '{' '}' and the size information from the display.
+			 * That is not actually part of the parsable expression.
+			 */
+			set_text = TRUE;
+		} else if (!cell_has_expr (cell) && VALUE_IS_FLOAT (cell->value)) {
+			GOFormat *fmt = cell_get_format (cell);
+			gnm_float f = value_get_as_float (cell->value);
+
+			switch (fmt->family) {
+			case GO_FORMAT_FRACTION:
+				text = cell_get_entered_text (cell);
+				g_strchug (text);
+				g_strchomp (text);
+				set_text = TRUE;
+				break;
+
+			case GO_FORMAT_PERCENTAGE: {
+				GString *new_str = g_string_new (NULL);
+				gnm_fmt_general_float (new_str, f * 100, -1);
+				cursor_pos = g_utf8_strlen (new_str->str, -1);
+				g_string_append_c (new_str, '%');
+				text = g_string_free (new_str, FALSE);
+				set_text = TRUE;
+				break;
+			}
+
+			case GO_FORMAT_NUMBER:
+			case GO_FORMAT_CURRENCY:
+			case GO_FORMAT_ACCOUNTING: {
+				GString *new_str = g_string_new (NULL);
+				gnm_fmt_general_float (new_str, f, -1);
+				text = g_string_free (new_str, FALSE);
+				set_text = TRUE;
+				break;
+			}
+
+			case GO_FORMAT_DATE: {
+				GString *fstr = g_string_new (format_month_before_day ()
+							      ? "m/d/yyyy"
+							      : "d/m/yyyy");
+				GOFormat *new_fmt;
+
+				if (!close_to_int (f, 1e-6 / (24 * 60 * 60))) {
+					g_string_append_c (fstr, ' ');
+					guess_time_format (fstr, f - gnm_floor (f));
+				}
+				new_fmt = go_format_new_from_XL (fstr->str, FALSE);
+				g_string_free (fstr, TRUE);
+
+				text = format_value (new_fmt, cell->value, NULL, -1,
+						     workbook_date_conv (sv->sheet->workbook));
+				set_text = TRUE;
+				go_format_unref (new_fmt);
+				break;
+			}
+
+			case GO_FORMAT_TIME:
+				if (f >= 0 && f <= 1) {
+					GString *fstr = g_string_new (NULL);
+					GOFormat *new_fmt;
+
+					guess_time_format (fstr, f);
+					new_fmt = go_format_new_from_XL (fstr->str, FALSE);
+					g_string_free (fstr, TRUE);
+
+					text = format_value (new_fmt, cell->value, NULL, -1,
+							     workbook_date_conv (sv->sheet->workbook));
+					set_text = TRUE;
+					go_format_unref (new_fmt);
+				}
+				break;
+
+			default:
+				break;
+			}
+		}
+
+		if (!text)
+			text = cell_get_entered_text (cell);
+
+		if (set_text)
 			gtk_entry_set_text (wbcg_get_entry (wbcg), text);
 
 		if (cell->value != NULL) {
 			GOFormat *fmt = VALUE_FMT (cell->value);
-			if (fmt != NULL && style_format_is_markup (fmt))
+			if (fmt != NULL && go_format_is_markup (fmt))
 				wbcg_edit_init_markup (wbcg,
 					pango_attr_list_copy (fmt->markup));
 		}
@@ -758,14 +934,9 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 		wbcg->auto_complete = NULL;
 
 	/* Give the focus to the edit line */
-	if (!cursorp) {
-		GtkEntry *w = wbcg_get_entry (wbcg);
-		gtk_window_set_focus (GTK_WINDOW (wbcg->toplevel), GTK_WIDGET (w));
-		/* the gtk-entry-select-on-focus property of GtkEntry is not
-		 * what we want here.  Rather than trying some gtk-2.2 specific
-		 * magic just undo the selection change here */
-		gtk_editable_select_region (GTK_EDITABLE (w), -1, -1);
-	}
+	if (!cursorp)
+		gtk_window_set_focus (wbcg_toplevel (wbcg),
+			(GtkWidget *) wbcg_get_entry (wbcg));
 
 	wbcg->wb_control.editing = TRUE;
 	wbcg->wb_control.editing_sheet = sv->sheet;
@@ -800,6 +971,9 @@ wbcg_edit_start (WorkbookControlGUI *wbcg,
 	wb_control_update_action_sensitivity (WORKBOOK_CONTROL (wbcg));
 
 	inside_editing = FALSE;
+
+	gtk_editable_set_position (GTK_EDITABLE (wbcg_get_entry (wbcg)), cursor_pos);
+
 	return TRUE;
 }
 
