@@ -2,7 +2,7 @@
 /*
  * workbook-view.c: View functions for the workbook
  *
- * Copyright (C) 2000-2004 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2000-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,7 +24,7 @@
 #include "workbook-view.h"
 
 #include "workbook-control-priv.h"
-#include "workbook.h"
+#include "workbook-priv.h"
 #include "application.h"
 #include "sheet.h"
 #include "sheet-view.h"
@@ -47,6 +47,7 @@
 #include "auto-format.h"
 
 #include <goffice/app/file.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/app/io-context.h>
 #include <gsf/gsf.h>
 #include <gsf/gsf-impl-utils.h>
@@ -58,6 +59,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
+#include <stdlib.h>
 #include "mathfunc.h"
 #include <goffice/utils/go-file.h>
 
@@ -66,13 +68,51 @@ enum {
 	LAST_SIGNAL
 };
 
+/**
+ * wb_view_get_workbook :
+ * @wbv : #WorkbookView
+ *
+ * Return the #Workbook assciated with @wbv
+ **/
 Workbook *
-wb_view_workbook (WorkbookView const *wbv)
+wb_view_get_workbook (WorkbookView const *wbv)
 {
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), NULL);
 	return wbv->wb;
 }
 
+/**
+ * wb_view_get_doc :
+ * @wbv : #WorkbookView
+ *
+ * Return the #Workbook assciated with @wbv cast to a #GODoc
+ **/
+GODoc *
+wb_view_get_doc (WorkbookView const *wbv)
+{
+	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), NULL);
+	return GO_DOC (wbv->wb);
+}
+
+/**
+ * wb_view_get_index_in_wb :
+ * @wbv : #WorkbookView
+ *
+ * Returns 0 based index of wbv within workbook, or -1 if there is no workbook.
+ **/
+int
+wb_view_get_index_in_wb (WorkbookView const *wbv)
+{
+	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), -1);
+	if (NULL != wbv->wb) {
+		unsigned i = wbv->wb->wb_views->len;
+		while (i-- > 0)
+			if (g_ptr_array_index (wbv->wb->wb_views, i) == wbv)
+				return i;
+	}
+	return -1;
+}
+
 Sheet *
 wb_view_cur_sheet (WorkbookView const *wbv)
 {
@@ -133,21 +173,6 @@ wb_view_sheet_add (WorkbookView *wbv, Sheet *new_sheet)
 	g_object_unref (new_view);
 }
 
-void
-wb_view_sheet_remove (WorkbookView *wbv, Sheet *sheet)
-{
-	SheetView *sv;
-
-	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
-
-	WORKBOOK_VIEW_FOREACH_CONTROL (wbv, control,
-		wb_control_sheet_remove (control, sheet););
-
-	sv = sheet_get_view (sheet, wbv);
-	if (sv)
-		sv_dispose (sv);
-}
-
 gboolean
 wb_view_is_protected (WorkbookView *wbv, gboolean check_sheet)
 {
@@ -206,7 +231,8 @@ wb_view_format_feedback (WorkbookView *wbv, gboolean display)
 	if (sv == NULL)
 		return;
 
-	style = sheet_style_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row);
+	style = (GnmStyle *)sheet_style_get (sv->sheet,
+		sv->edit_pos.col, sv->edit_pos.row);
 	sf_style = gnm_style_get_format (style);
 	if (go_format_is_general (sf_style) &&
 	    (cell = sheet_cell_get (sv->sheet, sv->edit_pos.col, sv->edit_pos.row)) &&
@@ -314,31 +340,48 @@ wb_view_edit_line_set (WorkbookView *wbv, WorkbookControl *optional_wbc)
 
 	sv = wbv->current_sheet_view;
 	if (sv != NULL) {
-		GnmCell *cell;
 		char    *text;
-		GnmExprArray const *ar;
+		GnmCell const *cell = sheet_cell_get (sv->sheet,
+			sv->edit_pos.col, sv->edit_pos.row);
 
-		cell = sheet_cell_get (sv->sheet,
-				       sv->edit_pos.col,
-				       sv->edit_pos.row);
-
-		if (cell) {
+		if (NULL != cell) {
 			text = cell_get_entered_text (cell);
-			/* If this is part of an array we add '{' '}' and size
-			 * information to the display.  That is not actually
-			 * part of the parsable expression, but it is a useful
-			 * extension to the simple '{' '}' that MS excel(tm)
-			 * uses.
-			 */
-			if (NULL != (ar = cell_is_array(cell))) {
-				/* No need to worry about locale for the comma
-				 * this syntax is not parsed
-				 */
-				char *tmp = g_strdup_printf (
-					"{%s}(%d,%d)[%d][%d]", text,
-					ar->rows, ar->cols, ar->y, ar->x);
-				g_free (text);
-				text = tmp;
+
+			if (cell_has_expr (cell)) {
+				GnmExpr const *expr = cell->base.texpr->expr;
+				GnmCell const *corner;
+				int x = 0, y = 0;
+				char *tmp;
+
+				/* If this is part of an array we add '{' '}'
+				 * and size information to the display.  That
+				 * is not actually part of the parsable
+				 * expression, but it is a useful extension to
+				 * the simple '{' '}' that MS excel(tm) uses. */
+				switch (GNM_EXPR_GET_OPER (expr)) {
+				case GNM_EXPR_OP_ARRAY_ELEM :
+					corner = sheet_cell_get (cell->base.sheet,
+						cell->pos.col - (x = expr->array_elem.x),
+						cell->pos.row - (y = expr->array_elem.y));
+
+					g_return_if_fail (corner != NULL);
+					g_return_if_fail (cell_has_expr (corner));
+
+					expr = corner->base.texpr->expr;
+
+				case GNM_EXPR_OP_ARRAY_CORNER :
+					tmp = g_strdup_printf ("{%s}(%d%c%d)[%d][%d]", text,
+						expr->array_corner.cols,
+						format_get_arg_sep (),
+						expr->array_corner.rows,
+						y, x);
+					g_free (text);
+					text = tmp;
+					break;
+
+				default :
+					break;
+				}
 			}
 		} else
 			text = g_strdup ("");
@@ -359,7 +402,7 @@ wb_view_auto_expr (WorkbookView *wbv, char const *descr, char const *func_name)
 	if (wbv->auto_expr_desc)
 		g_free (wbv->auto_expr_desc);
 	if (wbv->auto_expr)
-		gnm_expr_unref (wbv->auto_expr);
+		gnm_expr_free (wbv->auto_expr);
 
 	wbv->auto_expr_desc = g_strdup (descr);
 	wbv->auto_expr = gnm_expr_new_funcall (
@@ -415,6 +458,9 @@ wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display)
 	GnmExprList	*selection = NULL;
 	GnmValue	*v;
 	SheetView	*sv;
+	int             i, argc;
+	GnmExprConstPtr *argv;
+	GnmExprList     *l;
 
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 	g_return_if_fail (wbv->auto_expr != NULL);
@@ -423,12 +469,17 @@ wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display)
 	if (sv == NULL)
 		return;
 
-	selection_apply (sv, &accumulate_regions, FALSE, &selection);
+	sv_selection_apply (sv, &accumulate_regions, FALSE, &selection);
 
 	ei.pos = eval_pos_init_sheet (&ep, wbv->current_sheet);
 	ei.func_call = (GnmExprFunction const *)wbv->auto_expr;
 
-	v = function_call_with_list (&ei, selection, 0);
+	argc = gnm_expr_list_length (selection);
+	argv = g_new (GnmExprConstPtr, argc);
+	for (i = 0, l = selection; i < argc; i++, l = l->next)
+		argv[i] = l->data;
+	v = function_call_with_exprs (&ei, argc, argv, 0);
+	g_free (argv);
 
 	g_free (wbv->auto_expr_value_as_string);
 
@@ -441,19 +492,20 @@ wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display)
 		if (!wbv->auto_expr_use_max_precision) {
 			format = VALUE_FMT (v);
 			if (!format) {
-				const GnmExpr *fcall =
-					gnm_expr_new_funcall (gnm_expr_get_func_def (wbv->auto_expr),
-							      selection);
+				const GnmExprTop *fcall =
+					gnm_expr_top_new
+					(gnm_expr_new_funcall (gnm_expr_get_func_def (wbv->auto_expr),
+							       selection));
 				selection = NULL;
 				format = tmp_format =
 					auto_style_format_suggest (fcall, ei.pos);
-				gnm_expr_unref (fcall);
+				gnm_expr_top_unref (fcall);
 			}
 		}
 
 		if (format) {
 			format_value_gstring (str, format, v, NULL,
-					      -1, workbook_date_conv (wb_view_workbook (wbv)));
+					      -1, workbook_date_conv (wb_view_get_workbook (wbv)));
 			if (tmp_format)
 				go_format_unref (tmp_format);
 		} else {
@@ -474,7 +526,6 @@ wb_view_auto_expr_recalc (WorkbookView *wbv, gboolean display)
 static void
 wb_view_init_control (WorkbookControl *wbc)
 {
-	wb_control_update_title (wbc);
 }
 
 void
@@ -482,12 +533,12 @@ wb_view_attach_control (WorkbookView *wbv, WorkbookControl *wbc)
 {
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
 	g_return_if_fail (IS_WORKBOOK_CONTROL (wbc));
-	g_return_if_fail (wbc->wb_view == NULL);
+	g_return_if_fail (wb_control_view (wbc) == NULL);
 
 	if (wbv->wb_controls == NULL)
 		wbv->wb_controls = g_ptr_array_new ();
 	g_ptr_array_add (wbv->wb_controls, wbc);
-	wbc->wb_view = wbv;
+	g_object_set (G_OBJECT (wbc), "view", wbv, NULL);
 
 	if (wbv->wb != NULL)
 		wb_view_init_control (wbc);
@@ -497,14 +548,14 @@ void
 wb_view_detach_control (WorkbookControl *wbc)
 {
 	g_return_if_fail (IS_WORKBOOK_CONTROL (wbc));
-	g_return_if_fail (IS_WORKBOOK_VIEW (wbc->wb_view));
+	g_return_if_fail (IS_WORKBOOK_VIEW (wb_control_view (wbc)));
 
 	g_ptr_array_remove (wbc->wb_view->wb_controls, wbc);
 	if (wbc->wb_view->wb_controls->len == 0) {
 		g_ptr_array_free (wbc->wb_view->wb_controls, TRUE);
 		wbc->wb_view->wb_controls = NULL;
 	}
-	wbc->wb_view = NULL;
+	g_object_set (G_OBJECT (wbc), "view", NULL, NULL);
 }
 
 static GObjectClass *parent_class;
@@ -520,14 +571,14 @@ wb_view_finalize (GObject *object)
 			g_object_unref (G_OBJECT (control));
 		});
 		if (wbv->wb_controls != NULL)
-			g_warning ("Unexpected left over controls");
+			g_warning ("Unexpected left-over controls");
 	}
 
 	if (wbv->wb != NULL)
 		workbook_detach_view (wbv);
 
 	if (wbv->auto_expr) {
-		gnm_expr_unref (wbv->auto_expr);
+		gnm_expr_free (wbv->auto_expr);
 		wbv->auto_expr = NULL;
 	}
 	if (wbv->auto_expr_desc) {
@@ -621,8 +672,8 @@ wbv_save_to_uri (WorkbookView *wbv, GOFileSaver const *fs,
 
 	if (output != NULL) {
 		GError const *save_err;
-
 		g_printerr ("Writing %s\n", uri);
+
 		go_file_saver_save (fs, io_context, wbv, output);
 		if (!gsf_output_is_closed (output))
 			gsf_output_close (output);
@@ -672,7 +723,8 @@ wb_view_save_as (WorkbookView *wbv, GOFileSaver *fs, char const *uri,
 	g_return_val_if_fail (uri != NULL, FALSE);
 	g_return_val_if_fail (IS_GO_CMD_CONTEXT (context), FALSE);
 
-	wb = wb_view_workbook (wbv);
+	wb = wb_view_get_workbook (wbv);
+	g_object_ref (wb);
 	io_context = gnumeric_io_context_new (context);
 
 	go_cmd_context_set_sensitive (context, FALSE);
@@ -684,12 +736,13 @@ wb_view_save_as (WorkbookView *wbv, GOFileSaver *fs, char const *uri,
 	if (!has_error) {
 		if (workbook_set_saveinfo (wb,
 			go_file_saver_get_format_level (fs), fs) &&
-		    workbook_set_uri (wb, uri))
-			workbook_set_dirty (wb, FALSE);
+		    go_doc_set_uri (GO_DOC (wb), uri))
+			go_doc_set_dirty (GO_DOC (wb), FALSE);
 	}
 	if (has_error || has_warning)
 		gnumeric_io_error_display (io_context);
 	g_object_unref (G_OBJECT (io_context));
+	g_object_unref (wb);
 
 	return !has_error;
 }
@@ -716,7 +769,9 @@ wb_view_save (WorkbookView *wbv, GOCmdContext *context)
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), FALSE);
 	g_return_val_if_fail (IS_GO_CMD_CONTEXT (context), FALSE);
 
-	wb = wb_view_workbook (wbv);
+	wb = wb_view_get_workbook (wbv);
+	g_object_ref (wb);
+
 	fs = workbook_get_file_saver (wb);
 	if (fs == NULL)
 		fs = go_file_saver_get_default ();
@@ -726,18 +781,19 @@ wb_view_save (WorkbookView *wbv, GOCmdContext *context)
 		go_cmd_context_error_export (GO_CMD_CONTEXT (io_context),
 			_("Default file saver is not available."));
 	else {
-		char const *uri = workbook_get_uri (wb);
+		char const *uri = go_doc_get_uri (GO_DOC (wb));
 		wbv_save_to_uri (wbv, fs, uri, io_context);
 	}
 
 	has_error   = gnumeric_io_error_occurred (io_context);
 	has_warning = gnumeric_io_warning_occurred (io_context);
 	if (!has_error)
-		workbook_set_dirty (wb, FALSE);
+		go_doc_set_dirty (GO_DOC (wb), FALSE);
 	if (has_error || has_warning)
 		gnumeric_io_error_display (io_context);
 
 	g_object_unref (G_OBJECT (io_context));
+	g_object_unref (wb);
 
 	return !has_error;
 }
@@ -798,7 +854,7 @@ cb_cleanup_sendto (gpointer path)
 gboolean
 wb_view_sendto (WorkbookView *wbv, GOCmdContext *context)
 {
-	gboolean problem;
+	gboolean problem = FALSE;
 	IOContext	*io_context;
 	Workbook	*wb;
 	GOFileSaver	*fs;
@@ -806,7 +862,8 @@ wb_view_sendto (WorkbookView *wbv, GOCmdContext *context)
 	g_return_val_if_fail (IS_WORKBOOK_VIEW (wbv), FALSE);
 	g_return_val_if_fail (IS_GO_CMD_CONTEXT (context), FALSE);
 
-	wb = wb_view_workbook (wbv);
+	wb = wb_view_get_workbook (wbv);
+	g_object_ref (wb);
 	fs = workbook_get_file_saver (wb);
 	if (fs == NULL)
 		fs = go_file_saver_get_default ();
@@ -814,7 +871,7 @@ wb_view_sendto (WorkbookView *wbv, GOCmdContext *context)
 	io_context = gnumeric_io_context_new (context);
 	if (fs != NULL) {
 		char *template, *full_name, *uri;
-		char *basename = g_path_get_basename (workbook_get_uri (wb));
+		char *basename = g_path_get_basename (go_doc_get_uri (GO_DOC (wb)));
 
 #define GNM_SEND_DIR	".gnm-sendto-"
 #ifdef HAVE_MKDTEMP
@@ -871,7 +928,7 @@ wb_view_sendto (WorkbookView *wbv, GOCmdContext *context)
 			/* mutt does not handle urls with no destination
 			 * so pick something to arbitrary */
 			GError *err = NULL;
-			char *url, *tmp = go_url_encode (full_name);
+			char *url, *tmp = go_url_encode (full_name, 0);
 			url = g_strdup_printf ("mailto:someone?attach=%s", tmp);
 			g_free (tmp);
 #ifdef WITH_GNOME
@@ -904,6 +961,7 @@ wb_view_sendto (WorkbookView *wbv, GOCmdContext *context)
 
  out:
 	g_object_unref (G_OBJECT (io_context));
+	g_object_unref (wb);
 
 	return !problem;
 }
@@ -963,10 +1021,10 @@ wb_view_new_from_input  (GsfInput *input,
 		gboolean old;
 
 		new_wbv = workbook_view_new (NULL);
-		new_wb = wb_view_workbook (new_wbv);
+		new_wb = wb_view_get_workbook (new_wbv);
 		if (NULL != (input_name = gsf_input_name (input))) {
 			char *uri = go_shell_arg_to_uri (input_name);
-			workbook_set_uri (new_wb, uri);
+			go_doc_set_uri (GO_DOC (new_wb), uri);
 			g_free (uri);
 		}
 
@@ -985,7 +1043,7 @@ wb_view_new_from_input  (GsfInput *input,
 			new_wbv = NULL;			
 		} else {
 			workbook_recalc (new_wb);
-			workbook_set_dirty (new_wb, FALSE);
+			go_doc_set_dirty (GO_DOC (new_wb), FALSE);
 		}
 	} else
 		go_cmd_context_error_import (GO_CMD_CONTEXT (io_context),
