@@ -6,7 +6,6 @@
 #define IS_WORKBOOK(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), WORKBOOK_TYPE))
 
 #include "gnumeric.h"
-#include "summary.h"
 #include <goffice/app/file.h>
 #include <glib-object.h>
 
@@ -21,7 +20,7 @@ Sheet      *workbook_sheet_by_index	 (Workbook const *wb, int i);
 Sheet      *workbook_sheet_by_name       (Workbook const *wb, char const *sheet_name);
 void        workbook_sheet_attach        (Workbook *wb, Sheet *new_sheet);
 void        workbook_sheet_attach_at_pos (Workbook *wb, Sheet *new_sheet, int pos);
-Sheet	   *workbook_sheet_add		 (Workbook *wb, int pos, gboolean make_dirty);
+Sheet	   *workbook_sheet_add		 (Workbook *wb, int pos);
 void        workbook_sheet_delete        (Sheet *sheet);
 void        workbook_sheet_move          (Sheet *sheet, int direction);
 char       *workbook_sheet_get_free_name (Workbook *wb,
@@ -30,49 +29,20 @@ char       *workbook_sheet_get_free_name (Workbook *wb,
 					  gboolean handle_counter);
 gboolean    workbook_sheet_reorder       (Workbook *wb,
 					  GSList *new_order);
-gboolean    workbook_sheet_reorder_by_idx(Workbook *wb,
-					  GSList *new_order);
-gboolean    workbook_sheet_recolor       (Workbook *wb,
-					  GSList *sheets,
-					  GSList *fore,
-					  GSList *back);
 gboolean    workbook_sheet_rename        (Workbook *wb,
 					  GSList *sheet_indices,
 					  GSList *new_names,
 					  GOCmdContext *cc);
-gboolean    workbook_sheet_rename_check  (Workbook *wb,
-					  GSList *sheet_indices,
-					  GSList *new_names,
-					  GSList *sheet_indices_deleted,
-					  GOCmdContext *cc);
-gboolean    workbook_sheet_change_protection  (Workbook *wb,
-					       GSList *sheets,
-					       GSList *locks);
-gboolean    workbook_sheet_change_visibility  (Workbook *wb,
-					       GSList *sheets,
-					       GSList *visibility);
 
 unsigned    workbook_find_command	(Workbook *wb,
 					 gboolean is_undo, gpointer cmd);
 
 
 /* IO Routines */
-gboolean       workbook_set_uri		(Workbook *wb, char const *uri);
-char const    *workbook_get_uri		(Workbook const *wb);
-
 gboolean       workbook_set_saveinfo	(Workbook *wb,
 					 FileFormatLevel, GOFileSaver *);
 GOFileSaver *workbook_get_file_saver	(Workbook *wb);
 
-gboolean    workbook_is_pristine	(Workbook const *wb);
-void        workbook_set_dirty		(Workbook *wb, gboolean is_dirty);
-gboolean    workbook_is_dirty		(Workbook const *wb);
-void        workbook_set_placeholder	(Workbook *wb, gboolean is_placeholder);
-gboolean    workbook_is_placeholder	(Workbook const *wb);
-
-void         workbook_add_summary_info    (Workbook *wb, SummaryItem *sit);
-SummaryInfo *workbook_metadata    	  (Workbook const *wb);
-
 /* See also sheet_cell_foreach_range */
 GnmValue   *workbook_foreach_cell_in_range (GnmEvalPos const  *pos,
 					    GnmValue const *cell_range,
