@@ -37,7 +37,6 @@
 #include "validation.h"
 #include "hlink.h"
 #include "input-msg.h"
-#include "gnm-format.h"
 #include "cell.h"
 #include "position.h"
 #include "expr.h"
@@ -60,6 +59,7 @@
 #include <goffice/app/go-plugin.h>
 #include <goffice/app/error-info.h>
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-format.h>
 
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-input.h>
@@ -263,7 +263,7 @@ typedef struct {
 	struct {
 		char            *title;
 		char            *msg;
-		GnmExpr	const	*expr [2];
+		GnmExprTop const *texpr[2];
 		ValidationStyle  style;
 		ValidationType	 type;
 		ValidationOp	 op;
@@ -290,6 +290,7 @@ typedef struct {
 	int outline_symbols_below;
 	int outline_symbols_right;
 	int text_is_rtl;
+	int is_protected;
 	GnmSheetVisibility visibility;
 	GnmColor *tab_color;
 
@@ -453,7 +454,7 @@ xml_sax_attr_elem (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 	default :
 		g_assert_not_reached ();
-	};
+	}
 }
 
 static void
@@ -466,9 +467,10 @@ xml_sax_sheet_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	GnmColor *color = NULL;
 
 	state->hide_col_header = state->hide_row_header =
-	state->display_formulas = state->hide_zero =
-	state->hide_grid = state->display_outlines =
-	state->outline_symbols_below = state->outline_symbols_right = state->text_is_rtl = -1;
+		state->display_formulas = state->hide_zero =
+		state->hide_grid = state->display_outlines =
+		state->outline_symbols_below = state->outline_symbols_right =
+		state->text_is_rtl = state->is_protected = -1;
 	state->visibility = GNM_SHEET_VISIBILITY_VISIBLE;
 	state->tab_color = NULL;
 	state->sheet_zoom = 1.; /* default */
@@ -494,6 +496,8 @@ xml_sax_sheet_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 			state->visibility = tmpi;
 		else if (xml_sax_attr_bool (attrs, "RTL_Layout", &tmp))
 			state->text_is_rtl = tmp;
+		else if (xml_sax_attr_bool (attrs, "Protected", &tmp))
+			state->is_protected = tmp;
 		else if (xml_sax_attr_color (attrs, "TabColor", &color))
 			state->tab_color = color;
 		else
@@ -670,10 +674,7 @@ xml_sax_selection_range (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	GnmRange r;
 	if (xml_sax_attr_range (attrs, &r))
 		sv_selection_add_range (
-			sheet_get_view (state->sheet, state->wb_view),
-			r.start.col, r.start.row,
-			r.start.col, r.start.row,
-			r.end.col, r.end.row);
+			sheet_get_view (state->sheet, state->wb_view), &r);
 }
 
 static void
@@ -899,11 +900,9 @@ xml_sax_styleregion_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 		else if (!strcmp (attrs[0], "Format"))
 			gnm_style_set_format_text (state->style, (char *)attrs[1]);
 		else if (xml_sax_attr_int (attrs, "Hidden", &val))
-			gnm_style_set_content_hidden (state->style, val);
+			gnm_style_set_contents_hidden (state->style, val);
 		else if (xml_sax_attr_int (attrs, "Locked", &val))
-			gnm_style_set_content_locked (state->style, val);
-		else if (xml_sax_attr_int (attrs, "Locked", &val))
-			gnm_style_set_content_locked (state->style, val);
+			gnm_style_set_contents_locked (state->style, val);
 		else if (xml_sax_attr_int (attrs, "Orient", &val))
 			; /* ignore old useless attribute */
 		else
@@ -1014,8 +1013,8 @@ xml_sax_validation (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	g_return_if_fail (state->validation.title == NULL);
 	g_return_if_fail (state->validation.msg == NULL);
-	g_return_if_fail (state->validation.expr[0] == NULL);
-	g_return_if_fail (state->validation.expr[1] == NULL);
+	g_return_if_fail (state->validation.texpr[0] == NULL);
+	g_return_if_fail (state->validation.texpr[1] == NULL);
 
 	state->validation.style = VALIDATION_STYLE_NONE;
 	state->validation.type = VALIDATION_TYPE_ANY;
@@ -1056,8 +1055,8 @@ xml_sax_validation_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 				state->validation.op,
 				state->validation.title,
 				state->validation.msg,
-				state->validation.expr[0],
-				state->validation.expr[1],
+				state->validation.texpr[0],
+				state->validation.texpr[1],
 				state->validation.allow_blank,
 				state->validation.use_dropdown));
 
@@ -1065,7 +1064,7 @@ xml_sax_validation_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	state->validation.title = NULL;
 	g_free (state->validation.msg);
 	state->validation.msg = NULL;
-	state->validation.expr[0] = state->validation.expr[1] = NULL;
+	state->validation.texpr[0] = state->validation.texpr[1] = NULL;
 }
 
 static void
@@ -1074,17 +1073,17 @@ xml_sax_validation_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int const i = gsf_state->node->user_data.v_int;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	GnmParsePos pos;
 
-	g_return_if_fail (state->validation.expr [i] == NULL);
+	g_return_if_fail (state->validation.texpr[i] == NULL);
 
-	expr = gnm_expr_parse_str_simple (gsf_state->content->str,
+	texpr = gnm_expr_parse_str_simple (gsf_state->content->str,
 		parse_pos_init_sheet (&pos, state->sheet));
 
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 
-	state->validation.expr [i] = expr;
+	state->validation.texpr[i] = texpr;
 }
 
 static void
@@ -1094,8 +1093,8 @@ xml_sax_condition (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	int dummy;
 
-	g_return_if_fail (state->cond.expr[0] == NULL);
-	g_return_if_fail (state->cond.expr[1] == NULL);
+	g_return_if_fail (state->cond.texpr[0] == NULL);
+	g_return_if_fail (state->cond.texpr[1] == NULL);
 
 	state->cond.op = GNM_STYLE_COND_CUSTOM;
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
@@ -1119,7 +1118,7 @@ xml_sax_condition_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 			(sc = gnm_style_conditions_new ()));
 	gnm_style_conditions_insert (sc, &state->cond, -1);
 
-	state->cond.expr[0] = state->cond.expr[1] = NULL;
+	state->cond.texpr[0] = state->cond.texpr[1] = NULL;
 }
 
 static void
@@ -1128,17 +1127,17 @@ xml_sax_condition_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	XMLSaxParseState *state = (XMLSaxParseState *)gsf_state->user_state;
 
 	int const i = gsf_state->node->user_data.v_int;
-	GnmExpr const *expr;
+	GnmExprTop const *texpr;
 	GnmParsePos pos;
 
-	g_return_if_fail (state->cond.expr [i] == NULL);
+	g_return_if_fail (state->cond.texpr[i] == NULL);
 
-	expr = gnm_expr_parse_str_simple (gsf_state->content->str,
+	texpr = gnm_expr_parse_str_simple (gsf_state->content->str,
 		parse_pos_init_sheet (&pos, state->sheet));
 
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 
-	state->cond.expr [i] = expr;
+	state->cond.texpr[i] = texpr;
 }
 
 static void
@@ -1219,9 +1218,11 @@ xml_sax_style_region_borders (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	if (pattern >= STYLE_BORDER_NONE) {
 		GnmStyleElement const type = gsf_state->node->user_data.v_int;
+		StyleBorderLocation const loc =
+			STYLE_BORDER_TOP + (int)(type - MSTYLE_BORDER_TOP);
 		GnmBorder *border =
 			style_border_fetch ((StyleBorderType)pattern, colour,
-					    style_border_get_orientation (type));
+					    style_border_get_orientation (loc));
 		gnm_style_set_border (state->style, type, border);
 	}
 }
@@ -1290,14 +1291,14 @@ xml_cell_set_array_expr (GnmCell *cell, char const *text,
 			 int const cols, int const rows)
 {
 	GnmParsePos pp;
-	GnmExpr const *expr = gnm_expr_parse_str_simple (text,
+	GnmExprTop const *texpr = gnm_expr_parse_str_simple (text,
 		parse_pos_init_cell (&pp, cell));
 
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 	cell_set_array_formula (cell->base.sheet,
 				cell->pos.col, cell->pos.row,
 				cell->pos.col + cols-1, cell->pos.row + rows-1,
-				expr);
+				texpr);
 }
 
 /**
@@ -1362,8 +1363,6 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	int const expr_id = state->expr_id;
 	int const value_type = state->value_type;
 	GOFormat *value_fmt = state->value_fmt;
-	gpointer const id = GINT_TO_POINTER (expr_id);
-	gpointer expr = NULL;
 
 	/* Clean out the state before any error checking */
 	state->cell.row = state->cell.col = -1;
@@ -1382,9 +1381,6 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	if (cell == NULL)
 		return;
 
-	if (expr_id > 0)
-		expr = g_hash_table_lookup (state->expr_map, id);
-
 	is_post_52_array = (array_cols > 0) && (array_rows > 0);
 
 	if (gsf_state->content->len > 0) {
@@ -1411,23 +1407,23 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 		if (expr_id > 0) {
 			gpointer id = GINT_TO_POINTER (expr_id);
-			gpointer expr =
+			GnmExprTop const *texpr =
 				g_hash_table_lookup (state->expr_map, id);
-			if (expr == NULL) {
+			if (texpr == NULL) {
 				if (cell_has_expr (cell))
 					g_hash_table_insert (state->expr_map, id,
-							     (gpointer)cell->base.expression);
+							     (gpointer)cell->base.texpr);
 				else
-					g_warning ("XML-IO : Shared expression with no expession ??");
+					g_warning ("XML-IO : Shared expression with no expression ??");
 			} else if (!is_post_52_array)
 				g_warning ("XML-IO : Duplicate shared expression");
 		}
 	} else if (expr_id > 0) {
-		gpointer expr = g_hash_table_lookup (state->expr_map,
+		GnmExprTop const *texpr = g_hash_table_lookup (state->expr_map,
 			GINT_TO_POINTER (expr_id));
 
-		if (expr != NULL)
-			cell_set_expr (cell, expr);
+		if (texpr != NULL)
+			cell_set_expr (cell, texpr);
 		else
 			g_warning ("XML-IO : Missing shared expression");
 	} else if (is_new_cell)
@@ -1478,7 +1474,7 @@ xml_sax_object_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	/* Class renamed between 1.0.x and 1.2.x */
 	else if (!strcmp (type_name, "GnmGraph"))
-		so = sheet_object_graph_new (FALSE);
+		so = sheet_object_graph_new (NULL);
 
 	/* Class renamed in 1.2.2 */
 	else if (!strcmp (type_name, "CellComment"))
@@ -1555,7 +1551,7 @@ xml_sax_named_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 	GnmParseError  perr;
 	GnmParsePos    pos;
-	GnmExpr const	*expr;
+	GnmExprTop const *texpr;
 
 	g_return_if_fail (state->name.name != NULL);
 	g_return_if_fail (state->name.value != NULL);
@@ -1571,21 +1567,26 @@ xml_sax_named_expr_end (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	}
 
 	parse_error_init (&perr);
-	expr = gnm_expr_parse_str (state->name.value, &pos,
+	texpr = gnm_expr_parse_str (state->name.value, &pos,
 				   GNM_EXPR_PARSE_DEFAULT,
 				   gnm_expr_conventions_default, &perr);
-	if (expr != NULL) {
+	if (texpr != NULL) {
 		char *err = NULL;
-		expr_name_add (&pos, state->name.name, expr, &err, TRUE, NULL);
+		expr_name_add (&pos, state->name.name,
+			       texpr,
+			       &err, TRUE, NULL);
 		if (err != NULL) {
 			gnm_io_warning (state->context, err);
 			g_free (err);
 		}
 	} else
-		state->delayed_names = g_list_prepend (state->delayed_names,
-			expr_name_add (&pos, state->name.name, 
-				gnm_expr_new_constant (value_new_string (state->name.value)),
-				NULL, TRUE, NULL));
+		state->delayed_names =
+			g_list_prepend (state->delayed_names,
+			expr_name_add (&pos,
+				       state->name.name, 
+				       gnm_expr_top_new_constant
+				       (value_new_string (state->name.value)),
+				       NULL, TRUE, NULL));
 
 	parse_error_free (&perr);
 
@@ -1653,7 +1654,7 @@ xml_sax_paper (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 #define GNM	0
 
-static GsfXMLInNS content_ns[] = {
+static GsfXMLInNS const content_ns[] = {
 	GSF_XML_IN_NS (GNM, "http://www.gnumeric.org/v10.dtd"),
 	GSF_XML_IN_NS (GNM, "http://www.gnumeric.org/v9.dtd"),
 	GSF_XML_IN_NS (GNM, "http://www.gnumeric.org/v8.dtd"),
@@ -1933,7 +1934,7 @@ gnm_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 	/* init */
 	state.context = io_context;
 	state.wb_view = wb_view;
-	state.wb = wb_view_workbook (wb_view);
+	state.wb = wb_view_get_workbook (wb_view);
 	state.sheet = NULL;
 	state.version = GNM_XML_UNKNOWN;
 	state.attribute.name = state.attribute.value = NULL;
@@ -1946,7 +1947,7 @@ gnm_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 	state.value_type = -1;
 	state.value_fmt = NULL;
 	state.validation.title = state.validation.msg = NULL;
-	state.validation.expr[0] = state.validation.expr[1] = NULL;
+	state.validation.texpr[0] = state.validation.texpr[1] = NULL;
 	state.expr_map = g_hash_table_new (g_direct_hash, g_direct_equal);
 	state.delayed_names = NULL;
 	state.so = NULL;
