@@ -4,7 +4,7 @@
  * xml-sax-write.c : export .gnumeric and the clipboard subset using a the sax
  * 			like wrappers in libgsf
  *
- * Copyright (C) 2003-2005 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2006 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -34,7 +34,6 @@
 #include <sheet.h>
 #include <sheet-view.h>
 #include <sheet-style.h>
-#include <summary.h>
 #include <style-color.h>
 #include <style-conditions.h>
 #include <expr.h>
@@ -62,6 +61,7 @@
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-output-gzip.h>
 #include <gsf/gsf-doc-meta-data.h>
+#include <gsf/gsf-opendoc-utils.h>
 #include <gsf/gsf-utils.h>
 #include <locale.h>
 
@@ -137,36 +137,10 @@ xml_write_attributes (GnmOutputXML *state)
 }
 
 static void
-xml_write_summary (GnmOutputXML *state)
+xml_write_meta_data (GnmOutputXML *state)
 {
-	SummaryInfo *summary_info = workbook_metadata (state->wb);
-	GList *items, *ptr;
-
-	if (summary_info == NULL)
-		return;
-	items = summary_info_as_list (summary_info);
-	if (items == NULL)
-		return;
-
-	gsf_xml_out_start_element (state->output, GNM "Summary");
-	for (ptr = items ; ptr != NULL ; ptr = ptr->next) {
-		SummaryItem *sit = ptr->data;
-		if (sit == NULL)
-			continue;
-		gsf_xml_out_start_element (state->output, GNM "Item");
-		gsf_xml_out_simple_element (state->output, GNM "name", sit->name);
-		if (sit->type == SUMMARY_INT) {
-			gsf_xml_out_simple_int_element (state->output,
-				GNM "val-int", sit->v.i);
-		} else {
-			char *text = summary_item_as_text (sit);
-			gsf_xml_out_simple_element (state->output, GNM "val-string", text);
-			g_free (text);
-		}
-		gsf_xml_out_end_element (state->output);	/* </Item> */
-	}
-	gsf_xml_out_end_element (state->output); /* </Summary> */
-	g_list_free (items);
+	gsf_opendoc_metadata_write (state->output, 
+		go_doc_get_meta_data (GO_DOC (state->wb)));
 }
 
 static void
@@ -258,7 +232,7 @@ xml_write_print_repeat_range (GnmOutputXML *state,
 	if (range->use) {
 		gsf_xml_out_start_element (state->output, name);
 		gsf_xml_out_add_cstr_unchecked (state->output, "value",
-			range_name (&range->range));
+			range_as_string (&range->range));
 		gsf_xml_out_end_element (state->output);
 	}
 }
@@ -390,10 +364,10 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 		gsf_xml_out_add_int (state->output, "Shade", gnm_style_get_pattern (style));
 	if (gnm_style_is_element_set (style, MSTYLE_INDENT))
 		gsf_xml_out_add_int (state->output, "Indent", gnm_style_get_indent (style));
-	if (gnm_style_is_element_set (style, MSTYLE_CONTENT_LOCKED))
-		gsf_xml_out_add_bool (state->output, "Locked", gnm_style_get_content_locked (style));
-	if (gnm_style_is_element_set (style, MSTYLE_CONTENT_HIDDEN))
-		gsf_xml_out_add_bool (state->output, "Hidden", gnm_style_get_content_hidden (style));
+	if (gnm_style_is_element_set (style, MSTYLE_CONTENTS_LOCKED))
+		gsf_xml_out_add_bool (state->output, "Locked", gnm_style_get_contents_locked (style));
+	if (gnm_style_is_element_set (style, MSTYLE_CONTENTS_HIDDEN))
+		gsf_xml_out_add_bool (state->output, "Hidden", gnm_style_get_contents_hidden (style));
 	if (gnm_style_is_element_set (style, MSTYLE_FONT_COLOR))
 		gnm_xml_out_add_color (state->output, "Fore", gnm_style_get_font_color (style));
 	if (gnm_style_is_element_set (style, MSTYLE_COLOR_BACK))
@@ -476,13 +450,13 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 			gsf_xml_out_add_cstr (state->output, "Message", v->msg->str);
 
 		parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
-		if (v->expr[0] != NULL &&
-		    (tmp = gnm_expr_as_string (v->expr[0], &pp, state->exprconv)) != NULL) {
+		if (v->texpr[0] != NULL &&
+		    (tmp = gnm_expr_top_as_string (v->texpr[0], &pp, state->exprconv)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 			g_free (tmp);
 		}
-		if (v->expr[1] != NULL &&
-		    (tmp = gnm_expr_as_string (v->expr[1], &pp, state->exprconv)) != NULL) {
+		if (v->texpr[1] != NULL &&
+		    (tmp = gnm_expr_top_as_string (v->texpr[1], &pp, state->exprconv)) != NULL) {
 			gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 			g_free (tmp);
 		}
@@ -509,13 +483,13 @@ xml_write_gnmstyle (GnmOutputXML *state, GnmStyle const *style)
 				gsf_xml_out_start_element (state->output, GNM "Condition");
 				gsf_xml_out_add_int (state->output, "Operator", cond->op);
 				parse_pos_init_sheet (&pp, (Sheet *)state->sheet);
-				if (cond->expr[0] != NULL &&
-				    (tmp = gnm_expr_as_string (cond->expr[0], &pp, state->exprconv)) != NULL) {
+				if (cond->texpr[0] != NULL &&
+				    (tmp = gnm_expr_top_as_string (cond->texpr[0], &pp, state->exprconv)) != NULL) {
 					gsf_xml_out_simple_element (state->output, GNM "Expression0", tmp);
 					g_free (tmp);
 				}
-				if (cond->expr[1] != NULL &&
-				    (tmp = gnm_expr_as_string (cond->expr[1], &pp, state->exprconv)) != NULL) {
+				if (cond->texpr[1] != NULL &&
+				    (tmp = gnm_expr_top_as_string (cond->texpr[1], &pp, state->exprconv)) != NULL) {
 					gsf_xml_out_simple_element (state->output, GNM "Expression1", tmp);
 					g_free (tmp);
 				}
@@ -654,8 +628,8 @@ xml_write_cols_rows (GnmOutputXML *state)
 static void
 xml_write_selection_info (GnmOutputXML *state)
 {
-	GList *ptr, *copy;
-	const SheetView *sv = sheet_get_view (state->sheet, state->wb_view);
+	GSList *ptr, *copy;
+	SheetView const *sv = sheet_get_view (state->sheet, state->wb_view);
 	if (!sv) return;  /* Hidden.  */
 
 	gsf_xml_out_start_element (state->output, GNM "Selections");
@@ -663,31 +637,30 @@ xml_write_selection_info (GnmOutputXML *state)
 	gsf_xml_out_add_int (state->output, "CursorRow", sv->edit_pos_real.row);
 
 	/* Insert the selections in REVERSE order */
-	copy = g_list_copy (sv->selections);
-	ptr = g_list_reverse (copy);
+	copy = g_slist_copy (sv->selections);
+	ptr = g_slist_reverse (copy);
 	for (; ptr != NULL ; ptr = ptr->next) {
 		GnmRange const *r = ptr->data;
 		gsf_xml_out_start_element (state->output, GNM "Selection");
 		xml_out_add_range (state->output, r);
 		gsf_xml_out_end_element (state->output); /* </gnm:Selection> */
 	}
-	g_list_free (copy);
+	g_slist_free (copy);
 
 	gsf_xml_out_end_element (state->output); /* </gnm:Selections> */
 }
 
 static void
 xml_write_cell_and_position (GnmOutputXML *state,
-			     GnmExpr const *expr, GnmValue const *val,
+			     GnmExprTop const *texpr, GnmValue const *val,
 			     GnmParsePos const *pp)
 {
-	GnmExprArray const *ar = NULL;
 	gboolean write_contents = TRUE;
-	gboolean const is_shared_expr = (expr != NULL) &&
-		gnm_expr_is_shared (expr);
+	gboolean const is_shared_expr = (texpr != NULL) &&
+		gnm_expr_top_is_shared (texpr);
 
 	/* Only the top left corner of an array needs to be saved (>= 0.53) */
-	if (expr && NULL != (ar = gnm_expr_is_array (expr)) && (ar->y != 0 || ar->x != 0))
+	if (texpr && gnm_expr_top_is_array_elem (texpr))
 		return; /* DOM version would write <Cell Col= Row=/> */
 
 	gsf_xml_out_start_element (state->output, GNM "Cell");
@@ -696,11 +669,11 @@ xml_write_cell_and_position (GnmOutputXML *state,
 
 	/* As of version 0.53 we save the ID of shared expressions */
 	if (is_shared_expr) {
-		gpointer id = g_hash_table_lookup (state->expr_map, (gpointer) expr);
+		gpointer id = g_hash_table_lookup (state->expr_map, (gpointer) texpr);
 
 		if (id == NULL) {
 			id = GINT_TO_POINTER (g_hash_table_size (state->expr_map) + 1);
-			g_hash_table_insert (state->expr_map, (gpointer)expr, id);
+			g_hash_table_insert (state->expr_map, (gpointer)texpr, id);
 		} else
 			write_contents = FALSE;
 
@@ -709,15 +682,15 @@ xml_write_cell_and_position (GnmOutputXML *state,
 
 	/* As of version 0.53 we save the size of the array as attributes */
 	/* As of version 0.57 the attributes are in the Cell not the Content */
-	if (ar != NULL) {
-	        gsf_xml_out_add_int (state->output, "Rows", ar->rows);
-	        gsf_xml_out_add_int (state->output, "Cols", ar->cols);
+	if (texpr && gnm_expr_top_get_array_corner (texpr)) {
+	        gsf_xml_out_add_int (state->output, "Rows", texpr->expr->array_corner.rows);
+	        gsf_xml_out_add_int (state->output, "Cols", texpr->expr->array_corner.cols);
 	}
 
 	if (write_contents) {
 		GString *str = g_string_sized_new (1000);
 
-		if (NULL == expr) {
+		if (!texpr) {
 			if (val != NULL) {
 				gsf_xml_out_add_int (state->output, "ValueType", val->type);
 				if (VALUE_FMT (val) != NULL) {
@@ -731,7 +704,7 @@ xml_write_cell_and_position (GnmOutputXML *state,
 			}
 		} else {
 			g_string_append_c (str, '=');
-			gnm_expr_as_gstring (str, expr, pp, state->exprconv);
+			gnm_expr_top_as_gstring (str, texpr, pp, state->exprconv);
 		}
 
 		gsf_xml_out_add_cstr (state->output, NULL, str->str);
@@ -744,7 +717,7 @@ static GnmValue *
 cb_write_cell (Sheet *sheet, int col, int row, GnmCell const *cell, GnmOutputXML *state)
 {
 	GnmParsePos pp;
-	xml_write_cell_and_position (state, cell->base.expression, cell->value,
+	xml_write_cell_and_position (state, cell->base.texpr, cell->value,
 		parse_pos_init_cell (&pp, cell));
 	return NULL;
 }
@@ -768,7 +741,7 @@ xml_write_merged_regions (GnmOutputXML *state)
 	gsf_xml_out_start_element (state->output, GNM "MergedRegions");
 	for (; ptr != NULL ; ptr = ptr->next)
 		gsf_xml_out_simple_element (state->output,
-			GNM "Merge", range_name (ptr->data));
+			GNM "Merge", range_as_string (ptr->data));
 	gsf_xml_out_end_element (state->output); /* </gnm:MergedRegions> */
 }
 
@@ -862,7 +835,7 @@ xml_write_sheet_filters (GnmOutputXML *state)
 		filter = ptr->data;
 		gsf_xml_out_start_element (state->output, GNM "Filter");
 		gsf_xml_out_add_cstr_unchecked (state->output, "Area",
-			range_name (&filter->r));
+			range_as_string (&filter->r));
 
 		for (i = filter->fields->len ; i-- > 0 ; ) {
 			cond = gnm_filter_get_condition (filter, i);
@@ -1020,7 +993,7 @@ xml_write_objects (GnmOutputXML *state, GSList *ptr)
 
 		tmp = g_strconcat (GNM, type_name, NULL);
 		gsf_xml_out_start_element (state->output, tmp);
-		gsf_xml_out_add_cstr (state->output, "ObjectBound", range_name (&so->anchor.cell_bound));
+		gsf_xml_out_add_cstr (state->output, "ObjectBound", range_as_string (&so->anchor.cell_bound));
 		snprintf (buffer, sizeof (buffer), "%.3g %.3g %.3g %.3g",
 			  so->anchor.offset [0], so->anchor.offset [1],
 			  so->anchor.offset [2], so->anchor.offset [3]);
@@ -1150,7 +1123,6 @@ xml_io_conventions (void)
 	res->range_ref_handler = gnm_1_0_rangeref_as_string;
 	res->range_sep_colon = TRUE;
 	res->sheet_sep_exclamation = TRUE;
-	res->dots_in_names = TRUE;
 	res->output_sheet_name_sep = "!";
 	res->output_argument_sep  = ",";
 	res->output_array_col_sep = ",";
@@ -1183,7 +1155,7 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 	}
 
 	state.wb_view	= wb_view;
-	state.wb	= wb_view_workbook (wb_view);
+	state.wb	= wb_view_get_workbook (wb_view);
 	state.sheet	= NULL;
 	state.output	= gsf_xml_out_new (output);
 	state.exprconv	= xml_io_conventions ();
@@ -1205,7 +1177,7 @@ gnm_xml_file_save (GOFileSaver const *fs, IOContext *io_context,
 
 	xml_write_version	    (&state);
 	xml_write_attributes	    (&state);
-	xml_write_summary	    (&state);
+	xml_write_meta_data	    (&state);
 	xml_write_conventions	    (&state);
 	xml_write_sheet_names	    (&state);
 	xml_write_named_expressions (&state, state.wb->names);
@@ -1277,7 +1249,7 @@ gnm_cellregion_to_xml (GnmCellRegion const *cr)
 	gsf_xml_out_add_int (state.output, "Rows", cr->rows);
 	gsf_xml_out_add_int (state.output, "BaseCol", cr->base.col);
 	gsf_xml_out_add_int (state.output, "BaseRow", cr->base.row);
-	if (cr->not_as_content)
+	if (cr->not_as_contents)
 		gsf_xml_out_add_bool (state.output, "NotAsContent", TRUE);
 
 	if (cr->styles != NULL) {
@@ -1292,20 +1264,20 @@ gnm_cellregion_to_xml (GnmCellRegion const *cr)
 		for (ptr = cr->merged ; ptr != NULL ; ptr = ptr->next) {
 			gsf_xml_out_start_element (state.output, GNM "Merge");
 			gsf_xml_out_add_cstr_unchecked (state.output, NULL,
-				range_name (ptr->data));
+				range_as_string (ptr->data));
 			gsf_xml_out_end_element (state.output); /* </Merge> */
 		}
 	}
 
 	pp.wb = NULL; /* NOTE SNEAKY : ensure that sheet names have explicit workbooks */
 	pp.sheet = cr->origin_sheet;
-	if (cr->content != NULL) {
+	if (cr->contents != NULL) {
 		gsf_xml_out_start_element (state.output, GNM "Cells");
-		for (ptr = cr->content; ptr != NULL ; ptr = ptr->next) {
+		for (ptr = cr->contents; ptr != NULL ; ptr = ptr->next) {
 			cc = ptr->data;
 			pp.eval.col = cr->base.col + cc->col_offset,
 			pp.eval.row = cr->base.row + cc->row_offset;
-			xml_write_cell_and_position (&state, cc->expr, cc->val, &pp);
+			xml_write_cell_and_position (&state, cc->texpr, cc->val, &pp);
 		}
 		gsf_xml_out_end_element (state.output); /* </Cells> */
 	}
